import { Configuration } from './configuration';
import { AxiosPromise, AxiosInstance, AxiosRequestConfig } from 'axios';
import { RequestArgs, BaseAPI } from './base';
export interface ATimeZone {
    'id': string;
}
export interface ATimeZoneWithOffset {
    'id': string;
    'offset': number;
}
export interface AbsenceApproval {
    'approved': boolean;
    'approvedBy': TDMemberProfile;
    'approvedAt': string;
}
export interface AbsenceApprovalWebhookEvent {
    'meta': KMetaMod;
    'absence': AbsenceRecord;
    'member': TDMemberProfile;
    'approved': AbsenceApprovalWebhookEventApproved;
}
export interface AbsenceApprovalWebhookEventApproved {
    'old': boolean;
    'new': boolean;
}
export interface AbsenceCommonSubscriptionFilter {
    'reasons': Array<AbsenceReasonRecord>;
}
export interface AbsenceCommonSubscriptionFilterIn {
    'reasons': Array<string>;
}
export interface AbsenceEvent {
    'profile': TDMemberWithTeam;
    'events': Array<AbsenceRecord>;
}
export interface AbsenceIdentifier {
    'className': string;
    'id': string;
}
export interface AbsenceIdentifierId {
    'id': string;
}
export declare const AbsenceListMode: {
    readonly All: "All";
    readonly WithAccessibleReasonUnapproved: "WithAccessibleReasonUnapproved";
    readonly WithAccessibleReasonAll: "WithAccessibleReasonAll";
};
export type AbsenceListMode = typeof AbsenceListMode[keyof typeof AbsenceListMode];
export interface AbsenceReasonRecord {
    'id': string;
    'archived': boolean;
    'name': string;
    'description': string;
    'defaultAvailability': boolean;
    'approvalRequired': boolean;
    'icon': string;
    'etag'?: number | null;
}
export interface AbsenceRecord {
    'id': string;
    'archived': boolean;
    'member': TDMemberProfile;
    'icon': string;
    'reason'?: AbsenceReasonRecord;
    'description'?: string | null;
    'since': string;
    'till': string;
    'location'?: TDLocation;
    'available': boolean;
    'approval'?: AbsenceApproval;
    'customFields': {
        [key: string]: CFValue | undefined;
    };
}
export interface AbsenceWebhookEvent {
    'meta': KMetaMod;
    'absence': AbsenceRecord;
    'member': TDMemberProfile;
    'icon'?: AbsenceWebhookEventIcon | null;
    'reason'?: AbsenceWebhookEventReason | null;
    'description'?: AbsenceWebhookEventIcon | null;
    'since'?: AbsenceWebhookEventSince | null;
    'till'?: AbsenceWebhookEventSince | null;
    'location'?: AbsenceWebhookEventLocation | null;
    'available'?: AbsenceWebhookEventAvailable | null;
}
export interface AbsenceWebhookEventAvailable {
    'old': boolean;
    'new': boolean;
}
export interface AbsenceWebhookEventIcon {
    'old': string;
    'new': string;
}
export interface AbsenceWebhookEventLocation {
    'old': TDLocation;
    'new': TDLocation;
}
export interface AbsenceWebhookEventReason {
    'old': AbsenceReasonRecord;
    'new': AbsenceReasonRecord;
}
export interface AbsenceWebhookEventSince {
    'old': string;
    'new': string;
}
export interface AbsencesAbsenceReasonsGetRequest {
    'name': string;
    'description': string;
    'defaultAvailability': boolean;
    'approvalRequired': boolean;
    'icon'?: string | null;
}
export interface AbsencesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<AbsenceRecord>;
}
export interface AbsencesGetDefaultResponse {
    'error': string;
    'error_description': string;
}
export interface AbsencesGetRequest {
    'member': string;
    'reason': string;
    'description': string;
    'location'?: string | null;
    'since': string;
    'till': string;
    'available'?: boolean;
    'icon': string;
    'customFieldValues'?: Array<CustomFieldInputValue> | null;
}
export interface AbsencesIdApprovePostRequest {
    'approve': boolean;
}
export interface AbsencesIdDeleteRequest {
    'member'?: string | null;
    'reason'?: string | null;
    'description'?: string | null;
    'location'?: string | null;
    'since'?: string | null;
    'till'?: string | null;
    'available': boolean;
    'icon'?: string | null;
    'customFieldValues'?: Array<CustomFieldInputValue> | null;
}
export interface AccessRecord {
    'time': string;
    'address': string;
    'userAgent'?: string | null;
}
export declare const AccessType: {
    readonly Public: "PUBLIC";
    readonly Confidential: "CONFIDENTIAL";
    readonly Restricted: "RESTRICTED";
};
export type AccessType = typeof AccessType[keyof typeof AccessType];
export interface AdHocConnectedAppMetadata {
    'lastSentServerUrl'?: string | null;
    'connectionStatus'?: AppConnectionStatus;
    'uninstallationStatus'?: AppUninstallationStatus;
    'uninstallationStartedAt'?: string | null;
}
export interface AdministrationUserAgreementEnabledGetRequest {
    'enabled': boolean;
}
export interface AdministrationUserAgreementGetRequest {
    'newContent': string;
    'invalidate': boolean;
}
export interface AdministrationUserAgreementStatusGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<UAUserAgreementStatus>;
}
export declare const AllChannelsFilter: {
    readonly CreatedByMe: "CreatedByMe";
    readonly Subscribed: "Subscribed";
};
export type AllChannelsFilter = typeof AllChannelsFilter[keyof typeof AllChannelsFilter];
export interface AllChannelsListEntry {
    'channelId': string;
    'name': string;
    'description': string;
    'access': M2Access;
    'icon'?: string | null;
    'created': string;
    'subscribers': number;
    'subscribed': boolean;
    'archived': boolean;
}
export declare const AllChannelsSortColumn: {
    readonly Name: "Name";
    readonly Subscribers: "Subscribers";
    readonly Created: "Created";
};
export type AllChannelsSortColumn = typeof AllChannelsSortColumn[keyof typeof AllChannelsSortColumn];
export interface AllReactionsToItemRecord {
    'id': string;
    'emojiReactions'?: Array<EmojiReactionRecord> | null;
}
export interface ApiIcon {
    'icon': string;
}
export declare const AppConnectionStatus: {
    readonly Connecting: "CONNECTING";
    readonly FailedToConnect: "FAILED_TO_CONNECT";
    readonly Reconnecting: "RECONNECTING";
    readonly Connected: "CONNECTED";
};
export type AppConnectionStatus = typeof AppConnectionStatus[keyof typeof AppConnectionStatus];
export interface AppMessageDeliveryClientErrorDTO {
    'className': string;
    'message'?: string | null;
    'causeClassName'?: string | null;
    'causeMessage'?: string | null;
}
export interface AppParameter {
    'key': string;
    'value': string;
}
export interface AppUiEnabledState {
    'enabledByDefault': boolean;
    'enabledForMe'?: boolean | null;
}
export interface AppUiExtContextData {
    'context': PermissionContextApi;
    'extensions'?: Array<AppUiExtensionApi> | null;
    'appUiEnabledState': AppUiEnabledState;
}
export interface AppUiExtensionApi {
    'className': string;
    'iframeUrl'?: string | null;
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<DocumentFolderMenuItemVisibilityFilterApi>;
    'uniqueCode': string;
}
export interface AppUiExtensionIn {
    'className': string;
    'iframeUrl'?: string | null;
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<ChatMessageMenuItemVisibilityFilterIn>;
    'uniqueCode': string;
}
export declare const AppUninstallationStatus: {
    readonly SendingUninstallPayload: "SENDING_UNINSTALL_PAYLOAD";
    readonly RetryingSendingUninstallPayload: "RETRYING_SENDING_UNINSTALL_PAYLOAD";
    readonly Failed: "FAILED";
    readonly Uninstalled: "UNINSTALLED";
};
export type AppUninstallationStatus = typeof AppUninstallationStatus[keyof typeof AppUninstallationStatus];
export interface AppUserActionExecutionResult {
    'className': string;
    'permissionsToRequest': Array<AuthCodeFlowPermissionsRequest>;
    'message': string | null;
}
export interface AppUserActionExecutionResultAuthCodeFlowRequired {
    'permissionsToRequest': Array<AuthCodeFlowPermissionsRequest>;
}
export interface AppUserActionExecutionResultFailure {
    'message': string;
}
export interface AppUserActionExecutionResultSuccess {
    'message'?: string | null;
}
export interface ApplicationAuthorizationRequestedEvent {
    'meta': KMetaMod;
    'application': ESApp;
    'scope': AuthScope;
}
export interface ApplicationAuthorizedWebhookEvent {
    'meta': KMetaMod;
    'application': ESApp;
    'scope': AuthScope;
}
export interface ApplicationHomepageUiExtensionApi {
    'iframeUrl'?: string | null;
}
export interface ApplicationHomepageUiExtensionIn {
    'iframeUrl'?: string | null;
}
export interface ApplicationIdentifier {
    'className': string;
    'clientId': string;
    'id': string;
}
export interface ApplicationIdentifierClientId {
    'clientId': string;
}
export interface ApplicationIdentifierId {
    'id': string;
}
export interface ApplicationMetadata {
    'className': string;
    'project': PRProject;
    'channel': M2ChannelRecord;
    'subscription': string;
    'subscriptionName': string;
    'lastSentServerUrl': string;
    'connectionStatus': AppConnectionStatus;
    'uninstallationStatus'?: AppUninstallationStatus;
    'uninstallationStartedAt'?: string | null;
    'marketplaceAppId': string;
}
export interface ApplicationPayload {
    'className': string;
    'newClientSecret': string;
    'clientId': string;
    'userId': string | null;
    'verificationToken'?: string | null;
    'refreshToken': string;
    'scope': string;
    'webhookId': string;
    'payload': WebhookEvent;
    'clientSecret': string;
    'serverUrl': string;
    'state'?: string | null;
    'actionId': string;
    'actionValue': string;
    'message': MessageContext;
    'menuItemUniqueCode': string;
    'context': ApplicationUnfurlContext;
    'newServerUrl': string;
    'link': string;
}
export interface ApplicationSshKeyWebhookEvent {
    'meta': KMetaMod;
    'application': ESApp;
    'fingerprint'?: AbsenceWebhookEventIcon | null;
    'comment'?: AbsenceWebhookEventIcon | null;
}
export declare const ApplicationType: {
    readonly Application: "Application";
    readonly InternalApp: "InternalApp";
    readonly MarketplaceApp: "MarketplaceApp";
};
export type ApplicationType = typeof ApplicationType[keyof typeof ApplicationType];
export interface ApplicationUnfurl {
    'queueItemId': string;
    'content': ApplicationUnfurlContent;
}
export interface ApplicationUnfurlContent {
    'className': string;
    'icon'?: ApiIcon;
    'title': string;
    'url': string;
    'style': MessageStyle;
    'outline'?: MessageOutlineBase;
    'sections': Array<MessageSectionElement>;
}
export interface ApplicationUnfurlContentImage {
    'icon'?: ApiIcon;
    'title': string;
    'url': string;
}
export interface ApplicationUnfurlContentMessage {
    'style': MessageStyle;
    'outline'?: MessageOutlineBase;
    'sections': Array<MessageSectionElement>;
}
export interface ApplicationUnfurlContext {
    'className': string;
    'userId': string;
    'todoItemId': string;
    'channel': string;
    'messageId': ChatMessageIdentifier;
    'reviewId': string;
    'field'?: CodeReviewUnfurlContextField;
    'documentId': string;
    'projectId': string;
    'repo': string;
    'commitId': string;
    'issueId': string;
    'articleId': string;
}
export interface ApplicationUnfurlDomain {
    'domain': string;
    'status': RightStatus;
    'modificationAuthor'?: CPrincipal;
    'modificationTimestamp'?: string | null;
}
export interface ApplicationUnfurlPattern {
    'prefix': string;
    'linkReplacement': string;
    'status': RightStatus;
    'modificationAuthor'?: CPrincipal;
    'modificationTimestamp'?: string | null;
}
export interface ApplicationUnfurlPatternRequest {
    'prefix': string;
    'linkReplacement': string;
}
export interface ApplicationUnfurlQueueItem {
    'id': string;
    'target': string;
    'context'?: ApplicationUnfurlContext;
    'authorUserId'?: ProfileIdentifier;
    'etag': number;
}
export interface ApplicationUnfurlTarget {
    'className': string;
    'domain': string;
    'pattern': string;
}
export interface ApplicationUnfurlTargetDomain {
    'domain': string;
}
export interface ApplicationUnfurlTargetPattern {
    'pattern': string;
}
export interface ApplicationUnfurlTargetWebhookEvent {
    'meta': KMetaMod;
    'application': ESApp;
    'target': ApplicationUnfurlTarget;
}
export interface ApplicationUninstalledPayload {
    'serverUrl': string;
    'clientId': string;
    'verificationToken'?: string | null;
}
export interface ApplicationWebhookEvent {
    'meta': KMetaMod;
    'application': ESApp;
    'name'?: AbsenceWebhookEventIcon | null;
    'owner'?: ApplicationWebhookEventOwner | null;
    'archived'?: AbsenceWebhookEventAvailable | null;
    'endpointURI'?: AbsenceWebhookEventIcon | null;
    'clientIdChanged': boolean;
    'clientSecretChanged': boolean;
    'verificationTokenChanged': boolean;
    'signingKeyChanged': boolean;
}
export interface ApplicationWebhookEventOwner {
    'old': TDMemberProfile;
    'new': TDMemberProfile;
}
export interface ApplicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest {
    'contextIdentifier': PermissionContextIdentifier;
    'updates': Array<RightUpdateDTO>;
}
export interface ApplicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest {
    'contextIdentifier': PermissionContextIdentifier;
    'rightCodes': Array<PermissionIdentifier>;
}
export interface ApplicationsApplicationAuthorizationsRequiredRightsGetRequest {
    'rightCodesToAdd': Array<PermissionIdentifier>;
    'rightCodesToRemove': Array<PermissionIdentifier>;
    'requestRightsInAuthorizedContexts': boolean;
}
export interface ApplicationsApplicationDeleteRequest {
    'name'?: string;
    'pictureAttachmentId'?: string | null;
    'defaultExternalPicture'?: string | null;
    'clientSecret'?: string;
    'clientCredentialsFlowEnabled'?: boolean;
    'codeFlowEnabled'?: boolean;
    'codeFlowRedirectURIs'?: string | null;
    'pkceRequired'?: boolean;
    'implicitFlowEnabled'?: boolean;
    'implicitFlowRedirectURIs'?: string | null;
    'endpointUri'?: string | null;
    'endpointSslVerification'?: boolean;
    'hasVerificationToken'?: boolean;
    'hasPublicKeySignature'?: boolean;
    'hasSigningKey'?: boolean;
    'appLevelAuth'?: EndpointAppLevelAuthUpdateType;
    'sslKeystoreAuth'?: string | null;
    'basicAuthUsername'?: string;
    'basicAuthPassword'?: string;
    'bearerAuthToken'?: string;
}
export interface ApplicationsApplicationPermanentTokensGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ESApplicationPermanentToken>;
}
export interface ApplicationsApplicationPermanentTokensGet200Response1 {
    'first': ESApplicationPermanentToken;
    'second': string;
}
export interface ApplicationsApplicationPermanentTokensGetRequest {
    'name': string;
    'scope': string;
    'expires'?: string | null;
}
export interface ApplicationsApplicationPermanentTokensTokenIdDeleteRequest {
    'name'?: string | null;
    'scope'?: string | null;
    'expires'?: string | null;
}
export interface ApplicationsApplicationSshKeysGetRequest {
    'publicKey': string;
    'comment': string;
}
export interface ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest {
    'contextIdentifier': PermissionContextIdentifier;
}
export interface ApplicationsApplicationUnfurlDomainsAuthorizePostRequest {
    'domains': Array<string>;
    'approve': boolean;
}
export interface ApplicationsApplicationUnfurlPatternsAuthorizePostRequest {
    'patterns': Array<string>;
    'approve': boolean;
}
export interface ApplicationsApplicationWebhooksGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<FullWebhookDTO>;
}
export interface ApplicationsApplicationWebhooksGetRequest {
    'name': string;
    'description'?: string | null;
    'endpoint'?: EndpointCreateDTO;
    'endpointAuth'?: EndpointAuthCreateDTO;
    'enabled'?: boolean;
    'acceptedHttpResponseCodes': Array<number>;
    'doRetries'?: boolean;
    'payloadFields'?: string | null;
}
export interface ApplicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest {
    'headers': Array<CustomHttpHeaderDTO>;
}
export interface ApplicationsApplicationWebhooksWebhookIdDeleteRequest {
    'name'?: string;
    'description'?: string | null;
    'enabled'?: boolean;
    'endpoint'?: ExternalEndpointUpdateDTO;
    'endpointAuth'?: EndpointAuthUpdateDTO;
    'acceptedHttpResponseCodes'?: Array<number>;
    'doRetries'?: boolean;
    'payloadFields'?: string | null;
}
export interface ApplicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest {
    'name': string;
    'subscription': CustomGenericSubscriptionIn;
}
export interface ApplicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest {
    'name'?: string;
    'enabled'?: boolean;
    'subscription'?: CustomGenericSubscriptionIn;
}
export interface ApplicationsPagedGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ESApp>;
}
export interface ApplicationsPostRequest {
    'name': string;
    'pictureAttachmentId'?: string | null;
    'defaultExternalPicture'?: string | null;
    'clientId'?: string | null;
    'clientSecret'?: string | null;
    'clientCredentialsFlowEnabled'?: boolean | null;
    'codeFlowEnabled'?: boolean | null;
    'codeFlowRedirectURIs'?: string | null;
    'pkceRequired'?: boolean | null;
    'implicitFlowEnabled'?: boolean | null;
    'implicitFlowRedirectURIs'?: string | null;
    'endpointUri'?: string | null;
    'endpointSslVerification'?: boolean | null;
    'appLevelAuth'?: EndpointAuthCreate;
    'sslKeystoreAuth'?: string | null;
    'hasSigningKey'?: boolean | null;
    'hasPublicKeySignature'?: boolean | null;
    'basicAuthUsername'?: string | null;
    'basicAuthPassword'?: string | null;
    'bearerAuthToken'?: string | null;
    'connectToSpace'?: boolean | null;
    'state'?: string | null;
}
export interface ApplicationsSubscriptionFilter {
    'application'?: ESApp;
}
export interface ApplicationsSubscriptionFilterIn {
    'application'?: string | null;
}
export interface ApplicationsUiExtensionsPatchRequest {
    'contextIdentifier': PermissionContextIdentifier;
    'extensions': Array<AppUiExtensionIn>;
}
export interface ApplicationsUnfurlsDomainsPatchRequest {
    'domains': Array<string>;
}
export interface ApplicationsUnfurlsPatternsPatchRequest {
    'patterns': Array<ApplicationUnfurlPatternRequest>;
}
export interface ApplicationsUnfurlsQueueContentPostRequest {
    'unfurls': Array<ApplicationUnfurl>;
}
export interface ApplicationsUnfurlsQueueRequestExternalAuthPostRequest {
    'queueItemId': string;
    'message': ApplicationUnfurlContentMessage;
}
export interface ApplicationsUnfurlsQueueResetExternalAuthRequestsPostRequest {
    'userId': ProfileIdentifier;
}
export declare const AppsOrdering: {
    readonly NameAToZ: "NAME_A_TO_Z";
    readonly NameZToA: "NAME_Z_TO_A";
    readonly LastEdited: "LAST_EDITED";
};
export type AppsOrdering = typeof AppsOrdering[keyof typeof AppsOrdering];
export interface ArticleAppUnfurlContext {
    'articleId': string;
}
export interface ArticleChannelRecord {
    'id': string;
    'archived': boolean;
    'channel': M2ChannelRecord;
    'channelContent'?: M2ChannelContentRecord;
    'reactions': AllReactionsToItemRecord;
}
export interface ArticleContentRecord {
    'id': string;
    'archived': boolean;
    'content': string;
    'attachments'?: Array<AttachmentInfo> | null;
}
export interface ArticleDetailsRecord {
    'id': string;
    'archived': boolean;
    'event'?: MeetingRecord;
    'teams'?: Array<TDTeam> | null;
    'locations'?: Array<TDLocation> | null;
    'externalEntityInfo'?: ExternalEntityInfoRecord;
}
export interface ArticleImportResult {
    'externalId'?: string | null;
    'article'?: ArticleRecord;
    'error'?: string | null;
}
export interface ArticleMarkdownImage {
    'alt': string;
    'src': string;
}
export interface ArticlePreviewRecord {
    'id': string;
    'archived': boolean;
    'previewImages': Array<ArticleMarkdownImage>;
    'preview': string;
    'previewAttachments'?: Array<AttachmentInfo> | null;
    'wordsNumber'?: number | null;
    'cut'?: boolean | null;
}
export interface ArticleRecord {
    'id': string;
    'archived': boolean;
    'title': string;
    'created': string;
    'author': TDMemberProfile;
    'aliases': Array<BGArticleAlias>;
    'archivedBy'?: TDMemberProfile;
    'archivedAt'?: string | null;
    'editable': boolean;
    'event'?: MeetingRecord;
    'teams'?: Array<TDTeam> | null;
    'locations'?: Array<TDLocation> | null;
    'externalEntityInfo'?: ExternalEntityInfoRecord;
    'channel': M2ChannelRecord;
    'channelContent'?: M2ChannelContentRecord;
    'reactions': AllReactionsToItemRecord;
    'previewImages': Array<ArticleMarkdownImage>;
    'preview': string;
    'previewAttachments'?: Array<AttachmentInfo> | null;
    'wordsNumber'?: number | null;
    'cut'?: boolean | null;
    'content': string;
    'attachments'?: Array<AttachmentInfo> | null;
}
export interface AssignedPerson {
    'profile'?: TDMemberProfile;
    'role'?: string | null;
}
export interface Attachment {
    'className': string;
    'profile': TDMemberProfile;
    'deletedIdentity': string;
    'unfurl': Unfurl;
    'id': string;
    'onlyVisibleToYou'?: boolean | null;
    'name'?: string | null;
    'width': number;
    'height': number;
    'sizeBytes': number;
    'previewBytes'?: string | null;
    'variants'?: Array<ImageAttachmentVariant> | null;
    'filename': string;
}
export interface AttachmentIn {
    'className': string;
    'id': string;
    'name'?: string | null;
    'width': number;
    'height': number;
    'sizeBytes': number;
    'previewBytes'?: string | null;
    'variants'?: Array<ImageAttachmentVariant> | null;
    'filename': string;
}
export interface AttachmentInfo {
    'details'?: Attachment;
}
export interface AttachmentUnfurlDetails {
    'className': string;
    'icon'?: string | null;
    'title': string;
    'image': ImageAttachment;
    'targetRef': DeployTargetRecord;
    'targetName'?: string | null;
    'showLinkIcon'?: boolean | null;
    'skipDetailsRender'?: boolean | null;
    'session': CallSession;
    'message': MCMessage;
    'inlineUnfurls'?: Array<AttachmentInfo> | null;
    'id': string;
    'channelId'?: string | null;
    'text': string;
    'details'?: ArticleDetailsRecord;
    'author': CPrincipal;
    'created': string;
    'time': number;
    'attachments'?: Array<AttachmentInfo> | null;
    'mentions'?: Array<EntityMention> | null;
    'meeting': DTOMeeting;
    'compact'?: boolean | null;
    'jobId': string;
    'jobName': string;
    'projectRef': PRProject;
    'repoName': string;
    'jobExecutionDisplayStatusFilter'?: JobExecutionDisplayStatus;
    'jobTriggerFilter'?: JobTriggerType;
    'branch'?: Branch;
    'jobExecutionId': string;
    'article': ArticleRecord;
    'content'?: ArticleContentRecord;
    'channel'?: ArticleChannelRecord;
    'header': MCMessage;
    'headerAttachments': Array<AttachmentInfo>;
    'lines': Array<CodeLine>;
    'selectedLineIndex': number;
    'selectedLinesCount': number;
    'sourceBranch'?: string | null;
    'targetBranch'?: string | null;
    'anchor': CodeSnippetAnchor;
}
export interface AuthCodeFlowPermissionsRequest {
    'scope': string;
    'purpose': string;
}
export interface AuthConfig {
    'dontRememberMeTtl': number;
    'adminRememberMeTtl': number;
    'userRememberMeTtl': number;
}
export interface AuthModuleUsage {
    'authModule': ESAuthModule;
    'profiles': number;
}
export interface AuthModulesConfigDeleteRequest {
    'dontRememberMeTtl'?: number | null;
    'adminRememberMeTtl'?: number | null;
    'userRememberMeTtl'?: number | null;
}
export interface AuthModulesGetRequest {
    'key': string;
    'name': string;
    'enabled': boolean;
    'iconDataURI'?: string | null;
    'settings': ESAuthModuleSettings;
}
export interface AuthModulesIdDeleteRequest {
    'key'?: string | null;
    'name'?: string | null;
    'enabled'?: boolean | null;
    'iconDataURI'?: string | null;
    'settings'?: ESAuthModuleSettings;
}
export interface AuthModulesIdLoginsIdentifierChangePostRequest {
    'oldPassword': string;
    'newPassword': string;
}
export interface AuthModulesIdSamlMetadataPostRequest {
    'idpUrl': string;
    'idpEntityId': string;
    'idpCertificateSHA256': string;
    'spEntityId': string;
    'sslKeystore'?: SSLKeystore;
    'contactProfileId'?: string | null;
}
export interface AuthModulesReorderPostRequest {
    'order': Array<string>;
}
export interface AuthModulesTestBuiltInPostRequest {
    'settings': ESBuiltinAuthModuleSettings;
    'username': string;
    'password': string;
}
export interface AuthModulesTestLdapPostRequest {
    'settings': ESLdapAuthModuleSettings;
    'username': string;
    'password': string;
}
export interface AuthModulesThrottledLoginsDelete200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ThrottledLogin>;
}
export interface AuthScope {
    'className': string;
    'project': PRProject;
}
export interface AutomationJobEvent {
    'meta': KMetaMod;
    'executionId': string;
    'project': PRProject;
    'repositoryName': string;
    'jobName': string;
    'executionNumber': number;
    'trigger': JobExecutionTrigger;
    'triggerTime': string;
    'status'?: AutomationJobEventStatus | null;
    'failureReasons'?: AutomationJobEventFailureReasons | null;
    'stoppedBy'?: AutomationJobEventStoppedBy | null;
    'startTime'?: AutomationJobEventStartTime | null;
    'endTime'?: AutomationJobEventStartTime | null;
}
export interface AutomationJobEventFailureReasons {
    'old': Array<FailureConditionType>;
    'new': Array<FailureConditionType>;
}
export interface AutomationJobEventStartTime {
    'old': string;
    'new': string;
}
export interface AutomationJobEventStatus {
    'old': JobExecutionStatus;
    'new': JobExecutionStatus;
}
export interface AutomationJobEventStoppedBy {
    'old': CPrincipal;
    'new': CPrincipal;
}
export interface AutomationJobExecutionUnfurlDetails {
    'jobExecutionId': string;
    'projectRef': PRProject;
    'repoName': string;
}
export interface AutomationJobSubscriptionFilter {
    'projects': Array<PRProject>;
    'repositoryName'?: string | null;
    'branchSpec'?: Array<string> | null;
    'jobs'?: Array<string> | null;
}
export interface AutomationJobSubscriptionFilterIn {
    'projects'?: Array<string> | null;
    'repositoryName'?: string | null;
    'branchSpec'?: Array<string> | null;
    'jobs'?: Array<string> | null;
}
export interface AutomationJobUnfurlDetails {
    'jobId': string;
    'jobName': string;
    'projectRef': PRProject;
    'repoName': string;
    'jobExecutionDisplayStatusFilter'?: JobExecutionDisplayStatus;
    'jobTriggerFilter'?: JobTriggerType;
    'branch'?: Branch;
}
export interface AutomationServiceMetadata {
    'project': PRProject;
}
export interface AutonumberCFParameters {
    'prefix': string;
    'suffix': string;
}
export interface AutonumberCFValue {
    'value'?: string | null;
}
export interface AvatarCropSquare {
    'x': number;
    'y': number;
    'length': number;
}
export interface BGArticleAlias {
    'alias': string;
    'created': string;
}
export interface BGStats {
    'totalBlogs': number;
    'teams': Array<BGStatsTeamsInner>;
    'projects'?: Array<BGStatsProjectsInner> | null;
    'locations': Array<BGStatsLocationsInner>;
}
export interface BGStatsLocationsInner {
    'first': TDLocation;
    'second': number;
}
export interface BGStatsProjectsInner {
    'first': PRProject;
    'second': number;
}
export interface BGStatsTeamsInner {
    'first': TDTeam;
    'second': number;
}
export interface BillingAdminOverdraftsGetRequest {
    'storage': number;
    'bandwidth': number;
    'ciCredits': number;
}
export interface BillingAdminTrialPutRequest {
    'trialTier': Tier;
}
export interface BillingFeedChannel {
    'notificationDefaults'?: ChannelSpecificDefaults;
}
export interface BillingReport {
    'plans': Array<PurchasedBillingPlan>;
    'reportDays': Array<BillingReportDay>;
    'bandwidthAllocation'?: number | null;
    'ciAllocation'?: number | null;
    'serverToday': string;
    'earliestBillingDate': string;
}
export interface BillingReportDay {
    'date': string;
    'activeUsers': number;
    'userUsage': number;
    'userCost': number;
    'userLimit'?: number | null;
    'activeGuests'?: number | null;
    'paidActiveGuests'?: number | null;
    'freeActiveGuests'?: number | null;
    'guestCost'?: number | null;
    'guestLimit'?: number | null;
    'storageAllocationB'?: number | null;
    'storageTotalUsage': number;
    'storageFilesUsage': number;
    'storageGitUsage': number;
    'storagePackagesUsage': number;
    'storageCost': number;
    'bandwidthTotalUsageDelta': number;
    'bandwidthTotalUsage': number;
    'bandwidthFilesUsageDelta': number;
    'bandwidthFilesUsage': number;
    'bandwidthGitUsageDelta': number;
    'bandwidthGitUsage': number;
    'bandwidthPackagesUsageDelta': number;
    'bandwidthPackagesUsage': number;
    'bandwidthAutomationExternalTrafficUsageDelta': number;
    'bandwidthAutomationExternalTrafficUsage': number;
    'bandwidthAutomationInternalUsageDelta': number;
    'bandwidthAutomationInternalUsage': number;
    'bandwidthCost': number;
    'ciUsageDelta': number;
    'ciUsage': number;
    'ciCost': number;
    'appUsage': number;
    'chatUsage': number;
    'totalCost': number;
}
export interface BirthdayEvent {
    'profile': TDMemberWithTeam;
    'birthday': string;
}
export interface BlockNode {
    'className': string;
    'lang': string;
    'children': Array<InlineNode>;
    'attrs'?: RtTableCellAttrs;
    'startNumber': number;
    'level': number;
    'textAlign': RtTextAlign;
}
export interface BlockNodeWithMarks {
    'className': string;
    'children': Array<InlineNode>;
    'attrs'?: RtTableCellAttrs;
    'startNumber': number;
    'level': number;
    'textAlign': RtTextAlign;
}
export interface BlogCalendarEvent {
    'starts'?: string | null;
    'ends'?: string | null;
    'timezone'?: ATimeZone;
    'rooms'?: Array<string> | null;
    'allDay'?: boolean | null;
}
export interface BlogCommonSubscriptionFilter {
    'teams': Array<TDTeam>;
    'locations': Array<TDLocation>;
    'authors': Array<TDMemberProfile>;
}
export interface BlogCommonSubscriptionFilterIn {
    'teams': Array<string>;
    'locations': Array<string>;
    'authors': Array<string>;
}
export interface BlogEventMenuActionContext {
    'blogEventId': string;
}
export interface BlogGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ArticleRecord>;
}
export interface BlogGetRequest {
    'title': string;
    'content': string;
    'locations'?: Array<string> | null;
    'teams'?: Array<string> | null;
    'event'?: BlogCalendarEvent;
}
export interface BlogIdDeleteRequest {
    'title'?: string | null;
    'content'?: string | null;
    'locations'?: Array<string> | null;
    'teams'?: Array<string> | null;
    'event'?: BlogCalendarEvent;
}
export interface BlogImportPostRequest {
    'metadata': ImportMetadata;
    'articles': Array<ExternalArticle>;
}
export interface BlogPublicationDetails {
    'teams'?: Array<TDTeam> | null;
    'locations'?: Array<TDLocation> | null;
    'event'?: CalendarEvent;
    'article'?: ArticleRecord;
}
export interface BlogPublicationDetailsIn {
    'teams'?: Array<string> | null;
    'locations'?: Array<string> | null;
    'event'?: CalendarEventIn;
    'article'?: string | null;
}
export interface BlogWebhookEvent {
    'meta': KMetaMod;
    'article': ArticleRecord;
    'action'?: string | null;
    'author'?: ApplicationWebhookEventOwner | null;
    'created'?: AutomationJobEventStartTime | null;
    'title'?: AbsenceWebhookEventIcon | null;
    'textChanged': boolean;
    'unpublished'?: AbsenceWebhookEventAvailable | null;
    'teams'?: BlogWebhookEventTeams | null;
    'locations'?: BlogWebhookEventLocations | null;
    'externalEntityInfo'?: BlogWebhookEventExternalEntityInfo | null;
}
export interface BlogWebhookEventExternalEntityInfo {
    'old': number;
    'new': number;
}
export interface BlogWebhookEventLocations {
    'old': Array<TDLocation>;
    'new': Array<TDLocation>;
}
export interface BlogWebhookEventTeams {
    'old': Array<TDTeam>;
    'new': Array<TDTeam>;
}
export interface BoardColumn {
    'name': string;
    'statuses': Array<IssueStatus>;
}
export interface BoardColumns {
    'columns': Array<BoardColumn>;
}
export interface BoardIdentifier {
    'className': string;
    'id': string;
}
export interface BoardIdentifierId {
    'id': string;
}
export interface BoardInfo {
    'owners': BoardOwners;
    'columns': BoardColumns;
    'description'?: string | null;
    'swimlaneKey'?: string | null;
    'issueFields': Array<BoardIssueField>;
}
export interface BoardIssueField {
    'className': string;
    'field': BuiltInIssueField;
    'customField': CustomField;
}
export interface BoardIssueFieldBuiltIn {
    'field': BuiltInIssueField;
}
export interface BoardIssueFieldCustom {
    'customField': CustomField;
}
export interface BoardIssueInputField {
    'className': string;
    'field': BuiltInIssueField;
    'key': ExtendedTypeKey;
    'fieldId': string;
}
export interface BoardMemberOwners {
    'members': Array<TDMemberProfile>;
}
export interface BoardOwners {
    'members': BoardMemberOwners;
    'teams': BoardTeamOwners;
}
export interface BoardRecord {
    'id': string;
    'archived': boolean;
    'name': string;
    'info': BoardInfo;
    'from'?: string | null;
    'to'?: string | null;
}
export interface BoardTeamOwners {
    'teams': Array<TDTeam>;
}
export interface BookContainerInfo {
    'book': KBBook;
    'grantedRights'?: Array<string> | null;
}
export interface BooleanCFValue {
    'value': boolean;
}
export interface Branch {
    'branchName': string;
}
export interface BranchInfo {
    'head': string;
    'ref': string;
}
export declare const BuiltInIssueField: {
    readonly Id: "ID";
    readonly DueDate: "DUE_DATE";
    readonly Attachments: "ATTACHMENTS";
    readonly Status: "STATUS";
    readonly Title: "TITLE";
    readonly Description: "DESCRIPTION";
    readonly Tags: "TAGS";
    readonly Assignee: "ASSIGNEE";
    readonly AttachmentCount: "ATTACHMENT_COUNT";
    readonly SubitemsCount: "SUBITEMS_COUNT";
};
export type BuiltInIssueField = typeof BuiltInIssueField[keyof typeof BuiltInIssueField];
export interface BusinessEntityRelationEvent {
    'meta': KMetaMod;
    'relation': string;
    'entity': string;
    'member': TDMemberProfile;
    'since'?: AbsenceWebhookEventSince | null;
    'till'?: AbsenceWebhookEventSince | null;
    'archived'?: AbsenceWebhookEventAvailable | null;
}
export declare const BusyStatus: {
    readonly Busy: "Busy";
    readonly Free: "Free";
};
export type BusyStatus = typeof BusyStatus[keyof typeof BusyStatus];
export interface CApplicationPrincipalDetails {
    'app': ESApp;
}
export interface CAutomationTaskPrincipalDetails {
    'project': PRProject;
    'name': string;
    'id': string;
}
export interface CBuiltInServicePrincipalDetails {
    'name': string;
    'key'?: string | null;
}
export declare const CDomainStatus: {
    readonly Available: "AVAILABLE";
    readonly Occupied: "OCCUPIED";
    readonly Blocked: "BLOCKED";
};
export type CDomainStatus = typeof CDomainStatus[keyof typeof CDomainStatus];
export interface CFAbsenceIdentifier {
    'absence': AbsenceIdentifier;
}
export interface CFCommitIdentifier {
    'className': string;
    'commitHash': string;
    'project': ProjectIdentifier;
    'repository': string;
}
export interface CFCommitIdentifierCommitHash {
    'commitHash': string;
}
export interface CFCommitIdentifierFull {
    'project': ProjectIdentifier;
    'repository': string;
    'commitHash': string;
}
export interface CFCommitInfoBase {
    'className': string;
    'commit': CommitInfo;
    'commitHash': string;
}
export interface CFCommitInfoBaseCFCommitInfo {
    'commit': CommitInfo;
}
export interface CFCommitInfoBasePermissionDenied {
    'commitHash': string;
}
export interface CFConstraint {
    'className': string;
    'min'?: number | null;
    'max'?: number | null;
    'pattern'?: string | null;
    'message'?: string | null;
}
export interface CFCreateParameters {
    'className': string;
    'contactKind'?: TDContactKind;
    'values': Array<string>;
    'documentScope': DocumentCFScopeInput;
    'prefix': string;
    'suffix': string;
    'vcsCFScope': VcsCFScopeInput;
}
export interface CFEntityIdentifier {
    'className': string;
    'absence': AbsenceIdentifier;
    'issue': IssueIdentifier;
    'membership': TeamMembershipIdentifier;
    'profile': ProfileIdentifier;
    'team': TeamIdentifier;
}
export interface CFEntityTypeIdentifier {
    'className': string;
    'issueTracker': IssueTrackerIdentifier;
}
export interface CFEnumValue {
    'id': string;
    'value': string;
    'principal'?: CPrincipalDetails;
}
export interface CFEnumValueIdentifier {
    'className': string;
    'id': string;
    'name': string;
}
export interface CFEnumValueIdentifierId {
    'id': string;
}
export interface CFEnumValueIdentifierName {
    'name': string;
}
export interface CFEnumValueModification {
    'className': string;
    'name': string;
    'valueIdentifier': CFEnumValueIdentifier;
    'newName': string;
}
export interface CFEnumValueModificationCreate {
    'name': string;
}
export interface CFEnumValueModificationRemove {
    'valueIdentifier': CFEnumValueIdentifier;
}
export interface CFEnumValueModificationUpdate {
    'valueIdentifier': CFEnumValueIdentifier;
    'newName': string;
}
export interface CFEnumValuesModification {
    'valuesToAdd': Array<EnumValueData>;
    'valuesToUpdate': Array<EnumValueData>;
    'valuesToDelete': Array<string>;
}
export interface CFIdentifier {
    'className': string;
    'id': string;
    'name': string;
}
export interface CFIdentifierId {
    'id': string;
}
export interface CFIdentifierName {
    'name': string;
}
export interface CFInputValue {
    'className': string;
    'value': string | null;
    'location'?: string | null;
    'commits': Array<CFCommitIdentifier>;
    'documents': Array<string>;
    'enumValueIdentifiers': Array<CFEnumValueIdentifier>;
    'target'?: TargetIdentifier;
    'values': Array<string>;
    'enumValueIdentifier'?: CFEnumValueIdentifier;
    'commit'?: CFCommitIdentifier;
    'issue'?: IssueIdentifier;
    'deployment'?: DeploymentIdentifier;
    'team'?: string | null;
    'project'?: ProjectIdentifier;
    'href'?: string | null;
    'issues': Array<IssueIdentifier>;
    'profiles': Array<ProfileIdentifier>;
    'profile'?: ProfileIdentifier;
    'documentId'?: string | null;
}
export interface CFIssueIdentifier {
    'issue': IssueIdentifier;
}
export interface CFIssueTrackerEntityType {
    'issueTracker': IssueTrackerIdentifier;
}
export interface CFMembershipIdentifier {
    'membership': TeamMembershipIdentifier;
}
export interface CFParameters {
    'className': string;
    'vcsCFScope': VcsCFScope;
    'contactKind'?: TDContactKind;
    'values': Array<CFEnumValue>;
    'documentScope': DocumentCFScope;
    'prefix': string;
    'suffix': string;
}
export interface CFProfileIdentifier {
    'profile': ProfileIdentifier;
}
export interface CFTeamIdentifier {
    'team': TeamIdentifier;
}
export interface CFType {
    'className': string;
    'values': Array<EnumValueData>;
}
export interface CFUpdateParameters {
    'className': string;
    'contactKind'?: TDContactKind;
    'documentScope': DocumentCFScopeInput;
    'modifications': Array<CFEnumValueModification>;
    'prefix': string;
    'suffix': string;
    'vcsCFScope': VcsCFScopeInput;
    'values': Array<string>;
}
export interface CFValue {
    'className': string;
    'value': string | null;
    'documents': Array<Document>;
    'values': Array<string>;
    'target'?: DeployTargetRecord;
    'issue'?: Issue;
    'profile'?: TDMemberProfile;
    'team'?: TDTeam;
    'issues': Array<Issue>;
    'location'?: TDLocation;
    'deployment'?: DeploymentRecord;
    'href'?: string | null;
    'project'?: PRProject;
    'commits': Array<CFCommitInfoBase>;
    'commit'?: CFCommitInfoBase;
    'profiles': Array<TDMemberProfile>;
    'document'?: Document;
}
export interface CPrincipal {
    'name': string;
    'details'?: CPrincipalDetails;
}
export interface CPrincipalDetails {
    'className': string;
    'name': string;
    'email': string;
    'app': ESApp;
    'project': PRProject;
    'id': string;
    'user': TDMemberProfile;
    'key'?: string | null;
}
export interface CRight {
    'typeCode': string;
    'code': string;
    'title': string;
    'description'?: string | null;
    'featureFlag'?: FeatureFlag;
}
export interface CUserPrincipalDetails {
    'user': TDMemberProfile;
}
export interface CUserWithEmailPrincipalDetails {
    'name': string;
    'email': string;
}
export interface CalendarEvent {
    'calendarEvent'?: boolean | null;
    'starts'?: string | null;
    'ends'?: string | null;
    'timezone'?: ATimeZone;
    'rooms'?: Array<TDLocation> | null;
    'allDay'?: boolean | null;
}
export interface CalendarEventIn {
    'calendarEvent'?: boolean | null;
    'starts'?: string | null;
    'ends'?: string | null;
    'timezone'?: ATimeZone;
    'rooms'?: Array<string> | null;
    'allDay'?: boolean | null;
}
export interface CalendarEventSpec {
    'start': string;
    'end': string;
    'recurrenceRule'?: RecurrenceRule;
    'allDay': boolean;
    'timezone': ATimeZone;
    'parentId'?: string | null;
    'initialMeetingStart'?: string | null;
    'busyStatus'?: BusyStatus;
    'nextChainId'?: string | null;
}
export interface CalendarIdentifier {
    'className': string;
    'id': string;
}
export interface CalendarIdentifierId {
    'id': string;
}
export interface CalendarInfo {
    'id': string;
    'name'?: string | null;
    'defaultColor'?: string | null;
    'freeBusyOnly': boolean;
    'readOnly': boolean;
    'exposeToCalDav': boolean;
    'exposeToGoogle': boolean;
    'sourceUrl'?: string | null;
    'syncAttemptCount'?: number | null;
    'archived': boolean;
}
export interface CalendarsEventParticipationsIdPatchRequest {
    'newStatus': EventParticipationStatus;
}
export interface CalendarsMeetingsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<DTOMeeting>;
}
export interface CalendarsMeetingsGetRequest {
    'summary': string;
    'description'?: string | null;
    'occurrenceRule': CalendarEventSpec;
    'locations'?: Array<string>;
    'profiles'?: Array<string>;
    'externalParticipants'?: Array<string>;
    'teams'?: Array<string>;
    'visibility'?: MeetingVisibility;
    'modificationPreference'?: MeetingModificationPreference;
    'joiningPreference'?: MeetingJoiningPreference;
    'notifyOnExport'?: boolean;
    'organizer'?: string | null;
    'conferenceData'?: EventConferenceData;
    'attachments'?: Array<MeetingAttachment> | null;
    'calendarId'?: CalendarIdentifier;
}
export interface CalendarsMeetingsIdConferenceRoomsDeleteRequest {
    'roomId': string;
    'dateTime': string;
}
export interface CalendarsMeetingsIdDeleteRequest {
    'summary'?: string | null;
    'description'?: string | null;
    'occurrenceRule'?: CalendarEventSpec;
    'locationsDiff'?: Diff;
    'profilesDiff'?: Diff;
    'externalParticipantsDiff'?: Diff;
    'teamsDiff'?: Diff;
    'visibility'?: MeetingVisibility;
    'modificationPreference'?: MeetingModificationPreference;
    'joiningPreference'?: MeetingJoiningPreference;
    'notifyOnExport'?: boolean;
    'organizer'?: string | null;
    'targetDate'?: string | null;
    'modificationKind'?: RecurrentModification;
    'conferenceData'?: EventConferenceData;
    'attachments'?: Array<MeetingAttachment> | null;
    'calendarId'?: CalendarIdentifier;
}
export interface CalendarsMeetingsIdParticipationStatusPatchRequest {
    'profileId': string;
    'status': EventParticipationStatus;
    'targetDate'?: string | null;
    'modificationKind'?: RecurrentModification;
}
export interface CalendarsMeetingsProfileParticipationGet200ResponseInner {
    'first': string;
    'second': EventParticipationStatus;
}
export interface CallSession {
    'id': string;
    'archived': boolean;
    'room': Room;
    'description': string;
    'start': string;
    'end'?: string | null;
    'channel'?: M2ChannelRecord;
    'topology'?: ConnectionTopology;
    'resourcesPrepared': boolean;
    'active'?: boolean | null;
    'initialSession'?: CallSession;
    'subSessions': Array<CallSession>;
    'participations': Array<SessionParticipationRecord>;
}
export interface CallSessionUnfurlDetails {
    'session': CallSession;
}
export interface CallsPostRequest {
    'participants': Array<ProfileIdentifier>;
    'private': boolean;
    'channelId'?: string | null;
    'description'?: string | null;
}
export interface CertificateInfo {
    'certificateType': string;
    'version': number;
    'serialNumber': string;
    'issuedBy': string;
    'issuedTo': string;
    'validFrom': string;
    'validTo': string;
    'algorithm': string;
    'fingerprint': Fingerprint;
}
export interface ChangeClientSecretPayload {
    'newClientSecret': string;
    'clientId': string;
    'userId'?: string | null;
    'verificationToken'?: string | null;
}
export interface ChangeInReview {
    'repository': string;
    'change': GitCommitChange;
    'read': boolean;
}
export interface ChangeServerUrlPayload {
    'newServerUrl': string;
    'clientId': string;
    'userId'?: string | null;
    'verificationToken'?: string | null;
}
export interface ChannelContactThread {
    'className': string;
    'parent': M2ChannelRecord;
    'text'?: string | null;
    'messageId': string;
    'messageAuthor'?: CPrincipal;
    'attachments'?: string | null;
    'notificationDefaults': ChannelSpecificDefaults;
    'codeDiscussion': CodeDiscussionRecord;
}
export interface ChannelEvent {
    'meta': KMetaMod;
    'channel': M2ChannelRecord;
    'name'?: AbsenceWebhookEventIcon | null;
    'description'?: AbsenceWebhookEventIcon | null;
    'icon'?: AbsenceWebhookEventIcon | null;
    'restored'?: boolean | null;
    'archived'?: boolean | null;
}
export interface ChannelIdentifier {
    'className': string;
    'application': ApplicationIdentifier;
    'article': string;
    'channel': ChatChannel;
    'contactKey': string;
    'id': string;
    'issue': IssueIdentifier;
    'member': ProfileIdentifier;
    'codeReview': ReviewIdentifier;
    'message': string;
}
export interface ChannelIdentifierApplication {
    'application': ApplicationIdentifier;
}
export interface ChannelIdentifierArticle {
    'article': string;
}
export interface ChannelIdentifierChannel {
    'channel': ChatChannel;
}
export interface ChannelIdentifierContactKey {
    'contactKey': string;
}
export interface ChannelIdentifierId {
    'id': string;
}
export interface ChannelIdentifierIssue {
    'issue': IssueIdentifier;
}
export interface ChannelIdentifierProfile {
    'member': ProfileIdentifier;
}
export interface ChannelIdentifierReview {
    'codeReview': ReviewIdentifier;
}
export interface ChannelIdentifierThread {
    'message': string;
}
export interface ChannelInfoAttachment {
    'details'?: Attachment;
    'time': string;
    'messageId': string;
    'channelId': string;
    'contactKey': string;
}
export interface ChannelItemRecord {
    'text': string;
    'details'?: M2ItemContentDetails;
    'author': CPrincipal;
    'created': string;
    'time': number;
    'reactions'?: AllReactionsToItemRecord;
    'thread'?: M2ChannelRecord;
    'projectedItem'?: ChannelItemRecord;
    'attachments'?: Array<AttachmentInfo> | null;
    'externalId'?: string | null;
    'pending'?: boolean | null;
    'id': string;
    'archived': boolean;
    'edited'?: string | null;
    'pinned'?: boolean | null;
    'suggestedParticipants'?: Array<CPrincipal> | null;
    'mentions'?: Array<EntityMention> | null;
    'channelId'?: string | null;
    'importerAppId'?: string | null;
    'issues': Array<Issue>;
}
export interface ChannelItemSnapshot {
    'id': string;
    'channelId'?: string | null;
    'text': string;
    'details'?: M2ItemContentDetails;
    'author': CPrincipal;
    'created': string;
    'time': number;
    'attachments'?: Array<AttachmentInfo> | null;
    'mentions'?: Array<EntityMention> | null;
}
export interface ChannelItemSyncRecord {
    'modType': SyncRecordModType;
    'etag': string;
    'chatMessage': ChannelItemRecord;
}
export interface ChannelMessageMenuActionContext {
    'channelIdentifier': ChannelIdentifier;
    'messageIdentifier': ChatMessageIdentifier;
}
export interface ChannelParticipant {
    'principal': CPrincipal;
    'messageCount': number;
    'pendingMessageCount'?: number | null;
}
export interface ChannelPermissionContext {
    'channel': M2ChannelRecord;
}
export interface ChannelPermissionContextIdentifier {
    'channel': string;
}
export interface ChannelPermissionTarget {
    'channel': ChannelIdentifier;
}
export interface ChannelSpecificDefaults {
    'subscribed'?: boolean | null;
    'filter': NotificationFilter;
    'push': boolean;
    'email': M2EmailNotificationType;
    'threadsSubscribed'?: boolean | null;
}
export interface ChatChannel {
    'className': string;
    'id': string;
    'name': string;
}
export interface ChatChannelFromId {
    'id': string;
}
export interface ChatChannelFromName {
    'name': string;
}
export interface ChatChannelSubscriptionFilter {
    'channel'?: string | null;
    'contact'?: ChatContactRecord;
}
export interface ChatChannelSubscriptionFilterIn {
    'channel'?: string | null;
}
export interface ChatContactDetails {
    'className': string;
    'app': ESApp;
    'icon'?: string | null;
    'users': Array<TDMemberProfile>;
    'subject'?: string | null;
    'type': string;
    'name': string;
    'fontIcon'?: string | null;
    'fontIconColor'?: PrivateFeedColor;
    'access'?: M2Access;
    'resolvedImpl'?: boolean | null;
    'extra'?: ContactExtraTag;
    'user': TDMemberProfile;
    'parent': ChatContactRecord;
    'message': MessageInfo;
    'authors'?: Array<CPrincipal> | null;
    'replies'?: number | null;
    'absorbingParent'?: boolean | null;
}
export interface ChatContactDetailsApp {
    'app': ESApp;
    'icon'?: string | null;
}
export interface ChatContactDetailsConversation {
    'users': Array<TDMemberProfile>;
    'subject'?: string | null;
}
export interface ChatContactDetailsDefault {
    'type': string;
    'name': string;
    'icon'?: string | null;
    'fontIcon'?: string | null;
    'fontIconColor'?: PrivateFeedColor;
    'access'?: M2Access;
    'resolvedImpl'?: boolean | null;
    'extra'?: ContactExtraTag;
}
export interface ChatContactDetailsProfile {
    'user': TDMemberProfile;
}
export interface ChatContactDetailsThread {
    'parent': ChatContactRecord;
    'message': MessageInfo;
    'authors'?: Array<CPrincipal> | null;
    'replies'?: number | null;
    'absorbingParent'?: boolean | null;
    'resolvedImpl'?: boolean | null;
}
export interface ChatContactRecord {
    'id': string;
    'archived': boolean;
    'key': string;
    'details': ChatContactDetails;
    'channelType': string;
    'lastMessage'?: MessageInfo;
    'unreadStatus'?: M2UnreadStatus;
    'readTime'?: string | null;
    'subscribedSince': string;
    'pinned': boolean;
    'pinnedPrevId'?: string | null;
    'draft'?: string | null;
    'draftTime'?: number | null;
    'lastChildMessageTime'?: number | null;
    'deleted'?: boolean | null;
    'muted'?: boolean | null;
}
export interface ChatHistoryRange {
    'messages': Array<ChannelItemRecord>;
    'firstAfterLimitMessage'?: ChannelItemRecord;
}
export interface ChatMessage {
    'className': string;
    'style'?: MessageStyle;
    'outline'?: MessageOutlineBase;
    'sections': Array<MessageSectionElement>;
    'messageData'?: string | null;
    'text': string;
}
export interface ChatMessageBlock {
    'style'?: MessageStyle;
    'outline'?: MessageOutlineBase;
    'sections': Array<MessageSectionElement>;
    'messageData'?: string | null;
}
export interface ChatMessageCreatedEvent {
    'channelId': string;
    'threadId'?: string | null;
    'message': ChannelItemRecord;
}
export interface ChatMessageDeletedEvent {
    'channelId': string;
    'threadId'?: string | null;
    'message': ChannelItemRecord;
}
export interface ChatMessageIdentifier {
    'className': string;
    'externalId': string;
    'id': string;
}
export interface ChatMessageIdentifierExternalId {
    'externalId': string;
}
export interface ChatMessageIdentifierInternalId {
    'id': string;
}
export interface ChatMessageMenuItemUiExtensionApi {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<ChatMessageMenuItemVisibilityFilterApi>;
}
export interface ChatMessageMenuItemUiExtensionIn {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<ChatMessageMenuItemVisibilityFilterIn>;
}
export interface ChatMessageMenuItemVisibilityFilterApi {
    'className': string;
}
export interface ChatMessageMenuItemVisibilityFilterIn {
    'className': string;
}
export interface ChatMessageReactionAddedEvent {
    'messageId': string;
    'channelId': string;
    'threadId'?: string | null;
    'actor': CPrincipal;
    'emoji': string;
    'newCount': number;
}
export interface ChatMessageReactionRemovedEvent {
    'messageId': string;
    'channelId': string;
    'threadId'?: string | null;
    'actor': CPrincipal;
    'emoji': string;
    'newCount': number;
}
export interface ChatMessageText {
    'text': string;
}
export interface ChatMessageUnfurlContext {
    'channel': string;
    'messageId': ChatMessageIdentifier;
}
export interface ChatMessageUpdatedEvent {
    'channelId': string;
    'threadId'?: string | null;
    'message': ChannelItemRecord;
    'edited'?: AutomationJobEventStartTime | null;
    'pinned'?: AbsenceWebhookEventAvailable | null;
}
export interface ChatModification {
    'className': string;
    'id': string;
    'text': string;
    'attachments'?: Array<Attachment> | null;
    'pending': boolean;
    'temporaryId': string;
    'draftTag'?: string | null;
    'postponeId'?: string | null;
}
export interface ChatSubscriptionFilterIn {
    'className': string;
    'channel'?: string | null;
}
export interface ChatsChannelsAllChannelsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<AllChannelsListEntry>;
}
export interface ChatsChannelsChannelAdministratorGetRequest {
    'profile': ProfileIdentifier;
}
export interface ChatsChannelsChannelAttachmentsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ChannelInfoAttachment>;
}
export interface ChatsChannelsChannelDescriptionPatchRequest {
    'description': string;
}
export interface ChatsChannelsChannelIconPatchRequest {
    'icon'?: string | null;
}
export interface ChatsChannelsChannelSubscribersTeamsDelete200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDTeam>;
}
export interface ChatsChannelsChannelSubscribersTeamsDeleteRequest {
    'teams': Array<TeamIdentifier>;
}
export interface ChatsChannelsChannelSubscribersUsersDelete200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDMemberProfile>;
}
export interface ChatsChannelsChannelSubscribersUsersDeleteRequest {
    'profiles': Array<ProfileIdentifier>;
}
export interface ChatsChannelsConversationsChannelConvertPostRequest {
    'channelName': string;
}
export interface ChatsChannelsConversationsChannelSubjectPatchRequest {
    'subject': string;
}
export interface ChatsChannelsConversationsPostRequest {
    'profileIds': Array<string>;
    'subject'?: string | null;
}
export interface ChatsChannelsDmPostRequest {
    'profile': string;
}
export interface ChatsChannelsIsNameFreePostRequest {
    'name': string;
}
export interface ChatsChannelsPostRequest {
    'name': string;
    'description': string;
    'private': boolean;
}
export interface ChatsMessagesDeleteMessagePostRequest {
    'channel': ChannelIdentifier;
    'id': ChatMessageIdentifier;
}
export interface ChatsMessagesEditMessagePostRequest {
    'channel': ChannelIdentifier;
    'message': ChatMessageIdentifier;
    'content': ChatMessage;
    'attachments'?: Array<AttachmentIn> | null;
    'unfurlLinks'?: boolean | null;
    'resolveNames'?: boolean | null;
}
export interface ChatsMessagesImportPostRequest {
    'channel': ChannelIdentifier;
    'messages': Array<ImportMessage>;
    'suppressNotifications'?: boolean;
}
export interface ChatsMessagesPinPatchRequest {
    'channel': ChannelIdentifier;
    'message': ChatMessageIdentifier;
}
export interface ChatsMessagesPinnedMessagesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ChannelItemRecord>;
}
export interface ChatsMessagesSendMessagePostRequest {
    'channel'?: ChannelIdentifier;
    'content': ChatMessage;
    'attachments'?: Array<AttachmentIn> | null;
    'unfurlLinks'?: boolean | null;
    'externalId'?: string | null;
    'resolveNames'?: boolean | null;
    'pending'?: boolean | null;
    'recipient'?: MessageRecipient;
}
export interface ChatsMessagesSendPostRequest {
    'channel': string;
    'text': string;
    'pending'?: boolean;
    'temporaryId'?: string | null;
}
export interface Checklist {
    'id': string;
    'archived': boolean;
    'projectId'?: string | null;
    'project'?: PRProject;
    'issue'?: Issue;
    'owner'?: TDMemberProfile;
    'name': string;
    'root'?: PlanItem;
    'rootTag'?: PlanningTag;
    'description'?: string | null;
    'totalItemsCount': number;
    'doneItemsCount': number;
    'updatedTime'?: string | null;
}
export interface ChecklistBodyIn {
    'className': string;
    'text': string;
}
export interface ChecklistDocumentBody {
    'checklist': Checklist;
    'canConvertItemsToIssues': boolean;
}
export interface ChecklistDocumentBodyCreateIn {
    'bodyIn'?: ChecklistBodyIn;
}
export interface ChecklistDocumentBodyEditIn {
    'description'?: string | null;
    'owner'?: string | null;
    'tag'?: string | null;
    'topics'?: Array<string>;
    'bodyIn'?: ChecklistBodyIn;
}
export interface ChecklistIdentifier {
    'className': string;
    'id': string;
}
export interface ChecklistIdentifierId {
    'id': string;
}
export declare const ChecklistSorting: {
    readonly Updated: "UPDATED";
    readonly Title: "TITLE";
    readonly FavTitle: "FAV_TITLE";
};
export type ChecklistSorting = typeof ChecklistSorting[keyof typeof ChecklistSorting];
export interface ChecklistsChecklistItemsPlanItemDeleteRequest {
    'itemText'?: string;
    'itemDone'?: boolean;
}
export interface ChecklistsChecklistItemsPlanItemMovePostRequest {
    'targetParent': PlanItemIdentifier;
    'afterItem'?: PlanItemIdentifier;
}
export interface ChecklistsChecklistItemsPostRequest {
    'parentItem'?: PlanItemIdentifier;
    'itemText': string;
}
export interface CheckoutRevisionDTO {
    'repoName': string;
    'cloneDir'?: string | null;
    'branch': Branch;
    'commit'?: string | null;
}
export declare const ClientBrowser: {
    readonly Other: "Other";
    readonly Chrome: "Chrome";
    readonly Firefox: "Firefox";
    readonly Safari: "Safari";
    readonly Edge: "Edge";
    readonly Opera: "Opera";
};
export type ClientBrowser = typeof ClientBrowser[keyof typeof ClientBrowser];
export declare const ClientDeviceType: {
    readonly Other: "Other";
    readonly Phone: "Phone";
    readonly Tablet: "Tablet";
    readonly Desktop: "Desktop";
};
export type ClientDeviceType = typeof ClientDeviceType[keyof typeof ClientDeviceType];
export declare const ClientIDE: {
    readonly Other: "Other";
    readonly AppCode: "AppCode";
    readonly AndroidStudio: "AndroidStudio";
    readonly CLion: "CLion";
    readonly DataGrip: "DataGrip";
    readonly DataSpell: "DataSpell";
    readonly GoLand: "GoLand";
    readonly IdeaCommunity: "IDEACommunity";
    readonly IdeaEdu: "IDEAEdu";
    readonly IdeaUltimate: "IDEAUltimate";
    readonly Mps: "MPS";
    readonly PhpStorm: "PhpStorm";
    readonly PyCharmCommunity: "PyCharmCommunity";
    readonly PyCharmEdu: "PyCharmEdu";
    readonly PyCharmProfessional: "PyCharmProfessional";
    readonly RubyMine: "RubyMine";
    readonly WebStorm: "WebStorm";
};
export type ClientIDE = typeof ClientIDE[keyof typeof ClientIDE];
export interface ClientInfo {
    'os'?: ClientOS;
    'osVersion'?: string | null;
    'browser'?: ClientBrowser;
    'browserVersion'?: string | null;
    'ide'?: ClientIDE;
    'ideVersion'?: string | null;
    'device'?: string | null;
    'deviceType'?: ClientDeviceType;
    'version'?: string | null;
}
export declare const ClientOS: {
    readonly Other: "Other";
    readonly Windows: "Windows";
    readonly MacOs: "MacOS";
    readonly Linux: "Linux";
    readonly Android: "Android";
    readonly IOs: "iOS";
};
export type ClientOS = typeof ClientOS[keyof typeof ClientOS];
export interface ClientSideActionContext {
    'className': string;
    'jobId': string;
    'projectId': ProjectIdentifier;
    'projectKey': ProjectKey;
    'repository': string;
    'commitId': string;
    'issueNumber': number;
    'projectRepos': ProjectReposRecord;
    'issueCommitsRef': CommitLinksContainer;
    'url': string;
    'withBackUrl': boolean;
    'openInNewTab': boolean;
    'app': ESApp;
    'context': PermissionContextIdentifier;
    'tab': string;
    'feed': string;
    'feedId'?: string | null;
    'workspaceId': string;
}
export interface CodeDiscussionAddedFeedEvent {
    'codeDiscussion': CodeDiscussionRecord;
    'codeReview': CodeReviewRecord;
}
export interface CodeDiscussionAnchor {
    'project': ProjectKey;
    'repository': string;
    'revision': string;
    'filename'?: string | null;
    'line'?: number | null;
    'oldLine'?: number | null;
    'interpolatedLineState'?: InterpolatedLineState;
}
export interface CodeDiscussionRecord {
    'id': string;
    'projectId': string;
    'project'?: PRProject;
    'anchor': CodeDiscussionAnchor;
    'endAnchor'?: CodeDiscussionAnchor;
    'created': string;
    'channel': M2ChannelRecord;
    'resolvable'?: boolean | null;
    'resolved': boolean;
    'snippet'?: CodeDiscussionSnippet;
    'suggestedEdit'?: CodeDiscussionSuggestedEdit;
    'resolvedBy'?: CPrincipal;
    'pending'?: boolean | null;
    'review'?: CodeReviewRecord;
    'feedItemId'?: string | null;
    'reviews'?: Array<CodeReviewRecord> | null;
    'archived': boolean;
}
export interface CodeDiscussionSnippet {
    'className': string;
    'lines': Array<CodeLine>;
}
export interface CodeDiscussionSnippetInlineDiffSnippet {
    'lines': Array<InlineDiffLine>;
}
export interface CodeDiscussionSnippetPlainSnippet {
    'lines': Array<CodeLine>;
}
export interface CodeDiscussionSuggestedEdit {
    'suggestionCommitId': string;
    'status'?: CodeDiscussionSuggestedEditState;
    'resolvedBy'?: CPrincipal;
    'filePath': string;
    'hasConflicts': boolean;
    'identicalContents'?: boolean | null;
    'startLineIndex': number;
    'endLineIndexInclusive': number;
}
export interface CodeDiscussionSuggestedEditRequest {
    'path': string;
    'commitId': string;
    'startLineIndex': number;
    'endLineIndexInclusive': number;
    'suggestedContent': string;
}
export declare const CodeDiscussionSuggestedEditState: {
    readonly Accepted: "Accepted";
    readonly Rejected: "Rejected";
    readonly Deleted: "Deleted";
};
export type CodeDiscussionSuggestedEditState = typeof CodeDiscussionSuggestedEditState[keyof typeof CodeDiscussionSuggestedEditState];
export interface CodeLine {
    'text': string;
    'index'?: number | null;
    'offset': number;
    'syntax'?: Array<SyntaxMarkup> | null;
}
export interface CodeReviewDescription {
    'text': string;
    'unfurls': Array<Attachment>;
}
export interface CodeReviewDetailedInfo {
    'shortInfo': CodeReviewRecord;
    'commits': Array<RevisionsInReview>;
    'discussionCounter': CodeReviewDiscussionCounter;
    'unboundDiscussionCounter'?: CodeReviewUnboundDiscussionCounter;
    'branches': Array<TrackedBranchesInReview>;
}
export interface CodeReviewDiscussionAddedFeedEvent {
    'discussion': CodeReviewDiscussionRecord;
}
export interface CodeReviewDiscussionCounter {
    'id': string;
    'discussionCounter': DiscussionCounter;
}
export interface CodeReviewDiscussionRecord {
    'id': string;
    'review': CodeReviewRecord;
    'created': string;
    'channel'?: M2ChannelRecord;
    'resolved'?: boolean | null;
    'archived': boolean;
}
export interface CodeReviewMenuActionContext {
    'projectIdentifier': ProjectIdentifier;
    'codeReviewIdentifier': ReviewIdentifier;
}
export interface CodeReviewMenuItemUiExtensionApi {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<CodeReviewMenuItemVisibilityFilterApi>;
}
export interface CodeReviewMenuItemUiExtensionIn {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<CodeReviewMenuItemVisibilityFilterIn>;
}
export interface CodeReviewMenuItemVisibilityFilterApi {
    'className': string;
}
export interface CodeReviewMenuItemVisibilityFilterIn {
    'className': string;
}
export interface CodeReviewParticipant {
    'user': TDMemberProfile;
    'role': CodeReviewParticipantRole;
    'state'?: ReviewerState;
    'isApproveSticky'?: boolean | null;
    'theirTurn'?: boolean | null;
    'qualityGateSlots'?: Array<CodeReviewParticipantQualityGateSlot> | null;
    'codeOwnerSlots'?: Array<CodeReviewParticipantCodeOwnerSlot> | null;
    'addedAt'?: string | null;
}
export interface CodeReviewParticipantCodeOwnerSlot {
    'rule'?: Array<string> | null;
    'pattern': string;
}
export interface CodeReviewParticipantQualityGateSlot {
    'rule': Array<string>;
    'index': number;
}
export interface CodeReviewParticipantRecord {
    'id': string;
    'projectId': string;
    'role': CodeReviewParticipantRole;
    'profile': TDMemberProfile;
    'reviewerState'?: ReviewerState;
    'theirTurn'?: boolean | null;
    'archived': boolean;
}
export declare const CodeReviewParticipantRole: {
    readonly Reviewer: "Reviewer";
    readonly Author: "Author";
    readonly Watcher: "Watcher";
};
export type CodeReviewParticipantRole = typeof CodeReviewParticipantRole[keyof typeof CodeReviewParticipantRole];
export interface CodeReviewParticipantSlotBase {
    'className': string;
    'rule': Array<string>;
    'pattern': string;
    'index': number;
}
export interface CodeReviewParticipants {
    'id': string;
    'participants'?: Array<CodeReviewParticipant> | null;
    'reviewers': Array<CodeReviewParticipantRecord>;
    'authors': Array<CodeReviewParticipantRecord>;
    'watchers': Array<CodeReviewParticipantRecord>;
}
export interface CodeReviewPendingMessageCounter {
    'id': string;
    'archived': boolean;
}
export interface CodeReviewRecord {
    'className': string;
    'id': string;
    'project': ProjectKey;
    'projectId': string;
    'number': number;
    'title': string;
    'state': CodeReviewState;
    'canBeReopened'?: boolean | null;
    'createdAt': number;
    'createdBy'?: TDMemberProfile;
    'timestamp'?: number | null;
    'turnBased'?: boolean | null;
    'feedChannel'?: M2ChannelRecord;
    'feedChannelId'?: string | null;
    'issueIds': Array<string>;
    'discussionCounter': DiscussionCounter;
    'description'?: string | null;
    'unfurls': Array<Attachment>;
    'commits': Array<ReviewCommit>;
    'participants'?: Array<CodeReviewParticipant> | null;
    'reviewers': Array<CodeReviewParticipantRecord>;
    'authors': Array<CodeReviewParticipantRecord>;
    'watchers': Array<CodeReviewParticipantRecord>;
    'branchPairs': Array<MergeRequestBranchPair>;
    'externalLink'?: ExternalCodeReviewLink;
}
export declare const CodeReviewState: {
    readonly Opened: "Opened";
    readonly Closed: "Closed";
    readonly Deleted: "Deleted";
};
export type CodeReviewState = typeof CodeReviewState[keyof typeof CodeReviewState];
export declare const CodeReviewStateFilter: {
    readonly Opened: "Opened";
    readonly Closed: "Closed";
    readonly RequiresAuthorAttention: "RequiresAuthorAttention";
    readonly NeedsReview: "NeedsReview";
    readonly Merged: "Merged";
};
export type CodeReviewStateFilter = typeof CodeReviewStateFilter[keyof typeof CodeReviewStateFilter];
export interface CodeReviewSubscriptionFilter {
    'project'?: PRProject;
    'repository'?: string | null;
    'authors': Array<TDMemberProfile>;
    'participants': Array<TDMemberProfile>;
    'branchSpec': Array<string>;
    'pathSpec': Array<string>;
    'titleRegex': string;
}
export interface CodeReviewSubscriptionFilterIn {
    'project'?: string | null;
    'repository'?: string | null;
    'authors': Array<string>;
    'participants': Array<string>;
    'branchSpec': Array<string>;
    'pathSpec': Array<string>;
    'titleRegex': string;
}
export interface CodeReviewUnboundDiscussionCounter {
    'id': string;
}
export interface CodeReviewUnfurlContext {
    'reviewId': string;
    'field'?: CodeReviewUnfurlContextField;
}
export declare const CodeReviewUnfurlContextField: {
    readonly Title: "Title";
    readonly Description: "Description";
};
export type CodeReviewUnfurlContextField = typeof CodeReviewUnfurlContextField[keyof typeof CodeReviewUnfurlContextField];
export interface CodeReviewWebhookEvent {
    'projectKey': ProjectKey;
    'repository': string;
    'reviewId': string;
    'title': string;
    'isMergeRequest': boolean;
}
export interface CodeReviewWithCount {
    'review': CodeReviewRecord;
    'messagesCount': number;
    'authors': Array<GitCommitterProfile>;
    'participants': CodeReviewParticipants;
}
export interface CodeSnippetAnchor {
    'projectKey': ProjectKey;
    'repository': string;
    'revision': string;
    'filename': string;
    'lineIndex'?: number | null;
    'linesCount'?: number | null;
}
export interface CodeSuggestedEditHeadContentDetails {
    'codeDiscussion': CodeDiscussionRecord;
    'accepted'?: boolean | null;
}
export declare const ColumnSortOrder: {
    readonly Asc: "ASC";
    readonly Desc: "DESC";
};
export type ColumnSortOrder = typeof ColumnSortOrder[keyof typeof ColumnSortOrder];
export interface CommandDetail {
    'name': string;
    'description': string;
}
export interface Commands {
    'commands': Array<CommandDetail>;
}
export declare const CommitExecutionStatus: {
    readonly Scheduled: "SCHEDULED";
    readonly Pending: "PENDING";
    readonly ReadyToStart: "READY_TO_START";
    readonly FailedToStart: "FAILED_TO_START";
    readonly Running: "RUNNING";
    readonly Failing: "FAILING";
    readonly Succeeded: "SUCCEEDED";
    readonly Terminated: "TERMINATED";
    readonly Failed: "FAILED";
    readonly Hanging: "HANGING";
};
export type CommitExecutionStatus = typeof CommitExecutionStatus[keyof typeof CommitExecutionStatus];
export interface CommitInfo {
    'project': PRProject;
    'repository': string;
    'commitId': string;
    'message': string;
    'commitDate': string;
    'authorName': string;
    'authorEmail': string;
    'authorProfile'?: TDMemberProfile;
}
export interface CommitLinksContainer {
    'className': string;
    'id': string;
}
export interface CommitMessageUnfurlContext {
    'projectId': string;
    'repo': string;
    'commitId': string;
}
export interface CommitMessageUnfurlsRecord {
    'id': string;
    'unfurls': Array<Unfurl>;
}
export interface CommitSetReviewRecord {
    'id': string;
    'project': ProjectKey;
    'projectId': string;
    'number': number;
    'title': string;
    'state': CodeReviewState;
    'canBeReopened'?: boolean | null;
    'createdAt': number;
    'createdBy'?: TDMemberProfile;
    'timestamp'?: number | null;
    'turnBased'?: boolean | null;
    'feedChannel'?: M2ChannelRecord;
    'feedChannelId'?: string | null;
    'issueIds': Array<string>;
    'discussionCounter': DiscussionCounter;
    'description'?: string | null;
    'unfurls': Array<Attachment>;
    'commits': Array<ReviewCommit>;
    'participants'?: Array<CodeReviewParticipant> | null;
    'reviewers': Array<CodeReviewParticipantRecord>;
    'authors': Array<CodeReviewParticipantRecord>;
    'watchers': Array<CodeReviewParticipantRecord>;
}
export interface ComposerPackageDependency {
    'name': string;
    'version': string;
    'type': string;
}
export interface ComposerPackageType {
    'id': string;
}
export interface ComposerPackageVersionDetails {
    'type': PackageType;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'description'?: string | null;
    'homepage'?: string | null;
    'dependencies': Array<ComposerPackageDependency>;
    'keywords'?: Array<string> | null;
    'license'?: string | null;
    'projectUrl'?: string | null;
    'repositoryUrl'?: string | null;
    'repositoryRevision'?: string | null;
    'readme'?: string | null;
}
export interface ComputePoolIdentifier {
    'className': string;
    'id': string;
    'value': string;
}
export interface ComputePoolIdentifierId {
    'id': string;
}
export interface ComputePoolIdentifierKey {
    'value': string;
}
export interface ComputePoolKey {
    'value': string;
}
export interface ConfigurePermissionsAction {
    'app': ESApp;
    'context': PermissionContextIdentifier;
}
export declare const ConnectionTopology: {
    readonly Mesh: "MESH";
    readonly Sfu: "SFU";
};
export type ConnectionTopology = typeof ConnectionTopology[keyof typeof ConnectionTopology];
export interface ContactCFParameters {
    'contactKind'?: TDContactKind;
}
export interface ContactCFValue {
    'value'?: string | null;
}
export interface ContactExtraTag {
    'primary': string;
    'secondary': string;
    'issueStatus'?: string | null;
}
export interface ContactListCFValue {
    'values': Array<string>;
}
export interface ContainerHelmChart {
    'name'?: string | null;
    'description'?: string | null;
    'tags'?: Array<string> | null;
    'projectUrl'?: string | null;
    'sourceUrl'?: string | null;
    'version'?: string | null;
    'apiVersion'?: string | null;
    'appVersion'?: string | null;
    'dependencies': Array<ContainerHelmChartDependency>;
    'type'?: string | null;
}
export interface ContainerHelmChartDependency {
    'name': string;
    'version': string;
    'repository'?: string | null;
    'condition'?: string | null;
    'tags'?: Array<string> | null;
}
export interface ContainerImage {
    'name'?: string | null;
    'description'?: string | null;
    'tags'?: Array<string> | null;
    'projectUrl'?: string | null;
    'sourceUrl'?: string | null;
    'version'?: string | null;
    'platform'?: ContainerImagePlatform;
    'history': Array<ContainerImageLayer>;
    'children': Array<ContainerManifest>;
    'config'?: ContainerImageConfig;
    'annotation'?: ContainerImageAnnotation;
}
export interface ContainerImageAnnotation {
    'created'?: number | null;
    'buildName'?: string | null;
    'buildUrl'?: string | null;
    'revision'?: string | null;
    'vendor'?: string | null;
    'documentationUrl'?: string | null;
    'licenses'?: string | null;
}
export interface ContainerImageConfig {
    'userName'?: string | null;
    'ports'?: Array<string> | null;
    'volumes'?: Array<string> | null;
    'env'?: Array<string> | null;
    'workingDir'?: string | null;
    'entryPoint'?: string | null;
    'cmd'?: string | null;
    'labels'?: Array<string> | null;
}
export interface ContainerImageLayer {
    'id': string;
    'created'?: number | null;
    'statement': string;
    'command': string;
    'size'?: number | null;
}
export interface ContainerImagePlatform {
    'os': string;
    'osVersion'?: string | null;
    'arch'?: string | null;
}
export interface ContainerManifest {
    'name': string;
    'digest': string;
    'tags': Array<string>;
    'created': number;
    'schemaVersion': number;
    'mediaType': string;
    'manifestType': string;
    'totalSize': number;
    'image'?: ContainerImage;
    'chart'?: ContainerHelmChart;
}
export interface ContainerPackageType {
    'id': string;
}
export interface ContainerPackageVersionDetails {
    'type': PackageType;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'schemaVersion': number;
    'mediaType': string;
    'manifestType': string;
    'image'?: ContainerImage;
    'chart'?: ContainerHelmChart;
}
export interface ContextMenuItemUiExtensionApi {
    'className': string;
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<DocumentFolderMenuItemVisibilityFilterApi>;
}
export interface ContextMenuItemUiExtensionIn {
    'className': string;
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<ChatMessageMenuItemVisibilityFilterIn>;
}
export interface CratesPackageDependency {
    'name': string;
    'versionReq': string;
    'features': Array<string>;
    'optional': boolean;
    'defaultFeatures': boolean;
    'target'?: string | null;
    'kind': string;
    'registry'?: string | null;
}
export interface CratesPackageType {
    'id': string;
}
export interface CratesPackageVersionDetails {
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'dependencies': Array<CratesPackageDependency>;
    'description'?: string | null;
    'documentation'?: string | null;
    'homepage'?: string | null;
    'readme'?: string | null;
    'keywords': Array<string>;
    'categories': Array<string>;
    'license'?: string | null;
    'licenseFileContent'?: string | null;
    'gitRepository'?: string | null;
    'links'?: string | null;
}
export interface Currency {
    'code': string;
    'sign'?: string | null;
    'signAsPrefix': boolean;
    'hasCents': boolean;
}
export interface CustomBoardIssueInputField {
    'key': ExtendedTypeKey;
    'fieldId': string;
}
export interface CustomEmojiAddedEvent {
    'emoji': string;
    'owner': CPrincipal;
    'attachmentId': string;
    'uploadedAt': string;
}
export interface CustomEmojiDeletedEvent {
    'emoji': string;
    'owner': CPrincipal;
}
export interface CustomEmojiInfo {
    'name': string;
    'provider': CPrincipal;
    'uploaded': string;
    'attachmentId'?: string | null;
    'deleted'?: boolean | null;
}
export interface CustomEmojiUpdatedEvent {
    'emoji': string;
    'owner': CPrincipal;
    'attachmentId'?: AbsenceWebhookEventIcon | null;
    'uploadedAt'?: AutomationJobEventStartTime | null;
}
export interface CustomField {
    'extendedType': ExtendedType;
    'id': string;
    'name': string;
    'description'?: string | null;
    'key'?: string | null;
    'type': CFType;
    'constraint'?: CFConstraint;
    'required': boolean;
    'private': boolean;
    'access'?: AccessType;
    'defaultValue': CFValue;
    'order': number;
    'scope'?: ExtendedTypeScope;
    'deleted'?: boolean | null;
    'parameters'?: CFParameters;
    'archived': boolean;
}
export interface CustomFieldData {
    'id': string;
    'name': string;
    'type': CustomFieldType;
    'multivalued': boolean;
    'parameters'?: CFParameters;
    'required': boolean;
    'defaultValue': CFValue;
    'constraint'?: CFConstraint;
    'description'?: string | null;
    'order': number;
    'archived': boolean;
}
export interface CustomFieldInputValue {
    'fieldId': string;
    'value': CFInputValue;
}
export declare const CustomFieldType: {
    readonly Autonumber: "AUTONUMBER";
    readonly Boolean: "BOOLEAN";
    readonly Date: "DATE";
    readonly DateTime: "DATE_TIME";
    readonly Document: "DOCUMENT";
    readonly Enum: "ENUM";
    readonly Integer: "INTEGER";
    readonly Issue: "ISSUE";
    readonly Location: "LOCATION";
    readonly OpenEnum: "OPEN_ENUM";
    readonly Percentage: "PERCENTAGE";
    readonly Profile: "PROFILE";
    readonly Project: "PROJECT";
    readonly String: "STRING";
    readonly Team: "TEAM";
    readonly Url: "URL";
    readonly Commit: "COMMIT";
};
export type CustomFieldType = typeof CustomFieldType[keyof typeof CustomFieldType];
export interface CustomFieldValue {
    'field': CustomField;
    'value': CFValue;
}
export interface CustomFieldValueData {
    'customField': CustomFieldData;
    'value': CFValue;
}
export interface CustomFieldValueUpdate {
    'field': CFIdentifier;
    'newValue': CFInputValue;
}
export interface CustomFieldsRecord {
    'id': string;
    'values': Array<CustomFieldValue>;
}
export interface CustomFieldsTypeKeyAllValuesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<CustomFieldsRecord>;
}
export interface CustomFieldsTypeKeyEntityIdValuesGetRequest {
    'values': Array<CustomFieldInputValue>;
    'scope': ExtendedTypeScope;
}
export interface CustomFieldsTypeKeyEnumValuesCustomFieldIdGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<EnumValueData>;
}
export interface CustomFieldsTypeKeyEnumValuesCustomFieldIdGetRequest {
    'valuesToAdd': Array<EnumValueData>;
    'scope': ExtendedTypeScope;
}
export interface CustomFieldsTypeKeyFieldsGetRequest {
    'name': string;
    'description'?: string | null;
    'type': CFType;
    'constraint'?: CFConstraint;
    'required': boolean;
    'private': boolean;
    'access'?: AccessType;
    'defaultValue': CFInputValue;
    'openEnumValuesModification'?: CFEnumValuesModification;
    'cfParameters'?: CFCreateParameters;
    'scope': ExtendedTypeScope;
}
export interface CustomFieldsTypeKeyFieldsIdArchivePostRequest {
    'scope': ExtendedTypeScope;
}
export interface CustomFieldsTypeKeyFieldsIdDeleteRequest {
    'name'?: string | null;
    'description'?: string | null;
    'constraint'?: CFConstraint;
    'required'?: boolean | null;
    'private'?: boolean | null;
    'access'?: AccessType;
    'defaultValue'?: CFInputValue;
    'enumValues'?: Array<EnumValueData> | null;
    'openEnumValuesModification'?: CFEnumValuesModification;
    'cfParameters'?: CFUpdateParameters;
    'scope': ExtendedTypeScope;
}
export interface CustomFieldsTypeKeyFieldsIdFilterValuesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<CFValue>;
}
export interface CustomFieldsTypeKeyFieldsReorderPostRequest {
    'customFieldOrder': Array<string>;
    'scope': ExtendedTypeScope;
}
export interface CustomFieldsV2EntityTypeFieldsCustomFieldDeleteRequest {
    'name'?: string | null;
    'parameters'?: CFUpdateParameters;
    'required'?: boolean | null;
    'defaultValue'?: CFInputValue;
    'constraint'?: CFConstraint;
    'description'?: string | null;
}
export interface CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest {
    'enumValueModifications': Array<CFEnumValueModification>;
}
export interface CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<CFEnumValue>;
}
export interface CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest {
    'enumValueToAdd': string;
}
export interface CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1 {
    'enumValueToUpdate': CFEnumValueIdentifier;
    'newName': string;
}
export interface CustomFieldsV2EntityTypeFieldsGetRequest {
    'name': string;
    'type': CustomFieldType;
    'multivalued'?: boolean;
    'parameters'?: CFCreateParameters;
    'required'?: boolean;
    'defaultValue'?: CFInputValue;
    'constraint'?: CFConstraint;
    'description'?: string | null;
    'order'?: number | null;
}
export interface CustomFieldsV2EntityTypeFieldsReorderPostRequest {
    'customFields': Array<CFIdentifier>;
}
export interface CustomFieldsV2ValuesEntityCustomFieldGetRequest {
    'newValue': CFInputValue;
}
export interface CustomFieldsV2ValuesEntityGetRequest {
    'customFieldValues': Array<CustomFieldValueUpdate>;
}
export interface CustomGenericSubscriptionIn {
    'subjectCode': string;
    'filters': Array<SubscriptionFilterIn>;
    'eventTypeCodes': Array<string>;
}
export interface CustomHttpHeaderDTO {
    'name': string;
    'value': string;
    'description'?: string | null;
}
export interface DTOFullRightWithoutType {
    'code': string;
    'title': string;
    'description'?: string | null;
    'grantedForUsers': boolean;
    'featureFlag'?: FeatureFlag;
}
export interface DTOMeeting {
    'id': string;
    'archived': boolean;
    'summary': string;
    'description'?: string | null;
    'locations': Array<TDLocation>;
    'profiles': Array<TDMemberProfile>;
    'teams': Array<TDTeam>;
    'occurrenceRule': CalendarEventSpec;
    'origin': MeetingOrigin;
    'conferenceLink'?: string | null;
    'visibility': MeetingVisibility;
    'modificationPreference': MeetingModificationPreference;
    'joiningPreference'?: MeetingJoiningPreference;
    'organizer'?: MeetingOrganizer;
    'etag': number;
    'privateDataSubstituted': boolean;
    'canModify': boolean;
    'canDelete': boolean;
    'canJoin': boolean;
    'externalParticipants': Array<string>;
    'linkToExternalSource'?: string | null;
    'eventAttachments'?: Array<MeetingAttachment> | null;
    'conferenceData'?: EventConferenceData;
    'channelRef'?: M2ChannelRecord;
    'externalSource'?: EventExternalSource;
    'calendar'?: CalendarInfo;
    'canLeaveOrRsvp'?: boolean | null;
}
export interface DTOMeetingRSVP {
    'id': string;
    'archived': boolean;
    'member': TDMemberProfile;
    'meeting': DTOMeeting;
    'status': EventParticipationStatus;
}
export interface DTORight {
    'typeCode': string;
    'code': string;
}
export interface DTORightType {
    'typeCode': string;
    'title': string;
}
export interface DTORightTypeRight {
    'rightType': DTORightType;
    'rights': Array<DTOFullRightWithoutType>;
}
export interface DTORightsDeps {
    'code': string;
    'deps': Array<string>;
}
export interface DTORightsGroup {
    'title': string;
    'priority': number;
    'rights': Array<DTORight>;
}
export interface DTORightsWithHierarchy {
    'rights': Array<DTORightTypeRight>;
    'rightsGroups': Array<DTORightsGroup>;
    'deps': Array<DTORightsDeps>;
}
export declare const DarkTheme: {
    readonly Auto: "AUTO";
    readonly Enabled: "ENABLED";
    readonly Disabled: "DISABLED";
};
export type DarkTheme = typeof DarkTheme[keyof typeof DarkTheme];
export interface DartPackageDependency {
    'name': string;
    'version': string;
}
export interface DartPackageType {
    'id': string;
}
export interface DartPackageVersionDetails {
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'description': string;
    'homePage': string;
    'repositoryUrl': string;
    'issueTracker': string;
    'documentation': string;
    'license': string;
    'readme': string;
    'changelog': string;
    'dependencies'?: Array<DartPackageDependency> | null;
    'devDependencies'?: Array<DartPackageDependency> | null;
    'dependencyOverrides'?: Array<DartPackageDependency> | null;
    'environment'?: Array<DartPackageDependency> | null;
}
export declare const DashboardContainerType: {
    readonly OldPersonal: "OLD_PERSONAL";
    readonly Project: "PROJECT";
    readonly Personal: "PERSONAL";
};
export type DashboardContainerType = typeof DashboardContainerType[keyof typeof DashboardContainerType];
export interface DashboardIdentifier {
    'className': string;
    'id': string;
    'profile': ProfileIdentifier;
    'project': ProjectIdentifier;
}
export interface DashboardIdentifierId {
    'id': string;
}
export interface DashboardIdentifierPersonal {
    'profile': ProfileIdentifier;
}
export interface DashboardIdentifierProject {
    'project': ProjectIdentifier;
    'profile': ProfileIdentifier;
}
export interface DashboardItemData {
    'name': string;
    'isEnabled': boolean;
}
export interface DashboardPreferencesRecord {
    'id': string;
    'archived': boolean;
    'data'?: Array<Array<DashboardItemData>> | null;
    'type'?: DashboardContainerType;
}
export interface DataProducerOptions {
    'sfuOptions': SFUDataProducerOptions;
    'closed': boolean;
    'userConnectionId'?: number | null;
    'routerId': string;
}
export interface DateCFConstraint {
    'min'?: string | null;
    'max'?: string | null;
    'message'?: string | null;
}
export interface DateCFValue {
    'value'?: string | null;
}
export interface DateTimeCFConstraint {
    'min'?: string | null;
    'max'?: string | null;
    'message'?: string | null;
}
export interface DateTimeCFValue {
    'value'?: string | null;
}
export interface DateTimeViewParams {
    'format'?: UnfurlDateTimeFormat;
    'style'?: UnfurlDateTimeStyle;
    'template'?: string | null;
}
export interface DeleteAutomationLegacyChannelAction {
    'jobId': string;
    'projectId': ProjectIdentifier;
}
export interface DeleteMessage {
    'id': string;
}
export interface DeletePersonalFeedAction {
    'feed': string;
    'feedId'?: string | null;
}
export interface DeletedAttachment {
    'deletedIdentity': string;
}
export interface DeployTargetRecord {
    'id': string;
    'projectId': string;
    'name': string;
    'key': string;
    'description': string;
    'createdAt': string;
    'lastUpdated'?: string | null;
    'lastDeployed'?: string | null;
    'channel': M2ChannelRecord;
    'archived': boolean;
    'current'?: DeploymentInfo;
    'next'?: DeploymentInfo;
    'createdBy'?: CPrincipal;
    'channelId': string;
    'repositories': Array<DeployTargetRepositoryDTO>;
    'manualControl': boolean;
    'hangTimeoutMinutes'?: number | null;
    'failTimeoutMinutes'?: number | null;
}
export interface DeployTargetRepositoryDTO {
    'repositoryName': string;
    'missing': boolean;
    'repositoryId'?: string | null;
}
export interface DeploymentCFInputValue {
    'deployment'?: DeploymentIdentifier;
}
export interface DeploymentCFValue {
    'deployment'?: DeploymentRecord;
}
export interface DeploymentCommitRefDetails {
    'repositoryName': string;
    'branch': string;
    'commit': string;
    'timestamp'?: number | null;
    'head': boolean;
    'totalCommits'?: number | null;
    'totalMerges'?: number | null;
    'totalIssues'?: number | null;
    'rollback': boolean;
    'missing': boolean;
}
export interface DeploymentCommitReference {
    'repositoryName': string;
    'branch': string;
    'commit'?: string | null;
}
export interface DeploymentData {
    'id': string;
    'version': string;
    'createdAt': string;
    'scheduledStart'?: string | null;
    'startedAt'?: string | null;
    'finishedAt'?: string | null;
    'targetKey': string;
    'status': DeploymentStatus;
    'syncStatus': DeploymentSyncStatus;
    'externalLink'?: ExternalLink;
    'commitRefs': Array<DeploymentCommitRefDetails>;
}
export interface DeploymentIdentifier {
    'className': string;
    'id': string;
    'status': DeploymentIdentifierStatus;
    'version': string;
}
export interface DeploymentIdentifierId {
    'id': string;
}
export interface DeploymentIdentifierStatus {
    'status': DeploymentIdentifierStatus;
}
export interface DeploymentIdentifierVersion {
    'version': string;
}
export interface DeploymentInfo {
    'className': string;
    'id': string;
    'version': string;
    'createdAt': string;
    'scheduledStart'?: string | null;
    'startedAt'?: string | null;
    'finishedAt'?: string | null;
    'targetKey': string;
    'status': DeploymentStatus;
    'syncStatus': DeploymentSyncStatus;
    'externalLink'?: ExternalLink;
    'commitRefs': Array<DeploymentCommitRefDetails>;
    'description'?: string | null;
    'channel': M2ChannelRecord;
    'target': DeployTargetRecord;
    'jobIds'?: Array<string> | null;
    'archived': boolean;
    'totalCommits'?: number | null;
    'totalMerges'?: number | null;
    'totalIssues'?: number | null;
}
export interface DeploymentM2ChannelInfo {
    'notificationDefaults': ChannelSpecificDefaults;
    'deployment': DeploymentRecord;
    'project'?: PRProject;
}
export interface DeploymentM2DetailsWithThread {
    'deployment': DeploymentRecord;
}
export interface DeploymentRecord {
    'id': string;
    'version': string;
    'scheduledStart'?: string | null;
    'startedAt'?: string | null;
    'finishedAt'?: string | null;
    'createdAt': string;
    'status': DeploymentStatus;
    'description'?: string | null;
    'channel': M2ChannelRecord;
    'target': DeployTargetRecord;
    'targetKey': string;
    'syncStatus': DeploymentSyncStatus;
    'commitRefs': Array<DeploymentCommitRefDetails>;
    'jobIds'?: Array<string> | null;
    'externalLink'?: ExternalLink;
    'archived': boolean;
    'totalCommits'?: number | null;
    'totalMerges'?: number | null;
    'totalIssues'?: number | null;
}
export declare const DeploymentStatus: {
    readonly Scheduled: "SCHEDULED";
    readonly Deploying: "DEPLOYING";
    readonly Failed: "FAILED";
    readonly Current: "CURRENT";
    readonly Obsolete: "OBSOLETE";
    readonly Hanging: "HANGING";
};
export type DeploymentStatus = typeof DeploymentStatus[keyof typeof DeploymentStatus];
export declare const DeploymentSyncStatus: {
    readonly Synced: "SYNCED";
    readonly Dirty: "DIRTY";
    readonly Syncing: "SYNCING";
    readonly Failed: "FAILED";
};
export type DeploymentSyncStatus = typeof DeploymentSyncStatus[keyof typeof DeploymentSyncStatus];
export interface DeploymentTargetM2ChannelInfo {
    'notificationDefaults': ChannelSpecificDefaults;
    'deployTarget': DeployTargetRecord;
    'project': PRProject;
}
export interface DeploymentWebhookEvent {
    'meta': KMetaMod;
    'projectKey': string;
    'targetId': string;
    'deploymentId': string;
    'statusMod'?: DeploymentWebhookEventStatusMod | null;
}
export interface DeploymentWebhookEventStatusMod {
    'old': DeploymentStatus;
    'new': DeploymentStatus;
}
export interface DeploymentsSubscriptionFilter {
    'project'?: PRProject;
    'repositories'?: Array<string> | null;
    'targetIdentifiers'?: Array<string> | null;
}
export interface DeploymentsSubscriptionFilterIn {
    'project': string;
    'repositories'?: Array<string> | null;
    'targetIdentifiers'?: Array<string> | null;
}
export interface Diff {
    'added': Array<string>;
    'removed': Array<string>;
    'removeAll': boolean;
}
export interface DiffContext {
    'left'?: DiffSide;
    'right': DiffSide;
}
export declare const DiffLineType: {
    readonly Added: "ADDED";
    readonly Deleted: "DELETED";
    readonly Modified: "MODIFIED";
    readonly FilteredAdded: "FILTERED_ADDED";
    readonly FilteredDeleted: "FILTERED_DELETED";
    readonly Filtered: "FILTERED";
    readonly FilteredModified: "FILTERED_MODIFIED";
    readonly ConflictOld: "CONFLICT_OLD";
    readonly ConflictNew: "CONFLICT_NEW";
};
export type DiffLineType = typeof DiffLineType[keyof typeof DiffLineType];
export interface DiffSide {
    'className': string;
    'revision': string;
    'path': string;
}
export interface DiffSideEmpty {
    'revision': string;
}
export interface DiffSideNonEmpty {
    'revision': string;
    'path': string;
}
export interface DiscussionCounter {
    'resolved': number;
    'unresolved': number;
    'unresolvedSuggestedEdits': number;
    'acceptedSuggestedEdits': number;
    'rejectedSuggestedEdits': number;
}
export interface Document {
    'id': string;
    'title': string;
    'alias': string;
    'publicationDetails'?: PublicationDetails;
    'folderRef'?: DocumentFolder;
    'bodyType'?: DocumentBodyType;
    'bodyInfo'?: DocumentBodyInfo;
    'archived': boolean;
    'archivedBy'?: CPrincipal;
    'archivedAt'?: string | null;
    'createdBy'?: CPrincipal;
    'created'?: string | null;
    'modifiedBy'?: CPrincipal;
    'modified': string;
    'documentBody': DocumentBodyInfo;
}
export interface DocumentAppUnfurlContext {
    'documentId': string;
}
export interface DocumentBodyCreateIn {
    'className': string;
    'docContent': TextDocumentContent;
    'blobId': string;
    'bodyIn'?: ChecklistBodyIn;
}
export interface DocumentBodyInfo {
    'className': string;
    'docBodyType': DocumentBodyType;
    'docBodyId': string;
    'fileDocument': FileDocumentRecord;
    'versionId': string;
    'contentType': string;
    'fileSize': number;
    'id': string;
    'resetCounter': number;
    'version'?: number | null;
    'type': DraftDocumentType;
    'text': string;
    'model'?: string | null;
    'attachments'?: Array<AttachmentInfo> | null;
    'checklist': Checklist;
    'canConvertItemsToIssues': boolean;
    'textDocument': TextDocumentRecord;
}
export declare const DocumentBodyType: {
    readonly Text: "TEXT";
    readonly Checklist: "CHECKLIST";
    readonly File: "FILE";
};
export type DocumentBodyType = typeof DocumentBodyType[keyof typeof DocumentBodyType];
export interface DocumentBodyUpdateIn {
    'className': string;
    'blobId': string;
    'docContent': TextDocumentContent;
    'type': DraftDocumentType;
    'description'?: string | null;
    'owner'?: string | null;
    'tag'?: string | null;
    'topics'?: Array<string>;
    'bodyIn'?: ChecklistBodyIn;
}
export interface DocumentCFInputValue {
    'documentId'?: string | null;
}
export interface DocumentCFParameters {
    'documentScope': DocumentCFScope;
}
export interface DocumentCFParametersInput {
    'documentScope': DocumentCFScopeInput;
}
export interface DocumentCFScope {
    'className': string;
    'project': PRProject;
}
export interface DocumentCFScopeInput {
    'className': string;
    'projectId': string;
}
export interface DocumentCFScopeInputProject {
    'projectId': string;
}
export interface DocumentCFScopeProject {
    'project': PRProject;
}
export interface DocumentCFValue {
    'document'?: Document;
}
export interface DocumentContainerInfo {
    'className': string;
    'owner': TDMemberProfile;
    'grantedRights'?: Array<string> | null;
    'book': KBBook;
}
export interface DocumentCustomSubscriptionFilter {
    'project'?: PRProject;
    'folders'?: Array<DocumentFolder> | null;
    'documents': Array<Document>;
}
export interface DocumentCustomSubscriptionFilterIn {
    'project'?: string | null;
    'folders'?: Array<string> | null;
    'documents': Array<string>;
}
export interface DocumentEditorsChangedEvent {
    'meta': KMetaMod;
    'document': string;
}
export interface DocumentFolder {
    'id': string;
    'archived': boolean;
    'parent'?: DocumentFolder;
    'name': string;
    'created': string;
    'createdBy'?: CPrincipal;
    'updated': string;
    'updatedBy'?: CPrincipal;
    'cover'?: Document;
}
export interface DocumentFolderMenuActionContext {
    'folderIdentifier': FolderIdentifier;
    'projectIdentifier'?: ProjectIdentifier;
}
export interface DocumentFolderMenuItemUiExtensionApi {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<DocumentFolderMenuItemVisibilityFilterApi>;
}
export interface DocumentFolderMenuItemUiExtensionIn {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<DocumentFolderMenuItemVisibilityFilterIn>;
}
export interface DocumentFolderMenuItemVisibilityFilterApi {
    'className': string;
}
export interface DocumentFolderMenuItemVisibilityFilterIn {
    'className': string;
}
export interface DocumentFolderWithChildren {
    'id': string;
    'archived': boolean;
    'containerLinkId': string;
    'containerInfo': DocumentContainerInfo;
    'subfolders': Array<DocumentFolderWithChildren>;
    'documents': Array<Document>;
}
export interface DocumentHistoryUnfurlDetails {
    'document': string;
    'title': string;
    'version'?: string | null;
    'base'?: string | null;
    'preview'?: string | null;
    'version2'?: string | null;
    'base2'?: string | null;
    'preview2'?: string | null;
}
export interface DocumentListCFInputValue {
    'documents': Array<string>;
}
export interface DocumentListCFValue {
    'documents': Array<Document>;
}
export interface DocumentMark {
    'className': string;
    'attrs': RtLinkAttrs;
}
export interface DocumentMenuActionContext {
    'documentId': string;
    'projectIdentifier'?: ProjectIdentifier;
}
export interface DocumentMenuItemUiExtensionApi {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<DocumentMenuItemVisibilityFilterApi>;
}
export interface DocumentMenuItemUiExtensionIn {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<DocumentMenuItemVisibilityFilterIn>;
}
export interface DocumentMenuItemVisibilityFilterApi {
    'className': string;
}
export interface DocumentMenuItemVisibilityFilterIn {
    'className': string;
}
export interface DocumentMetaWebhookEvent {
    'meta': KMetaMod;
    'document': string;
    'deleted'?: AbsenceWebhookEventAvailable | null;
    'published'?: AbsenceWebhookEventAvailable | null;
}
export interface DocumentWebhookEvent {
    'meta': KMetaMod;
    'document': string;
    'changeAuthors': Array<CPrincipal>;
    'version': string;
    'base': string;
}
export declare const DraftDocumentType: {
    readonly RichText: "RICH_TEXT";
    readonly Wysiwyg: "WYSIWYG";
    readonly Markdown: "MARKDOWN";
    readonly NewRichText: "NEW_RICH_TEXT";
};
export type DraftDocumentType = typeof DraftDocumentType[keyof typeof DraftDocumentType];
export interface DryCleanupResults {
    'className': string;
    'error': string;
    'totalSize': number;
    'totalCount': number;
    'packageVersions': Array<PackageVersionRef>;
}
export interface DryCleanupResultsFailure {
    'error': string;
}
export interface DryCleanupResultsResults {
    'totalSize': number;
    'totalCount': number;
    'packageVersions': Array<PackageVersionRef>;
}
export interface DryRunResult {
    'teams': Array<TDTeam>;
    'projects': Array<PRProject>;
    'locations': Array<TDLocation>;
    'records': Array<string>;
}
export interface DslEvaluationConfig {
    'experimentalOptIns': Array<string>;
    'runtimeInfo': DslRuntimeInfo;
}
export interface DslRuntimeInfo {
    'mavenRepoUrl': string;
    'mavenCoordinates': MavenCoordinates;
}
export declare const DurationTextFormat: {
    readonly Weeks: "WEEKS";
    readonly Days: "DAYS";
    readonly Hours: "HOURS";
};
export type DurationTextFormat = typeof DurationTextFormat[keyof typeof DurationTextFormat];
export interface ESApp {
    'id': string;
    'owner'?: TDMemberProfile;
    'ownerApp'?: ESApp;
    'clientId': string;
    'name': string;
    'picture'?: string | null;
    'defaultExternalPicture'?: string | null;
    'createdAt'?: string | null;
    'kind'?: string | null;
    'presentableName'?: string | null;
    'applicationType'?: ApplicationType;
    'clientCredentialsFlowEnabled'?: boolean | null;
    'codeFlowEnabled'?: boolean | null;
    'codeFlowRedirectURIs'?: string | null;
    'pkceRequired'?: boolean | null;
    'implicitFlowEnabled'?: boolean | null;
    'implicitFlowRedirectURIs'?: string | null;
    'endpointURI'?: string | null;
    'hasVerificationToken'?: boolean | null;
    'hasSigningKey'?: boolean | null;
    'hasPublicKeySignature'?: boolean | null;
    'endpointSslVerification'?: boolean | null;
    'basicAuthUsername'?: string | null;
    'hasBearerToken'?: boolean | null;
    'sslKeystoreAuth'?: string | null;
    'archived': boolean;
    'domains': Array<ApplicationUnfurlDomain>;
    'patterns': Array<ApplicationUnfurlPattern>;
    'contexts': Array<AppUiExtContextData>;
    'metadata'?: ApplicationMetadata;
    'settings': ESAppSettings;
}
export interface ESAppSettings {
    'clientCredentialsFlowEnabled'?: boolean | null;
    'codeFlowEnabled'?: boolean | null;
    'codeFlowRedirectURIs'?: string | null;
    'pkceRequired'?: boolean | null;
    'implicitFlowEnabled'?: boolean | null;
    'implicitFlowRedirectURIs'?: string | null;
    'endpoint'?: EndpointDTO;
    'endpointAuth'?: EndpointAuthDTO;
}
export interface ESApplicationPassword {
    'id': string;
    'profile': TDMemberProfile;
    'name': string;
    'scope': XScopeApi;
    'created': string;
    'lastAccess'?: AccessRecord;
}
export interface ESApplicationPermanentToken {
    'id': string;
    'name': string;
    'application': ESApp;
    'scope': string;
    'apiScope': XScopeApi;
    'created': string;
    'expires'?: string | null;
    'lastAccess'?: AccessRecord;
}
export interface ESApprovedScope {
    'id': string;
    'scope': XScopeApi;
    'offlineAllowed': boolean;
}
export interface ESAuthModule {
    'id': string;
    'key': string;
    'name': string;
    'ordinal': number;
    'enabled': boolean;
    'iconDataURI'?: string | null;
    'settings': ESAuthModuleSettings;
}
export interface ESAuthModuleSettings {
    'className': string;
    'passwordModule'?: boolean | null;
    'federatedModule'?: boolean | null;
    'type': LdapModuleType;
    'registerNewUsers': boolean;
    'serverUrl': string;
    'connectionTimeout': number;
    'readTimeout': number;
    'sslKeystore'?: SSLKeystore;
    'teamMappings': Array<ESTeamMapping>;
    'referralIgnored': boolean;
    'filter': string;
    'bindUserDN': string;
    'bindUserPassword': string;
    'attributeNames': ESSamlAttributeNames;
    'passwordStrengthPolicy': PasswordStrength;
    'domains': Array<string>;
    'idpUrl': string;
    'idpEntityId': string;
    'idpCertificateSHA256': string;
    'spEntityId': string;
    'contactProfileId'?: string | null;
    'githubUrl': string;
    'clientId': string;
    'clientSecret': string;
    'organizations': Array<string>;
    'discoveryUrl'?: string | null;
    'issuer': string;
    'authorizationEndpoint': string;
    'tokenEndpoint': string;
    'tokenKeysEndpoint': string;
    'userInfoEndpoint': string;
    'hubUrl': string;
    'orgAuthProviderName'?: string | null;
    'groups'?: Array<string> | null;
    'tenantId': string;
    'emailVerified': boolean;
}
export interface ESAuthenticationSession {
    'id': string;
    'profile': TDMemberProfile;
    'created': string;
    'expires': string;
    'lastAccess'?: AccessRecord;
    'current': boolean;
}
export interface ESAzureAuthModuleSettings {
    'tenantId': string;
    'clientId': string;
    'clientSecret': string;
    'registerNewUsers': boolean;
    'emailVerified': boolean;
}
export interface ESBuiltinAuthModuleSettings {
    'passwordStrengthPolicy': PasswordStrength;
    'domains'?: Array<string> | null;
}
export interface ESBuiltinProfileLoginDetails {
    'passwordChangeRequest'?: string | null;
}
export interface ESComposerRepositorySettings {
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
}
export interface ESContainerRegistrySettings {
    'immutableTags': boolean;
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
}
export interface ESCratesRegistrySettings {
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
}
export interface ESDartRepositorySettings {
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
}
export interface ESDefaultProfileLoginDetails {
    'login'?: string | null;
    'firstName'?: string | null;
    'lastName'?: string | null;
    'email'?: string | null;
    'emailVerified': boolean;
    'avatarUrl'?: string | null;
}
export interface ESExternalPasswordAuthModuleSettings {
    'className': string;
    'type': LdapModuleType;
    'registerNewUsers': boolean;
    'serverUrl': string;
    'connectionTimeout': number;
    'readTimeout': number;
    'sslKeystore'?: SSLKeystore;
    'teamMappings': Array<ESTeamMapping>;
    'referralIgnored': boolean;
    'filter': string;
    'bindUserDN': string;
    'bindUserPassword': string;
    'attributeNames': ESLdapAttributeNames;
}
export interface ESFederatedAuthModuleSettings {
    'className': string;
    'idpUrl': string;
    'idpEntityId': string;
    'idpCertificateSHA256': string;
    'spEntityId': string;
    'sslKeystore'?: SSLKeystore;
    'registerNewUsers': boolean;
    'contactProfileId'?: string | null;
    'attributeNames': ESSamlAttributeNames;
    'githubUrl': string;
    'clientId': string;
    'clientSecret': string;
    'organizations': Array<string>;
    'discoveryUrl'?: string | null;
    'issuer': string;
    'authorizationEndpoint': string;
    'tokenEndpoint': string;
    'tokenKeysEndpoint': string;
    'userInfoEndpoint': string;
    'domains': Array<string>;
    'hubUrl': string;
    'orgAuthProviderName'?: string | null;
    'groups'?: Array<string> | null;
    'tenantId': string;
    'emailVerified': boolean;
}
export interface ESFileRepositorySettings {
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
}
export interface ESGithubAuthModuleSettings {
    'githubUrl': string;
    'clientId': string;
    'clientSecret': string;
    'registerNewUsers': boolean;
    'organizations': Array<string>;
}
export interface ESGithubProfileLoginDetails {
    'login'?: string | null;
    'firstName'?: string | null;
    'lastName'?: string | null;
    'email'?: string | null;
    'emailVerified': boolean;
    'avatarUrl'?: string | null;
    'organizations'?: Array<string> | null;
    'allEmails'?: Array<ESGithubProfileLoginDetailsAllEmailsInner> | null;
}
export interface ESGithubProfileLoginDetailsAllEmailsInner {
    'first': string;
    'second': boolean;
}
export interface ESGoogleAuthModuleSettings {
    'clientId': string;
    'clientSecret': string;
    'registerNewUsers': boolean;
    'domains': Array<string>;
}
export interface ESHiddenAuthModuleSettings {
    'passwordModule'?: boolean | null;
    'federatedModule'?: boolean | null;
}
export interface ESHubAuthModuleSettings {
    'hubUrl': string;
    'clientId': string;
    'clientSecret': string;
    'registerNewUsers'?: boolean | null;
    'orgAuthProviderName'?: string | null;
    'groups'?: Array<string> | null;
}
export interface ESHubProfileLoginDetails {
    'login'?: string | null;
    'hubAuthModuleLogin'?: string | null;
    'firstName'?: string | null;
    'lastName'?: string | null;
    'email'?: string | null;
    'emailVerified': boolean;
    'avatarUrl'?: string | null;
}
export interface ESLdapAttributeNames {
    'fullNameAttributeName'?: string | null;
    'usernameAttributeName'?: string | null;
    'emailAttributeName'?: string | null;
    'groupsAttributeName'?: string | null;
}
export interface ESLdapAuthModuleSettings {
    'type': LdapModuleType;
    'registerNewUsers': boolean;
    'serverUrl': string;
    'connectionTimeout': number;
    'readTimeout': number;
    'sslKeystore'?: SSLKeystore;
    'teamMappings': Array<ESTeamMapping>;
    'referralIgnored': boolean;
    'filter': string;
    'bindUserDN': string;
    'bindUserPassword': string;
    'attributeNames': ESLdapAttributeNames;
}
export interface ESMavenRepositorySettings {
    'enableSnapshots': boolean;
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
    'checkVulnerabilities'?: boolean | null;
}
export interface ESNpmRegistrySettings {
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
    'checkVulnerabilities'?: boolean | null;
}
export interface ESNuGetFeedSettings {
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
    'checkVulnerabilities'?: boolean | null;
}
export interface ESOAuth2AuthModuleSettings {
    'className': string;
    'githubUrl': string;
    'clientId': string;
    'clientSecret': string;
    'registerNewUsers': boolean;
    'organizations': Array<string>;
    'discoveryUrl'?: string | null;
    'issuer': string;
    'authorizationEndpoint': string;
    'tokenEndpoint': string;
    'tokenKeysEndpoint': string;
    'userInfoEndpoint': string;
    'domains': Array<string>;
    'hubUrl': string;
    'orgAuthProviderName'?: string | null;
    'groups'?: Array<string> | null;
    'tenantId': string;
    'emailVerified': boolean;
}
export interface ESOAuthApp {
    'className': string;
    'name': string;
    'clientId': string;
    'app': ESApp;
}
export interface ESOAuthConsent {
    'clientApplication': ESOAuthApp;
    'approvedScopes': Array<ESApprovedScope>;
    'refreshTokens': Array<ESRefreshToken>;
}
export interface ESOAuthInternalApp {
    'name': string;
    'clientId': string;
}
export interface ESOAuthUserApp {
    'app': ESApp;
}
export interface ESOIDCAuthModuleSettings {
    'clientId': string;
    'clientSecret': string;
    'registerNewUsers': boolean;
    'discoveryUrl'?: string | null;
    'issuer': string;
    'authorizationEndpoint': string;
    'tokenEndpoint': string;
    'tokenKeysEndpoint': string;
    'userInfoEndpoint': string;
    'domains': Array<string>;
}
export interface ESPackageRepositorySettings {
    'className': string;
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
    'enableSnapshots': boolean;
    'checkVulnerabilities'?: boolean | null;
    'immutableTags': boolean;
}
export interface ESPasswordAuthModuleSettings {
    'className': string;
    'type': LdapModuleType;
    'registerNewUsers': boolean;
    'serverUrl': string;
    'connectionTimeout': number;
    'readTimeout': number;
    'sslKeystore'?: SSLKeystore;
    'teamMappings': Array<ESTeamMapping>;
    'referralIgnored': boolean;
    'filter': string;
    'bindUserDN': string;
    'bindUserPassword': string;
    'attributeNames': ESLdapAttributeNames;
    'passwordStrengthPolicy': PasswordStrength;
    'domains'?: Array<string> | null;
}
export interface ESPersonalToken {
    'id': string;
    'name': string;
    'profile': TDMemberProfile;
    'scope': string;
    'apiScope': XScopeApi;
    'created': string;
    'expires'?: string | null;
    'lastAccess'?: AccessRecord;
}
export interface ESProfileLogin {
    'identifier': string;
    'authModule': ESAuthModule;
    'details': ESProfileLoginDetails;
    'access'?: AccessRecord;
}
export interface ESProfileLoginDetails {
    'className': string;
    'login'?: string | null;
    'hubAuthModuleLogin'?: string | null;
    'firstName'?: string | null;
    'lastName'?: string | null;
    'email'?: string | null;
    'emailVerified': boolean;
    'avatarUrl'?: string | null;
    'passwordChangeRequest'?: string | null;
    'organizations'?: Array<string> | null;
    'allEmails'?: Array<ESGithubProfileLoginDetailsAllEmailsInner> | null;
}
export interface ESPythonPackageIndexSettings {
    'retentionPolicyParams'?: RetentionPolicyParams;
    'immutablePackageVersions'?: boolean | null;
}
export interface ESRefreshToken {
    'id': string;
    'scope': XScopeApi;
    'lastAccess'?: AccessRecord;
}
export interface ESSamlAttributeNames {
    'loginAttributeName'?: string | null;
    'firstNameAttributeName'?: string | null;
    'lastNameAttributeName'?: string | null;
    'fullNameAttributeName'?: string | null;
    'emailAttributeName'?: string | null;
    'emailVerified': boolean;
}
export interface ESSamlAuthModuleSettings {
    'idpUrl': string;
    'idpEntityId': string;
    'idpCertificateSHA256': string;
    'spEntityId': string;
    'sslKeystore'?: SSLKeystore;
    'registerNewUsers': boolean;
    'contactProfileId'?: string | null;
    'attributeNames': ESSamlAttributeNames;
}
export interface ESTeamMapping {
    'teamId': string;
    'externalGroupName': string;
}
export interface EbsSnapshotApi {
    'snapshotId': string;
    'snapshotName'?: string | null;
    'snapshotState': SnapshotState;
    'execution'?: number | null;
    'claimId'?: number | null;
}
export interface EbsVolumeApi {
    'volumeId': string;
    'volumeName': string;
    'az'?: string | null;
    'volumeState': VolumeState;
    'execution'?: number | null;
    'claimId'?: number | null;
}
export interface EditMessage {
    'text': string;
    'id': string;
    'attachments'?: Array<Attachment> | null;
    'pending'?: boolean | null;
}
export interface EmailBouncedEvent {
    'email': string;
    'reason': string;
}
export interface EmojiReaction {
    'emoji': string;
    'count': number;
    'meReacted': boolean;
}
export interface EmojiReactionRecord {
    'id': string;
    'itemId': string;
    'emoji': string;
    'count': number;
    'meReacted': boolean;
    'order': number;
}
export interface EmojiSearchMatchData {
    'emoji': string;
    'matched'?: string | null;
    'matchType': EmojiSearchMatchType;
}
export declare const EmojiSearchMatchType: {
    readonly Name: "Name";
    readonly OtherName: "OtherName";
    readonly Emoticon: "Emoticon";
    readonly Unicode: "Unicode";
};
export type EmojiSearchMatchType = typeof EmojiSearchMatchType[keyof typeof EmojiSearchMatchType];
export interface EmojisAddPostRequest {
    'emoji': string;
    'attachmentId': string;
}
export interface EmojisDeletePostRequest {
    'emoji': string;
}
export interface EmojisRecordUsagePostRequest {
    'emojis': Array<string>;
}
export interface EmojisSearchGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<EmojiSearchMatchData>;
}
export declare const EndpointAppLevelAuthUpdateType: {
    readonly PublicKeySignature: "PublicKeySignature";
    readonly SigningKey: "SigningKey";
    readonly Bearer: "Bearer";
    readonly Basic: "Basic";
};
export type EndpointAppLevelAuthUpdateType = typeof EndpointAppLevelAuthUpdateType[keyof typeof EndpointAppLevelAuthUpdateType];
export interface EndpointAuthCreate {
    'className': string;
    'username': string;
    'password': string;
    'token': string;
}
export interface EndpointAuthCreateBasic {
    'username': string;
    'password': string;
}
export interface EndpointAuthCreateBearer {
    'token': string;
}
export interface EndpointAuthCreateDTO {
    'appLevelAuth': EndpointAuthCreate;
    'sslKeystore'?: SSLKeystoreEndpointAuth;
}
export interface EndpointAuthDTO {
    'appLevelAuth': EndpointAuthRead;
    'sslKeystore'?: SSLKeystoreEndpointAuth;
    'verificationToken': boolean;
}
export interface EndpointAuthRead {
    'className': string;
    'username': string;
}
export interface EndpointAuthReadBasic {
    'username': string;
}
export interface EndpointAuthUpdateDTO {
    'appLevelAuth'?: EndpointAppLevelAuthUpdateType;
    'basicAuthUsername'?: string;
    'basicAuthPassword'?: string;
    'bearerAuthToken'?: string;
    'hasVerificationToken'?: boolean;
    'sslKeystoreAuth'?: string | null;
}
export interface EndpointCreateDTO {
    'url'?: string | null;
    'sslVerification': boolean;
}
export interface EndpointDTO {
    'url'?: string | null;
    'sslVerification': boolean;
}
export interface EntityMention {
    'position': number;
    'length': number;
    'link': string;
}
export interface EnumCFInputParameters {
    'values': Array<string>;
}
export interface EnumCFInputValue {
    'enumValueIdentifier'?: CFEnumValueIdentifier;
}
export interface EnumCFParameters {
    'values': Array<CFEnumValue>;
}
export interface EnumCFType {
    'values': Array<EnumValueData>;
}
export interface EnumCFValue {
    'value'?: EnumValueData;
}
export interface EnumListCFInputValue {
    'enumValueIdentifiers': Array<CFEnumValueIdentifier>;
}
export interface EnumListCFType {
    'values': Array<EnumValueData>;
}
export interface EnumListCFValue {
    'values': Array<EnumValueData>;
}
export interface EnumValueData {
    'id'?: string | null;
    'value': string;
    'principal'?: CPrincipalDetails;
}
export declare const EnumValueOrdering: {
    readonly Asc: "NAME_ASC";
    readonly Desc: "NAME_DESC";
};
export type EnumValueOrdering = typeof EnumValueOrdering[keyof typeof EnumValueOrdering];
export interface EstimationDTO {
    'start': string;
    'predictedDuration': number;
}
export interface EventConferenceData {
    'kind': EventConferenceKind;
    'url'?: string | null;
}
export declare const EventConferenceKind: {
    readonly None: "NONE";
    readonly GoogleMeet: "GOOGLE_MEET";
    readonly SpaceTelekom: "SPACE_TELEKOM";
    readonly Custom: "CUSTOM";
};
export type EventConferenceKind = typeof EventConferenceKind[keyof typeof EventConferenceKind];
export interface EventExternalSource {
    'sourceName': string;
    'externalId': string;
    'sourceLink'?: string | null;
    'externalTimestamp'?: number | null;
    'sourceHash'?: string | null;
}
export declare const EventParticipationStatus: {
    readonly Going: "GOING";
    readonly NotGoing: "NOT_GOING";
    readonly NotSure: "NOT_SURE";
    readonly WaitingForResponse: "WAITING_FOR_RESPONSE";
};
export type EventParticipationStatus = typeof EventParticipationStatus[keyof typeof EventParticipationStatus];
export interface EventSubjectInfoDTO {
    'code': string;
    'parentCode'?: string | null;
    'name': string;
    'description': string;
    'events': Array<EventTypeInfoDTO>;
    'defaultFilters': Array<SubscriptionFilter>;
    'featureFlag'?: string | null;
}
export interface EventTypeInfoDTO {
    'code': string;
    'name': string;
    'description': string;
    'rights': Array<CRight>;
    'featureFlag'?: string | null;
    'sortOrder': number;
    'payloadApiClassName'?: string | null;
}
export declare const ExecutionStatus: {
    readonly Scheduled: "SCHEDULED";
    readonly Pending: "PENDING";
    readonly ReadyToStart: "READY_TO_START";
    readonly Running: "RUNNING";
    readonly Finishing: "FINISHING";
    readonly Finished: "FINISHED";
    readonly Terminating: "TERMINATING";
    readonly Terminated: "TERMINATED";
    readonly Hibernating: "HIBERNATING";
    readonly Hibernated: "HIBERNATED";
    readonly Restarting: "RESTARTING";
    readonly Failed: "FAILED";
};
export type ExecutionStatus = typeof ExecutionStatus[keyof typeof ExecutionStatus];
export interface ExtendedType {
    'key': string;
    'displayName': string;
    'apiClassName'?: string | null;
    'scopeType': ExtendedTypeScopeType;
}
export interface ExtendedTypeKey {
    'key': string;
    'scope': ExtendedTypeScope;
}
export interface ExtendedTypeScope {
    'className': string;
    'containerId': string;
}
export interface ExtendedTypeScopeContainer {
    'containerId': string;
}
export declare const ExtendedTypeScopeType: {
    readonly Org: "Org";
    readonly Container: "Container";
};
export type ExtendedTypeScopeType = typeof ExtendedTypeScopeType[keyof typeof ExtendedTypeScopeType];
export interface ExternalArticle {
    'title': string;
    'content': string;
    'authorId': string;
    'created': string;
    'teams'?: Array<string> | null;
    'locations'?: Array<string> | null;
    'externalId'?: string | null;
    'externalUrl'?: string | null;
}
export interface ExternalCheckDTO {
    'repository': string;
    'revision': string;
    'executionStatus': CommitExecutionStatus;
    'url': string;
    'externalServiceName': string;
    'taskName': string;
    'taskId': string;
    'taskBuildId'?: string | null;
    'timestamp': number;
    'description'?: string | null;
}
export interface ExternalCodeReviewLink {
    'id': string;
    'url': string;
}
export interface ExternalEndpointUpdateDTO {
    'url'?: string | null;
    'sslVerification'?: boolean;
}
export interface ExternalEntityInfoRecord {
    'id': string;
    'archived': boolean;
    'externalId'?: string | null;
    'externalName'?: string | null;
    'externalUrl'?: string | null;
    'transaction': ImportTransactionRecord;
}
export interface ExternalIssue {
    'summary': string;
    'description'?: string | null;
    'status': string;
    'assignee'?: string | null;
    'externalId': string;
    'externalName'?: string | null;
    'externalUrl'?: string | null;
}
export interface ExternalLink {
    'label'?: string | null;
    'url': string;
}
export interface ExternalLinkPattern {
    'id': string;
    'archived': boolean;
    'prefix': string;
    'linkReplacement': string;
}
export interface ExternalLinkPatternsDeleteRequest {
    'pattern': string;
    'linkReplacement': string;
}
export interface FailureConditionDTO {
    'className': string;
}
export declare const FailureConditionType: {
    readonly NonZeroExitCode: "NON_ZERO_EXIT_CODE";
    readonly TestFailed: "TEST_FAILED";
    readonly OutOfMemory: "OUT_OF_MEMORY";
    readonly TimeOut: "TIME_OUT";
};
export type FailureConditionType = typeof FailureConditionType[keyof typeof FailureConditionType];
export interface FeatureFlag {
    'name': string;
    'description': string;
    'status': FeatureFlagStatus;
    'owner': string;
    'since'?: FeatureFlagDate;
    'issueNumber'?: number | null;
}
export interface FeatureFlagDate {
    'year': number;
    'month': number;
    'day': number;
}
export declare const FeatureFlagStatus: {
    readonly Wip: "WIP";
    readonly Experimental: "EXPERIMENTAL";
    readonly Internal: "INTERNAL";
};
export type FeatureFlagStatus = typeof FeatureFlagStatus[keyof typeof FeatureFlagStatus];
export interface FeatureFlagWebhookEvent {
    'meta': KMetaMod;
    'name': string;
    'issueNumber'?: number | null;
    'enabledForAll'?: AbsenceWebhookEventAvailable | null;
    'selfManageable'?: AbsenceWebhookEventAvailable | null;
    'addedTeams'?: Array<TDTeam> | null;
    'addedProfiles'?: Array<TDTeam> | null;
    'removedTeams'?: Array<TDMemberProfile> | null;
    'removedProfiles'?: Array<TDMemberProfile> | null;
}
export interface FeedEvent {
    'className': string;
    'state': CodeReviewState;
    'commits': Array<UnfurlDetailsCommit> | null;
    'changeType': ReviewerChangedType;
    'projectKey': string;
    'review'?: CodeReviewRecord;
    'repository': string;
    'branch': string;
    'branchType': MergeRequestBranchType;
    'discussion': CodeReviewDiscussionRecord;
    'sourceBranch': string;
    'targetBranch': string;
    'codeDiscussion': CodeDiscussionRecord;
    'codeReview': CodeReviewRecord;
    'track': boolean;
    'reviewId': string;
    'reviewNumber': number;
    'reviewType': ReviewType;
    'description'?: CodeReviewDescription;
    'descriptionEditedByProfileIds': Array<TDMemberProfile>;
    'uid': TDMemberProfile;
    'oldTitle': string;
    'newTitle': string;
}
export interface FileAttachment {
    'id': string;
    'sizeBytes': number;
    'filename': string;
}
export interface FileData {
    'type': PackageType;
    'repository': string;
    'name': string;
    'path': string;
    'fileType': FileDataType;
    'created': number;
    'size': number;
}
export declare const FileDataType: {
    readonly File: "FILE";
    readonly Dir: "DIR";
};
export type FileDataType = typeof FileDataType[keyof typeof FileDataType];
export interface FileDetails {
    'type': PackageType;
    'repository': string;
    'name': string;
    'created': number;
    'lastModified'?: number | null;
    'downloads': number;
    'diskSize': number;
    'authors'?: Array<CPrincipal> | null;
    'description'?: string | null;
    'path': string;
    'contentType'?: string | null;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
}
export interface FileDocumentBody {
    'versionId': string;
    'contentType': string;
    'fileSize': number;
}
export interface FileDocumentBodyCreateIn {
    'blobId': string;
}
export interface FileDocumentBodyInfo {
    'fileDocument': FileDocumentRecord;
}
export interface FileDocumentBodyUpdateIn {
    'blobId': string;
}
export interface FileDocumentRecord {
    'id': string;
    'archived': boolean;
    'body': FileDocumentBody;
}
export interface FileSetInReview {
    'repository': string;
    'commit': string;
    'paths': Array<string>;
}
export interface FileType {
    'id': string;
}
export interface Fingerprint {
    'md5': string;
    'sha1': string;
    'sha256': string;
    'sha384': string;
    'sha512': string;
}
export interface FolderIdentifier {
    'className': string;
    'id': string;
}
export interface FolderIdentifierId {
    'id': string;
}
export interface Fraction {
    'numerator': number;
    'denominator': number;
}
export interface FractionCFValue {
    'value'?: Fraction;
}
export interface FullWebhookDTO {
    'webhook': WebhookRecord;
    'status'?: WebhookDeliveryStatusDTO;
}
export declare const Gender: {
    readonly Female: "Female";
    readonly Male: "Male";
    readonly Other: "Other";
    readonly NotSpecified: "NotSpecified";
};
export type Gender = typeof Gender[keyof typeof Gender];
export interface GetMessagesResponse {
    'messages': Array<ChannelItemRecord>;
    'nextStartFromDate'?: string | null;
    'orgLimitReached': boolean;
}
export interface GitAuthorInfo {
    'name': string;
    'email': string;
}
export interface GitCherryPickResult {
    'success': boolean;
    'message': string;
    'resultCommitId'?: string | null;
}
export interface GitCommitChange {
    'changeType': GitCommitChangeType;
    'old'?: GitFile;
    'new'?: GitFile;
    'revision': string;
    'diffSize'?: GitDiffSize;
    'path'?: string | null;
}
export declare const GitCommitChangeType: {
    readonly Added: "ADDED";
    readonly Deleted: "DELETED";
    readonly Modified: "MODIFIED";
};
export type GitCommitChangeType = typeof GitCommitChangeType[keyof typeof GitCommitChangeType];
export interface GitCommitChanges {
    'changes': Array<GitCommitChange>;
    'overflow': boolean;
}
export interface GitCommitFileRequest {
    'path': string;
    'content': GitFileContent;
}
export interface GitCommitInfo {
    'id': string;
    'message': string;
    'authorDate': number;
    'commitDate': number;
    'author': GitAuthorInfo;
    'authorProfile'?: TDMemberProfile;
    'committer': GitAuthorInfo;
    'committerProfile'?: TDMemberProfile;
    'parents': Array<string>;
    'heads': Array<string>;
    'signature'?: GitCommitSignature;
    'committerIsSpace'?: boolean | null;
}
export interface GitCommitInfoWithChanges {
    'commit': GitCommitInfo;
    'changes': GitCommitChanges;
}
export interface GitCommitResult {
    'fromHead'?: string | null;
    'toHead'?: string | null;
    'success': boolean;
    'message': string;
    'resultCommitId'?: string | null;
}
export interface GitCommitSignature {
    'className': string;
    'description': string;
    'signature': string;
    'signedDataB64': string;
    'revokeComment'?: string | null;
}
export interface GitCommitSignatureExpiredKey {
    'description': string;
}
export interface GitCommitSignatureRaw {
    'signature': string;
    'signedDataB64': string;
}
export interface GitCommitSignatureRevokedKey {
    'description': string;
    'revokeComment'?: string | null;
}
export interface GitCommitSignatureVerified {
    'description': string;
}
export interface GitCommitWithGraph {
    'repositoryName': string;
    'commit': GitCommitInfo;
    'commitMessageUnfurls': Array<Unfurl>;
    'reviews': Array<CodeReviewRecord>;
    'issueIds': Array<string>;
    'deployments': Array<GitCommitWithGraphDeploymentsInner>;
    'layout'?: GitGraphLayoutLine;
    'unreachable': boolean;
}
export interface GitCommitWithGraphDeploymentsInner {
    'first': string;
    'second'?: string | null;
}
export interface GitCommitterProfile {
    'author': string;
    'email': string;
    'profile'?: TDMemberProfile;
}
export interface GitDiffSize {
    'added': number;
    'deleted': number;
}
export declare const GitEntryType: {
    readonly ExeFile: "EXE_FILE";
    readonly File: "FILE";
    readonly Dir: "DIR";
    readonly GitLink: "GIT_LINK";
    readonly SymLink: "SYM_LINK";
};
export type GitEntryType = typeof GitEntryType[keyof typeof GitEntryType];
export interface GitFile {
    'commit': string;
    'path': string;
    'blob': string;
    'type': GitEntryType;
}
export interface GitFileContent {
    'className': string;
    'value': string;
    'executable'?: boolean | null;
}
export interface GitFileContentBase64 {
    'value': string;
    'executable'?: boolean | null;
}
export interface GitFileContentText {
    'value': string;
    'executable'?: boolean | null;
}
export declare const GitGraphEdgeLineStyle: {
    readonly Solid: "SOLID";
    readonly Dashed: "DASHED";
    readonly Dotted: "DOTTED";
};
export type GitGraphEdgeLineStyle = typeof GitGraphEdgeLineStyle[keyof typeof GitGraphEdgeLineStyle];
export declare const GitGraphEdgeType: {
    readonly Up: "UP";
    readonly Down: "DOWN";
};
export type GitGraphEdgeType = typeof GitGraphEdgeType[keyof typeof GitGraphEdgeType];
export interface GitGraphLayoutEdge {
    'from': number;
    'to': number;
    'type': GitGraphEdgeType;
    'style': GitGraphEdgeLineStyle;
    'hasArrow': boolean;
    'color': number;
}
export interface GitGraphLayoutLine {
    'nodes': Array<GitGraphLayoutNode>;
    'edges': Array<GitGraphLayoutEdge>;
}
export interface GitGraphLayoutNode {
    'pos': number;
    'color': number;
}
export interface GitMergeBranchResult {
    'status': GitMergeStatus;
    'message': string;
    'resultCommitId'?: string | null;
}
export declare const GitMergeMode: {
    readonly Ff: "FF";
    readonly FfOnly: "FF_ONLY";
    readonly NoFf: "NO_FF";
};
export type GitMergeMode = typeof GitMergeMode[keyof typeof GitMergeMode];
export interface GitMergeResultHttp {
    'status': GitMergeStatusHttp;
    'resultCommitId'?: string | null;
}
export declare const GitMergeStatus: {
    readonly Success: "Success";
    readonly SuccessfulFastForward: "SuccessfulFastForward";
    readonly Conflict: "Conflict";
    readonly AlreadyMerged: "AlreadyMerged";
    readonly MultipleMergeBase: "MultipleMergeBase";
    readonly TooManyChanges: "TooManyChanges";
    readonly Error: "Error";
};
export type GitMergeStatus = typeof GitMergeStatus[keyof typeof GitMergeStatus];
export declare const GitMergeStatusHttp: {
    readonly SuccessfulMerge: "SUCCESSFUL_MERGE";
    readonly SuccessfulFastForward: "SUCCESSFUL_FAST_FORWARD";
    readonly AlreadyMerged: "ALREADY_MERGED";
};
export type GitMergeStatusHttp = typeof GitMergeStatusHttp[keyof typeof GitMergeStatusHttp];
export interface GitMergedFile {
    'name': string;
    'oldName'?: string | null;
    'baseId'?: string | null;
    'sourceId'?: string | null;
    'targetId'?: string | null;
    'diffSize'?: GitDiffSize;
    'entryType': GitEntryType;
    'conflicting': boolean;
}
export declare const GitRebaseMode: {
    readonly Ff: "FF";
    readonly NoFf: "NO_FF";
};
export type GitRebaseMode = typeof GitRebaseMode[keyof typeof GitRebaseMode];
export interface GitRebaseResult {
    'success': boolean;
    'message': string;
    'resultCommitIds': Array<string>;
}
export interface GitRebaseResultHttp {
    'resultCommitIds': Array<string>;
}
export interface GitRepositorySettings {
    'version': string;
    'mirror'?: GitRepositorySettingsMirrorSettings;
    'encoding'?: string | null;
    'pushRestrictions'?: GitRepositorySettingsPushRestrictions;
    'protectedBranches'?: Array<GitRepositorySettingsProtectedBranch> | null;
    'preReceiveHook'?: GitRepositorySettingsPreReceiveHook;
}
export interface GitRepositorySettingsMirrorSettings {
    'url': string;
    'refsSpec'?: Array<string> | null;
    'detachHEAD': boolean;
    'username'?: string | null;
    'password'?: string | null;
    'useKeyAuth'?: boolean | null;
    'key'?: string | null;
    'keyPassphrase'?: string | null;
    'keyError'?: string | null;
    'publicKey'?: string | null;
    'keyFingerprints'?: Array<string> | null;
    'fetchPeriodically': boolean;
    'fetchBeforeGitCall': boolean;
}
export interface GitRepositorySettingsPreReceiveHook {
    'serverUrl'?: string | null;
    'script'?: string | null;
    'scriptOid'?: string | null;
}
export interface GitRepositorySettingsProtectedBranch {
    'pattern': Array<string>;
    'regex'?: boolean | null;
    'allowCreate'?: Array<string> | null;
    'allowPush'?: Array<string> | null;
    'allowDelete'?: Array<string> | null;
    'allowForcePush'?: Array<string> | null;
    'qualityGate'?: GitRepositorySettingsQualityGate;
    'safeMerge'?: GitRepositorySettingsSafeMerge;
    'linearHistory'?: boolean | null;
}
export interface GitRepositorySettingsPushRestrictions {
    'checkAuthenticatedCommitter'?: boolean | null;
    'requireSignature'?: boolean | null;
    'verifySignature'?: boolean | null;
    'maxFileSize'?: string | null;
    'defaultForbiddenFilesList'?: boolean | null;
    'forbiddenFileNames'?: Array<string> | null;
    'commitMessageRegex'?: string | null;
    'commitMessageNegativeRegex'?: string | null;
    'windowsCompatibility'?: boolean | null;
}
export interface GitRepositorySettingsQualityGate {
    'allowMergeFor'?: Array<string> | null;
    'allowBypassFor'?: Array<string> | null;
    'externalStatus'?: Array<string> | null;
    'automationJobs'?: Array<string> | null;
    'approvals'?: Array<GitRepositorySettingsQualityGateApproval> | null;
    'codeOwnersApproval'?: boolean | null;
}
export interface GitRepositorySettingsQualityGateApproval {
    'minApprovals': number;
    'approvedBy': Array<string>;
}
export interface GitRepositorySettingsSafeMerge {
    'configOid': string;
}
export declare const GitSquashMode: {
    readonly All: "ALL";
    readonly Auto: "AUTO";
    readonly None: "NONE";
};
export type GitSquashMode = typeof GitSquashMode[keyof typeof GitSquashMode];
export interface GlobalNotificationSettings {
    'emailNotificationsEnabled': boolean;
    'notificationEmail'?: TDProfileEmail;
    'pushNotificationEnabled': boolean;
    'desktopInactivityTimeout'?: number | null;
    'multiClientEnabled': boolean;
}
export interface GlobalRole {
    'className': string;
}
export interface GoogleIntegrationDisconnectedEvent {
    'member': TDMemberProfile;
}
export interface GpgKeyData {
    'fingerprint': string;
    'publicKey': string;
    'userIds': Array<GpgKeyDataKeyUserId>;
    'comment': string;
    'created': string;
    'added': string;
    'expires'?: string | null;
    'revoked': boolean;
    'revokedAt'?: string | null;
    'revokeComment'?: string | null;
}
export interface GpgKeyDataKeyUserId {
    'id': string;
    'verified': boolean;
}
export interface GuestType {
    'className': string;
}
export interface HADefaultValue {
    'className': string;
    'elements': {
        [key: string]: HADefaultValue | undefined;
    };
    'entryName': string;
    'expression': string;
    'paramName': string;
}
export interface HADefaultValueCollection {
    'elements': Array<HADefaultValue>;
}
export interface HADefaultValueConst {
    'className': string;
    'entryName': string;
    'expression': string;
}
export interface HADefaultValueConstEnumEntry {
    'entryName': string;
}
export interface HADefaultValueConstPrimitive {
    'expression': string;
}
export interface HADefaultValueMap {
    'elements': {
        [key: string]: HADefaultValue | undefined;
    };
}
export interface HADefaultValueReference {
    'paramName': string;
}
export interface HADeprecation {
    'message': string;
    'since': string;
    'forRemoval': boolean;
}
export interface HADescription {
    'text': string;
    'helpTopic'?: string | null;
}
export interface HADto {
    'id': string;
    'name': string;
    'fields': Array<HADtoField>;
    'hierarchyRole': HierarchyRole;
    'hierarchyRole2': HierarchyRole2;
    'extends'?: HADto;
    'implements': Array<HADto>;
    'inheritors': Array<HADto>;
    'description'?: HADescription;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
    'record': boolean;
    'errorsField'?: HAField;
}
export interface HADtoField {
    'field': HAField;
    'extension': boolean;
}
export interface HAEndpoint {
    'resource': HAResource;
    'method': HAMethod;
    'parameters': Array<HAParameter>;
    'requestBody'?: HATypeObject;
    'responseBody'?: HAType;
    'path': HAPath;
    'displayName': string;
    'functionName': string;
    'doc'?: string | null;
    'description'?: HADescription;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
    'rights'?: Array<HARight> | null;
    'featureFlag'?: string | null;
}
export interface HAEnum {
    'id': string;
    'name': string;
    'values': Array<string>;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
}
export interface HAExperimental {
    'message'?: string | null;
}
export interface HAField {
    'name': string;
    'type': HAType;
    'defaultValue'?: HADefaultValue;
    'optional': boolean;
    'description'?: HADescription;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
}
export interface HAInlineError {
    'className': string;
    'fields': Array<string>;
    'message': string;
}
export interface HAInlineErrorInaccessibleFields {
    'fields': Array<string>;
    'message': string;
}
export interface HAMenuId {
    'menuId': string;
    'context': HADto;
}
export declare const HAMethod: {
    readonly RestCreate: "REST_CREATE";
    readonly RestQuery: "REST_QUERY";
    readonly RestGet: "REST_GET";
    readonly RestUpdate: "REST_UPDATE";
    readonly RestDelete: "REST_DELETE";
    readonly HttpGet: "HTTP_GET";
    readonly HttpPost: "HTTP_POST";
    readonly HttpPatch: "HTTP_PATCH";
    readonly HttpPut: "HTTP_PUT";
    readonly HttpDelete: "HTTP_DELETE";
};
export type HAMethod = typeof HAMethod[keyof typeof HAMethod];
export interface HAModel {
    'dto': Array<HADto>;
    'enums': Array<HAEnum>;
    'urlParams': Array<HAUrlParameter>;
    'resources': Array<HAResource>;
    'allResources': Array<HAResource>;
    'menuIds': Array<HAMenuId>;
}
export interface HAParameter {
    'field': HAField;
    'path': boolean;
}
export interface HAPath {
    'segments': Array<HAPathSegment>;
}
export interface HAPathSegment {
    'className': string;
    'value': string;
    'prefix': string;
    'name': string;
}
export interface HAPathSegmentConst {
    'value': string;
}
export interface HAPathSegmentPrefixedVar {
    'prefix': string;
    'name': string;
}
export interface HAPathSegmentVar {
    'name': string;
}
export declare const HAPrimitive: {
    readonly Byte: "Byte";
    readonly Short: "Short";
    readonly Int: "Int";
    readonly Long: "Long";
    readonly Float: "Float";
    readonly Double: "Double";
    readonly Boolean: "Boolean";
    readonly String: "String";
    readonly Date: "Date";
    readonly DateTime: "DateTime";
    readonly Duration: "Duration";
};
export type HAPrimitive = typeof HAPrimitive[keyof typeof HAPrimitive];
export interface HAResource {
    'id': string;
    'path': HAPath;
    'displaySingular': string;
    'displayPlural': string;
    'parentResource'?: HAResource;
    'endpoints': Array<HAEndpoint>;
    'nestedResources': Array<HAResource>;
}
export interface HARight {
    'rightUniqueCode': string;
    'title': string;
    'description'?: string | null;
}
export interface HAType {
    'className': string;
    'elementType': HAType;
    'nullable': boolean;
    'tags': Array<string>;
    'dto': HADto;
    'enum': HAEnum;
    'valueType': HAType;
    'fields': Array<HAField>;
    'kind': HATypeObjectKind;
    'primitive': HAPrimitive;
    'urlParam': HAUrlParameter;
}
export interface HATypeArray {
    'elementType': HAType;
    'nullable': boolean;
    'tags': Array<string>;
}
export interface HATypeDto {
    'dto': HADto;
    'nullable': boolean;
    'tags': Array<string>;
}
export interface HATypeEnum {
    'enum': HAEnum;
    'nullable': boolean;
    'tags': Array<string>;
}
export interface HATypeMap {
    'valueType': HAType;
    'nullable': boolean;
    'tags': Array<string>;
}
export interface HATypeObject {
    'fields': Array<HAField>;
    'kind': HATypeObjectKind;
    'nullable': boolean;
    'tags': Array<string>;
}
export declare const HATypeObjectKind: {
    readonly Pair: "PAIR";
    readonly Triple: "TRIPLE";
    readonly Batch: "BATCH";
    readonly SyncBatch: "SYNC_BATCH";
    readonly Mod: "MOD";
    readonly RequestBody: "REQUEST_BODY";
};
export type HATypeObjectKind = typeof HATypeObjectKind[keyof typeof HATypeObjectKind];
export interface HATypePrimitive {
    'primitive': HAPrimitive;
    'nullable': boolean;
    'tags': Array<string>;
}
export interface HATypeRef {
    'dto': HADto;
    'nullable': boolean;
    'tags': Array<string>;
}
export interface HATypeUrlParam {
    'urlParam': HAUrlParameter;
    'nullable': boolean;
    'tags': Array<string>;
}
export interface HAUrlParameter {
    'id': string;
    'name': string;
    'options': Array<HAUrlParameterOption>;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
}
export interface HAUrlParameterOption {
    'className': string;
    'value': string;
    'optionName': string;
    'description'?: HADescription;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
    'parameter': HAField;
    'parameters': Array<HAField>;
    'prefixRequired': boolean;
}
export interface HAUrlParameterOptionConst {
    'value': string;
    'optionName': string;
    'description'?: HADescription;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
}
export interface HAUrlParameterOptionVar {
    'parameter': HAField;
    'parameters': Array<HAField>;
    'prefixRequired': boolean;
    'optionName': string;
    'description'?: HADescription;
    'deprecation'?: HADeprecation;
    'experimental'?: HAExperimental;
}
export declare const HierarchyRole: {
    readonly Sealed: "SEALED";
    readonly Open: "OPEN";
    readonly Final: "FINAL";
    readonly Abstract: "ABSTRACT";
    readonly Interface: "INTERFACE";
};
export type HierarchyRole = typeof HierarchyRole[keyof typeof HierarchyRole];
export declare const HierarchyRole2: {
    readonly SealedClass: "SEALED_CLASS";
    readonly OpenClass: "OPEN_CLASS";
    readonly FinalClass: "FINAL_CLASS";
    readonly AbstractClass: "ABSTRACT_CLASS";
    readonly Interface: "INTERFACE";
    readonly SealedInterface: "SEALED_INTERFACE";
};
export type HierarchyRole2 = typeof HierarchyRole2[keyof typeof HierarchyRole2];
export interface HolidaysEvent {
    'profile': TDMemberProfile;
    'holidays': Array<PublicHoliday>;
}
export interface IdeNames {
    'backendFullName': string;
    'backendShortName': string;
    'clientFullName': string;
    'clientShortName': string;
    'usesSeparateClientProduct': boolean;
}
export interface IdeType {
    'id': string;
    'toolboxId': string;
    'names': IdeNames;
    'description': string;
    'defaultWarmupProfile': string;
}
export interface IdeVersionDetails {
    'ideName': string;
    'version': string;
    'versionDisplayName': string;
    'build': string;
    'releaseDate'?: string | null;
    'quality': IdeVersionQuality;
}
export interface IdeVersionQuality {
    'name': string;
    'order': number;
}
export declare const IdeVersionSortOrder: {
    readonly ByReleaseDate: "ByReleaseDate";
    readonly ByVersionAndQuality: "ByVersionAndQuality";
};
export type IdeVersionSortOrder = typeof IdeVersionSortOrder[keyof typeof IdeVersionSortOrder];
export interface ImageAttachment {
    'id': string;
    'name'?: string | null;
    'width': number;
    'height': number;
    'previewBytes'?: string | null;
    'variants'?: Array<ImageAttachmentVariant> | null;
}
export interface ImageAttachmentMeta {
    'variants': Array<ImageAttachmentVariantsMeta>;
}
export interface ImageAttachmentVariant {
    'id': string;
    'name'?: string | null;
    'width': number;
    'height': number;
}
export interface ImageAttachmentVariantsMeta {
    'id': string;
    'name': string;
    'width': number;
    'height': number;
}
export interface ImportExistsPolicy {
    'className': string;
}
export interface ImportMessage {
    'className': string;
    'messageId': ChatMessageIdentifierExternalId;
    'message': ChatMessage;
    'author': PrincipalIn;
    'createdAtUtc': number;
    'editedAtUtc'?: number | null;
    'attachments'?: Array<AttachmentIn> | null;
}
export interface ImportMessageCreate {
    'messageId': ChatMessageIdentifierExternalId;
    'message': ChatMessage;
    'author': PrincipalIn;
    'createdAtUtc': number;
    'editedAtUtc'?: number | null;
    'attachments'?: Array<AttachmentIn> | null;
}
export interface ImportMessageDelete {
    'messageId': ChatMessageIdentifierExternalId;
}
export interface ImportMessageUpdate {
    'messageId': ChatMessageIdentifierExternalId;
    'message': ChatMessage;
    'author': PrincipalIn;
    'editedAtUtc'?: number | null;
    'attachments'?: Array<AttachmentIn> | null;
}
export interface ImportMetadata {
    'importSource': string;
}
export interface ImportMissingPolicy {
    'className': string;
}
export interface ImportTransactionRecord {
    'id': string;
    'archived': boolean;
    'importer': CPrincipal;
    'externalSource': string;
    'imported': string;
}
export interface InaccessibleContainerInfo {
    'grantedRights'?: Array<string> | null;
}
export interface InaccessibleDocumentBody {
    'docBodyType': DocumentBodyType;
    'docBodyId': string;
}
export interface InitPayload {
    'clientSecret': string;
    'serverUrl': string;
    'state'?: string | null;
    'clientId': string;
    'userId'?: string | null;
    'verificationToken'?: string | null;
}
export interface InlineDiffLine {
    'text': string;
    'type'?: DiffLineType;
    'oldLineNum'?: number | null;
    'newLineNum'?: number | null;
    'oldFileOffset': number;
    'newFileOffset': number;
    'syntax'?: Array<SyntaxMarkup> | null;
    'deletes'?: Array<TextRange> | null;
    'inserts'?: Array<TextRange> | null;
}
export interface InlineNode {
    'className': string;
    'value': string;
    'marks': Array<DocumentMark>;
    'attrs': RtImageAttrs;
    'soft': boolean;
    'emojiName': string;
}
export interface InlineNodeWithMarks {
    'className': string;
    'value': string;
    'marks': Array<DocumentMark>;
    'attrs': RtImageAttrs;
    'soft': boolean;
    'emojiName': string;
}
export interface InlineUnfurlDetails {
    'className': string;
    'team': TDTeam;
    'strikeThrough': boolean | null;
    'targetRef': DeployTargetRecord;
    'targetName'?: string | null;
    'showLinkIcon'?: boolean | null;
    'skipDetailsRender'?: boolean | null;
    'utcMilliseconds': number;
    'params'?: DateTimeViewParams;
    'role': TDRole;
    'profile': TDMemberProfile;
    'draft': string;
    'title': string;
    'document': string;
    'project': PRProject;
    'repository': string;
    'repositoryId'?: string | null;
    'commitId': string;
    'message': string;
    'messageUnfurls'?: CommitMessageUnfurlsRecord;
    'commitDate': string;
    'authorDate'?: string | null;
    'author': GitAuthorInfo;
    'authorProfile'?: TDMemberProfile;
    'hideAuthorAndDate'?: boolean | null;
    'withBranchTags'?: boolean | null;
    'reviewId'?: string | null;
    'repoRef': ProjectPackageRepository;
    'packageName'?: string | null;
    'version'?: string | null;
    'base'?: string | null;
    'preview'?: string | null;
    'version2'?: string | null;
    'base2'?: string | null;
    'preview2'?: string | null;
    'jobId': string;
    'jobName': string;
    'projectRef': PRProject;
    'repoName': string;
    'jobExecutionDisplayStatusFilter'?: JobExecutionDisplayStatus;
    'jobTriggerFilter'?: JobTriggerType;
    'branch'?: Branch;
    'status': IssueStatus;
    'article': ArticleRecord;
    'content'?: ArticleContentRecord;
    'channel': string;
    'details'?: ArticleDetailsRecord;
    'snapshotId': string;
    'baseSnapshotId': string;
    'branchHead': string;
    'deleted': boolean;
    'isDefault'?: boolean | null;
    'deploymentRef': DeploymentRecord;
    'showDetails'?: boolean | null;
    'showStatus'?: boolean | null;
    'location': TDLocation;
    'since': number;
    'till': number;
    'sprint': SprintRecord;
    'removed'?: boolean | null;
    'session': CallSession;
    'importTransaction': ImportTransactionRecord;
    'meeting': DTOMeeting;
    'compact'?: boolean | null;
    'topic': Topic;
    'textBefore': string;
    'textAfter': string;
    'projectId': string;
    'issueId': string;
    'name': string;
    'root': boolean;
    'folder'?: DocumentFolder;
    'checklist': Checklist;
    'app': ESApp;
    'tag': PlanningTag;
    'textSize'?: MessageTextSize;
    'review': CodeReviewRecord;
    'withIcon'?: boolean | null;
    'withBranchPair': boolean;
    'defaultBranchInRepo'?: string | null;
    'hideIfCannotResolve'?: boolean | null;
    'reviewState'?: CodeReviewState;
    'isMerged'?: boolean | null;
    'jobExecutionId': string;
    'issue': Issue;
    'contactKey': string;
}
export interface IntCFConstraint {
    'min'?: number | null;
    'max'?: number | null;
    'message'?: string | null;
}
export interface IntCFValue {
    'value'?: number | null;
}
export interface IntListCFValue {
    'values': Array<number>;
}
export declare const InterpolatedLineState: {
    readonly Normal: "Normal";
    readonly Deleted: "Deleted";
    readonly Modified: "Modified";
};
export type InterpolatedLineState = typeof InterpolatedLineState[keyof typeof InterpolatedLineState];
export interface Invitation {
    'id': string;
    'expiresAt': string;
    'inviteeEmail': string;
    'inviteeEmailBlocked': boolean;
    'inviteeEmailBlockedReason'?: string | null;
    'inviteeFirstName'?: string | null;
    'inviteeLastName'?: string | null;
    'invitee'?: TDMemberProfile;
    'inviter': CPrincipal;
    'team'?: TDTeam;
    'role'?: TDRole;
    'project'?: PRProject;
    'projectRole'?: ProjectTeamRole;
    'globalRole'?: GlobalRole;
    'revoked'?: boolean | null;
}
export interface InvitationLink {
    'id': string;
    'name': string;
    'createdBy': CPrincipal;
    'createdAt': string;
    'expiresAt'?: string | null;
    'inviteeLimit': number;
    'inviteeUsage': number;
    'team'?: TDTeam;
    'position'?: TDRole;
    'project'?: PRProject;
    'globalRole'?: GlobalRole;
    'projectRole': ProjectTeamRole;
    'link'?: string | null;
    'deleted': boolean;
}
export interface Issue {
    'id': string;
    'archived': boolean;
    'projectId'?: string | null;
    'projectRef': PRProject;
    'trackerRef'?: IssueTracker;
    'number': number;
    'createdBy': CPrincipal;
    'creationTime': string;
    'assignee'?: TDMemberProfile;
    'status': IssueStatus;
    'dueDate'?: string | null;
    'externalEntityInfo'?: ExternalEntityInfoRecord;
    'tags': Array<PlanningTag>;
    'title': string;
    'attachmentsCount'?: number | null;
    'subItemsCount'?: number | null;
    'doneSubItemsCount'?: number | null;
    'commentsCount'?: number | null;
    'deletedBy'?: CPrincipal;
    'deletedTime'?: string | null;
    'parents': Array<Issue>;
    'messagePermalink'?: string | null;
    'subItemsList': Checklist;
    'checklists': Array<Checklist>;
    'sprints': Array<SprintRecord>;
    'description'?: string | null;
    'channel': M2ChannelRecord;
    'attachments': Array<AttachmentInfo>;
    'unfurls'?: Array<AttachmentInfo> | null;
    'customFields': {
        [key: string]: CFValue | undefined;
    };
}
export interface IssueAssigneeChangedDetails {
    'oldAssignee'?: TDMemberProfile;
    'newAssignee'?: TDMemberProfile;
}
export interface IssueAttachmentsChangedDetails {
    'addedNames'?: Array<string> | null;
    'removedNames'?: Array<string> | null;
}
export interface IssueCFInputValue {
    'issue'?: IssueIdentifier;
}
export interface IssueCFValue {
    'issue'?: Issue;
}
export interface IssueChangedM2Details {
    'className': string;
    'oldDescription'?: string | null;
    'newDescription'?: string | null;
    'issue'?: Issue;
    'originMessage'?: MessageLink;
    'message': ChannelItemSnapshot;
    'oldDueDate'?: string | null;
    'newDueDate'?: string | null;
    'oldStatus': IssueStatus;
    'newStatus': IssueStatus;
    'addedNames'?: Array<string> | null;
    'removedNames'?: Array<string> | null;
    'addedTags'?: Array<PlanningTag> | null;
    'removedTags'?: Array<PlanningTag> | null;
    'oldAssignee'?: TDMemberProfile;
    'newAssignee'?: TDMemberProfile;
    'oldTitle': string;
    'newTitle': string;
    'addedChecklists'?: Array<Checklist> | null;
    'removedChecklists'?: Array<Checklist> | null;
}
export interface IssueChecklistsChangedDetails {
    'addedChecklists'?: Array<Checklist> | null;
    'removedChecklists'?: Array<Checklist> | null;
}
export interface IssueCommits {
    'id': string;
}
export interface IssueCreatedDetails {
    'issue'?: Issue;
    'originMessage'?: MessageLink;
}
export interface IssueDescriptionChangedDetails {
    'oldDescription'?: string | null;
    'newDescription'?: string | null;
}
export interface IssueDueDateChangedDetails {
    'oldDueDate'?: string | null;
    'newDueDate'?: string | null;
}
export interface IssueIdentifier {
    'className': string;
    'project': ProjectIdentifier;
    'externalId': string;
    'id': string;
    'key': string;
}
export interface IssueIdentifierExternalId {
    'project': ProjectIdentifier;
    'externalId': string;
}
export interface IssueIdentifierId {
    'id': string;
}
export interface IssueIdentifierKey {
    'key': string;
}
export interface IssueImportResult {
    'message': string;
    'created'?: Array<IssueImportResultItem> | null;
    'updated'?: Array<IssueImportResultItem> | null;
    'skipped'?: Array<IssueImportResultItem> | null;
    'missingAttributes'?: Array<string> | null;
}
export interface IssueImportResultItem {
    'externalId'?: string | null;
    'issue'?: Issue;
    'error'?: string | null;
}
export interface IssueImportTransactionWebhookEvent {
    'meta': KMetaMod;
    'importTransaction': ImportTransactionRecord;
}
export interface IssueListCFInputValue {
    'issues': Array<IssueIdentifier>;
}
export interface IssueListCFValue {
    'issues': Array<Issue>;
}
export declare const IssueListGrouping: {
    readonly None: "NONE";
    readonly Tag: "TAG";
    readonly Assignee: "ASSIGNEE";
    readonly Status: "STATUS";
    readonly Project: "PROJECT";
};
export type IssueListGrouping = typeof IssueListGrouping[keyof typeof IssueListGrouping];
export interface IssueMentionedDetails {
    'message': ChannelItemSnapshot;
}
export interface IssueMenuActionContext {
    'projectIdentifier': ProjectIdentifier;
    'issueIdentifier': IssueIdentifier;
}
export interface IssueMenuItemUiExtensionApi {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<IssueMenuItemVisibilityFilterApi>;
}
export interface IssueMenuItemUiExtensionIn {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<IssueMenuItemVisibilityFilterIn>;
}
export interface IssueMenuItemVisibilityFilterApi {
    'className': string;
}
export interface IssueMenuItemVisibilityFilterIn {
    'className': string;
}
export interface IssueStatus {
    'id': string;
    'archived': boolean;
    'name': string;
    'resolved': boolean;
    'color': string;
}
export interface IssueStatusChangedDetails {
    'oldStatus': IssueStatus;
    'newStatus': IssueStatus;
}
export interface IssueStatusData {
    'existingId'?: string | null;
    'name': string;
    'resolved': boolean;
    'color': string;
    'originalStatus'?: IssueStatus;
}
export interface IssueStatusWithUsages {
    'status': IssueStatus;
    'usages': number;
}
export interface IssueTagsChangedDetails {
    'addedTags'?: Array<PlanningTag> | null;
    'removedTags'?: Array<PlanningTag> | null;
}
export interface IssueTitleChangedDetails {
    'oldTitle': string;
    'newTitle': string;
}
export interface IssueTracker {
    'id': string;
    'archived': boolean;
    'name': string;
}
export interface IssueTrackerIdentifier {
    'className': string;
    'project': ProjectIdentifier;
    'id': string;
}
export interface IssueTrackerIdentifierDefaultProjectTracker {
    'project': ProjectIdentifier;
}
export interface IssueTrackerIdentifierId {
    'id': string;
}
export interface IssueUnfurlContext {
    'issueId': string;
}
export interface IssueWebhookCustomFieldUpdate {
    'customField': CustomField;
    'mod'?: IssueWebhookCustomFieldUpdateMod | null;
}
export interface IssueWebhookCustomFieldUpdateMod {
    'old': CFValue;
    'new': CFValue;
}
export interface IssueWebhookEvent {
    'meta': KMetaMod;
    'issue': Issue;
    'title'?: AbsenceWebhookEventIcon | null;
    'description'?: AbsenceWebhookEventIcon | null;
    'assignee'?: ApplicationWebhookEventOwner | null;
    'status'?: IssueWebhookEventStatus | null;
    'dueDate'?: AbsenceWebhookEventSince | null;
    'tagDelta'?: IssueWebhookEventTagDelta | null;
    'topicDelta'?: IssueWebhookEventTopicDelta | null;
    'checklistDelta'?: IssueWebhookEventChecklistDelta | null;
    'sprintDelta'?: IssueWebhookEventSprintDelta | null;
    'customFieldUpdate'?: IssueWebhookCustomFieldUpdate;
    'deleted'?: AbsenceWebhookEventAvailable | null;
}
export interface IssueWebhookEventChecklistDelta {
    'old': Array<Checklist>;
    'new': Array<Checklist>;
}
export interface IssueWebhookEventSprintDelta {
    'old': Array<SprintRecord>;
    'new': Array<SprintRecord>;
}
export interface IssueWebhookEventStatus {
    'old': IssueStatus;
    'new': IssueStatus;
}
export interface IssueWebhookEventTagDelta {
    'old': Array<PlanningTag>;
    'new': Array<PlanningTag>;
}
export interface IssueWebhookEventTopicDelta {
    'old': Array<Topic>;
    'new': Array<Topic>;
}
export declare const IssuesSorting: {
    readonly Updated: "UPDATED";
    readonly Created: "CREATED";
    readonly Title: "TITLE";
    readonly Due: "DUE";
};
export type IssuesSorting = typeof IssuesSorting[keyof typeof IssuesSorting];
export interface JobDTO {
    'id': string;
    'name': string;
    'repoName': string;
    'repository': RepositoryInProject;
    'archive': boolean;
}
export interface JobExecutionDTO {
    'executionId': string;
    'executionNumber': number;
    'jobId': string;
    'jobName': string;
    'projectId': string;
    'branch': string;
    'status': ExecutionStatus;
    'triggeredTime': number;
    'startedTime'?: number | null;
    'finishedTime'?: number | null;
    'changesCount': number;
    'predictedEndTime'?: EstimationDTO;
    'failureConditions': Array<FailureConditionDTO>;
}
export declare const JobExecutionDisplayStatus: {
    readonly Awaiting: "Awaiting";
    readonly Running: "Running";
    readonly Restarting: "Restarting";
    readonly Finishing: "Finishing";
    readonly Stopped: "Stopped";
    readonly Succeeded: "Succeeded";
    readonly Failed: "Failed";
    readonly NoSuitableWorkers: "NoSuitableWorkers";
    readonly WaitingForWorkers: "WaitingForWorkers";
};
export type JobExecutionDisplayStatus = typeof JobExecutionDisplayStatus[keyof typeof JobExecutionDisplayStatus];
export declare const JobExecutionStatus: {
    readonly Started: "Started";
    readonly Succeeded: "Succeeded";
    readonly Failed: "Failed";
    readonly Terminated: "Terminated";
};
export type JobExecutionStatus = typeof JobExecutionStatus[keyof typeof JobExecutionStatus];
export interface JobExecutionTrigger {
    'className': string;
    'reviewId': string;
    'branches': Array<string>;
    'commit': string;
    'principal': CPrincipal;
}
export interface JobExecutionTriggerCodeReviewClosed {
    'reviewId': string;
}
export interface JobExecutionTriggerCodeReviewOpened {
    'reviewId': string;
}
export interface JobExecutionTriggerGitBranchDeleted {
    'branches': Array<string>;
}
export interface JobExecutionTriggerGitPush {
    'commit': string;
}
export interface JobExecutionTriggerManual {
    'principal': CPrincipal;
}
export interface JobParameter {
    'name': string;
    'value': string;
}
export interface JobSubscription {
    'id': string;
    'jobId': string;
    'project': PRProject;
    'state': JobSubscriptionState;
    'archived': boolean;
}
export interface JobSubscriptionState {
    'type': JobSubscriptionType;
    'containsMyCommit': boolean;
    'runByMe': boolean;
}
export declare const JobSubscriptionType: {
    readonly AllFinished: "ALL_FINISHED";
    readonly Failed: "FAILED";
};
export type JobSubscriptionType = typeof JobSubscriptionType[keyof typeof JobSubscriptionType];
export declare const JobTriggerType: {
    readonly Manual: "MANUAL";
    readonly GitPush: "GIT_PUSH";
    readonly Schedule: "SCHEDULE";
    readonly GitBranchDeleted: "GIT_BRANCH_DELETED";
    readonly CodeReviewOpened: "CODE_REVIEW_OPENED";
    readonly CodeReviewClosed: "CODE_REVIEW_CLOSED";
};
export type JobTriggerType = typeof JobTriggerType[keyof typeof JobTriggerType];
export interface KBArticle {
    'id': string;
    'archived': boolean;
    'title': string;
    'book': KBBook;
    'folder': KBFolder;
    'documentId': string;
    'documentRef': Document;
    'created': string;
    'createdBy'?: CPrincipal;
    'updated': string;
    'updatedBy'?: CPrincipal;
    'alias': string;
}
export interface KBBook {
    'id': string;
    'archived': boolean;
    'name': string;
    'summary': string;
    'updated': number;
    'updatedBy'?: CPrincipal;
    'alias': string;
    'contexts'?: Array<KBBookContext> | null;
    'project'?: PRProject;
    'rootFolder': KBFolder;
}
export interface KBBookContext {
    'className': string;
    'project': PRProject;
}
export interface KBFolder {
    'id': string;
    'archived': boolean;
    'name': string;
    'parent'?: KBFolder;
    'subfolders': Array<KBFolder>;
    'articles': Array<KBArticle>;
    'book': KBBook;
    'cover'?: KBArticle;
    'alias': string;
    'created': string;
    'createdBy'?: CPrincipal;
    'updated': string;
    'updatedBy'?: CPrincipal;
}
export interface KMetaMod {
    'principal': CPrincipal;
    'timestamp': string;
    'method': string;
}
export interface KbProjectContext {
    'project': PRProject;
}
export interface KotlinPlatform {
    'name': string;
    'targets': Array<string>;
}
export interface LastChanges {
    'lastChanges': Array<RevisionInfo>;
    'totalChanges': number;
}
export interface LaunchResult {
    'className': string;
    'message': string;
    'executionId': string;
}
export interface LaunchResultError {
    'message': string;
}
export interface LaunchResultSuccess {
    'executionId': string;
}
export declare const LdapModuleType: {
    readonly Ad: "AD";
    readonly OpenLdap: "OPEN_LDAP";
};
export type LdapModuleType = typeof LdapModuleType[keyof typeof LdapModuleType];
export interface ListCommandsPayload {
    'clientId': string;
    'userId'?: string | null;
    'verificationToken'?: string | null;
}
export declare const LoadDirection: {
    readonly Prev: "Prev";
    readonly Next: "Next";
};
export type LoadDirection = typeof LoadDirection[keyof typeof LoadDirection];
export interface LocalCodeDiscussionAnchorIn {
    'revision': string;
    'filename': string;
    'line'?: number | null;
    'oldLine'?: number | null;
}
export interface LocationCFInputValue {
    'location'?: string | null;
}
export interface LocationCFValue {
    'location'?: TDLocation;
}
export interface LocationEvent {
    'meta': KMetaMod;
    'location': TDLocation;
}
export declare const LocationType: {
    readonly Region: "Region";
    readonly Campus: "Campus";
    readonly Building: "Building";
    readonly Floor: "Floor";
    readonly Room: "Room";
    readonly ConferenceRoom: "ConferenceRoom";
};
export type LocationType = typeof LocationType[keyof typeof LocationType];
export interface M2AbsenceItemApproveDeletedContent {
    'absence': AbsenceRecord;
    'by': TDMemberProfile;
    'approve': boolean;
}
export interface M2AbsenceItemApprovedContent {
    'absence': AbsenceRecord;
    'by': TDMemberProfile;
    'approve': boolean;
}
export interface M2AbsenceItemContent {
    'absence': AbsenceRecord;
    'by'?: TDMemberProfile;
}
export interface M2AbsenceItemDeletedContent {
    'absence': AbsenceRecord;
    'by'?: TDMemberProfile;
}
export interface M2AbsenceItemUpdatedContent {
    'absence': AbsenceRecord;
    'reason'?: AbsenceWebhookEventReason | null;
    'description'?: AbsenceWebhookEventIcon | null;
    'since'?: AbsenceWebhookEventSince | null;
    'till'?: AbsenceWebhookEventSince | null;
    'by'?: TDMemberProfile;
}
export declare const M2Access: {
    readonly Private: "Private";
    readonly Public: "Public";
};
export type M2Access = typeof M2Access[keyof typeof M2Access];
export interface M2BlogItemContent {
    'article': ArticleRecord;
    'articleContent': ArticleContentRecord;
    'articleDetails': ArticleDetailsRecord;
    'articleChannel': ArticleChannelRecord;
}
export interface M2BlogItemContentDetails {
    'className': string;
    'article': ArticleRecord;
    'articleContent': ArticleContentRecord;
    'articleDetails': ArticleDetailsRecord;
    'articleChannel': ArticleChannelRecord;
    'articlePreview': ArticlePreviewRecord;
}
export interface M2BlogItemPreview {
    'article': ArticleRecord;
    'articlePreview': ArticlePreviewRecord;
    'articleDetails': ArticleDetailsRecord;
    'articleChannel': ArticleChannelRecord;
}
export interface M2ChannelAutomationJobFeedInfo {
    'jobSubscription': JobSubscription;
    'jobName': string;
    'repoName'?: string | null;
    'notificationDefaults': ChannelSpecificDefaults;
}
export interface M2ChannelContact {
    'defaultName': string;
    'key': string;
    'ext'?: M2ChannelContactInfo;
}
export interface M2ChannelContactArticle {
    'article': ArticleRecord;
    'notificationDefaults': ChannelSpecificDefaults;
}
export interface M2ChannelContactCallDiscussion {
    'session': CallSession;
    'notificationDefaults': ChannelSpecificDefaults;
}
export interface M2ChannelContactInfo {
    'className': string;
    'member': TDMemberProfile;
    'notificationDefaults': ChannelSpecificDefaults;
    'memberTeams'?: ProfileMembershipRecord;
    'projectKey'?: ProjectKey;
    'issue': Issue;
    'deployTarget': DeployTargetRecord;
    'project': PRProject;
    'channelId': string;
    'subject'?: string | null;
    'members': Array<TDMemberProfile>;
    'codeDiscussionId': string;
    'codeDiscussion': CodeDiscussionRecord;
    'cause'?: M2ObsoleteCause;
    'workspaceId': string;
    'workspaceName': string;
    'name': string;
    'canHaveThreads': boolean;
    'color'?: PrivateFeedColor;
    'icon'?: string | null;
    'jobSubscription': JobSubscription;
    'jobName': string;
    'repoName'?: string | null;
    'codeReviewDiscussion': string;
    'supportReplies'?: boolean | null;
    'session': CallSession;
    'parent': M2ChannelRecord;
    'text'?: string | null;
    'messageId': string;
    'messageAuthor'?: CPrincipal;
    'attachments'?: string | null;
    'deployment': DeploymentRecord;
    'app': ESApp;
    'article': ArticleRecord;
    'group': string;
    'access': M2Access;
    'description': string;
    'iconId'?: string | null;
    'teams'?: Array<TDTeam> | null;
    'canEdit'?: boolean | null;
    'codeReviewId': string;
    'codeReview'?: CodeReviewRecord;
    'participants'?: CodeReviewParticipants;
    'pendingMessageCounter'?: CodeReviewPendingMessageCounter;
}
export interface M2ChannelContactObsolete {
    'cause'?: M2ObsoleteCause;
}
export interface M2ChannelContactThread {
    'parent': M2ChannelRecord;
    'text'?: string | null;
    'messageId': string;
    'messageAuthor'?: CPrincipal;
    'attachments'?: string | null;
}
export interface M2ChannelContentApplication {
    'app': ESApp;
    'notificationDefaults': ChannelSpecificDefaults;
}
export interface M2ChannelContentArticle {
    'article': ArticleRecord;
    'articleContent': ArticleContentRecord;
    'details'?: ArticleDetailsRecord;
    'channel'?: ArticleChannelRecord;
}
export interface M2ChannelContentCallDiscussion {
    'session': CallSession;
}
export interface M2ChannelContentCodeDiscussion {
    'codeDiscussionId': string;
    'notificationDefaults': ChannelSpecificDefaults;
    'codeDiscussion'?: CodeDiscussionRecord;
}
export interface M2ChannelContentCodeDiscussionInReview {
    'notificationDefaults': ChannelSpecificDefaults;
    'codeDiscussion': CodeDiscussionRecord;
    'parent': M2ChannelRecord;
    'messageId': string;
}
export interface M2ChannelContentCodeReviewDiscussion {
    'codeReviewDiscussion': string;
    'notificationDefaults': ChannelSpecificDefaults;
}
export interface M2ChannelContentCodeReviewFeed {
    'codeReviewId': string;
    'notificationDefaults': ChannelSpecificDefaults;
    'codeReview'?: CodeReviewRecord;
    'participants'?: CodeReviewParticipants;
    'pendingMessageCounter'?: CodeReviewPendingMessageCounter;
    'project'?: PRProject;
}
export interface M2ChannelContentInfo {
    'className': string;
    'member': TDMemberProfile;
    'notificationDefaults': ChannelSpecificDefaults;
    'memberTeams'?: ProfileMembershipRecord;
    'projectKey'?: ProjectKey;
    'issue': Issue;
    'article': ArticleRecord;
    'articleContent': ArticleContentRecord;
    'details'?: ArticleDetailsRecord;
    'channel'?: ArticleChannelRecord;
    'record': ChannelItemRecord;
    'parent': M2ChannelRecord;
    'session': CallSession;
    'deployTarget': DeployTargetRecord;
    'project': PRProject;
    'channelId': string;
    'subject'?: string | null;
    'members': Array<TDMemberProfile>;
    'codeDiscussionId': string;
    'codeDiscussion'?: CodeDiscussionRecord;
    'cause'?: M2ObsoleteCause;
    'workspaceId': string;
    'workspaceName': string;
    'name': string;
    'canHaveThreads': boolean;
    'color'?: PrivateFeedColor;
    'icon'?: string | null;
    'jobSubscription': JobSubscription;
    'jobName': string;
    'repoName'?: string | null;
    'codeReviewDiscussion': string;
    'supportReplies'?: boolean | null;
    'deployment': DeploymentRecord;
    'app': ESApp;
    'group': string;
    'access': M2Access;
    'description': string;
    'iconId'?: string | null;
    'teams'?: Array<TDTeam> | null;
    'canEdit'?: boolean | null;
    'codeReviewId': string;
    'codeReview'?: CodeReviewRecord;
    'participants'?: CodeReviewParticipants;
    'pendingMessageCounter'?: CodeReviewPendingMessageCounter;
}
export interface M2ChannelContentMember {
    'member': TDMemberProfile;
    'notificationDefaults': ChannelSpecificDefaults;
    'memberTeams'?: ProfileMembershipRecord;
}
export interface M2ChannelContentNamedPrivateChannel {
    'name': string;
    'notificationDefaults'?: ChannelSpecificDefaults;
    'canHaveThreads': boolean;
    'color'?: PrivateFeedColor;
    'icon'?: string | null;
}
export interface M2ChannelContentRecord {
    'id': string;
    'content'?: M2ChannelContentInfo;
}
export interface M2ChannelContentThread {
    'record': ChannelItemRecord;
    'parent': M2ChannelRecord;
}
export interface M2ChannelIssueInfo {
    'projectKey'?: ProjectKey;
    'issue': Issue;
    'notificationDefaults': ChannelSpecificDefaults;
}
export interface M2ChannelRdWsTimelineInfo {
    'notificationDefaults': ChannelSpecificDefaults;
    'workspaceId': string;
    'workspaceName': string;
}
export interface M2ChannelRecord {
    'id': string;
    'contact': M2ChannelContact;
    'totalMessages': number;
    'lastMessage'?: MessageInfo;
    'participants'?: Array<ChannelParticipant> | null;
    'channelArchived'?: boolean | null;
    'archived': boolean;
    'pinnedMessages'?: Array<ChannelItemRecord> | null;
    'content'?: M2ChannelContentInfo;
}
export interface M2DraftEditorAddedItemContent {
    'id': string;
    'title': string;
}
export interface M2DraftEditorTeamAddedItemContent {
    'id': string;
    'team': TDTeam;
    'title': string;
}
export declare const M2EmailNotificationType: {
    readonly Instant: "INSTANT";
    readonly Delayed: "DELAYED";
    readonly None: "NONE";
};
export type M2EmailNotificationType = typeof M2EmailNotificationType[keyof typeof M2EmailNotificationType];
export interface M2ExternalStatusFailureItemContent {
    'projectId'?: string | null;
    'repository': string;
    'branch': string;
    'revisionInfo'?: RevisionAuthorInfo;
    'changesInfo'?: LastChanges;
    'url': string;
    'externalServiceName': string;
    'taskName': string;
    'timestamp'?: number | null;
    'description'?: string | null;
}
export interface M2ExternalStatusSucceedItemContent {
    'projectId'?: string | null;
    'repository': string;
    'branch': string;
    'revisionInfo'?: RevisionAuthorInfo;
    'changesInfo'?: LastChanges;
    'url': string;
    'externalServiceName': string;
    'taskName': string;
    'timestamp'?: number | null;
    'description'?: string | null;
}
export interface M2ItemContentDetails {
    'className': string;
    'absence': AbsenceRecord;
    'by': TDMemberProfile;
    'approve': boolean;
    'team': TDTeam;
    'state': CodeReviewState;
    'commits': Array<UnfurlDetailsCommit> | null;
    'changeType': ReviewerChangedType;
    'projectKey': string;
    'review'?: CodeReviewRecord;
    'repository': string;
    'branch': string;
    'branchType': MergeRequestBranchType;
    'discussion': CodeReviewDiscussionRecord;
    'sourceBranch': string;
    'targetBranch': string;
    'codeDiscussion': CodeDiscussionRecord;
    'codeReview': CodeReviewRecord;
    'track': boolean;
    'reviewId': string;
    'reviewNumber': number;
    'reviewType': ReviewType;
    'description'?: AbsenceWebhookEventIcon | null;
    'descriptionEditedByProfileIds': Array<TDMemberProfile>;
    'uid': TDMemberProfile;
    'oldTitle': string;
    'newTitle': string;
    'projectId'?: string | null;
    'revisionInfo'?: RevisionAuthorInfo;
    'changesInfo'?: LastChanges;
    'url': string;
    'externalServiceName': string;
    'taskName': string;
    'timestamp'?: number | null;
    'sticker': Sticker;
    'pack'?: StickerPackInfo;
    'poll': PollRecord;
    'oldDescription'?: string | null;
    'newDescription'?: string | null;
    'issue'?: Issue;
    'originMessage'?: MessageLink;
    'message': ChannelItemSnapshot;
    'oldDueDate'?: string | null;
    'newDueDate'?: string | null;
    'oldStatus': IssueStatus;
    'newStatus': IssueStatus;
    'addedNames'?: Array<string> | null;
    'removedNames'?: Array<string> | null;
    'addedTags'?: Array<PlanningTag> | null;
    'removedTags'?: Array<PlanningTag> | null;
    'oldAssignee'?: TDMemberProfile;
    'newAssignee'?: TDMemberProfile;
    'addedChecklists'?: Array<Checklist> | null;
    'removedChecklists'?: Array<Checklist> | null;
    'deployment': DeploymentRecord;
    'markdown'?: boolean | null;
    'mentions'?: Array<EntityMention> | null;
    'action': string;
    'success': boolean;
    'details': string;
    'session': CallSession;
    'article': ArticleRecord;
    'articleContent': ArticleContentRecord;
    'articleDetails': ArticleDetailsRecord;
    'articleChannel': ArticleChannelRecord;
    'articlePreview': ArticlePreviewRecord;
    'pkg': PackageVersionInfo;
    'membership': TDMembership;
    'leave': boolean;
    'accepted'?: boolean | null;
    'id': string;
    'title': string;
    'taskExecutionId': string;
    'taskExecutionName': string;
    'repoName': string;
    'branchName': string;
    'commit': string;
    'shortCommitMessage': string;
    'project': ProjectKey;
    'finishDateTime': number;
    'triggerInfo': string;
    'member': TDMemberProfile;
    'style'?: MessageStyle;
    'outline'?: MCOutline;
    'content': Array<MCElement>;
    'extension'?: M2ItemContentDetails;
    'principals': Array<CPrincipal>;
    'othersDisplayNames': Array<string>;
    'reason'?: AbsenceWebhookEventReason | null;
    'since'?: AbsenceWebhookEventSince | null;
    'till'?: AbsenceWebhookEventSince | null;
}
export interface M2MaintenanceActionContent {
    'action': string;
    'success': boolean;
    'details': string;
}
export interface M2MemberContent {
    'className': string;
    'member': TDMemberProfile;
}
export interface M2MemberJoinsContent {
    'member': TDMemberProfile;
}
export interface M2MemberLeavesContent {
    'member': TDMemberProfile;
}
export interface M2MembershipContent {
    'className': string;
    'membership': TDMembership;
    'leave': boolean;
}
export interface M2MembershipCreatedContent {
    'membership': TDMembership;
}
export interface M2MembershipRequestedContent {
    'membership': TDMembership;
    'leave': boolean;
}
export interface M2MembershipTerminatedContent {
    'membership': TDMembership;
}
export declare const M2ObsoleteCause: {
    readonly PermissionsDenied: "PermissionsDenied";
};
export type M2ObsoleteCause = typeof M2ObsoleteCause[keyof typeof M2ObsoleteCause];
export interface M2PackageContentDetails {
    'className': string;
    'pkg': PackageVersionInfo;
}
export interface M2PackageCreatedDetails {
    'pkg': PackageVersionInfo;
}
export interface M2PackageDeletedDetails {
    'pkg': PackageVersionInfo;
}
export interface M2PollContent {
    'poll': PollRecord;
}
export interface M2PrivateConversationChannelContent {
    'channelId': string;
    'subject'?: string | null;
    'members': Array<TDMemberProfile>;
    'notificationDefaults'?: ChannelSpecificDefaults;
}
export interface M2SharedChannelContent {
    'name': string;
    'group': string;
    'access': M2Access;
    'description': string;
    'iconId'?: string | null;
    'notificationDefaults': ChannelSpecificDefaults;
    'teams'?: Array<TDTeam> | null;
    'canEdit'?: boolean | null;
    'project'?: PRProject;
}
export interface M2TaskExecutionFailureItemContent {
    'taskExecutionId': string;
    'taskExecutionName': string;
    'repoName': string;
    'branchName': string;
    'commit': string;
    'shortCommitMessage': string;
    'project': ProjectKey;
    'finishDateTime': number;
    'triggerInfo': string;
    'details': string;
}
export interface M2TaskExecutionSucceedItemContent {
    'taskExecutionId': string;
    'taskExecutionName': string;
    'repoName': string;
    'branchName': string;
    'commit': string;
    'shortCommitMessage': string;
    'project': ProjectKey;
    'finishDateTime': number;
    'triggerInfo': string;
    'details': string;
}
export interface M2TextItemContent {
    'markdown'?: boolean | null;
    'mentions'?: Array<EntityMention> | null;
}
export interface M2UnreadStatus {
    'unread': boolean;
    'counter': number;
    'id': string;
}
export interface MCAction {
    'id': string;
    'value': string;
    'clientSideAction'?: MCClientSideAction;
}
export interface MCButton {
    'text': string;
    'style'?: MessageButtonStyle;
    'action': MCAction;
    'disabled'?: boolean | null;
}
export interface MCClientSideAction {
    'context'?: ClientSideActionContext;
}
export interface MCElement {
    'defaultText': string;
    'details'?: MCElementDetails;
}
export interface MCElementDetails {
    'className': string;
    'fields': Array<MCFieldsFieldsInner>;
    'text': string;
    'style'?: MessageStyle;
    'content': string;
    'markdown': boolean;
    'accessory'?: MCElement;
    'size'?: MessageTextSize;
    'action': MCAction;
    'disabled'?: boolean | null;
    'src': string;
    'name': string;
    'elements': Array<MCInlineElement>;
    'header'?: MCText;
    'footer'?: MCText;
    'textSize'?: MessageTextSize;
}
export interface MCFields {
    'fields': Array<MCFieldsFieldsInner>;
}
export interface MCFieldsFieldsInner {
    'first'?: MCElement;
    'second'?: MCElement;
}
export interface MCGroup {
    'elements': Array<MCElement>;
}
export interface MCIcon {
    'name': string;
    'style'?: MessageStyle;
}
export interface MCImage {
    'src': string;
}
export interface MCInlineElement {
    'defaultText': string;
    'details'?: MCInlineElementDetails;
}
export interface MCInlineElementDetails {
    'className': string;
    'content': string;
    'markdown': boolean;
    'style'?: MessageStyle;
    'text': string;
    'action': MCAction;
    'disabled'?: boolean | null;
    'name': string;
    'ts': number;
    'format'?: MessageTimestampFormat;
    'strikethrough': boolean;
}
export interface MCInlineGroup {
    'elements': Array<MCInlineElement>;
    'accessory'?: MCElement;
    'textSize'?: MessageTextSize;
}
export interface MCInlineText {
    'content': string;
    'markdown': boolean;
    'style'?: MessageStyle;
}
export interface MCMessage {
    'style'?: MessageStyle;
    'outline'?: MCOutline;
    'content': Array<MCElement>;
    'extension'?: M2ItemContentDetails;
}
export interface MCOutline {
    'className': string;
    'elements': Array<MCInlineElement>;
    'icon'?: MCElement;
    'text'?: MCText;
}
export interface MCOutlineLegacy {
    'icon'?: MCElement;
    'text'?: MCText;
}
export interface MCOutlineV2 {
    'elements': Array<MCInlineElement>;
}
export interface MCParagraph {
    'accessory'?: MCElement;
    'elements': Array<MCElement>;
}
export interface MCSection {
    'elements': Array<MCElement>;
    'style'?: MessageStyle;
    'header'?: MCText;
    'footer'?: MCText;
}
export interface MCTag {
    'text': string;
    'style'?: MessageStyle;
}
export interface MCText {
    'content': string;
    'markdown': boolean;
    'accessory'?: MCElement;
    'style'?: MessageStyle;
    'size'?: MessageTextSize;
}
export interface MCTimestamp {
    'ts': number;
    'style'?: MessageStyle;
    'format'?: MessageTimestampFormat;
    'strikethrough': boolean;
}
export interface MarketplaceAppMetadata {
    'marketplaceAppId': string;
    'lastSentServerUrl': string;
    'connectionStatus': AppConnectionStatus;
    'uninstallationStatus'?: AppUninstallationStatus;
    'uninstallationStartedAt'?: string | null;
}
export interface MavenChecksum {
    'algorithm': string;
    'value': string;
}
export interface MavenCoordinates {
    'groupId': string;
    'artifactId': string;
    'version': string;
}
export interface MavenPackageDependency {
    'scope': string;
    'group': string;
    'artifact': string;
    'version'?: string | null;
}
export interface MavenPackageFile {
    'name': string;
    'created': number;
    'length': number;
    'checksums': Array<MavenChecksum>;
}
export interface MavenPackageParent {
    'group': string;
    'artifact': string;
    'version': string;
}
export interface MavenPackageType {
    'id': string;
}
export interface MavenPackageVersionDetails {
    'type': PackageType;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'packaging'?: string | null;
    'packageName'?: string | null;
    'description'?: string | null;
    'url'?: string | null;
    'licenses': Array<string>;
    'scmUrl'?: string | null;
    'dependencies': Array<MavenPackageDependency>;
    'kotlinPlatforms'?: Array<KotlinPlatform> | null;
    'parent'?: MavenPackageParent;
    'pathPrefix'?: string | null;
    'files': Array<MavenPackageFile>;
}
export interface MavenRepositoryConnectionSettings {
    'nexusStagingProfile'?: NexusStagingProfile;
}
export interface MdMarkup {
    'unfurl': Array<Unfurl>;
}
export interface MdTextDocumentContent {
    'markdown': string;
}
export interface MeCodeReviewParticipantRecord {
    'id': string;
    'role'?: CodeReviewParticipantRole;
    'theirTurn'?: boolean | null;
    'reviewerState'?: ReviewerState;
    'isApproveSticky'?: boolean | null;
    'review': CodeReviewRecord;
    'participants': CodeReviewParticipants;
    'pendingCounter': CodeReviewPendingMessageCounter;
    'archived': boolean;
}
export interface MediaAttachment {
    'className': string;
    'id': string;
    'name'?: string | null;
    'width': number;
    'height': number;
    'sizeBytes': number;
    'previewBytes'?: string | null;
    'variants'?: Array<ImageAttachmentVariant> | null;
}
export declare const MediaSource: {
    readonly User: "USER";
    readonly Screen: "SCREEN";
};
export type MediaSource = typeof MediaSource[keyof typeof MediaSource];
export interface MeetingAttachment {
    'fileUrl'?: string | null;
    'title'?: string | null;
    'mimeType'?: string | null;
    'fileId'?: string | null;
    'source'?: string | null;
    'iconLink'?: string | null;
}
export declare const MeetingJoiningPreference: {
    readonly Nobody: "NOBODY";
    readonly Everyone: "EVERYONE";
};
export type MeetingJoiningPreference = typeof MeetingJoiningPreference[keyof typeof MeetingJoiningPreference];
export interface MeetingMenuActionContext {
    'meetingId': string;
}
export interface MeetingMenuItemUiExtensionApi {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
}
export interface MeetingMenuItemUiExtensionIn {
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
}
export interface MeetingModWebhookEvent {
    'meta': KMetaMod;
    'meeting': DTOMeeting;
}
export declare const MeetingModificationPreference: {
    readonly Organizer: "ORGANIZER";
    readonly Participants: "PARTICIPANTS";
    readonly Everyone: "EVERYONE";
};
export type MeetingModificationPreference = typeof MeetingModificationPreference[keyof typeof MeetingModificationPreference];
export interface MeetingOccurrenceTime {
    'start': string;
    'end': string;
}
export interface MeetingOrganizer {
    'className': string;
    'applicationRef'?: ESApp;
    'email': string;
    'profileRef': TDMemberProfile;
}
export interface MeetingOrganizerApplication {
    'applicationRef'?: ESApp;
}
export interface MeetingOrganizerExternalUser {
    'email': string;
}
export interface MeetingOrganizerUser {
    'profileRef': TDMemberProfile;
}
export declare const MeetingOrigin: {
    readonly User: "User";
    readonly GoogleCalendar: "GoogleCalendar";
};
export type MeetingOrigin = typeof MeetingOrigin[keyof typeof MeetingOrigin];
export interface MeetingRecord {
    'id': string;
    'archived': boolean;
    'starts': string;
    'finishes': string;
    'timezone'?: ATimeZone;
    'allDay': boolean;
    'rooms': Array<TDLocation>;
    'participants': Array<Participant>;
    'article'?: ArticleRecord;
}
export declare const MeetingVisibility: {
    readonly Participants: "PARTICIPANTS";
    readonly Everyone: "EVERYONE";
};
export type MeetingVisibility = typeof MeetingVisibility[keyof typeof MeetingVisibility];
export interface MeetingWithOccurrenceTime {
    'meetingId': string;
    'occurrenceTime': MeetingOccurrenceTime;
}
export interface MemberCommonSubscriptionFilter {
    'teams': Array<TDTeam>;
    'locations': Array<TDLocation>;
    'profiles'?: Array<TDMemberProfile> | null;
}
export interface MemberCommonSubscriptionFilterIn {
    'teams': Array<string>;
    'locations': Array<string>;
    'profiles'?: Array<ProfileIdentifier> | null;
}
export interface MembersAddedItemDetails {
    'principals': Array<CPrincipal>;
    'othersDisplayNames': Array<string>;
}
export interface MembershipEvent {
    'profile': TDMemberProfile;
    'membership': Array<TDMembership>;
}
export interface MenuActionContext {
    'className': string;
    'channelIdentifier': ChannelIdentifier;
    'messageIdentifier': ChatMessageIdentifier;
    'projectIdentifier': ProjectIdentifier;
    'codeReviewIdentifier': ReviewIdentifier;
    'meetingId': string;
    'documentId': string;
    'folderIdentifier': FolderIdentifier;
    'blogEventId': string;
    'issueIdentifier': IssueIdentifier;
}
export interface MenuActionPayload {
    'menuItemUniqueCode': string;
    'context'?: MenuActionContext;
    'clientId': string;
    'userId': string;
    'verificationToken'?: string | null;
}
export interface MenuItemUiExtensionApi {
    'className': string;
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<DocumentFolderMenuItemVisibilityFilterApi>;
}
export interface MenuItemUiExtensionIn {
    'className': string;
    'displayName': string;
    'description'?: string | null;
    'menuItemUniqueCode': string;
    'visibilityFilters': Array<ChatMessageMenuItemVisibilityFilterIn>;
}
export interface MergeRequestBranch {
    'displayName': string;
    'ref': string;
    'deleted': boolean;
    'head'?: string | null;
}
export interface MergeRequestBranchDeletedEvent {
    'repository': string;
    'branch': string;
    'branchType': MergeRequestBranchType;
}
export interface MergeRequestBranchPair {
    'repository': string;
    'sourceBranch': string;
    'targetBranch': string;
    'sourceBranchRef': string;
    'sourceBranchInfo'?: MergeRequestBranch;
    'targetBranchInfo'?: MergeRequestBranch;
    'isMerged'?: boolean | null;
    'isStale'?: boolean | null;
}
export interface MergeRequestBranchRestoredEvent {
    'repository': string;
    'branch': string;
    'branchType': MergeRequestBranchType;
}
export declare const MergeRequestBranchType: {
    readonly Source: "SOURCE";
    readonly Target: "TARGET";
};
export type MergeRequestBranchType = typeof MergeRequestBranchType[keyof typeof MergeRequestBranchType];
export interface MergeRequestMergedEvent {
    'repository': string;
    'sourceBranch': string;
    'targetBranch': string;
}
export interface MergeRequestRecord {
    'id': string;
    'project': ProjectKey;
    'projectId': string;
    'number': number;
    'title': string;
    'state': CodeReviewState;
    'canBeReopened'?: boolean | null;
    'createdAt': number;
    'createdBy'?: TDMemberProfile;
    'timestamp'?: number | null;
    'turnBased'?: boolean | null;
    'feedChannel'?: M2ChannelRecord;
    'feedChannelId'?: string | null;
    'branchPairs': Array<MergeRequestBranchPair>;
    'externalLink'?: ExternalCodeReviewLink;
    'issueIds': Array<string>;
    'discussionCounter': DiscussionCounter;
    'description'?: string | null;
    'unfurls': Array<Attachment>;
    'commits': Array<ReviewCommit>;
    'participants'?: Array<CodeReviewParticipant> | null;
    'reviewers': Array<CodeReviewParticipantRecord>;
    'authors': Array<CodeReviewParticipantRecord>;
    'watchers': Array<CodeReviewParticipantRecord>;
}
export interface MessageAccessoryElement {
    'className': string;
    'text': string;
    'style'?: MessageStyle;
    'icon': ApiIcon;
    'src': string;
}
export interface MessageAction {
    'className': string;
    'url': string;
    'withBackUrl': boolean;
    'openInNewTab': boolean;
    'actionId': string;
    'payload': string;
}
export interface MessageActionPayload {
    'actionId': string;
    'actionValue': string;
    'message': MessageContext;
    'clientId': string;
    'userId': string;
    'verificationToken'?: string | null;
}
export interface MessageBlockElement {
    'className': string;
    'elements': Array<MessageInlineElement>;
    'accessory'?: MessageAccessoryElement;
    'textSize'?: MessageTextSize;
    'textStyle'?: MessageStyle;
    'elementsStyle'?: MessageStyle;
    'fields': Array<MessageFieldElement>;
    'nameTextSize'?: MessageTextSize;
    'valueTextSize'?: MessageTextSize;
    'style'?: MessageStyle;
    'size'?: MessageTextSize;
    'content': string;
}
export interface MessageButton {
    'text': string;
    'style': MessageButtonStyle;
    'action': MessageAction;
    'disabled'?: boolean | null;
}
export declare const MessageButtonStyle: {
    readonly Primary: "PRIMARY";
    readonly Regular: "REGULAR";
    readonly Danger: "DANGER";
    readonly Secondary: "SECONDARY";
};
export type MessageButtonStyle = typeof MessageButtonStyle[keyof typeof MessageButtonStyle];
export interface MessageContext {
    'messageId': string;
    'channelId': string;
    'messageData'?: string | null;
    'body'?: ChatMessage;
    'attachments'?: Array<Attachment> | null;
    'externalId'?: string | null;
    'createdTime': string;
}
export interface MessageControlElement {
    'className': string;
    'text': string;
    'style': MessageButtonStyle;
    'action': MessageAction;
    'disabled'?: boolean | null;
}
export interface MessageControlGroup {
    'elements': Array<MessageControlElement>;
}
export interface MessageField {
    'first': string;
    'second': string;
}
export interface MessageFieldElement {
    'className': string;
    'first': Array<MessageInlineElement>;
    'second': Array<MessageInlineElement>;
}
export interface MessageFieldV2 {
    'first': Array<MessageInlineElement>;
    'second': Array<MessageInlineElement>;
}
export interface MessageFields {
    'fields': Array<MessageFieldElement>;
    'nameTextSize'?: MessageTextSize;
    'valueTextSize'?: MessageTextSize;
}
export interface MessageForImport {
    'authorPrincipalId': string;
    'text': string;
    'createdAtUtc': number;
    'attachments'?: Array<AttachmentIn> | null;
}
export interface MessageIcon {
    'icon': ApiIcon;
    'style'?: MessageStyle;
}
export interface MessageImage {
    'src': string;
}
export interface MessageInfo {
    'id': string;
    'text': string;
    'time': number;
    'author': CPrincipal;
    'attachments'?: string | null;
    'inThirdPerson'?: boolean | null;
}
export interface MessageInlineElement {
    'className': string;
    'text': string;
    'style': MessageStyle;
    'icon': ApiIcon;
    'action': MessageAction;
    'disabled'?: boolean | null;
    'ts': number;
    'format'?: MessageTimestampFormat;
    'strikethrough': boolean;
}
export interface MessageInlineGroup {
    'accessory'?: MessageAccessoryElement;
    'elements': Array<MessageInlineElement>;
    'textSize'?: MessageTextSize;
    'textStyle'?: MessageStyle;
    'elementsStyle'?: MessageStyle;
}
export interface MessageInlineText {
    'text': string;
    'style'?: MessageStyle;
}
export interface MessageLink {
    'channelKey': string;
    'channelId': string;
    'messageId': string;
}
export interface MessageOutline {
    'icon'?: ApiIcon;
    'text': string;
}
export interface MessageOutlineBase {
    'className': string;
    'icon'?: ApiIcon;
    'text': string;
    'elements': Array<MessageInlineElement>;
}
export interface MessageOutlineElements {
    'elements': Array<MessageInlineElement>;
}
export interface MessagePayload {
    'message': MessageContext;
    'clientId': string;
    'userId': string;
    'verificationToken'?: string | null;
}
export interface MessageRecipient {
    'className': string;
    'channel': ChatChannel;
    'codeReview': string;
    'issue': string;
    'member': ProfileIdentifier;
}
export interface MessageRecipientChannel {
    'channel': ChatChannel;
}
export interface MessageRecipientCodeReview {
    'codeReview': string;
}
export interface MessageRecipientIssue {
    'issue': string;
}
export interface MessageRecipientMember {
    'member': ProfileIdentifier;
}
export interface MessageSection {
    'elements': Array<MessageBlockElement>;
    'header'?: string | null;
    'footer'?: string | null;
    'style'?: MessageStyle;
    'textSize'?: MessageTextSize;
}
export interface MessageSectionElement {
    'className': string;
    'elements': Array<MessageBlockElement>;
    'header'?: string | null;
    'footer'?: string | null;
    'style'?: MessageStyle;
    'textSize'?: MessageTextSize;
}
export declare const MessageStyle: {
    readonly Primary: "PRIMARY";
    readonly Secondary: "SECONDARY";
    readonly Success: "SUCCESS";
    readonly Error: "ERROR";
    readonly Warning: "WARNING";
};
export type MessageStyle = typeof MessageStyle[keyof typeof MessageStyle];
export interface MessageTag {
    'text': string;
    'style'?: MessageStyle;
}
export interface MessageText {
    'accessory'?: MessageAccessoryElement;
    'style'?: MessageStyle;
    'size'?: MessageTextSize;
    'content': string;
}
export declare const MessageTextSize: {
    readonly Large: "LARGE";
    readonly Regular: "REGULAR";
    readonly Small: "SMALL";
};
export type MessageTextSize = typeof MessageTextSize[keyof typeof MessageTextSize];
export interface MessageTimestamp {
    'ts': number;
    'style'?: MessageStyle;
    'format'?: MessageTimestampFormat;
    'strikethrough': boolean;
}
export declare const MessageTimestampFormat: {
    readonly RelativeToNow: "RELATIVE_TO_NOW";
    readonly Full: "FULL";
    readonly TimeOnly: "TIME_ONLY";
    readonly DateOnly: "DATE_ONLY";
};
export type MessageTimestampFormat = typeof MessageTimestampFormat[keyof typeof MessageTimestampFormat];
export interface MessagesRangePosition {
    'className': string;
    'date': string;
    'readTime': number;
    'subscribed'?: boolean | null;
    'id': string;
}
export interface MessagesRangePositionFirstOnDate {
    'date': string;
}
export interface MessagesRangePositionFirstUnread {
    'readTime': number;
    'subscribed'?: boolean | null;
}
export interface MessagesRangePositionMessageLink {
    'id': string;
}
export declare const MessagesSorting: {
    readonly FromOldestToNewest: "FromOldestToNewest";
    readonly FromNewestToOldest: "FromNewestToOldest";
};
export type MessagesSorting = typeof MessagesSorting[keyof typeof MessagesSorting];
export interface MetricsEvent {
    'id': string;
    'time': number;
    'props'?: Array<MetricsProp> | null;
    'points'?: Array<MetricsPoint> | null;
}
export interface MetricsPoint {
    'key': string;
    'value': number;
}
export interface MetricsProp {
    'key': string;
    'value': string;
}
export interface MetricsRequest {
    'client': ClientInfo;
    'events': Array<MetricsEvent>;
}
export interface NavBarMenuItem {
    'item': string;
    'enabled': boolean;
}
export interface NavigateUrlAction {
    'url': string;
    'withBackUrl': boolean;
    'openInNewTab': boolean;
}
export interface NavigateUrlActionContext {
    'url': string;
    'withBackUrl': boolean;
    'openInNewTab': boolean;
}
export interface NewMergeRequestFromIssueActionContext {
    'projectKey': ProjectKey;
    'repository': string;
    'commitId': string;
    'issueNumber': number;
    'projectRepos': ProjectReposRecord;
    'issueCommitsRef': CommitLinksContainer;
}
export interface NewMessage {
    'text': string;
    'temporaryId': string;
    'attachments'?: Array<Attachment> | null;
    'pending': boolean;
    'draftTag'?: string | null;
    'postponeId'?: string | null;
}
export interface NewUnfurlQueueItemsPayload {
    'clientId': string;
    'verificationToken'?: string | null;
}
export interface NexusStagingProfile {
    'id': string;
    'name': string;
}
export interface NodeWithChildren {
    'className': string;
    'lang': string;
    'children': Array<InlineNode>;
    'attrs'?: RtTableCellAttrs;
    'startNumber': number;
    'level': number;
    'textAlign': RtTextAlign;
}
export interface NonWorkingDays {
    'since': string;
    'till': string;
    'startsEarlier': boolean;
    'endsLater': boolean;
}
export interface NonWorkingDaysEvent {
    'profile': TDMemberProfile;
    'days': Array<NonWorkingDays>;
}
export declare const NotificationFilter: {
    readonly AllMessages: "ALL_MESSAGES";
    readonly Mentions: "MENTIONS";
    readonly Nothing: "NOTHING";
    readonly Mute: "MUTE";
};
export type NotificationFilter = typeof NotificationFilter[keyof typeof NotificationFilter];
export interface NotificationsChannelSubscriptionsGetRequest {
    'channel': ChatChannel;
    'name': string;
    'subscription': CustomGenericSubscriptionIn;
}
export interface NotificationsChannelSubscriptionsIdDeleteRequest {
    'name'?: string | null;
    'enabled'?: boolean | null;
    'subscription'?: CustomGenericSubscriptionIn;
}
export interface NotificationsPersonalCustomSubscriptionsGetRequest {
    'profile': ProfileIdentifier;
    'name': string;
    'feed': string;
    'subscription': CustomGenericSubscriptionIn;
}
export interface NotificationsPersonalCustomSubscriptionsIdDeleteRequest {
    'name'?: string | null;
    'enabled'?: boolean | null;
    'feed'?: string | null;
    'subscription'?: CustomGenericSubscriptionIn;
}
export interface NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest {
    'profile': ProfileIdentifier;
    'subjectCode': string;
    'feed': string;
    'enabled': boolean;
}
export interface NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest {
    'profile': ProfileIdentifier;
    'targetCode': string;
    'feed': string;
    'eventCodes': Array<string>;
}
export interface NotificationsPrivateFeedsGetRequest {
    'profile': ProfileIdentifier;
    'name': string;
    'icon': string;
    'color': PrivateFeedColor;
}
export interface NotificationsPrivateFeedsIdDeleteRequest {
    'name'?: string | null;
    'icon'?: string | null;
    'color'?: PrivateFeedColor;
}
export interface NpmPackageDependency {
    'name': string;
    'version': string;
    'type'?: string | null;
}
export interface NpmPackageType {
    'id': string;
}
export interface NpmPackageVersionDetails {
    'type': PackageType;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'description'?: string | null;
    'dependencies': Array<NpmPackageDependency>;
    'keywords': Array<string>;
    'license'?: string | null;
    'projectUrl'?: string | null;
    'repositoryUrl'?: string | null;
    'repositoryRevision'?: string | null;
    'readme'?: string | null;
    'unityVersion'?: string | null;
}
export interface NuGetDependency {
    'targetFramework'?: string | null;
    'id': string;
    'range': string;
}
export interface NuGetPackageType {
    'id': string;
}
export interface NuGetPackageVersionDetails {
    'type': PackageType;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'verbatimVersion': string;
    'description'?: string | null;
    'projectUrl'?: string | null;
    'license'?: string | null;
    'licenseUrl'?: string | null;
    'icon'?: string | null;
    'title'?: string | null;
    'dependencies': Array<NuGetDependency>;
}
export interface OIDCDiscovery {
    'issuer': string;
    'authorizationEndpoint': string;
    'tokenEndpoint'?: string | null;
    'userinfoEndpoint'?: string | null;
    'jwksUri': string;
}
export interface OpenEnumCFCreateParameters {
    'values': Array<string>;
}
export interface OpenEnumCFInputValue {
    'enumValueIdentifier'?: CFEnumValueIdentifier;
}
export interface OpenEnumCFUpdateParameters {
    'modifications': Array<CFEnumValueModification>;
}
export interface OpenEnumCFValue {
    'value'?: EnumValueData;
}
export interface OpenEnumListCFInputValue {
    'enumValueIdentifiers': Array<CFEnumValueIdentifier>;
}
export interface OpenEnumListCFValue {
    'values': Array<EnumValueData>;
}
export interface OpenPersonalFeedSettingsAction {
    'tab': string;
    'feed': string;
    'feedId'?: string | null;
}
export interface OrgDomainDTO {
    'domain': string;
    'expireAt'?: string | null;
    'orgUrl': string;
}
export declare const OrgIndustryDTO: {
    readonly SoftwareDevelopment: "SoftwareDevelopment";
    readonly Science: "Science";
    readonly Education: "Education";
    readonly Finance: "Finance";
    readonly Health: "Health";
    readonly Manufacturing: "Manufacturing";
    readonly Retail: "Retail";
    readonly Transportation: "Transportation";
    readonly Media: "Media";
    readonly Hospitality: "Hospitality";
    readonly ProfessionalServices: "ProfessionalServices";
    readonly NonProfit: "NonProfit";
    readonly Other: "Other";
};
export type OrgIndustryDTO = typeof OrgIndustryDTO[keyof typeof OrgIndustryDTO];
export declare const OrgSizeDTO: {
    readonly JustMe: "JUST_ME";
    readonly N210: "N_2_10";
    readonly N1150: "N_11_50";
    readonly N51250: "N_51_250";
    readonly N251500: "N_251_500";
    readonly N5011000: "N_501_1000";
    readonly N10015000: "N_1001_5000";
    readonly MoreThan5000: "MORE_THAN_5000";
};
export type OrgSizeDTO = typeof OrgSizeDTO[keyof typeof OrgSizeDTO];
export interface OrgThrottlingStatus {
    'throttledUntil'?: string | null;
}
export interface OrganizationDomainsGetRequest {
    'domain': string;
}
export interface OrganizationForUpdateDTO {
    'name'?: string;
    'slogan'?: string | null;
    'logoId'?: string | null;
    'logoSmall'?: string | null;
    'logo'?: string | null;
    'onboardingRequired'?: boolean;
    'userAgreementAccepted'?: boolean;
    'timezone'?: ATimeZone;
    'license'?: string | null;
    'orgSize'?: OrgSizeDTO;
    'orgIndustry'?: OrgIndustryDTO;
    'sendAnonymousDataAgreementAccepted'?: boolean;
    'marketplaceEnabled'?: boolean;
}
export interface OrganizationGetRequest {
    'orgData': OrganizationForUpdateDTO;
}
export interface OrganizationRecord {
    'id': string;
    'orgId': string;
    'name': string;
    'slogan'?: string | null;
    'logoId'?: string | null;
    'onboardingRequired'?: boolean | null;
    'allowDomainsEdit'?: boolean | null;
    'createdAt'?: number | null;
    'createdWithNavigationV2'?: boolean | null;
    'timezone'?: ATimeZone;
    'orgSize'?: OrgSizeDTO;
    'orgIndustry'?: OrgIndustryDTO;
    'sendAnonymousDataAgreementAccepted'?: boolean | null;
    'marketplaceEnabled'?: boolean | null;
    'slackWorkspace'?: string | null;
}
export interface Overdrafts {
    'storage': number;
    'bandwidth': number;
    'ciCredits': number;
}
export interface PRPrivateProject {
    'key': ProjectKey;
    'name': string;
    'accessAllowed': boolean;
    'admins': Array<TDMemberProfile>;
}
export interface PRProject {
    'id': string;
    'key': ProjectKey;
    'name': string;
    'private': boolean;
    'description'?: string | null;
    'icon'?: string | null;
    'latestRepositoryActivity'?: string | null;
    'createdAt'?: string | null;
    'archived': boolean;
    'boards': Array<BoardRecord>;
    'features'?: Array<ProjectFeatureState> | null;
    'guestProfiles': Array<TDMemberProfile>;
    'collaboratorsProfiles': Array<TDMemberProfile>;
    'collaboratorsTeams': Array<TDTeam>;
    'repos': Array<PRRepositoryInfo>;
    'memberProfiles': Array<TDMemberProfile>;
    'memberTeams': Array<TDTeam>;
    'adminProfiles': Array<TDMemberProfile>;
    'adminTeams': Array<TDTeam>;
    'trackers': Array<ProjectIssueTrackerItem>;
    'type': ProjectTeamType;
    'teams'?: Array<TDTeam> | null;
    'members'?: Array<ProjectTeamMemberRecord> | null;
    'team'?: TDTeam;
    'personalFeaturePins'?: Array<ToggleableProjectFeaturePins> | null;
    'featurePins'?: Array<ProjectFeaturePins> | null;
    'featuresUsage'?: Array<ProjectFeatureUsage> | null;
    'tags': Array<string>;
    'packages': Array<ProjectPackageRepository>;
    'enable': boolean;
    'hoursInDay': number;
    'daysInWeek': number;
    'format': DurationTextFormat;
}
export interface PRRepositoryInfo {
    'id'?: string | null;
    'name': string;
    'description': string;
    'latestActivity'?: string | null;
    'proxyPushNotification'?: string | null;
    'proxyPushNotificationBody'?: string | null;
    'state': RepositoryState;
    'initProgress'?: string | null;
    'readmeName'?: string | null;
    'monthlyActivity'?: RepositoryActivity;
    'defaultBranch'?: BranchInfo;
}
export interface PRTag {
    'name': string;
    'projectCount': number;
}
export interface PackageData {
    'type': PackageType;
    'repository': string;
    'name': string;
    'versions': number;
    'updated': number;
    'lastVersion': string;
}
export interface PackageMetadata {
    'connectionId'?: string | null;
}
export interface PackageOrigin {
    'className': string;
    'name'?: string | null;
    'url': string;
}
export interface PackageOriginBuild {
    'name'?: string | null;
    'url'?: string | null;
}
export interface PackageOriginUrl {
    'url': string;
}
export interface PackageRepository {
    'id': string;
    'type': PackageType;
    'name'?: string | null;
    'description'?: string | null;
    'public': boolean;
    'cleanupEnabled': boolean;
    'settings'?: ESPackageRepositorySettings;
    'mode': PackageRepositoryMode;
    'archived': boolean;
}
export interface PackageRepositoryConnection {
    'className': string;
    'id': string;
    'description'?: string | null;
    'enabled': boolean;
    'rememberDownloaded': boolean;
    'packageNameFilters'?: Array<string> | null;
    'url': string;
    'credentials': PackageRepositoryCredentials;
    'secretId'?: string | null;
    'secretValue'?: string | null;
    'settings'?: PackageRepositoryConnectionSettings;
    'repository': ProjectPackageRepository;
}
export interface PackageRepositoryConnectionRemote {
    'id': string;
    'description'?: string | null;
    'enabled': boolean;
    'rememberDownloaded': boolean;
    'packageNameFilters'?: Array<string> | null;
    'url': string;
    'credentials': PackageRepositoryCredentials;
    'secretId'?: string | null;
    'secretValue'?: string | null;
    'settings'?: PackageRepositoryConnectionSettings;
}
export interface PackageRepositoryConnectionSettings {
    'className': string;
    'nexusStagingProfile'?: NexusStagingProfile;
}
export interface PackageRepositoryConnectionSpace {
    'id': string;
    'description'?: string | null;
    'enabled': boolean;
    'rememberDownloaded': boolean;
    'packageNameFilters'?: Array<string> | null;
    'repository': ProjectPackageRepository;
}
export declare const PackageRepositoryCredentials: {
    readonly Anonymous: "ANONYMOUS";
    readonly Password: "PASSWORD";
    readonly Token: "TOKEN";
    readonly Header: "HEADER";
};
export type PackageRepositoryCredentials = typeof PackageRepositoryCredentials[keyof typeof PackageRepositoryCredentials];
export declare const PackageRepositoryEventAction: {
    readonly Published: "published";
    readonly Removed: "removed";
    readonly Vulnerabilities: "vulnerabilities";
};
export type PackageRepositoryEventAction = typeof PackageRepositoryEventAction[keyof typeof PackageRepositoryEventAction];
export interface PackageRepositoryIdentifier {
    'className': string;
    'id': string;
    'key': string;
    'type': string;
    'name': string;
}
export interface PackageRepositoryIdentifierId {
    'id': string;
}
export interface PackageRepositoryIdentifierKey {
    'key': string;
}
export interface PackageRepositoryIdentifierTypeAndName {
    'type': string;
    'name': string;
}
export declare const PackageRepositoryMode: {
    readonly Hosting: "HOSTING";
    readonly Proxy: "PROXY";
};
export type PackageRepositoryMode = typeof PackageRepositoryMode[keyof typeof PackageRepositoryMode];
export interface PackageType {
    'className': string;
    'id': string;
}
export interface PackageVersionData {
    'type': PackageType;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created'?: number | null;
    'lastAccessed'?: number | null;
    'downloads'?: number | null;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize'?: number | null;
    'path'?: string | null;
}
export interface PackageVersionDetails {
    'className': string;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'dependencies': Array<ComposerPackageDependency>;
    'description': string | null;
    'documentation': string;
    'homepage'?: string | null;
    'readme': string | null;
    'keywords': Array<string> | null;
    'categories': Array<string>;
    'license': string | null;
    'licenseFileContent'?: string | null;
    'gitRepository'?: string | null;
    'links'?: string | null;
    'type': PackageType;
    'packaging'?: string | null;
    'packageName'?: string | null;
    'url'?: string | null;
    'licenses': Array<string>;
    'scmUrl'?: string | null;
    'kotlinPlatforms'?: Array<KotlinPlatform> | null;
    'parent'?: MavenPackageParent;
    'pathPrefix'?: string | null;
    'files': Array<PythonPackageFile>;
    'homePage': string | null;
    'repositoryUrl': string | null;
    'issueTracker': string;
    'changelog': string;
    'devDependencies'?: Array<DartPackageDependency> | null;
    'dependencyOverrides'?: Array<DartPackageDependency> | null;
    'environment'?: Array<DartPackageDependency> | null;
    'notNormalizedName': string;
    'platform'?: string | null;
    'summary'?: string | null;
    'descriptionContentType'?: string | null;
    'authorFromPackageDetails'?: string | null;
    'authorEmail'?: string | null;
    'maintainer'?: string | null;
    'maintainerEmail'?: string | null;
    'classifiers': Array<string>;
    'projectUrls': Array<PythonPackageUrl>;
    'requiresDist': Array<string>;
    'requiresPython'?: string | null;
    'projectUrl'?: string | null;
    'repositoryRevision'?: string | null;
    'unityVersion'?: string | null;
    'verbatimVersion': string;
    'licenseUrl'?: string | null;
    'icon'?: string | null;
    'title'?: string | null;
    'schemaVersion': number;
    'mediaType': string;
    'manifestType': string;
    'image'?: ContainerImage;
    'chart'?: ContainerHelmChart;
}
export interface PackageVersionInfo {
    'type': PackageType;
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
}
export interface PackageVersionRef {
    'name': string;
    'version': string;
}
export interface PackagesExecutionResult {
    'className': string;
    'error': string;
}
export interface PackagesExecutionResultFailure {
    'error': string;
}
export interface PackagesPublishing {
    'publishingId': string;
    'source': PublishingSource;
    'created': number;
    'started'?: number | null;
    'completed'?: number | null;
    'successful': boolean;
    'error'?: string | null;
    'principal'?: CPrincipal;
}
export declare const PackagesSortColumn: {
    readonly Downloads: "Downloads";
    readonly LastAccessed: "LastAccessed";
    readonly Created: "Created";
    readonly Size: "Size";
};
export type PackagesSortColumn = typeof PackagesSortColumn[keyof typeof PackagesSortColumn];
export interface PackagesSubscriptionFilter {
    'project'?: PRProject;
    'repository'?: ProjectPackageRepository;
    'namePattern': Array<string>;
    'versionPattern'?: string | null;
}
export interface PackagesSubscriptionFilterIn {
    'project'?: string | null;
    'repository'?: string | null;
    'namePattern': Array<string>;
    'versionPattern'?: string | null;
}
export interface PageUiExtensionApi {
    'className': string;
    'iframeUrl'?: string | null;
}
export interface PageUiExtensionIn {
    'className': string;
    'iframeUrl'?: string | null;
}
export interface ParameterLastUsageDTO {
    'jobExecutionId': string;
    'stepExecutionId'?: string | null;
    'jobName': string;
    'usedAt': string;
}
export interface Participant {
    'user': TDMemberProfile;
    'status': EventParticipationStatus;
}
export declare const ParticipationState: {
    readonly None: "NONE";
    readonly Connected: "CONNECTED";
    readonly Disconnected: "DISCONNECTED";
};
export type ParticipationState = typeof ParticipationState[keyof typeof ParticipationState];
export declare const PasswordStrength: {
    readonly NoPolicy: "NO_POLICY";
    readonly Fair: "FAIR";
    readonly Good: "GOOD";
    readonly Strong: "STRONG";
};
export type PasswordStrength = typeof PasswordStrength[keyof typeof PasswordStrength];
export interface PercentageCFValue {
    'value'?: number | null;
}
export interface PermissionContextApi {
    'className': string;
    'channelId': string;
    'channel': M2ChannelRecord;
    'project': PRProject;
    'projectId': string;
}
export interface PermissionContextIdentifier {
    'className': string;
    'channel': string;
    'project': ProjectIdentifier;
}
export interface PermissionIdentifier {
    'className': string;
}
export declare const PermissionRoleType: {
    readonly Predefined: "PREDEFINED";
    readonly Custom: "CUSTOM";
    readonly Authorizable: "AUTHORIZABLE";
    readonly Anonymous: "ANONYMOUS";
};
export type PermissionRoleType = typeof PermissionRoleType[keyof typeof PermissionRoleType];
export interface PermissionRolesCreatePostRequest {
    'contextIdentifier': PermissionContextIdentifier;
    'name': string;
}
export interface PermissionRolesRoleId2FaRequirementGetRequest {
    'required'?: boolean | null;
}
export interface PermissionRolesRoleIdPermissionsDeleteRequest {
    'rightCodes': Array<string>;
}
export interface PermissionTarget {
    'className': string;
    'channel': ChannelIdentifier;
    'project': ProjectIdentifier;
    'profile': ProfileIdentifier;
    'team': string;
}
export interface PermissionsCheckPermissionPostRequest {
    'principal': PrincipalIn;
    'uniqueRightCode': PermissionIdentifier;
    'target': PermissionTarget;
}
export interface PersonalDocumentContainerInfo {
    'owner': TDMemberProfile;
    'grantedRights'?: Array<string> | null;
}
export interface PersonalSubscriptionEvent {
    'eventCode': string;
    'name': string;
    'memberOnly'?: boolean | null;
    'featureFlag'?: string | null;
}
export interface PersonalSubscriptionSettings {
    'feed': PrivateFeed;
    'subjectSettings': Array<PersonalSubscriptionSubjectSettings>;
    'enabledCodes': Array<string>;
}
export interface PersonalSubscriptionSubjectSettings {
    'subjectCode': string;
    'enabled': boolean;
}
export interface PersonalSubscriptionTarget {
    'subjectCode': string;
    'targetCode': string;
    'description': string;
    'events': Array<PersonalSubscriptionEvent>;
    'featureFlag'?: string | null;
}
export interface PingWebhookEvent {
    'webhookName': string;
}
export interface PlainParameterRecord {
    'id': string;
    'archived': boolean;
    'bundleId': string;
    'key': string;
    'description'?: string | null;
    'createdAt'?: string | null;
    'createdBy'?: CPrincipal;
    'updatedAt'?: string | null;
    'updatedBy'?: CPrincipal;
    'value': string;
}
export interface PlanItem {
    'id': string;
    'checklistId': string;
    'tag'?: PlanningTag;
    'simpleText'?: string | null;
    'simpleDone'?: boolean | null;
    'issue'?: Issue;
    'issueProblem'?: string | null;
    'canEditIssue'?: boolean | null;
    'hasChildren': boolean;
    'archived': boolean;
    'children': Array<PlanItem>;
    'topics': Array<Topic>;
}
export interface PlanItemChildren {
    'id': string;
    'children': Array<PlanItem>;
    'archived': boolean;
}
export interface PlanItemIdentifier {
    'className': string;
    'id': string;
}
export interface PlanItemIdentifierId {
    'id': string;
}
export interface PlanningTag {
    'id': string;
    'archived': boolean;
    'projectId': string;
    'parent'?: PlanningTag;
    'name': string;
}
export interface PollRecord {
    'id': string;
    'owner': TDMemberProfile;
    'question': string;
    'meVote': boolean;
    'countPeople': number;
    'anonymous': boolean;
    'closed': boolean;
    'extendable': boolean;
    'multiChoice': boolean;
    'ended': boolean;
    'expirationTime'?: string | null;
    'votes': Array<VoteGroup>;
}
export interface PostMessageAction {
    'actionId': string;
    'payload': string;
}
export interface PostUnfurlContentResult {
    'className': string;
    'queueItemId': string;
}
export interface PostUnfurlContentResultEntityNotFound {
    'queueItemId': string;
}
export interface PostUnfurlContentResultNotApproved {
    'queueItemId': string;
}
export interface PostUnfurlContentResultQueueItemNotFound {
    'queueItemId': string;
}
export interface PostUnfurlContentResultSuccess {
    'queueItemId': string;
}
export interface PostponeWorkspaceDeletionChannelAction {
    'workspaceId': string;
}
export interface PostponedChannelItem {
    'id': string;
    'channelId': string;
    'message': string;
    'attachments': Array<AttachmentInfo>;
    'etag': number;
    'time'?: string | null;
    'contactKey'?: string | null;
    'postpone': string;
    'markdown'?: boolean | null;
    'parentId'?: string | null;
    'archived': boolean;
}
export interface PrincipalIn {
    'className': string;
    'application': ApplicationIdentifier;
    'builtIn': string;
    'profile': ProfileIdentifier;
}
export interface PrincipalInApplication {
    'application': ApplicationIdentifier;
}
export interface PrincipalInBuiltIn {
    'builtIn': string;
}
export interface PrincipalInProfile {
    'profile': ProfileIdentifier;
}
export interface PrivateChannelPermissionContext {
    'channelId': string;
}
export interface PrivateFeed {
    'id': string;
    'name': string;
    'icon': string;
    'color': PrivateFeedColor;
}
export declare const PrivateFeedColor: {
    readonly Blue: "Blue";
    readonly Purple: "Purple";
    readonly Yellow: "Yellow";
    readonly Green: "Green";
    readonly Grey: "Grey";
    readonly Orange: "Orange";
    readonly Pink: "Pink";
};
export type PrivateFeedColor = typeof PrivateFeedColor[keyof typeof PrivateFeedColor];
export interface PrivateProjectPermissionContext {
    'projectId': string;
}
export interface ProducerOptions {
    'sfuOptions': SFUProducerOptions;
    'userConnectionId': number;
    'routerId': string;
    'source': MediaSource;
}
export interface Profile2FARequirement {
    'roleId': string;
    'roleCode': string;
    'roleName': string;
}
export interface ProfileCFInputValue {
    'profile'?: ProfileIdentifier;
}
export interface ProfileCFValue {
    'profile'?: TDMemberProfile;
}
export interface ProfileEvent {
    'meta': KMetaMod;
    'member': TDMemberProfile;
}
export interface ProfileIdentifier {
    'className': string;
    'externalId': string;
    'id': string;
    'username': string;
}
export interface ProfileIdentifierExternalId {
    'externalId': string;
}
export interface ProfileIdentifierId {
    'id': string;
}
export interface ProfileIdentifierUsername {
    'username': string;
}
export interface ProfileLinkPreview {
    'profile': TDMemberProfile;
}
export interface ProfileListCFInputValue {
    'profiles': Array<ProfileIdentifier>;
}
export interface ProfileListCFValue {
    'profiles': Array<TDMemberProfile>;
}
export interface ProfileMembershipRecord {
    'id': string;
    'memberships': Array<TDMembership>;
    'unapprovedMemberships'?: Array<TDMembership> | null;
}
export declare const ProfileOrder: {
    readonly Default: "DEFAULT";
    readonly FullName: "FULL_NAME";
};
export type ProfileOrder = typeof ProfileOrder[keyof typeof ProfileOrder];
export declare const ProfileOrgRelation: {
    readonly Member: "MEMBER";
    readonly Guest: "GUEST";
    readonly Any: "ANY";
};
export type ProfileOrgRelation = typeof ProfileOrgRelation[keyof typeof ProfileOrgRelation];
export interface ProfileOrganizationEvent {
    'meta': KMetaMod;
    'member': TDMemberProfile;
    'joinedOrganization': boolean;
    'leftOrganization': boolean;
}
export interface ProfilePermissionTarget {
    'profile': ProfileIdentifier;
}
export interface ProjectCFInputValue {
    'project'?: ProjectIdentifier;
}
export interface ProjectCFValue {
    'project'?: PRProject;
}
export interface ProjectCommonSubscriptionFilter {
    'project'?: PRProject;
}
export interface ProjectCommonSubscriptionFilterIn {
    'project'?: string | null;
}
export interface ProjectEvent {
    'meta': KMetaMod;
    'project': PRProject;
}
export declare const ProjectFeature: {
    readonly Documents: "DOCUMENTS";
    readonly Issues: "ISSUES";
    readonly Boards: "BOARDS";
    readonly Repositories: "REPOSITORIES";
    readonly CodeReviews: "CODE_REVIEWS";
    readonly Jobs: "JOBS";
    readonly Packages: "PACKAGES";
    readonly DevEnvironments: "DEV_ENVIRONMENTS";
    readonly Hosting: "HOSTING";
    readonly Deployments: "DEPLOYMENTS";
    readonly Tables: "TABLES";
    readonly Parameters: "PARAMETERS";
    readonly Secrets: "SECRETS";
};
export type ProjectFeature = typeof ProjectFeature[keyof typeof ProjectFeature];
export interface ProjectFeaturePinnedItem {
    'kind'?: ProjectPinnedItemKind;
    'id': string;
}
export interface ProjectFeaturePins {
    'feature': ProjectFeature;
    'items': Array<ProjectFeaturePinnedItem>;
}
export interface ProjectFeatureState {
    'feature': ProjectFeature;
    'enabled': boolean;
}
export interface ProjectFeatureUsage {
    'feature': ProjectFeature;
    'lastUsedAt'?: string | null;
    'firstUsedAt'?: string | null;
}
export interface ProjectIdentifier {
    'className': string;
    'id': string;
    'key': string;
}
export interface ProjectIdentifierId {
    'id': string;
}
export interface ProjectIdentifierKey {
    'key': string;
}
export interface ProjectIssueTrackerItem {
    'key': string;
    'tracker': IssueTracker;
}
export interface ProjectKey {
    'key': string;
}
export interface ProjectPackageRepository {
    'id': string;
    'project': PRProject;
    'name': string;
    'description'?: string | null;
    'repository': PackageRepository;
    'archived': boolean;
    'connections': Array<PackageRepositoryConnection>;
}
export interface ProjectPermissionContext {
    'project': PRProject;
}
export interface ProjectPermissionContextIdentifier {
    'project': ProjectIdentifier;
}
export interface ProjectPermissionTarget {
    'project': ProjectIdentifier;
}
export declare const ProjectPinnedItemKind: {
    readonly CodeRepository: "CodeRepository";
    readonly PlanningIssueBoard: "PlanningIssueBoard";
    readonly PackageRepository: "PackageRepository";
    readonly AutomationJob: "AutomationJob";
    readonly Document: "Document";
    readonly DocumentFolder: "DocumentFolder";
    readonly DeployTarget: "DeployTarget";
    readonly DocumentBook: "DocumentBook";
    readonly DevEnvironment: "DevEnvironment";
};
export type ProjectPinnedItemKind = typeof ProjectPinnedItemKind[keyof typeof ProjectPinnedItemKind];
export interface ProjectReposRecord {
    'id': string;
    'repos': Array<PRRepositoryInfo>;
}
export interface ProjectScope {
    'project': PRProject;
}
export interface ProjectTeamMemberRecord {
    'id': string;
    'project': PRProject;
    'profile': TDMemberProfile;
    'position'?: TDRole;
    'since': string;
    'archived': boolean;
}
export interface ProjectTeamRole {
    'className': string;
}
export declare const ProjectTeamType: {
    readonly Legacy: "LEGACY";
    readonly Members: "MEMBERS";
    readonly Attached: "ATTACHED";
};
export type ProjectTeamType = typeof ProjectTeamType[keyof typeof ProjectTeamType];
export interface ProjectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest {
    'value': string;
}
export interface ProjectsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PRProject>;
}
export interface ProjectsGetRequest {
    'key': ProjectKey;
    'name': string;
    'description'?: string | null;
    'private'?: boolean;
    'tags'?: Array<string>;
    'icon'?: string | null;
}
export interface ProjectsParamsDefaultBundleGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PlainParameterRecord>;
}
export interface ProjectsParamsDefaultBundleGetRequest {
    'project': ProjectIdentifier;
    'key': string;
    'value': string;
    'description'?: string | null;
}
export interface ProjectsParamsIdDeleteRequest {
    'value': string;
    'description'?: string | null;
}
export interface ProjectsParamsInDefaultBundleGetRequest {
    'projectId': string;
    'key': string;
    'value': string;
}
export interface ProjectsPlanningBoardsBoardDeleteRequest {
    'name'?: string | null;
    'description'?: string | null;
    'swimlaneKey'?: string | null;
    'columns'?: BoardColumns;
    'issueFields'?: Array<BoardIssueInputField> | null;
    'memberOwners'?: Array<string> | null;
    'teamOwners'?: Array<string> | null;
}
export interface ProjectsPlanningBoardsSprintsPostRequest {
    'board': BoardIdentifier;
    'name': string;
    'description'?: string | null;
    'from': string;
    'to': string;
    'launch'?: SprintLaunch;
}
export interface ProjectsPlanningBoardsSprintsSprintIssuesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<Issue>;
}
export interface ProjectsPlanningBoardsSprintsSprintLaunchPostRequest {
    'moveUnresolvedIssuesFromCurrentSprint': boolean;
    'notifySubscribers'?: boolean;
}
export interface ProjectsPlanningBoardsSprintsSprintPatchRequest {
    'name'?: string | null;
    'description'?: string | null;
    'from'?: string | null;
    'to'?: string | null;
    'launch'?: SprintLaunch;
}
export interface ProjectsProjectAccessAdminsTeamsPostRequest {
    'teamId': string;
}
export interface ProjectsProjectAutomationDeploymentTargetsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<DeployTargetRecord>;
}
export interface ProjectsProjectAutomationDeploymentTargetsGetRequest {
    'key': string;
    'name': string;
    'description': string;
    'repositories'?: Array<DeployTargetRepositoryDTO>;
}
export interface ProjectsProjectAutomationDeploymentTargetsTargetDeleteRequest {
    'name'?: string | null;
    'description'?: string | null;
    'repositories'?: Array<DeployTargetRepositoryDTO> | null;
    'manualControl'?: boolean | null;
    'hangTimeoutMinutes'?: number | null;
    'failTimeoutMinutes'?: number | null;
}
export interface ProjectsProjectAutomationDeploymentsFailPostRequest {
    'targetIdentifier': TargetIdentifier;
}
export interface ProjectsProjectAutomationDeploymentsFinishPostRequest {
    'targetIdentifier': TargetIdentifier;
    'deploymentIdentifier'?: DeploymentIdentifier;
    'version'?: string | null;
    'description'?: string | null;
    'commitRefs'?: Array<DeploymentCommitReference> | null;
    'externalLink'?: ExternalLink;
}
export interface ProjectsProjectAutomationDeploymentsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<DeploymentRecord>;
}
export interface ProjectsProjectAutomationDeploymentsGetRequest {
    'targetIdentifier': TargetIdentifier;
    'deploymentIdentifier': DeploymentIdentifier;
    'version'?: string | null;
    'scheduledStart'?: string | null;
    'description'?: string | null;
    'commitRefs'?: Array<DeploymentCommitReference> | null;
    'externalLink'?: ExternalLink;
}
export interface ProjectsProjectAutomationDeploymentsSchedulePostRequest {
    'targetIdentifier': TargetIdentifier;
    'version'?: string | null;
    'scheduledStart'?: string | null;
    'description'?: string | null;
    'commitRefs'?: Array<DeploymentCommitReference> | null;
    'externalLink'?: ExternalLink;
}
export interface ProjectsProjectAutomationDeploymentsStartPostRequest {
    'targetIdentifier': TargetIdentifier;
    'deploymentIdentifier'?: DeploymentIdentifier;
    'version'?: string | null;
    'description'?: string | null;
    'commitRefs'?: Array<DeploymentCommitReference> | null;
    'externalLink'?: ExternalLink;
    'syncWithAutomationJob'?: boolean;
}
export interface ProjectsProjectAutomationGraphExecutionsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<JobExecutionDTO>;
}
export interface ProjectsProjectAutomationJobsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<JobDTO>;
}
export interface ProjectsProjectAutomationJobsJobIdStartPostRequest {
    'branch': Branch;
    'parameters'?: Array<JobParameter> | null;
    'checkoutRevisions'?: Array<CheckoutRevisionDTO> | null;
}
export interface ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest {
    'commitMessage': string;
}
export interface ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest {
    'text': string;
    'attachments': Array<AttachmentIn>;
    'snippetContent'?: string;
}
export interface ProjectsProjectCodeReviewsCodeDiscussionsPostRequest {
    'text': string;
    'attachments'?: Array<AttachmentIn> | null;
    'diffContext'?: DiffContext;
    'repository': string;
    'revision'?: string | null;
    'filename'?: string | null;
    'line'?: number | null;
    'oldLine'?: number | null;
    'anchor'?: LocalCodeDiscussionAnchorIn;
    'endAnchor'?: LocalCodeDiscussionAnchorIn;
    'pending'?: boolean;
    'reviewId'?: ReviewIdentifier;
    'suggestedEdit'?: CodeDiscussionSuggestedEditRequest;
}
export interface ProjectsProjectCodeReviewsCommitSetReviewPostRequest {
    'repository': string;
    'revisions': Array<string>;
    'title'?: string | null;
    'authorProfileIds'?: Array<string> | null;
    'files'?: FileSetInReview;
}
export interface ProjectsProjectCodeReviewsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<CodeReviewWithCount>;
}
export interface ProjectsProjectCodeReviewsMergeRequestsPostRequest {
    'repository': string;
    'sourceBranch': string;
    'targetBranch': string;
    'title': string;
    'description'?: string | null;
    'reviewers'?: Array<ReviewerParam> | null;
}
export interface ProjectsProjectCodeReviewsReviewIdFilesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ChangeInReview>;
}
export interface ProjectsProjectCodeReviewsReviewIdMergeFilesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<GitMergedFile>;
}
export interface ProjectsProjectCodeReviewsReviewIdMergePutRequest {
    'deleteSourceBranch': boolean;
    'mergeMode': GitMergeMode;
}
export interface ProjectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest {
    'role': CodeReviewParticipantRole;
}
export interface ProjectsProjectCodeReviewsReviewIdRebasePutRequest {
    'deleteSourceBranch': boolean;
    'rebaseMode': GitRebaseMode;
    'squash': boolean;
    'squashedCommitMessage'?: string | null;
}
export interface ProjectsProjectCodeReviewsReviewIdRevisionsDeleteRequest {
    'revisions': Array<ReviewCommitIn>;
}
export interface ProjectsProjectCodeReviewsReviewIdStatePatchRequest {
    'state': CodeReviewState;
}
export interface ProjectsProjectCodeReviewsReviewIdTitlePatchRequest {
    'title': string;
}
export interface ProjectsProjectDeleteRequest {
    'key'?: ProjectKey;
    'name'?: string | null;
    'description'?: string | null;
    'private'?: boolean | null;
    'tags'?: Array<string> | null;
    'icon'?: string | null;
}
export interface ProjectsProjectDocumentsDocumentIdCopyPostRequest {
    'name': string;
    'folder': FolderIdentifier;
}
export interface ProjectsProjectDocumentsDocumentIdDeleteRequest {
    'name'?: string | null;
    'updateIn'?: DocumentBodyUpdateIn;
    'publicationDetailsIn'?: PublicationDetailsIn;
}
export interface ProjectsProjectDocumentsDocumentIdMovePatchRequest {
    'folder': FolderIdentifier;
}
export interface ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<Document>;
}
export interface ProjectsProjectDocumentsFoldersFolderMovePatchRequest {
    'parentFolder': FolderIdentifier;
}
export interface ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<DocumentFolder>;
}
export interface ProjectsProjectDocumentsFoldersPostRequest {
    'name': string;
    'parentFolder': FolderIdentifier;
}
export interface ProjectsProjectDocumentsPostRequest {
    'name': string;
    'folder': FolderIdentifier;
    'bodyIn': DocumentBodyCreateIn;
    'publicationDetailsIn'?: PublicationDetailsIn;
}
export interface ProjectsProjectPackagesRepositoriesGetRequest {
    'type': string;
    'name': string;
    'description'?: string | null;
    'public'?: boolean;
    'settings'?: ESPackageRepositorySettings;
    'mode'?: PackageRepositoryMode;
    'connection'?: PackageRepositoryConnection;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest {
    'retentionParams': RetentionPolicyParams;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryCleanupPostRequest {
    'retentionPolicyParams'?: RetentionPolicyParams;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PackagesPublishing>;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest {
    'source': PublishingSource;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryDeleteRequest {
    'name'?: string | null;
    'description'?: string | null;
    'public'?: boolean | null;
    'cleanupEnabled'?: boolean | null;
    'settings'?: ESPackageRepositorySettings;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PackageData>;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest {
    'connectionId'?: string | null;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest {
    'pin': boolean;
    'comment'?: string | null;
}
export interface ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PackageVersionData>;
}
export interface ProjectsProjectPlanningBoardsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<BoardRecord>;
}
export interface ProjectsProjectPlanningBoardsGetRequest {
    'name': string;
    'description'?: string | null;
}
export interface ProjectsProjectPlanningBoardsSprintsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<SprintRecord>;
}
export interface ProjectsProjectPlanningChecklistsChecklistIdDeleteRequest {
    'name'?: string;
    'description'?: string | null;
    'owner'?: string | null;
    'tag'?: string | null;
    'topics'?: Array<string>;
}
export interface ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest {
    'targetParentId': string;
    'afterItemId'?: string | null;
    'tabIndentedLines': string;
}
export interface ProjectsProjectPlanningChecklistsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<Checklist>;
}
export interface ProjectsProjectPlanningChecklistsImportPostRequest {
    'name': string;
    'tabIndentedLines': string;
}
export interface ProjectsProjectPlanningIssuesGetRequest {
    'title': string;
    'description'?: string | null;
    'assignee'?: ProfileIdentifier;
    'status': string;
    'dueDate'?: string | null;
    'tags'?: Array<string>;
    'checklists'?: Array<string>;
    'sprints'?: Array<SprintIdentifier>;
    'attachments'?: Array<AttachmentIn> | null;
    'fromMessage'?: MessageLink;
    'customFields'?: Array<CustomFieldInputValue> | null;
    'topics'?: Array<string> | null;
    'parents'?: Array<IssueIdentifier> | null;
}
export interface ProjectsProjectPlanningIssuesImportPostRequest {
    'metadata': ImportMetadata;
    'issues': Array<ExternalIssue>;
    'assigneeMissingPolicy'?: ImportMissingPolicy;
    'statusMissingPolicy'?: ImportMissingPolicy;
    'onExistsPolicy'?: ImportExistsPolicy;
    'dryRun': boolean;
    'notifySubscribers'?: boolean;
}
export interface ProjectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest {
    'attachments': Array<AttachmentIn>;
}
export interface ProjectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest {
    'codeReviewIds': Array<ReviewIdentifier>;
}
export interface ProjectsProjectPlanningIssuesIssueIdCommentsImportPostRequest {
    'comments': Array<MessageForImport>;
}
export interface ProjectsProjectPlanningIssuesIssueIdCommitsDeleteRequest {
    'repository': string;
    'commitIds': Array<string>;
}
export interface ProjectsProjectPlanningIssuesIssueIdDeleteRequest {
    'title'?: string;
    'description'?: string | null;
    'assignee'?: string | null;
    'status'?: string;
    'dueDate'?: string | null;
    'customFields'?: Array<CustomFieldInputValue>;
    'notifySubscribers'?: boolean;
}
export interface ProjectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest {
    'resolved': boolean;
}
export interface ProjectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest {
    'statusId'?: string | null;
}
export interface ProjectsProjectPlanningIssuesStatusesGetRequest {
    'statuses': Array<IssueStatusData>;
}
export interface ProjectsProjectPlanningTagsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PlanningTag>;
}
export interface ProjectsProjectPlanningTagsGetRequest {
    'parentTagId'?: string | null;
    'path': Array<string>;
}
export interface ProjectsProjectRepositoriesRepositoryCherryPickCommitPostRequest {
    'commit': string;
    'targetBranch': string;
}
export interface ProjectsProjectRepositoriesRepositoryCommitPostRequest {
    'baseCommit': string;
    'targetBranch': string;
    'commitMessage': string;
    'files': Array<GitCommitFileRequest>;
}
export interface ProjectsProjectRepositoriesRepositoryCommitsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<GitCommitInfo>;
}
export interface ProjectsProjectRepositoriesRepositoryDeleteBranchPostRequest {
    'branch': string;
}
export interface ProjectsProjectRepositoriesRepositoryDeleteRequest {
    'description'?: string;
    'defaultBranch'?: string | null;
    'initialize'?: boolean;
    'defaultSetup'?: boolean;
}
export interface ProjectsProjectRepositoriesRepositoryMergeBranchPostRequest {
    'sourceBranch': string;
    'targetBranch'?: string | null;
    'commitMessage'?: string | null;
    'mergeMode': GitMergeMode;
}
export interface ProjectsProjectRepositoriesRepositoryReadonlyGetRequest {
    'freeze': boolean;
}
export interface ProjectsProjectRepositoriesRepositoryRebaseBranchPostRequest {
    'sourceBranch': string;
    'targetBranch'?: string | null;
    'rebaseMode': GitRebaseMode;
    'squash': GitSquashMode;
    'squashedCommitMessage'?: string | null;
}
export interface ProjectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest {
    'branch'?: string | null;
    'changes'?: Array<string> | null;
    'executionStatus': CommitExecutionStatus;
    'url': string;
    'externalServiceName': string;
    'taskName': string;
    'taskId': string;
    'taskBuildId'?: string | null;
    'timestamp'?: number | null;
    'description'?: string | null;
}
export interface ProjectsProjectRepositoriesRepositorySettingsGetRequest {
    'settings': GitRepositorySettings;
}
export interface ProjectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest {
    'role'?: string | null;
}
export interface ProjectsProjectResponsibilitiesSubjectsPostRequest {
    'name'?: string | null;
}
export interface ProjectsProjectTopicsGetRequest {
    'name': string;
    'parentTopicId'?: string | null;
}
export interface ProjectsProjectTopicsIdPatchRequest {
    'name'?: string;
    'parentTopicId'?: string | null;
}
export interface ProjectsProjectTopicsRemoveTopicsPostRequest {
    'ids': Array<string>;
}
export interface ProjectsProjectTopicsSetResponsiblePostRequest {
    'topicId': string;
    'responsible': Array<ProfileIdentifier>;
}
export interface ProjectsRepositoriesFindGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<RepositoryDetails>;
}
export interface ProjectsResponsibilitiesPostRequest {
    'subjectId': string;
    'summary': string;
    'notes'?: string | null;
}
export interface ProjectsResponsibilitiesResponsibilityIdDeleteRequest {
    'summary': string;
    'notes'?: string | null;
}
export interface ProjectsSecretsDefaultBundleGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<SecretParameterRecord>;
}
export interface ProjectsSecretsDefaultBundleGetRequest {
    'project': ProjectIdentifier;
    'key': string;
    'valueBase64': string;
    'publicKeyId'?: string | null;
    'description'?: string | null;
}
export interface ProjectsSecretsIdDeleteRequest {
    'valueBase64': string;
    'publicKeyId'?: string | null;
    'description'?: string | null;
}
export interface ProjectsSecretsInDefaultBundleGetRequest {
    'projectId': string;
    'key': string;
    'valueBase64': string;
    'publicKeyId'?: string | null;
}
export interface ProjectsTagsTrackAccessPostRequest {
    'tag': string;
}
export interface ProjectsVaultGetRequest {
    'project': ProjectIdentifier;
    'url': string;
    'name': string;
    'namespace'?: string | null;
    'vaultNamespace'?: string | null;
    'appRoleEndpointPath': string;
    'appRoleId': string;
    'appRoleSecretIdBase64': string;
}
export interface ProjectsVaultIdDeleteRequest {
    'url': string;
    'name': string;
    'namespace'?: string | null;
    'vaultNamespace'?: string | null;
    'appRoleEndpointPath': string;
    'appRoleId': string;
    'appRoleSecretIdBase64'?: string | null;
}
export interface PublicHoliday {
    'id': string;
    'calendar': PublicHolidayCalendarRecord;
    'name': string;
    'date': string;
    'workingDay': boolean;
    'halfDay'?: boolean | null;
    'archived'?: boolean | null;
}
export interface PublicHolidayCalendarRecord {
    'id': string;
    'archived': boolean;
    'name': string;
    'firstDate'?: string | null;
    'lastDate'?: string | null;
    'eventsCount': number;
    'location': TDLocation;
}
export interface PublicHolidaysCalendarsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PublicHolidayCalendarRecord>;
}
export interface PublicHolidaysCalendarsGetRequest {
    'name': string;
    'location': string;
}
export interface PublicHolidaysCalendarsImportPostRequest {
    'calendar': string;
    'attachmentId': string;
}
export interface PublicHolidaysHolidaysGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<PublicHoliday>;
}
export interface PublicHolidaysHolidaysGetRequest {
    'calendar': string;
    'name': string;
    'date': string;
    'workingDay': boolean;
    'halfDay'?: boolean | null;
}
export interface PublicHolidaysHolidaysIdDeleteRequest {
    'calendar'?: string | null;
    'name'?: string | null;
    'date'?: string | null;
    'workingDay'?: boolean | null;
    'halfDay'?: boolean | null;
}
export interface PublicationDetails {
    'className': string;
    'teams'?: Array<TDTeam> | null;
    'locations'?: Array<TDLocation> | null;
    'event'?: CalendarEvent;
    'article'?: ArticleRecord;
}
export interface PublicationDetailsIn {
    'className': string;
    'teams'?: Array<string> | null;
    'locations'?: Array<string> | null;
    'event'?: CalendarEventIn;
    'article'?: string | null;
}
export interface PublishMessage {
    'id': string;
}
export interface PublishingSource {
    'className': string;
    'items': Array<PackageVersionRef>;
}
export interface PublishingSourcePackages {
    'items': Array<PackageVersionRef>;
}
export interface PurchasedBillingPlan {
    'id': string;
    'jetSalesId'?: string | null;
    'installationPublicKey'?: string | null;
    'plan': string;
    'billingPeriod': string;
    'since': string;
    'till': string;
    'mayUseTill'?: string | null;
    'currency': Currency;
    'addUserPrice': number;
    'addStoragePrice': number;
    'addBandwidthPrice': number;
    'addCiCreditPrice': number;
    'minActiveUsers': number;
    'prepaidUsers': number;
    'storagePerUser': number;
    'storageOverall': number;
    'bandwidthPerUser': number;
    'bandwidthOverall': number;
    'ciCredits': number;
    'ciCreditsReserve': number;
    'ciCreditsRateForExternalWorker'?: number | null;
    'integrations': number;
    'searchHistory': number;
    'balance': number;
    'hardLimitAmount': number;
    'hasCardVerifiedAdmin'?: boolean | null;
    'isTrial'?: boolean | null;
    'spentTrials'?: Array<string> | null;
    'trialBasePlan'?: string | null;
}
export interface PythonChecksum {
    'algorithm': string;
    'value': string;
}
export interface PythonPackageFile {
    'name': string;
    'created': number;
    'length': number;
    'checksums': Array<PythonChecksum>;
}
export interface PythonPackageType {
    'id': string;
}
export interface PythonPackageUrl {
    'title': string;
    'href': string;
}
export interface PythonPackageVersionDetails {
    'repository': string;
    'name': string;
    'version': string;
    'tags'?: Array<string> | null;
    'created': number;
    'accessed'?: number | null;
    'downloads': number;
    'pinned': boolean;
    'comment'?: string | null;
    'diskSize': number;
    'author'?: CPrincipal;
    'authors'?: Array<CPrincipal> | null;
    'origin'?: PackageOrigin;
    'metadata'?: {
        [key: string]: string | undefined;
    } | null;
    'notNormalizedName': string;
    'platform'?: string | null;
    'summary'?: string | null;
    'description'?: string | null;
    'descriptionContentType'?: string | null;
    'keywords': Array<string>;
    'homePage'?: string | null;
    'authorFromPackageDetails'?: string | null;
    'authorEmail'?: string | null;
    'maintainer'?: string | null;
    'maintainerEmail'?: string | null;
    'license'?: string | null;
    'classifiers': Array<string>;
    'projectUrls': Array<PythonPackageUrl>;
    'requiresDist': Array<string>;
    'requiresPython'?: string | null;
    'files': Array<PythonPackageFile>;
}
export interface QRCode {
    'width': number;
    'base64Bitmap': string;
}
export interface RdGatewayConnectionInfo {
    'originalLink': string;
    'relayLink': string;
    'relayAuthData': string;
    'version': string;
}
export interface RdInstanceType {
    'id': string;
    'name': string;
    'vcpus': number;
    'memMb': number;
    'volumeGb': number;
    'enabled': boolean;
}
export interface RdWarmupEvent {
    'meta': KMetaMod;
    'warmupExecution': RdWarmupExec;
}
export interface RdWarmupExec {
    'id': string;
    'projectId': string;
    'repositoryName': string;
    'branch': string;
    'revision': string;
    'devfile'?: string | null;
    'status': WarmupExecutionStatus;
    'startedAt'?: string | null;
    'finishedAt'?: string | null;
    'ideType': IdeType;
    'ideVersion': string;
    'ideQuality'?: string | null;
    'computeTaskId': string;
    'trigger': WarmupExecutionTrigger;
    'archived': boolean;
}
export interface RdWarmupSubscriptionFilter {
    'project'?: PRProject;
    'repositoryName'?: string | null;
    'branchSpec'?: Array<string> | null;
}
export interface RdWarmupSubscriptionFilterIn {
    'project': string;
    'repositoryName'?: string | null;
    'branchSpec'?: Array<string> | null;
}
export interface RdWorkspaceResources {
    'milliCpus': number;
    'memoryMb': number;
    'volumeGb': number;
}
export declare const RdWorkspaceState: {
    readonly Initializing: "Initializing";
    readonly Activating: "Activating";
    readonly Active: "Active";
    readonly Unhealthy: "Unhealthy";
    readonly Hibernating: "Hibernating";
    readonly Hibernated: "Hibernated";
    readonly Deleting: "Deleting";
    readonly Deleted: "Deleted";
    readonly Restarting: "Restarting";
    readonly Failed: "Failed";
    readonly FailedToStart: "FailedToStart";
    readonly Created: "Created";
    readonly Inactive: "Inactive";
};
export type RdWorkspaceState = typeof RdWorkspaceState[keyof typeof RdWorkspaceState];
export interface RdWorkspaceVcsData {
    'repoName': string;
    'branchName': string;
    'revision': string;
    'mergeRequestNumber'?: number | null;
    'containsNotPushedChanges': boolean;
}
export interface RdWorkspaceWithoutRefs {
    'id': string;
    'ideType': IdeType;
    'name': string;
    'state': RdWorkspaceState;
    'projectId': string;
    'number': number;
    'externalId': string;
    'vcsData'?: RdWorkspaceVcsData;
    'resources': RdWorkspaceResources;
    'created': string;
    'updated': string;
    'ideVersion'?: string | null;
}
export interface ReactionItemIdentifier {
    'className': string;
    'article': string;
    'channel': ChannelIdentifier;
    'message': ChatMessageIdentifier;
}
export interface ReactionItemIdentifierArticleReaction {
    'article': string;
}
export interface ReactionItemIdentifierChatMessageReaction {
    'channel': ChannelIdentifier;
    'message': ChatMessageIdentifier;
}
export interface RecurrenceRule {
    'freq': RecurrenceRuleFreq;
    'ends': RecurrenceRuleEnds;
}
export interface RecurrenceRuleEnds {
    'className': string;
    'date': string;
    'count': number;
}
export interface RecurrenceRuleEndsOnDate {
    'date': string;
}
export interface RecurrenceRuleEndsTotalCount {
    'count': number;
}
export interface RecurrenceRuleFreq {
    'className': string;
    'interval': number;
    'day': number;
    'weekday': Weekday;
    'shift': number;
    'weekdays': Array<Weekday>;
}
export interface RecurrenceRuleFreqDaily {
    'interval': number;
}
export interface RecurrenceRuleFreqMonthlyOnDate {
    'day': number;
    'interval': number;
}
export interface RecurrenceRuleFreqMonthlyOnFirstWeekday {
    'weekday': Weekday;
    'shift': number;
    'interval': number;
}
export interface RecurrenceRuleFreqMonthlyOnLastWeekday {
    'weekday': Weekday;
    'interval': number;
}
export interface RecurrenceRuleFreqMonthlyOnStartDate {
    'interval': number;
}
export interface RecurrenceRuleFreqWeekly {
    'weekdays': Array<Weekday>;
    'interval': number;
}
export interface RecurrenceRuleFreqYearly {
    'interval': number;
}
export declare const RecurrentModification: {
    readonly Me: "Me";
    readonly SinceMe: "SinceMe";
    readonly All: "All";
};
export type RecurrentModification = typeof RecurrentModification[keyof typeof RecurrentModification];
export interface RefreshTokenPayload {
    'refreshToken': string;
    'scope': string;
    'clientId': string;
    'userId': string;
    'verificationToken'?: string | null;
}
export interface RepoChanges {
    'heads': Array<RepoHeadsChange>;
    'commits': Array<GitCommitInfoWithChanges>;
    'totalNewCommits': number;
}
export interface RepoCommitsSubscriptionFilter {
    'project'?: PRProject;
    'repository': string;
    'spec': RepoCommitsSubscriptionFilterSpec;
}
export interface RepoCommitsSubscriptionFilterIn {
    'project'?: string | null;
    'repository': string;
    'spec': RepoCommitsSubscriptionFilterSpec;
}
export interface RepoCommitsSubscriptionFilterSpec {
    'authors': Array<string>;
    'committers': Array<string>;
    'branchSpec': Array<string>;
    'pathSpec': string;
    'messageRegex': string;
}
export interface RepoHeadsChange {
    'name': string;
    'oldId': string;
    'newId': string;
    'force'?: boolean | null;
}
export interface RepoHeadsSubscriptionFilter {
    'project'?: PRProject;
    'repository': string;
}
export interface RepoHeadsSubscriptionFilterIn {
    'project'?: string | null;
    'repository': string;
}
export interface RepositoryActivity {
    'lastActivity': Array<RepositoryActivityLastActivityInner>;
}
export interface RepositoryActivityLastActivityInner {
    'first': string;
    'second': number;
}
export interface RepositoryCommitRecord {
    'id': string;
    'repositoryId': string;
    'repositoryName': string;
    'revision': string;
    'message'?: string | null;
    'date': string;
    'authorName'?: string | null;
    'authorEmail'?: string | null;
    'committerName'?: string | null;
    'committerEmail'?: string | null;
    'authorProfile'?: TDMemberProfile;
}
export interface RepositoryDetails {
    'projectKey': string;
    'repository': string;
    'starred': boolean;
}
export interface RepositoryInProject {
    'name': string;
    'isDeleted': boolean;
}
export interface RepositoryInReview {
    'name': string;
    'deleted': boolean;
}
export declare const RepositoryState: {
    readonly Ready: "READY";
    readonly InProgress: "IN_PROGRESS";
    readonly Error: "ERROR";
};
export type RepositoryState = typeof RepositoryState[keyof typeof RepositoryState];
export interface RepositoryUrls {
    'httpUrl'?: string | null;
    'sshUrl'?: string | null;
    'sshHost'?: string | null;
}
export interface ResponsibilityRecord {
    'responsibilityId': string;
    'summary': string;
    'notes'?: string | null;
    'assignedPeople': Array<AssignedPerson>;
}
export interface RetentionPolicyParams {
    'numberOfDaysToRetain'?: number | null;
    'numberOfVersionsToRetain'?: number | null;
    'retainDownloadedOnce': boolean;
}
export interface ReviewBranchTrackEvent {
    'repository': string;
    'branch': string;
    'track': boolean;
}
export interface ReviewCommit {
    'repositoryId': string;
    'commitId': string;
}
export interface ReviewCommitIn {
    'repository': string;
    'commit': string;
}
export interface ReviewCompletionStateChangedEvent {
    'state': ReviewerState;
}
export interface ReviewCreatedEvent {
    'projectKey': string;
    'reviewId': string;
    'reviewNumber': number;
    'reviewType': ReviewType;
    'description'?: CodeReviewDescription;
    'descriptionEditedByProfileIds': Array<TDMemberProfile>;
    'commits'?: Array<UnfurlDetailsCommit> | null;
}
export interface ReviewIdentifier {
    'className': string;
    'id': string;
    'key': string;
    'number': number;
}
export interface ReviewIdentifierId {
    'id': string;
}
export interface ReviewIdentifierKey {
    'key': string;
}
export interface ReviewIdentifierNumber {
    'number': number;
}
export interface ReviewRevisionsChangedEvent {
    'commits': Array<RepositoryCommitRecord>;
    'changeType': ReviewRevisionsChangedType;
    'projectKey'?: string | null;
    'review'?: CodeReviewRecord;
}
export declare const ReviewRevisionsChangedType: {
    readonly Created: "Created";
    readonly Added: "Added";
    readonly Removed: "Removed";
};
export type ReviewRevisionsChangedType = typeof ReviewRevisionsChangedType[keyof typeof ReviewRevisionsChangedType];
export declare const ReviewSorting: {
    readonly CreatedAtAsc: "CreatedAtAsc";
    readonly CreatedAtDesc: "CreatedAtDesc";
    readonly LastUpdatedAsc: "LastUpdatedAsc";
    readonly LastUpdatedDesc: "LastUpdatedDesc";
};
export type ReviewSorting = typeof ReviewSorting[keyof typeof ReviewSorting];
export interface ReviewStateChangedEvent {
    'state': CodeReviewState;
    'review'?: CodeReviewRecord;
}
export interface ReviewTitleChangedEvent {
    'oldTitle': string;
    'newTitle': string;
}
export declare const ReviewType: {
    readonly CommitSetReview: "CommitSetReview";
    readonly MergeRequest: "MergeRequest";
};
export type ReviewType = typeof ReviewType[keyof typeof ReviewType];
export interface ReviewerChangedEvent {
    'uid': TDMemberProfile;
    'changeType': ReviewerChangedType;
}
export declare const ReviewerChangedType: {
    readonly Joined: "Joined";
    readonly Left: "Left";
};
export type ReviewerChangedType = typeof ReviewerChangedType[keyof typeof ReviewerChangedType];
export interface ReviewerParam {
    'profileId': string;
    'qualityGateSlot'?: CodeReviewParticipantSlotBase;
}
export declare const ReviewerState: {
    readonly Accepted: "Accepted";
    readonly Rejected: "Rejected";
};
export type ReviewerState = typeof ReviewerState[keyof typeof ReviewerState];
export interface RevisionAuthor {
    'authorName': string;
    'profile'?: TDMemberProfile;
}
export interface RevisionAuthorInfo {
    'revisionInfo': RevisionInfo;
    'author': RevisionAuthor;
}
export interface RevisionInfo {
    'revision': string;
    'date': number;
    'message': string;
}
export interface RevisionsInReview {
    'repository': RepositoryInReview;
    'commits': Array<GitCommitWithGraph>;
}
export interface RichTextParseMarkdownPostRequest {
    'text': string;
}
export interface RightDTO {
    'rightCode': PermissionIdentifier;
    'name': string;
    'group': string;
    'description': string;
    'targetName': string;
    'editable': boolean;
    'status': RightStatus;
    'modificationAuthor'?: CPrincipal;
    'modificationTimestamp'?: string | null;
    'dependencies': Array<string>;
}
export interface RightDescriptorDTO {
    'rightCode': PermissionIdentifier;
    'name': string;
    'group': string;
    'description': string;
    'targetName': string;
    'dependencies': Array<string>;
}
export declare const RightStatus: {
    readonly None: "NONE";
    readonly Requested: "REQUESTED";
    readonly Granted: "GRANTED";
    readonly Rejected: "REJECTED";
};
export type RightStatus = typeof RightStatus[keyof typeof RightStatus];
export interface RightUpdateDTO {
    'rightCode': PermissionIdentifier;
    'status': RightStatus;
}
export interface RoleDTO {
    'roleId': string;
    'type': PermissionRoleType;
    'code'?: string | null;
    'name': string;
    'description': string;
    'membersEditable': boolean;
    'rightsEditable'?: boolean | null;
}
export interface Room {
    'id': string;
    'archived': boolean;
    'name': string;
    'channel'?: M2ChannelRecord;
    'origin'?: RoomOrigin;
}
export declare const RoomOrigin: {
    readonly Unknown: "Unknown";
    readonly Chat: "Chat";
    readonly Adhoc: "Adhoc";
    readonly Meeting: "Meeting";
};
export type RoomOrigin = typeof RoomOrigin[keyof typeof RoomOrigin];
export interface RtBlockquote {
    'children': Array<BlockNode>;
}
export interface RtBreak {
    'soft': boolean;
    'marks': Array<DocumentMark>;
}
export interface RtBulletList {
    'children': Array<RtListItem>;
}
export interface RtCode {
    'lang': string;
    'children': Array<RtText>;
}
export interface RtDocument {
    'children': Array<BlockNode>;
}
export interface RtEmoji {
    'emojiName': string;
    'marks': Array<DocumentMark>;
}
export interface RtHeading {
    'level': number;
    'children': Array<InlineNode>;
    'textAlign': RtTextAlign;
}
export interface RtImage {
    'attrs': RtImageAttrs;
    'marks': Array<DocumentMark>;
}
export interface RtImageAttrs {
    'src': string;
    'title'?: string | null;
    'alt'?: string | null;
}
export interface RtLinkAttrs {
    'href': string;
    'title'?: string | null;
    'details'?: RtLinkDetails;
}
export interface RtLinkDetails {
    'className': string;
    'id': string;
    'userName': string;
    'value': string;
}
export interface RtLinkMark {
    'attrs': RtLinkAttrs;
}
export interface RtListItem {
    'children': Array<BlockNode>;
}
export interface RtMention {
    'attrs': RtMentionAttrs;
    'marks': Array<DocumentMark>;
}
export interface RtMentionAttrs {
    'className': string;
    'type': string;
    'id': string;
    'teamName': string;
    'href': string;
    'userName': string;
}
export interface RtOrderedList {
    'startNumber': number;
    'children': Array<RtListItem>;
}
export interface RtParagraph {
    'children': Array<InlineNode>;
    'textAlign': RtTextAlign;
}
export interface RtPredefinedMentionAttrs {
    'type': string;
}
export interface RtPredefinedMentionLinkDetails {
    'value': string;
}
export interface RtProfileLinkDetails {
    'id': string;
    'userName': string;
}
export interface RtProfileMentionAttrs {
    'id': string;
    'userName': string;
    'href': string;
}
export interface RtTable {
    'children': Array<RtTableRow>;
}
export interface RtTableCell {
    'children': Array<BlockNode>;
    'attrs'?: RtTableCellAttrs;
}
export interface RtTableCellAttrs {
    'rowspan': number;
    'colspan': number;
    'colwidth'?: Array<number> | null;
}
export interface RtTableHeader {
    'children': Array<BlockNode>;
    'attrs'?: RtTableCellAttrs;
}
export interface RtTableRow {
    'children': Array<RtTableRowContent>;
}
export interface RtTableRowContent {
    'className': string;
    'children': Array<BlockNode>;
    'attrs'?: RtTableCellAttrs;
}
export interface RtTeamLinkDetails {
    'id': string;
}
export interface RtTeamMentionAttrs {
    'id': string;
    'teamName': string;
    'href': string;
}
export interface RtText {
    'value': string;
    'marks': Array<DocumentMark>;
}
export declare const RtTextAlign: {
    readonly Left: "left";
    readonly Center: "center";
    readonly Right: "right";
    readonly Justify: "justify";
};
export type RtTextAlign = typeof RtTextAlign[keyof typeof RtTextAlign];
export interface RtUnfurl {
    'attrs': RtUnfurlAttrs;
    'marks': Array<DocumentMark>;
}
export interface RtUnfurlAttrs {
    'href': string;
    'title': string;
}
export interface SFUDataProducerOptions {
    'id': string;
}
export declare const SFUMediaKind: {
    readonly Video: "video";
    readonly Audio: "audio";
};
export type SFUMediaKind = typeof SFUMediaKind[keyof typeof SFUMediaKind];
export interface SFUProducerOptions {
    'id': string;
    'parameters': SFUProducerParameters;
}
export interface SFUProducerParameters {
    'mediaKind': SFUMediaKind;
    'rtpParameters': string;
    'paused': boolean;
    'closed': boolean;
}
export interface SPackageRepositoryWebhookEvent {
    'projectKey': ProjectKey;
    'repository': string;
    'repositoryType': PackageType;
    'action': PackageRepositoryEventAction;
    'packageInfo': PackageVersionRef;
}
export interface SRepoCommitsWebhookEvent {
    'projectKey': ProjectKey;
    'repository': string;
    'commit': GitCommitInfoWithChanges;
}
export interface SRepoHeadsWebhookEvent {
    'projectKey': ProjectKey;
    'repository': string;
    'changes': RepoChanges;
}
export interface SSLKeystore {
    'name': string;
    'created': string;
}
export interface SSLKeystoreEndpointAuth {
    'sslKeystore': string;
}
export interface SamlMetadataResponse {
    'className': string;
    'messages': Array<string>;
    'metadata': string;
}
export interface SamlMetadataResponseError {
    'messages': Array<string>;
}
export interface SamlMetadataResponseSuccess {
    'metadata': string;
}
export interface SecretParameterRecord {
    'id': string;
    'archived': boolean;
    'bundleId': string;
    'key': string;
    'description'?: string | null;
    'createdAt'?: string | null;
    'createdBy'?: CPrincipal;
    'updatedAt'?: string | null;
    'updatedBy'?: CPrincipal;
}
export interface SessionParticipationRecord {
    'id': string;
    'connectionId': number;
    'sessionId': string;
    'member'?: TDMemberProfile;
    'state': ParticipationState;
    'participant'?: TDCallParticipant;
    'description'?: string | null;
    'producers': Array<ProducerOptions>;
    'dataProducers': Array<DataProducerOptions>;
}
export interface SettingsValue {
    'emailNotificationsEnabled': boolean;
    'firstDayOfWeek': Weekday;
    'darkTheme'?: DarkTheme;
    'themeName'?: string | null;
    'calendarView'?: string | null;
    'notificationEmail'?: TDProfileEmail;
    'draftType'?: DraftDocumentType;
    'todoFilters'?: boolean | null;
    'defaultProject'?: PRProject;
}
export interface ShareTargetIdentifier {
    'className': string;
    'profileId': string;
    'teamId': string;
}
export interface ShareTargetIdentifierProfile {
    'profileId': string;
}
export interface ShareTargetIdentifierTeam {
    'teamId': string;
}
export interface SharingAccessType {
    'className': string;
}
export declare const SnapshotState: {
    readonly Unknown: "UNKNOWN";
    readonly Pending: "PENDING";
    readonly Completed: "COMPLETED";
    readonly Error: "ERROR";
};
export type SnapshotState = typeof SnapshotState[keyof typeof SnapshotState];
export declare const SortOrderType: {
    readonly Asc: "ASC";
    readonly Desc: "DESC";
};
export type SortOrderType = typeof SortOrderType[keyof typeof SortOrderType];
export interface SpaceNewsFeedChannel {
    'notificationDefaults'?: ChannelSpecificDefaults;
    'supportReplies'?: boolean | null;
}
export interface SprintIdentifier {
    'className': string;
    'board': BoardIdentifier;
    'id': string;
}
export interface SprintIdentifierCurrent {
    'board': BoardIdentifier;
}
export interface SprintIdentifierId {
    'id': string;
}
export interface SprintLaunch {
    'className': string;
    'moveUnresolvedIssues': boolean;
    'notifySubscribers': boolean;
    'time': string;
}
export interface SprintLaunchImmediate {
    'moveUnresolvedIssues': boolean;
    'notifySubscribers': boolean;
}
export interface SprintLaunchScheduled {
    'time': string;
    'moveUnresolvedIssues': boolean;
    'notifySubscribers': boolean;
}
export interface SprintRecord {
    'id': string;
    'archived': boolean;
    'board': BoardRecord;
    'name': string;
    'state': SprintState;
    'from': string;
    'to': string;
    'description'?: string | null;
    'launch': SprintLaunch;
}
export declare const SprintState: {
    readonly Closed: "CLOSED";
    readonly Current: "CURRENT";
    readonly Planned: "PLANNED";
};
export type SprintState = typeof SprintState[keyof typeof SprintState];
export interface SshKeyData {
    'fingerprint': string;
    'sha256fingerprint'?: string | null;
    'publicKey': string;
    'comment': string;
    'createdAt'?: string | null;
    'lastUsed'?: string | null;
    'lastResource'?: string | null;
}
export interface Sticker {
    'id': string;
    'symbol'?: string | null;
    'attachmentId'?: string | null;
    'width'?: number | null;
    'height'?: number | null;
    'variants'?: Array<StickerVariant> | null;
    'animated'?: boolean | null;
    'archived': boolean;
}
export interface StickerContentDetails {
    'sticker': Sticker;
    'pack'?: StickerPackInfo;
}
export interface StickerPackInfo {
    'id': string;
    'name': string;
    'favoriteByDefault': boolean;
    'archived': boolean;
    'private': boolean;
    'stickers': Array<Sticker>;
    'addedAt'?: string | null;
}
export interface StickerVariant {
    'id': string;
    'name'?: string | null;
    'width': number;
    'height': number;
}
export interface StringCFConstraint {
    'min'?: number | null;
    'max'?: number | null;
    'pattern'?: string | null;
    'message'?: string | null;
}
export interface StringCFValue {
    'value'?: string | null;
}
export interface StringListCFValue {
    'values': Array<string>;
}
export interface SubjectResponsibilitiesTable {
    'subjectId': string;
    'name'?: string | null;
    'reusedInProjects': Array<string>;
    'responsibilityRecords': Array<ResponsibilityRecord>;
}
export interface Subscription {
    'subjectCode': string;
    'filters': Array<SubscriptionFilter>;
    'eventTypeCodes': Array<string>;
}
export interface SubscriptionDTO {
    'id': string;
    'name': string;
    'enabled': boolean;
    'subscription': Subscription;
    'privateFeed'?: PrivateFeed;
    'application'?: ESApp;
    'requestedAuthentication'?: SubscriptionRequestedAuthorizations;
}
export interface SubscriptionFilter {
    'className': string;
    'application'?: ESApp;
    'projects': Array<PRProject>;
    'repositoryName'?: string | null;
    'branchSpec': Array<string>;
    'jobs'?: Array<string> | null;
    'project'?: PRProject;
    'repository': ProjectPackageRepository;
    'spec': RepoCommitsSubscriptionFilterSpec;
    'repositories'?: Array<string> | null;
    'targetIdentifiers'?: Array<string> | null;
    'teams': Array<TDTeam>;
    'locations': Array<TDLocation>;
    'profiles'?: Array<TDMemberProfile> | null;
    'authors': Array<TDMemberProfile>;
    'participants': Array<TDMemberProfile>;
    'pathSpec': Array<string>;
    'titleRegex': string;
    'folders'?: Array<DocumentFolder> | null;
    'documents': Array<Document>;
    'reasons': Array<AbsenceReasonRecord>;
    'namePattern': Array<string>;
    'versionPattern'?: string | null;
    'channel'?: string | null;
    'contact'?: ChatContactRecord;
}
export interface SubscriptionFilterIn {
    'className': string;
    'teams': Array<string>;
    'locations': Array<string>;
    'profiles'?: Array<ProfileIdentifier> | null;
    'project': string | null;
    'folders'?: Array<string> | null;
    'documents': Array<string>;
    'application'?: string | null;
    'repository': string;
    'namePattern': Array<string>;
    'versionPattern'?: string | null;
    'reasons': Array<string>;
    'repositories'?: Array<string> | null;
    'targetIdentifiers'?: Array<string> | null;
    'repositoryName'?: string | null;
    'branchSpec': Array<string>;
    'channel'?: string | null;
    'projects'?: Array<string> | null;
    'jobs'?: Array<string> | null;
    'authors': Array<string>;
    'participants': Array<string>;
    'pathSpec': Array<string>;
    'titleRegex': string;
    'spec': RepoCommitsSubscriptionFilterSpec;
}
export interface SubscriptionMetadata {
    'channel': M2ChannelRecord;
    'subscription': string;
    'subscriptionName': string;
}
export interface SubscriptionRequestedAuthorizations {
    'rights': Array<DTORight>;
    'rightCodes': Array<string>;
    'permissionContext'?: PermissionContextApi;
    'projects': Array<PRProject>;
}
export interface SupportProfileDTO {
    'profile': TDMemberProfile;
    'adminPermissionsGranted': boolean;
}
export interface SyncBatchInfo {
    'className': string;
    'etag': string;
    'batchSize': number;
}
export interface SyncBatchInfoSinceEtag {
    'etag': string;
    'batchSize': number;
}
export declare const SyncRecordModType: {
    readonly Created: "CREATED";
    readonly Updated: "UPDATED";
    readonly Archived: "ARCHIVED";
};
export type SyncRecordModType = typeof SyncRecordModType[keyof typeof SyncRecordModType];
export interface SyntaxMarkup {
    'type'?: SyntaxMarkupType;
    'range': TextRange;
}
export declare const SyntaxMarkupType: {
    readonly Keyword: "KEYWORD";
    readonly Comment: "COMMENT";
    readonly NumLiteral: "NUM_LITERAL";
    readonly StringLiteral: "STRING_LITERAL";
    readonly Declaration: "DECLARATION";
    readonly ParameterAssignment: "PARAMETER_ASSIGNMENT";
    readonly Identifier: "IDENTIFIER";
    readonly IdentifierType: "IDENTIFIER_TYPE";
    readonly IdentifierDeclaration: "IDENTIFIER_DECLARATION";
    readonly Key: "KEY";
    readonly Tag: "TAG";
    readonly Annotation: "ANNOTATION";
    readonly PreprocessorAnnotation: "PREPROCESSOR_ANNOTATION";
    readonly PreprocessorMessage: "PREPROCESSOR_MESSAGE";
    readonly Highlight: "HIGHLIGHT";
    readonly CodeFold: "CODE_FOLD";
    readonly Skip: "SKIP";
};
export type SyntaxMarkupType = typeof SyntaxMarkupType[keyof typeof SyntaxMarkupType];
export interface TDCallParticipant {
    'className': string;
    'app': ESApp;
    'member': TDMemberProfile;
}
export interface TDCallParticipantApp {
    'app': ESApp;
}
export interface TDCallParticipantUser {
    'member': TDMemberProfile;
}
export interface TDContactKind {
    'className': string;
    'messenger': string;
    'type': string;
}
export interface TDContactKindMessenger {
    'messenger': string;
}
export interface TDContactKindPhoneNumber {
    'type': string;
}
export interface TDLanguage {
    'id': string;
    'code': string;
    'name': string;
    'nativeName': string;
    'firstNameTitle': string;
    'lastNameTitle': string;
}
export interface TDLocation {
    'id': string;
    'name': string;
    'timezone'?: ATimeZoneWithOffset;
    'tz'?: string | null;
    'workdays'?: Array<number> | null;
    'phones': Array<string>;
    'emails': Array<string>;
    'equipment'?: Array<string> | null;
    'description'?: string | null;
    'address'?: string | null;
    'parent'?: TDLocation;
    'type': string;
    'mapId'?: string | null;
    'capacity'?: number | null;
    'channelId'?: string | null;
    'archived': boolean;
    'equipment2'?: Array<TDLocationEquipmentTypeRecord> | null;
}
export interface TDLocationEquipmentType {
    'name': string;
    'archived': boolean;
}
export interface TDLocationEquipmentTypeRecord {
    'id': string;
    'name': string;
    'archived': boolean;
}
export interface TDLocationMap {
    'id': string;
    'picture': string;
    'created': string;
    'width'?: number | null;
    'height'?: number | null;
}
export interface TDLocationMapPoint {
    'id': string;
    'mapId'?: string | null;
    'x': number;
    'y': number;
    'created': string;
    'memberLocation'?: string | null;
    'deleted'?: boolean | null;
}
export interface TDLocationWithTimeZone {
    'id': string;
    'name': string;
    'timezone': ATimeZoneWithOffset;
    'type'?: LocationType;
    'parentLocation'?: string | null;
}
export interface TDMemberInLocationMap {
    'profile': TDMemberProfile;
    'memberLocation': TDMemberLocation;
    'locationMapPoint'?: TDLocationMapPoint;
}
export interface TDMemberLocation {
    'id': string;
    'location': TDLocation;
    'locationMapPoints'?: Array<TDLocationMapPoint> | null;
    'since'?: string | null;
    'till'?: string | null;
    'member': TDMemberProfile;
    'archived': boolean;
}
export interface TDMemberProfile {
    'id': string;
    'username': string;
    'name': TDProfileName;
    'speaksEnglish': boolean;
    'smallAvatar'?: string | null;
    'avatar'?: string | null;
    'profilePicture'?: string | null;
    'languages': Array<TDProfileLanguage>;
    'archived': boolean;
    'notAMember': boolean;
    'joined'?: string | null;
    'leftAt'?: string | null;
    'external'?: boolean | null;
    'externalLight'?: boolean | null;
    'locationHistory': Array<TDMemberLocation>;
    'externalId'?: string | null;
    'memberships': Array<TDMembership>;
    'unapprovedMemberships'?: Array<TDMembership> | null;
    'onboardingRequired': boolean;
    'showBannerOnLandingPage'?: boolean | null;
    'showBannerOnProjectPage'?: boolean | null;
    'showBannerOnTeamDirectoryHomePage'?: boolean | null;
    'left'?: string | null;
    'absences': Array<AbsenceRecord>;
    'status': TwoFactorAuthenticationStatus;
    'emails': Array<TDProfileEmail>;
    'phones': Array<string>;
    'messengers': Array<string>;
    'links': Array<string>;
    'membershipHistory': Array<TDMembership>;
    'folderWithChildren': DocumentFolderWithChildren;
    'locations': Array<TDMemberLocation>;
    'topics': Array<Topic>;
    'managers': Array<TDMemberProfile>;
    'logins': Array<ESProfileLogin>;
    'about'?: string | null;
    'avatarCropSquare'?: AvatarCropSquare;
    'gender'?: Gender;
    'birthday'?: string | null;
    'customFields': {
        [key: string]: CFValue | undefined;
    };
}
export interface TDMemberWithTeam {
    'profile': TDMemberProfile;
    'teams': Array<TDTeam>;
}
export interface TDMembership {
    'id': string;
    'member': TDMemberProfile;
    'team': TDTeam;
    'role': TDRole;
    'lead': boolean;
    'manager'?: TDMemberProfile;
    'since'?: string | null;
    'till'?: string | null;
    'activeSince'?: string | null;
    'activeTill'?: string | null;
    'requiresApproval': boolean;
    'archived': boolean;
    'editFor'?: TDMembership;
    'pendingEdit'?: TDMembership;
    'approver'?: TDMemberProfile;
    'customFields': {
        [key: string]: CFValue | undefined;
    };
}
export interface TDMergedEvent {
    'events': Array<TDMergedEventEventsInner>;
    'profile': TDMemberProfile;
}
export interface TDMergedEventEventsInner {
    'first': TDMembership;
    'second': number;
}
export interface TDProfileEmail {
    'id': string;
    'email': string;
    'blocked'?: boolean | null;
}
export interface TDProfileLanguage {
    'name'?: TDProfileName;
    'language': TDLanguage;
    'languageCode'?: string | null;
}
export interface TDProfileName {
    'firstName': string;
    'lastName': string;
}
export interface TDProfileWorkingDays {
    'profile': TDMemberProfile;
    'workingDays': TDWorkingDays;
}
export interface TDRole {
    'id': string;
    'name': string;
    'parent'?: TDRole;
    'archived': boolean;
}
export interface TDStats {
    'totalMembers': number;
    'teams': Array<BGStatsTeamsInner>;
    'roles': Array<TDStatsRolesInner>;
    'locations': Array<BGStatsLocationsInner>;
}
export interface TDStatsRolesInner {
    'first': TDRole;
    'second': number;
}
export interface TDTeam {
    'id': string;
    'name': string;
    'description': string;
    'parent'?: TDTeam;
    'emails'?: Array<string> | null;
    'channelId'?: string | null;
    'archived': boolean;
    'disbanded'?: boolean | null;
    'disbandedAt'?: string | null;
    'externalId'?: string | null;
    'defaultManager'?: TDMemberProfile;
    'memberships': Array<TDMembership>;
    'customFields': {
        [key: string]: CFValue | undefined;
    };
}
export interface TDWorkingDays {
    'id': string;
    'dateStart'?: string | null;
    'dateEnd'?: string | null;
    'workingDaysSpec': WorkingDaysSpec;
}
export interface TabIndentedLinesBodyIn {
    'text': string;
}
export interface TargetCFInputValue {
    'target'?: TargetIdentifier;
}
export interface TargetCFValue {
    'target'?: DeployTargetRecord;
}
export interface TargetIdentifier {
    'className': string;
    'id': string;
    'key': string;
}
export interface TargetIdentifierId {
    'id': string;
}
export interface TargetIdentifierKey {
    'key': string;
}
export interface TeamAddedItemDetails {
    'team': TDTeam;
}
export interface TeamCFInputValue {
    'team'?: string | null;
}
export interface TeamCFValue {
    'team'?: TDTeam;
}
export interface TeamDirectoryInvitationLinksGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<InvitationLink>;
}
export interface TeamDirectoryInvitationLinksGet200Response1 {
    'first': string;
    'second': InvitationLink;
}
export interface TeamDirectoryInvitationLinksGetRequest {
    'name': string;
    'expiresAt': string;
    'inviteeLimit': number;
    'team'?: string | null;
    'role'?: string | null;
    'project'?: ProjectIdentifier;
    'projectRole'?: ProjectTeamRole;
    'globalRole'?: GlobalRole;
}
export interface TeamDirectoryInvitationLinksInvitationLinkIdDeleteRequest {
    'name'?: string | null;
    'expiresAt'?: string | null;
    'inviteeLimit'?: number | null;
    'team'?: string | null;
    'position'?: string | null;
    'project'?: ProjectIdentifier;
    'projectRole'?: ProjectTeamRole;
}
export interface TeamDirectoryInvitationsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<Invitation>;
}
export interface TeamDirectoryInvitationsGetRequest {
    'inviteeEmail': string;
    'inviteeFirstName'?: string | null;
    'inviteeLastName'?: string | null;
    'team'?: string | null;
    'role'?: string | null;
    'project'?: ProjectIdentifier;
    'projectRole'?: ProjectTeamRole;
    'globalRole'?: GlobalRole;
}
export interface TeamDirectoryInvitationsIdDeleteRequest {
    'inviteeEmail'?: string | null;
    'inviteeFirstName'?: string | null;
    'inviteeLastName'?: string | null;
    'team'?: string | null;
    'role'?: string | null;
    'project'?: ProjectIdentifier;
    'projectRole'?: ProjectTeamRole;
}
export interface TeamDirectoryLocationMapMemberPointsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDMemberInLocationMap>;
}
export interface TeamDirectoryLocationMapMemberPointsGetRequest {
    'memberLocationId': string;
    'x': number;
    'y': number;
    'mapId': string;
}
export interface TeamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest {
    'x'?: number | null;
    'y'?: number | null;
}
export interface TeamDirectoryLocationsGetRequest {
    'name': string;
    'timezone'?: string | null;
    'workdays'?: Array<number> | null;
    'phones'?: Array<string> | null;
    'emails'?: Array<string> | null;
    'equipment'?: Array<string> | null;
    'description'?: string | null;
    'address'?: string | null;
    'type'?: string | null;
    'parentId'?: string | null;
    'capacity'?: number | null;
}
export interface TeamDirectoryLocationsIdDeleteRequest {
    'name'?: string | null;
    'timezone'?: string | null;
    'customWorkdays'?: boolean | null;
    'workdays'?: Array<number> | null;
    'phones'?: Array<string> | null;
    'emails'?: Array<string> | null;
    'equipment'?: Array<string> | null;
    'description'?: string | null;
    'address'?: string | null;
    'type'?: string | null;
    'parentId'?: string | null;
    'mapId'?: string | null;
    'capacity'?: number | null;
}
export interface TeamDirectoryLocationsIdMapGetRequest {
    'mapPictureId': string;
}
export interface TeamDirectoryMemberLocationsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDMemberLocation>;
}
export interface TeamDirectoryMemberLocationsGetRequest {
    'member': ProfileIdentifier;
    'location': string;
    'since'?: string | null;
    'till'?: string | null;
    'previousLocation'?: string | null;
}
export interface TeamDirectoryMemberLocationsMemberLocationIdDeleteRequest {
    'location'?: string | null;
    'since'?: string | null;
    'till'?: string | null;
}
export interface TeamDirectoryMembershipEventsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDMergedEvent>;
}
export interface TeamDirectoryMembershipsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDMembership>;
}
export interface TeamDirectoryMembershipsGetRequest {
    'member': ProfileIdentifier;
    'teamId': string;
    'roleId': string;
    'lead'?: boolean;
    'manager'?: ProfileIdentifier;
    'activeSince'?: string | null;
    'activeTill'?: string | null;
    'previousMembershipId'?: string | null;
    'requiresApproval'?: boolean;
    'customFieldValues'?: Array<CustomFieldInputValue> | null;
}
export interface TeamDirectoryMembershipsMembershipIdDeleteRequest {
    'teamId'?: string | null;
    'roleId'?: string | null;
    'lead'?: boolean | null;
    'manager'?: ProfileIdentifier;
    'activeSince'?: string | null;
    'activeTill'?: string | null;
    'requiresApproval'?: boolean;
    'customFieldValues'?: Array<CustomFieldInputValue> | null;
}
export interface TeamDirectoryMembershipsMembershipIdRequestRevokePatchRequest {
    'till': string;
}
export interface TeamDirectoryMembershipsRequestsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TeamDirectoryMembershipsRequestsGet200ResponseDataInner>;
}
export interface TeamDirectoryMembershipsRequestsGet200ResponseDataInner {
    'first': TDMemberProfile;
    'second': TDMembership;
}
export interface TeamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest {
    'approved': boolean;
}
export interface TeamDirectoryProfilesDashboardsDashboardGetRequest {
    'data': Array<Array<DashboardItemData>>;
}
export interface TeamDirectoryProfilesGetRequest {
    'username': string;
    'firstName': string;
    'lastName': string;
    'emails'?: Array<string>;
    'phones'?: Array<string>;
    'birthday'?: string | null;
    'about'?: string | null;
    'messengers'?: Array<string>;
    'links'?: Array<string>;
    'notAMember'?: boolean;
    'joined'?: string | null;
    'left'?: string | null;
    'leftAt'?: string | null;
    'speaksEnglish'?: boolean | null;
    'pictureAttachmentId'?: string | null;
    'avatarCropSquare'?: AvatarCropSquare;
    'customFieldValues'?: Array<CustomFieldInputValue>;
    'externalId'?: string | null;
    'location'?: string | null;
    'guest'?: boolean | null;
    'guestType'?: GuestType;
    'project'?: ProjectIdentifier;
    'projectRole'?: ProjectTeamRole;
}
export interface TeamDirectoryProfilesProfile2FaTotpConfirmPostRequest {
    'code': number;
}
export interface TeamDirectoryProfilesProfileApplicationPasswordsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ESApplicationPassword>;
}
export interface TeamDirectoryProfilesProfileApplicationPasswordsGet200Response1 {
    'first': ESApplicationPassword;
    'second': string;
}
export interface TeamDirectoryProfilesProfileApplicationPasswordsGetRequest {
    'name': string;
    'scope': string;
}
export interface TeamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest {
    'name'?: string | null;
    'scope'?: string | null;
}
export interface TeamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest {
    'name'?: string;
    'description'?: string | null;
}
export interface TeamDirectoryProfilesProfileConvertToGuestPatchRequest {
    'dryrun': boolean;
    'guestType'?: GuestType;
}
export interface TeamDirectoryProfilesProfileConvertToMemberPatchRequest {
    'dryrun': boolean;
}
export interface TeamDirectoryProfilesProfileDeleteRequest {
    'username'?: string | null;
    'firstName'?: string | null;
    'lastName'?: string | null;
    'emails'?: Array<string> | null;
    'phones'?: Array<string> | null;
    'birthday'?: string | null;
    'about'?: string | null;
    'messengers'?: Array<string> | null;
    'links'?: Array<string> | null;
    'notAMember'?: boolean | null;
    'joined'?: string | null;
    'left'?: string | null;
    'leftAt'?: string | null;
    'speaksEnglish'?: boolean | null;
    'pictureAttachmentId'?: string | null;
    'avatarCropSquare'?: AvatarCropSquare;
    'customFieldValues'?: Array<CustomFieldInputValue> | null;
    'externalId'?: string | null;
}
export interface TeamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest {
    'comment'?: string;
}
export interface TeamDirectoryProfilesProfileGpgKeysGetRequest {
    'key': string;
    'comment'?: string;
}
export interface TeamDirectoryProfilesProfileNavBarMenuItemsGetRequest {
    'item': string;
    'enabled': boolean;
}
export interface TeamDirectoryProfilesProfileNavBarProjectsGetRequest {
    'project': ProjectIdentifier;
}
export interface TeamDirectoryProfilesProfileNotificationSettingsGetRequest {
    'emailNotificationsEnabled'?: boolean | null;
    'notificationEmail'?: string | null;
    'pushNotificationEnabled'?: boolean | null;
    'desktopInactivityTimeout'?: number | null;
}
export interface TeamDirectoryProfilesProfilePermanentTokensGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<ESPersonalToken>;
}
export interface TeamDirectoryProfilesProfilePermanentTokensGet200Response1 {
    'first': ESPersonalToken;
    'second': string;
}
export interface TeamDirectoryProfilesProfileReactivatePatchRequest {
    'joined'?: string | null;
}
export interface TeamDirectoryProfilesProfileSettingsGetRequest {
    'darkTheme'?: DarkTheme;
    'themeName'?: string | null;
    'firstDayOfWeek'?: Weekday;
    'draftType'?: DraftDocumentType;
    'typographySettings'?: TypographySettings;
    'todoFilters'?: boolean | null;
    'calendarView'?: string | null;
    'emailNotificationsEnabled'?: boolean | null;
    'notificationEmail'?: string | null;
    'preferredLanguage'?: string | null;
    'defaultProject'?: ProjectIdentifier;
}
export interface TeamDirectoryProfilesProfileSpokenLanguagesGetRequest {
    'language': string;
    'firstName'?: string | null;
    'lastName'?: string | null;
}
export interface TeamDirectoryProfilesProfileWorkingDaysGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDWorkingDays>;
}
export interface TeamDirectoryProfilesProfileWorkingDaysGetRequest {
    'dateStart'?: string | null;
    'dateEnd'?: string | null;
    'workingDaysSpec': WorkingDaysSpec;
}
export interface TeamDirectoryProfilesWorkingDaysGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TDProfileWorkingDays>;
}
export interface TeamDirectoryRolesGetRequest {
    'name': string;
    'parentId'?: string | null;
}
export interface TeamDirectoryRolesIdDeleteRequest {
    'name'?: string | null;
    'parentId'?: string | null;
}
export interface TeamDirectoryTeamsGetRequest {
    'name': string;
    'description'?: string | null;
    'emails'?: Array<string> | null;
    'parentId'?: string | null;
    'defaultManager'?: string | null;
    'customFieldValues'?: Array<CustomFieldInputValue> | null;
    'externalId'?: string | null;
}
export interface TeamDirectoryTeamsIdDeleteRequest {
    'name'?: string;
    'description'?: string | null;
    'emails'?: Array<string> | null;
    'parentId'?: string | null;
    'defaultManager'?: string | null;
    'customFieldValues'?: Array<CustomFieldInputValue> | null;
    'externalId'?: string | null;
}
export interface TeamEvent {
    'meta': KMetaMod;
    'team': TDTeam;
}
export interface TeamIdentifier {
    'className': string;
    'externalId': string;
    'id': string;
    'name': string;
}
export interface TeamIdentifierExternalId {
    'externalId': string;
}
export interface TeamIdentifierId {
    'id': string;
}
export interface TeamIdentifierName {
    'name': string;
}
export interface TeamMembershipEvent {
    'meta': KMetaMod;
    'membership': TDMembership;
    'team'?: TeamMembershipEventTeam | null;
    'role'?: TeamMembershipEventRole | null;
    'lead'?: AbsenceWebhookEventAvailable | null;
    'manager'?: ApplicationWebhookEventOwner | null;
    'since'?: AutomationJobEventStartTime | null;
    'till'?: AutomationJobEventStartTime | null;
    'deleted'?: AbsenceWebhookEventAvailable | null;
    'approvedBy'?: ApplicationWebhookEventOwner | null;
}
export interface TeamMembershipEventRole {
    'old': TDRole;
    'new': TDRole;
}
export interface TeamMembershipEventTeam {
    'old': TDTeam;
    'new': TDTeam;
}
export interface TeamMembershipIdentifier {
    'className': string;
    'id': string;
}
export interface TeamMembershipIdentifierId {
    'id': string;
}
export interface TeamPermissionTarget {
    'team': string;
}
export interface TextDocument {
    'id': string;
    'resetCounter': number;
    'version'?: number | null;
    'type': DraftDocumentType;
    'text': string;
    'model'?: string | null;
    'attachments'?: Array<AttachmentInfo> | null;
}
export interface TextDocumentBodyConvertTypeIn {
    'type': DraftDocumentType;
}
export interface TextDocumentBodyCreateInBase {
    'className': string;
    'docContent': TextDocumentContent;
}
export interface TextDocumentBodyCreateTypedIn {
    'docContent': TextDocumentContent;
}
export interface TextDocumentBodyInfo {
    'textDocument': TextDocumentRecord;
}
export interface TextDocumentBodyUpdateIn {
    'docContent': TextDocumentContent;
}
export interface TextDocumentBodyUpdateInBase {
    'className': string;
    'docContent': TextDocumentContent;
    'type': DraftDocumentType;
}
export interface TextDocumentContent {
    'className': string;
    'markdown': string;
}
export interface TextDocumentRecord {
    'id': string;
    'archived': boolean;
    'type': DraftDocumentType;
}
export interface TextRange {
    'start': number;
    'length': number;
}
export interface ThrottledLogin {
    'login': string;
    'throttledUntil': string;
}
export declare const Tier: {
    readonly Free: "Free";
    readonly Team: "Team";
    readonly Organization: "Organization";
    readonly Classroom: "Classroom";
    readonly Beta: "Beta";
    readonly Enterprise: "Enterprise";
};
export type Tier = typeof Tier[keyof typeof Tier];
export interface TimeInterval {
    'since': TimeOfDay;
    'till': TimeOfDay;
}
export interface TimeOfDay {
    'hours': number;
    'minutes': number;
    'seconds'?: number | null;
}
export interface TimeTrackingItem {
    'id': string;
    'user': TDMemberProfile;
    'project': PRProject;
    'date': string;
    'duration': string;
    'description'?: string | null;
    'subjectType': TimeTrackingSubjectType;
    'subject': string;
}
export interface TimeTrackingItemsGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TimeTrackingItem>;
}
export interface TimeTrackingItemsGetRequest {
    'subject': TimeTrackingSubjectIdentifier;
    'userId': ProfileIdentifier;
    'date': string;
    'duration': number;
    'description'?: string | null;
}
export interface TimeTrackingItemsItemIdDeleteRequest {
    'userId'?: ProfileIdentifier;
    'date'?: string | null;
    'duration'?: number | null;
    'description'?: string | null;
}
export interface TimeTrackingSubjectIdentifier {
    'className': string;
    'issue': IssueIdentifier;
}
export interface TimeTrackingSubjectIdentifierIssue {
    'issue': IssueIdentifier;
}
export declare const TimeTrackingSubjectType: {
    readonly Issue: "ISSUE";
};
export type TimeTrackingSubjectType = typeof TimeTrackingSubjectType[keyof typeof TimeTrackingSubjectType];
export interface TodayBillingReport {
    'plan': PurchasedBillingPlan;
    'activeUsers': number;
    'userUsage': number;
    'userCost': number;
    'activeGuests'?: number | null;
    'paidActiveGuests'?: number | null;
    'freeActiveGuests'?: number | null;
    'guestCost'?: number | null;
    'storageAllocationB'?: number | null;
    'storageTotalUsage': number;
    'storageCost': number;
    'bandwidthTotalUsage': number;
    'bandwidthCost': number;
    'ciUsage': number;
    'ciCost': number;
    'appUsage': number;
    'chatUsage': number;
    'totalCost': number;
}
export interface TodoContent {
    'originalText': string;
    'link'?: string | null;
    'kind'?: TodoItemContentKind;
    'principal'?: CPrincipal;
    'time'?: number | null;
    'text': string;
    'extId'?: string | null;
    'unfurls'?: Array<Unfurl> | null;
}
export interface TodoGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<TodoItemRecord>;
}
export interface TodoGetRequest {
    'text': string;
    'dueDate'?: string | null;
}
export interface TodoIdDeleteRequest {
    'text'?: string | null;
    'dueDate'?: string | null;
    'open'?: boolean | null;
}
export interface TodoItemContent {
    'className': string;
    'text': string;
    'originalText': string;
    'link'?: string | null;
    'kind'?: TodoItemContentKind;
    'principal'?: CPrincipal;
    'time'?: number | null;
    'extId'?: string | null;
    'unfurls'?: Array<Unfurl> | null;
    'mdMarkup': MdMarkup;
}
export declare const TodoItemContentKind: {
    readonly None: "NONE";
    readonly Regular: "REGULAR";
    readonly Message: "MESSAGE";
    readonly Issue: "ISSUE";
    readonly Blog: "BLOG";
    readonly Document: "DOCUMENT";
    readonly Meeting: "MEETING";
    readonly CodeReview: "CODE_REVIEW";
};
export type TodoItemContentKind = typeof TodoItemContentKind[keyof typeof TodoItemContentKind];
export interface TodoItemContentMdText {
    'text': string;
    'mdMarkup': MdMarkup;
}
export interface TodoItemContentText {
    'text': string;
}
export interface TodoItemRecord {
    'id': string;
    'archived': boolean;
    'created': string;
    'updated': string;
    'content': TodoItemContent;
    '_status': string;
    'dueDate'?: string | null;
    'dueTime'?: string | null;
    'notificationRequired'?: boolean | null;
}
export interface TodoItemUnfurlContext {
    'userId': string;
    'todoItemId': string;
}
export interface ToggleableProjectFeaturePinnedItem {
    'value': ProjectFeaturePinnedItem;
    'isEnabled': boolean;
}
export interface ToggleableProjectFeaturePins {
    'feature': ProjectFeature;
    'items': Array<ToggleableProjectFeaturePinnedItem>;
}
export interface TopLevelPageUiExtensionApi {
    'displayName': string;
    'description'?: string | null;
    'uniqueCode': string;
    'iframeUrl'?: string | null;
}
export interface TopLevelPageUiExtensionIn {
    'displayName': string;
    'description'?: string | null;
    'uniqueCode': string;
    'iframeUrl'?: string | null;
}
export interface Topic {
    'id': string;
    'archived': boolean;
    'name': string;
    'parent'?: Topic;
    'project'?: PRProject;
    'responsible': Array<TDMemberProfile>;
}
export interface TrackedBranchesInReview {
    'repository': string;
    'branches': Array<BranchInfo>;
}
export interface TrustedCertificate {
    'id': string;
    'alias': string;
    'data': string;
    'info': CertificateInfo;
    'archived': boolean;
}
export interface TrustedCertificatesGetRequest {
    'alias': string;
    'data': string;
    'archived': boolean;
}
export interface TrustedCertificatesIdDeleteRequest {
    'alias'?: string | null;
    'data'?: string | null;
    'archived'?: boolean | null;
}
export interface TwoFactorAuthenticationRequirement {
    'className': string;
}
export interface TwoFactorAuthenticationSecret {
    'secretKey': string;
    'qrCode': QRCode;
    'scratchCodes': Array<number>;
}
export declare const TwoFactorAuthenticationStatus: {
    readonly NotSetup: "NOT_SETUP";
    readonly Inactive: "INACTIVE";
    readonly Active: "ACTIVE";
};
export type TwoFactorAuthenticationStatus = typeof TwoFactorAuthenticationStatus[keyof typeof TwoFactorAuthenticationStatus];
export interface TypographySettings {
    'fontLigaturesInRegularText'?: boolean | null;
    'fontLigaturesInCodeSnippets'?: boolean | null;
    'limitLineLengthInChatMessages'?: boolean | null;
}
export interface UAJetBrainsUserAgreement {
    'content': string;
    'version': UAVersion;
    'updated': string;
}
export interface UAJetBrainsUserAgreementStatus {
    'className': string;
    'profile': TDMemberProfile;
    'version': UAVersion;
    'accepted': string;
}
export interface UAJetBrainsUserAgreementStatusAccepted {
    'profile': TDMemberProfile;
    'version': UAVersion;
    'accepted': string;
}
export interface UAJetBrainsUserAgreementStatusNotAccepted {
    'profile': TDMemberProfile;
}
export interface UAUserAgreement {
    'id': string;
    'content': string;
    'version': UAVersion;
    'updated': string;
}
export interface UAUserAgreementStatus {
    'className': string;
    'profile': TDMemberProfile;
    'userAgreement': UAUserAgreement;
    'accepted': string;
}
export interface UAUserAgreementStatusAccepted {
    'profile': TDMemberProfile;
    'userAgreement': UAUserAgreement;
    'accepted': string;
}
export interface UAUserAgreementStatusNotAccepted {
    'profile': TDMemberProfile;
}
export interface UAVersion {
    'major': number;
    'minor': number;
}
export interface Unfurl {
    'title': string;
    'link': string;
    'text': string;
    'details'?: UnfurlDetails;
    'sitename'?: string | null;
    'image'?: string | null;
    'imageMime'?: string | null;
    'imageWidth'?: number | null;
    'imageHeight'?: number | null;
    'video'?: string | null;
    'videoIFrame'?: string | null;
    'videoMime'?: string | null;
    'videoHeight'?: number | null;
    'videoWidth'?: number | null;
    'favicon'?: string | null;
    'alwaysInline'?: boolean | null;
}
export interface UnfurlActionPayload {
    'actionId': string;
    'actionValue': string;
    'link': string;
    'context': ApplicationUnfurlContext;
    'clientId': string;
    'userId': string;
    'verificationToken'?: string | null;
}
export interface UnfurlAttachment {
    'unfurl': Unfurl;
    'id': string;
    'onlyVisibleToYou'?: boolean | null;
}
export declare const UnfurlDateTimeFormat: {
    readonly RelativeToNow: "RELATIVE_TO_NOW";
    readonly Full: "FULL";
    readonly TimeOnly: "TIME_ONLY";
    readonly DateOnly: "DATE_ONLY";
};
export type UnfurlDateTimeFormat = typeof UnfurlDateTimeFormat[keyof typeof UnfurlDateTimeFormat];
export declare const UnfurlDateTimeStyle: {
    readonly Regular: "REGULAR";
    readonly Secondary: "SECONDARY";
    readonly Strikethrough: "STRIKETHROUGH";
    readonly Bold: "BOLD";
    readonly Code: "CODE";
};
export type UnfurlDateTimeStyle = typeof UnfurlDateTimeStyle[keyof typeof UnfurlDateTimeStyle];
export interface UnfurlDeployTargetDetails {
    'targetRef': DeployTargetRecord;
    'targetName'?: string | null;
    'showLinkIcon'?: boolean | null;
    'skipDetailsRender'?: boolean | null;
}
export interface UnfurlDeploymentDetails {
    'deploymentRef': DeploymentRecord;
    'showLinkIcon'?: boolean | null;
    'showDetails'?: boolean | null;
    'showStatus'?: boolean | null;
}
export interface UnfurlDetails {
    'className': string;
    'icon'?: string | null;
    'title': string;
    'image': ImageAttachment;
    'targetRef': DeployTargetRecord;
    'targetName'?: string | null;
    'showLinkIcon'?: boolean | null;
    'skipDetailsRender'?: boolean | null;
    'session': CallSession;
    'message': string;
    'inlineUnfurls'?: Array<AttachmentInfo> | null;
    'id': string;
    'channelId'?: string | null;
    'text': string;
    'details'?: ArticleDetailsRecord;
    'author': GitAuthorInfo;
    'created': string;
    'time': number;
    'attachments'?: Array<AttachmentInfo> | null;
    'mentions'?: Array<EntityMention> | null;
    'meeting': DTOMeeting;
    'compact'?: boolean | null;
    'jobId': string;
    'jobName': string;
    'projectRef': PRProject;
    'repoName': string;
    'jobExecutionDisplayStatusFilter'?: JobExecutionDisplayStatus;
    'jobTriggerFilter'?: JobTriggerType;
    'branch'?: Branch;
    'jobExecutionId': string;
    'article': ArticleRecord;
    'content'?: ArticleContentRecord;
    'channel': string;
    'header': MCMessage;
    'headerAttachments': Array<AttachmentInfo>;
    'lines': Array<CodeLine>;
    'selectedLineIndex': number;
    'selectedLinesCount': number;
    'sourceBranch'?: string | null;
    'targetBranch'?: string | null;
    'anchor': CodeSnippetAnchor;
    'team': TDTeam;
    'strikeThrough': boolean | null;
    'utcMilliseconds': number;
    'params'?: DateTimeViewParams;
    'role': TDRole;
    'profile': TDMemberProfile;
    'draft': string;
    'document': string;
    'project': PRProject;
    'repository': string;
    'repositoryId'?: string | null;
    'commitId': string;
    'messageUnfurls'?: CommitMessageUnfurlsRecord;
    'commitDate': string;
    'authorDate'?: string | null;
    'authorProfile'?: TDMemberProfile;
    'hideAuthorAndDate'?: boolean | null;
    'withBranchTags'?: boolean | null;
    'reviewId'?: string | null;
    'repoRef': ProjectPackageRepository;
    'packageName'?: string | null;
    'version'?: string | null;
    'base'?: string | null;
    'preview'?: string | null;
    'version2'?: string | null;
    'base2'?: string | null;
    'preview2'?: string | null;
    'status': IssueStatus;
    'snapshotId': string;
    'baseSnapshotId': string;
    'branchHead': string;
    'deleted': boolean;
    'isDefault'?: boolean | null;
    'deploymentRef': DeploymentRecord;
    'showDetails'?: boolean | null;
    'showStatus'?: boolean | null;
    'location': TDLocation;
    'since': number;
    'till': number;
    'sprint': SprintRecord;
    'removed'?: boolean | null;
    'importTransaction': ImportTransactionRecord;
    'topic': Topic;
    'textBefore': string;
    'textAfter': string;
    'projectId': string;
    'issueId': string;
    'name': string;
    'root': boolean;
    'folder'?: DocumentFolder;
    'checklist': Checklist;
    'app': ESApp;
    'tag': PlanningTag;
    'textSize'?: MessageTextSize;
    'review': CodeReviewRecord;
    'withIcon'?: boolean | null;
    'withBranchPair': boolean;
    'defaultBranchInRepo'?: string | null;
    'hideIfCannotResolve'?: boolean | null;
    'reviewState'?: CodeReviewState;
    'isMerged'?: boolean | null;
    'issue': Issue;
    'contactKey': string;
}
export interface UnfurlDetailsApplication {
    'app': ESApp;
}
export interface UnfurlDetailsArticle {
    'article': ArticleRecord;
    'content'?: ArticleContentRecord;
    'channel'?: ArticleChannelRecord;
    'details'?: ArticleDetailsRecord;
}
export interface UnfurlDetailsChat {
    'channel': string;
    'title': string;
}
export interface UnfurlDetailsChatLink {
    'contactKey': string;
    'title': string;
}
export interface UnfurlDetailsChecklist {
    'checklist': Checklist;
}
export interface UnfurlDetailsCodeDiffSnippet {
    'header': MCMessage;
    'headerAttachments': Array<AttachmentInfo>;
    'lines': Array<InlineDiffLine>;
    'selectedLineIndex': number;
    'selectedLinesCount': number;
    'sourceBranch'?: string | null;
    'targetBranch'?: string | null;
}
export interface UnfurlDetailsCodeReview {
    'review': CodeReviewRecord;
    'withIcon'?: boolean | null;
    'withBranchPair': boolean;
    'defaultBranchInRepo'?: string | null;
    'hideIfCannotResolve'?: boolean | null;
    'reviewState'?: CodeReviewState;
    'isMerged'?: boolean | null;
}
export interface UnfurlDetailsCodeSnippet {
    'anchor': CodeSnippetAnchor;
    'lines': Array<CodeLine>;
}
export interface UnfurlDetailsCommit {
    'project': PRProject;
    'repository': string;
    'repositoryId'?: string | null;
    'commitId': string;
    'message': string;
    'messageUnfurls'?: CommitMessageUnfurlsRecord;
    'commitDate': string;
    'authorDate'?: string | null;
    'author': GitAuthorInfo;
    'authorProfile'?: TDMemberProfile;
    'hideAuthorAndDate'?: boolean | null;
    'withBranchTags'?: boolean | null;
    'reviewId'?: string | null;
}
export interface UnfurlDetailsDateTime {
    'utcMilliseconds': number;
    'params'?: DateTimeViewParams;
}
export interface UnfurlDetailsDateTimeRange {
    'since': number;
    'till': number;
    'params'?: DateTimeViewParams;
}
export interface UnfurlDetailsDraft {
    'draft': string;
    'title': string;
    'document'?: Document;
    'strikeThrough'?: boolean | null;
}
export interface UnfurlDetailsFolder {
    'name': string;
    'root': boolean;
    'folder'?: DocumentFolder;
}
export interface UnfurlDetailsImage {
    'icon'?: string | null;
    'title': string;
    'image': ImageAttachment;
}
export interface UnfurlDetailsIssue {
    'issue': Issue;
    'strikeThrough'?: boolean | null;
    'compact'?: boolean | null;
}
export interface UnfurlDetailsIssueId {
    'projectId': string;
    'issueId': string;
}
export interface UnfurlDetailsIssueImportTransaction {
    'importTransaction': ImportTransactionRecord;
}
export interface UnfurlDetailsIssueStatus {
    'status': IssueStatus;
}
export interface UnfurlDetailsIssueTag {
    'tag': PlanningTag;
    'strikeThrough': boolean;
    'textSize'?: MessageTextSize;
}
export interface UnfurlDetailsIssueTopic {
    'topic': Topic;
    'strikeThrough': boolean;
}
export interface UnfurlDetailsLocation {
    'location': TDLocation;
    'strikeThrough'?: boolean | null;
}
export interface UnfurlDetailsMC {
    'message': MCMessage;
    'inlineUnfurls'?: Array<AttachmentInfo> | null;
}
export interface UnfurlDetailsMeeting {
    'meeting': DTOMeeting;
    'compact'?: boolean | null;
}
export interface UnfurlDetailsPackageDetails {
    'repoRef': ProjectPackageRepository;
    'packageName'?: string | null;
    'version'?: string | null;
}
export interface UnfurlDetailsProfile {
    'profile': TDMemberProfile;
    'strikeThrough'?: boolean | null;
}
export interface UnfurlDetailsProject {
    'project': PRProject;
    'strikeThrough'?: boolean | null;
}
export interface UnfurlDetailsRepositoryBranch {
    'project': PRProject;
    'repository': string;
    'branchHead': string;
    'deleted': boolean;
    'isDefault'?: boolean | null;
}
export interface UnfurlDetailsReviewDescriptionDiff {
    'snapshotId': string;
    'baseSnapshotId': string;
}
export interface UnfurlDetailsRole {
    'role': TDRole;
}
export interface UnfurlDetailsShortCommit {
    'project': PRProject;
    'repository': string;
    'commitId': string;
    'message': string;
    'strikeThrough': boolean;
}
export interface UnfurlDetailsSnapshotDiff {
    'snapshotId': string;
    'baseSnapshotId': string;
}
export interface UnfurlDetailsSprint {
    'project': PRProject;
    'sprint': SprintRecord;
    'removed'?: boolean | null;
}
export interface UnfurlDetailsTeam {
    'team': TDTeam;
    'strikeThrough'?: boolean | null;
}
export interface UnfurlDetailsTextDiff {
    'textBefore': string;
    'textAfter': string;
}
export interface UnfurlsBlockListEntry {
    'link': string;
    'prefix': boolean;
    'global': boolean;
}
export interface UnfurlsBlockUnfurlPostRequest {
    'link': string;
    'wholeHost': boolean;
}
export interface UnfurlsCheckBlockedPostRequest {
    'link': string;
}
export interface UnfurlsListBlockedGet200Response {
    'next': string;
    'totalCount'?: number | null;
    'data': Array<UnfurlsBlockListEntry>;
}
export interface UpdateErrorDTO {
    'message'?: string | null;
    'errorType': WorkerError;
}
export interface UploadsPostRequest {
    'storagePrefix': string;
    'mediaType'?: string | null;
}
export interface UrlCFValue {
    'href'?: string | null;
}
export interface VaultConnectionRecord {
    'id': string;
    'projectId': string;
    'url': string;
    'name'?: string | null;
    'namespace'?: string | null;
    'vaultNamespace'?: string | null;
    'appRoleEndpointPath': string;
    'appRoleId': string;
    'createdAt': string;
    'createdBy': CPrincipal;
    'updatedAt': string;
    'updatedBy': CPrincipal;
    'deleted': boolean;
}
export interface VcsCFScope {
    'className': string;
    'project': PRProject;
}
export interface VcsCFScopeInput {
    'className': string;
    'projectId': string;
}
export interface VcsCFScopeInputProject {
    'projectId': string;
}
export interface VcsCFScopeProject {
    'project': PRProject;
}
export interface VcsCommitCFInputValue {
    'commit'?: CFCommitIdentifier;
}
export interface VcsCommitCFParameters {
    'vcsCFScope': VcsCFScope;
}
export interface VcsCommitCFParametersInput {
    'vcsCFScope': VcsCFScopeInput;
}
export interface VcsCommitCFValue {
    'commit'?: CFCommitInfoBase;
}
export interface VcsCommitListCFInputValue {
    'commits': Array<CFCommitIdentifier>;
}
export interface VcsCommitListCFValue {
    'commits': Array<CFCommitInfoBase>;
}
export declare const VersionStatus: {
    readonly Outdated: "Outdated";
    readonly Active: "Active";
};
export type VersionStatus = typeof VersionStatus[keyof typeof VersionStatus];
export interface VideoAttachment {
    'id': string;
    'name'?: string | null;
    'width'?: number | null;
    'height'?: number | null;
    'sizeBytes': number;
    'previewBytes'?: string | null;
}
export interface VideoCallChannelItemContent {
    'session': CallSession;
}
export declare const VolumeState: {
    readonly Unknown: "UNKNOWN";
    readonly Creating: "CREATING";
    readonly Available: "AVAILABLE";
    readonly InUse: "IN_USE";
    readonly Deleting: "DELETING";
    readonly Deleted: "DELETED";
    readonly Error: "ERROR";
};
export type VolumeState = typeof VolumeState[keyof typeof VolumeState];
export interface VoteGroup {
    'variantName': string;
    'count': number;
    'meVote': boolean;
    'lastUsers': Array<TDMemberProfile>;
    'owner'?: TDMemberProfile;
}
export declare const WarmupExecutionStatus: {
    readonly Started: "Started";
    readonly Failed: "Failed";
    readonly Terminated: "Terminated";
    readonly SnapshotCreated: "SnapshotCreated";
    readonly SnapshotDeleted: "SnapshotDeleted";
};
export type WarmupExecutionStatus = typeof WarmupExecutionStatus[keyof typeof WarmupExecutionStatus];
export interface WarmupExecutionTrigger {
    'className': string;
    'user'?: TDMemberProfile;
    'cronExpressions': Array<string>;
}
export interface WarmupExecutionTriggerManual {
    'user'?: TDMemberProfile;
}
export interface WarmupExecutionTriggerScheduled {
    'cronExpressions': Array<string>;
}
export interface WebhookDeliveryStatusDTO {
    'className': string;
    'clientError': AppMessageDeliveryClientErrorDTO;
    'deliveryId': string;
    'sentTime': string;
    'responseCode': number;
    'message': string;
}
export interface WebhookDeliveryStatusDTOExpectedHttpClientError {
    'clientError': AppMessageDeliveryClientErrorDTO;
    'deliveryId': string;
    'sentTime': string;
}
export interface WebhookDeliveryStatusDTOFailedDelivery {
    'deliveryId': string;
    'responseCode': number;
    'message': string;
    'sentTime': string;
}
export interface WebhookDeliveryStatusDTOInternalError {
    'deliveryId': string;
    'sentTime': string;
}
export interface WebhookDeliveryStatusDTOSuccessful {
    'deliveryId': string;
    'responseCode': number;
    'sentTime': string;
}
export interface WebhookEvent {
    'className': string;
    'meta': KMetaMod;
    'application': ESApp;
    'fingerprint'?: AbsenceWebhookEventIcon | null;
    'comment'?: AbsenceWebhookEventIcon | null;
    'document': string;
    'membership': TDMembership;
    'team': TDTeam;
    'role'?: TeamMembershipEventRole | null;
    'lead'?: AbsenceWebhookEventAvailable | null;
    'manager'?: ApplicationWebhookEventOwner | null;
    'since'?: AbsenceWebhookEventSince | null;
    'till'?: AbsenceWebhookEventSince | null;
    'deleted'?: AbsenceWebhookEventAvailable | null;
    'approvedBy'?: ApplicationWebhookEventOwner | null;
    'emoji': string;
    'owner': CPrincipal;
    'published'?: AbsenceWebhookEventAvailable | null;
    'changeAuthors': Array<CPrincipal>;
    'version': string;
    'base': string;
    'issue': Issue;
    'title': AbsenceWebhookEventIcon | null;
    'description'?: AbsenceWebhookEventIcon | null;
    'assignee'?: ApplicationWebhookEventOwner | null;
    'status'?: AutomationJobEventStatus | null;
    'dueDate'?: AbsenceWebhookEventSince | null;
    'tagDelta'?: IssueWebhookEventTagDelta | null;
    'topicDelta'?: IssueWebhookEventTopicDelta | null;
    'checklistDelta'?: IssueWebhookEventChecklistDelta | null;
    'sprintDelta'?: IssueWebhookEventSprintDelta | null;
    'customFieldUpdate'?: IssueWebhookCustomFieldUpdate;
    'projectKey': ProjectKey;
    'repository': string;
    'reviewId': string;
    'isMergeRequest': boolean;
    'article': ArticleRecord;
    'action': PackageRepositoryEventAction;
    'author'?: ApplicationWebhookEventOwner | null;
    'created'?: AutomationJobEventStartTime | null;
    'textChanged': boolean;
    'unpublished'?: AbsenceWebhookEventAvailable | null;
    'teams'?: BlogWebhookEventTeams | null;
    'locations'?: BlogWebhookEventLocations | null;
    'externalEntityInfo'?: BlogWebhookEventExternalEntityInfo | null;
    'attachmentId': string;
    'uploadedAt': string;
    'channelId': string;
    'threadId'?: string | null;
    'message': ChannelItemRecord;
    'webhookName': string;
    'name': string;
    'archived'?: AbsenceWebhookEventAvailable | null;
    'endpointURI'?: AbsenceWebhookEventIcon | null;
    'clientIdChanged': boolean;
    'clientSecretChanged': boolean;
    'verificationTokenChanged': boolean;
    'signingKeyChanged': boolean;
    'absence': AbsenceRecord;
    'member': TDMemberProfile;
    'icon'?: AbsenceWebhookEventIcon | null;
    'reason': string;
    'location': TDLocation;
    'available'?: AbsenceWebhookEventAvailable | null;
    'messageId': string;
    'actor': CPrincipal;
    'newCount': number;
    'scope': AuthScope;
    'repositoryType': PackageType;
    'packageInfo': PackageVersionRef;
    'channel': M2ChannelRecord;
    'restored'?: boolean | null;
    'meeting': DTOMeeting;
    'warmupExecution': RdWarmupExec;
    'target': ApplicationUnfurlTarget;
    'relation': string;
    'entity': string;
    'issueNumber'?: number | null;
    'enabledForAll'?: AbsenceWebhookEventAvailable | null;
    'selfManageable'?: AbsenceWebhookEventAvailable | null;
    'addedTeams'?: Array<TDTeam> | null;
    'addedProfiles'?: Array<TDTeam> | null;
    'removedTeams'?: Array<TDMemberProfile> | null;
    'removedProfiles'?: Array<TDMemberProfile> | null;
    'targetId': string;
    'deploymentId': string;
    'statusMod'?: DeploymentWebhookEventStatusMod | null;
    'edited'?: AutomationJobEventStartTime | null;
    'pinned'?: AbsenceWebhookEventAvailable | null;
    'commit': GitCommitInfoWithChanges;
    'project': PRProject;
    'approved': AbsenceApprovalWebhookEventApproved;
    'email': string;
    'joinedOrganization': boolean;
    'leftOrganization': boolean;
    'importTransaction': ImportTransactionRecord;
    'changes': RepoChanges;
    'executionId': string;
    'repositoryName': string;
    'jobName': string;
    'executionNumber': number;
    'trigger': JobExecutionTrigger;
    'triggerTime': string;
    'failureReasons'?: AutomationJobEventFailureReasons | null;
    'stoppedBy'?: AutomationJobEventStoppedBy | null;
    'startTime'?: AutomationJobEventStartTime | null;
    'endTime'?: AutomationJobEventStartTime | null;
}
export interface WebhookRecord {
    'id': string;
    'archived': boolean;
    'app': ESApp;
    'subscriptions': Array<SubscriptionDTO>;
    'name': string;
    'description'?: string | null;
    'useAppEndpoint': boolean;
    'endpoint': EndpointDTO;
    'useAppEndpointAuth': boolean;
    'endpointAuth': EndpointAuthDTO;
    'enabled': boolean;
    'acceptedHttpResponseCodes': Array<number>;
    'doRetries': boolean;
    'payloadFields'?: string | null;
}
export interface WebhookRequestPayload {
    'verificationToken'?: string | null;
    'clientId': string;
    'webhookId': string;
    'payload': WebhookEvent;
}
export interface WeekDayLocation {
    'day': number;
    'remote': boolean;
}
export interface WeekDayTimeInterval {
    'day': number;
    'checked': boolean;
    'interval': TimeInterval;
}
export declare const Weekday: {
    readonly Sunday: "SUNDAY";
    readonly Monday: "MONDAY";
    readonly Tuesday: "TUESDAY";
    readonly Wednesday: "WEDNESDAY";
    readonly Thursday: "THURSDAY";
    readonly Friday: "FRIDAY";
    readonly Saturday: "SATURDAY";
};
export type Weekday = typeof Weekday[keyof typeof Weekday];
export interface Workday {
    'weekday': string;
    'working': boolean;
    'hours': number;
    'minutes'?: number | null;
}
export declare const WorkerCapability: {
    readonly Docker: "Docker";
    readonly DockerCompose: "DockerCompose";
    readonly BlockStorage: "BlockStorage";
    readonly Git: "Git";
};
export type WorkerCapability = typeof WorkerCapability[keyof typeof WorkerCapability];
export interface WorkerComputePoolDTO {
    'key': ComputePoolKey;
    'name': string;
}
export interface WorkerDTO {
    'id': string;
    'name': string;
    'owner'?: TDMemberProfile;
    'ownerPrincipal'?: CPrincipal;
    'info': WorkerInfoDTO;
    'computePool': WorkerComputePoolDTO;
    'version': WorkerVersionDTO;
    'lastAccessTime'?: string | null;
    'status': WorkerStatus;
    'scope': WorkerScope;
    'suspended': boolean;
    'projects'?: Array<PRProject> | null;
    'tags': Array<WorkerTagDTO>;
    'stepsStats': WorkerStepsStatsDTO;
    'workerStats': WorkerStatsDTO;
    'workerCapabilities': Array<WorkerCapability>;
    'permanentTokenInfo'?: WorkerPermanentTokenInfoDTO;
}
export declare const WorkerError: {
    readonly StartError: "StartError";
    readonly DownloadError: "DownloadError";
    readonly Error: "Error";
};
export type WorkerError = typeof WorkerError[keyof typeof WorkerError];
export interface WorkerInfoDTO {
    'osType'?: WorkerOSType;
    'osName'?: string | null;
    'osArch'?: string | null;
    'osVersion'?: string | null;
    'hostname'?: string | null;
}
export declare const WorkerOSType: {
    readonly Linux: "Linux";
    readonly Windows: "Windows";
    readonly Mac: "Mac";
    readonly Other: "Other";
};
export type WorkerOSType = typeof WorkerOSType[keyof typeof WorkerOSType];
export interface WorkerPermanentTokenInfoDTO {
    'created': string;
}
export declare const WorkerScope: {
    readonly Organization: "Organization";
    readonly Project: "Project";
};
export type WorkerScope = typeof WorkerScope[keyof typeof WorkerScope];
export interface WorkerScopeFilter {
    'className': string;
    'project': ProjectIdentifier;
}
export interface WorkerScopeFilterProject {
    'project': ProjectIdentifier;
}
export interface WorkerSortOrder {
    'className': string;
    'time': SortOrderType;
    'name': SortOrderType;
}
export interface WorkerSortOrderAccessTime {
    'time': SortOrderType;
}
export interface WorkerSortOrderName {
    'name': SortOrderType;
}
export interface WorkerStatsDTO {
    'volumeFreeSpace'?: string | null;
    'diskPressure': boolean;
    'cpuLimit'?: string | null;
    'memoryLimit'?: string | null;
}
export declare const WorkerStatus: {
    readonly Healthy: "Healthy";
    readonly Disconnected: "Disconnected";
    readonly Unhealthy: "Unhealthy";
    readonly Suspended: "Suspended";
    readonly GracefulShutdown: "GracefulShutdown";
    readonly Locked: "Locked";
    readonly WithWarning: "WithWarning";
    readonly NotInitialized: "NotInitialized";
};
export type WorkerStatus = typeof WorkerStatus[keyof typeof WorkerStatus];
export interface WorkerStepsStatsDTO {
    'activity': Array<RepositoryActivityLastActivityInner>;
}
export interface WorkerTagDTO {
    'name': string;
}
export interface WorkerVersionDTO {
    'workerVersion'?: string | null;
    'updateError'?: UpdateErrorDTO;
}
export interface WorkingDaysSpec {
    'days': Array<Workday>;
    'workingHours'?: Array<WeekDayTimeInterval> | null;
    'notificationHours'?: Array<WeekDayTimeInterval> | null;
    'timezone'?: ATimeZone;
    'boundWeekdays'?: Array<number> | null;
    'locations'?: Array<WeekDayLocation> | null;
}
export interface WorkspaceIdentifier {
    'className': string;
    'project': ProjectIdentifier;
    'number': number;
    'externalId': string;
    'id': string;
}
export interface WorkspaceIdentifierByProject {
    'project': ProjectIdentifier;
    'number': number;
}
export interface WorkspaceIdentifierExternalId {
    'externalId': string;
}
export interface WorkspaceIdentifierId {
    'id': string;
}
export interface XScopeApi {
    'className': string;
    'grants': Array<XScopeGrantApi>;
}
export interface XScopeApiScope {
    'grants': Array<XScopeGrantApi>;
}
export interface XScopeGrantApi {
    'context': PermissionContextApi;
    'right': RightDescriptorDTO;
}
export declare const SpaceApiAxiosParamCreator: (configuration?: Configuration) => {
    absencesAbsenceReasonsGet: (withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesAbsenceReasonsIdDelete: (id: string, _delete?: boolean, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesAbsenceReasonsIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesAbsenceReasonsIdPatch: (id: string, absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesAbsenceReasonsPost: (absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesGet: ($skip?: string, $top?: number, member?: string, members?: Array<string>, location?: string, team?: string, since?: string, till?: string, viewMode?: AbsenceListMode, reason?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesIdApprovePost: (id: string, absencesIdApprovePostRequest: AbsencesIdApprovePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesIdDelete: (id: string, _delete?: boolean, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesIdDeleteApprovalDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesIdPatch: (id: string, absencesIdDeleteRequest: AbsencesIdDeleteRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesMembermemberGet: (member: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    absencesPost: (absencesGetRequest: AbsencesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    administrationSupportPost: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    administrationUserAgreementEnabledGet: (options?: AxiosRequestConfig) => Promise<RequestArgs>;
    administrationUserAgreementEnabledPost: (administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    administrationUserAgreementGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    administrationUserAgreementPatch: (administrationUserAgreementGetRequest: AdministrationUserAgreementGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    administrationUserAgreementStatusGet: ($skip?: string, $top?: number, query?: string, accepted?: boolean, activeProfilesOnly?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    administrationUserAgreementStatusProfileGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationAuthorizationsAuthorizedContextsGet: (application: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationAuthorizationsAuthorizedRightsDelete: (application: string, contextIdentifier: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationAuthorizationsAuthorizedRightsGet: (application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationAuthorizationsAuthorizedRightsPatch: (application: string, applicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatch: (application: string, applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationAuthorizationsRequiredRightsGet: (application: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationAuthorizationsRequiredRightsPatch: (application: string, applicationsApplicationAuthorizationsRequiredRightsGetRequest: ApplicationsApplicationAuthorizationsRequiredRightsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationBearerTokenGet: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationClientSecretGet: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationClientSecretRegeneratePost: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationDelete: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationForceRemovePost: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationGet: (application: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationLastClientCredentialsAccessGet: (application: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationPatch: (application: string, $fields?: string, applicationsApplicationDeleteRequest?: ApplicationsApplicationDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationPermanentTokensCurrentDelete: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationPermanentTokensGet: (application: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationPermanentTokensPost: (application: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationPermanentTokensTokenIdDelete: (application: string, tokenId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationPermanentTokensTokenIdPatch: (application: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationPublicKeysGet: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationRestorePost: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationSigningKeyGet: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationSigningKeyRegeneratePost: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationSshKeysFingerprintDelete: (application: string, fingerprint: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationSshKeysGet: (application: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationSshKeysPost: (application: string, applicationsApplicationSshKeysGetRequest: ApplicationsApplicationSshKeysGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUiExtensionsDisableForEverybodyPatch: (application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUiExtensionsDisableForMePatch: (application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUiExtensionsEnableForEverybodyPatch: (application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUiExtensionsEnableForMePatch: (application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUiExtensionsGet: (application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUnfurlDomainsAuthorizePost: (application: string, applicationsApplicationUnfurlDomainsAuthorizePostRequest: ApplicationsApplicationUnfurlDomainsAuthorizePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUnfurlDomainsGet: (application: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUnfurlPatternsAuthorizePost: (application: string, applicationsApplicationUnfurlPatternsAuthorizePostRequest: ApplicationsApplicationUnfurlPatternsAuthorizePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationUnfurlPatternsGet: (application: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationVerificationTokenGet: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationVerificationTokenRegeneratePost: (application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksGet: (application: string, withArchived?: boolean, query?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksPost: (application: string, applicationsApplicationWebhooksGetRequest: ApplicationsApplicationWebhooksGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdBearerTokenGet: (application: string, webhookId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersGet: (application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersPost: (application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest: ApplicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdDelete: (application: string, webhookId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdPatch: (application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdPost: (application: string, webhookId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdSigningKeyGet: (application: string, webhookId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdSigningKeyRegeneratePost: (application: string, webhookId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsGet: (application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsPost: (application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest: ApplicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDelete: (application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdPatch: (application: string, webhookId: string, subscriptionId: string, $fields?: string, applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdRequestMissingRightsPost: (application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsAuthorizationsAuthorizedApplicationsGet: (contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsPagedGet: ($skip?: string, $top?: number, name?: string, owner?: Array<string>, withArchived?: boolean, withManaged?: boolean, ordering?: AppsOrdering, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsPost: (applicationsPostRequest: ApplicationsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsUiExtensionsPatch: (applicationsUiExtensionsPatchRequest: ApplicationsUiExtensionsPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsUnfurlsDomainsPatch: (applicationsUnfurlsDomainsPatchRequest: ApplicationsUnfurlsDomainsPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsUnfurlsPatternsPatch: (applicationsUnfurlsPatternsPatchRequest: ApplicationsUnfurlsPatternsPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsUnfurlsQueueContentPost: (applicationsUnfurlsQueueContentPostRequest: ApplicationsUnfurlsQueueContentPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsUnfurlsQueueGet: (batchSize: number, fromEtag?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsUnfurlsQueueRequestExternalAuthPost: (applicationsUnfurlsQueueRequestExternalAuthPostRequest: ApplicationsUnfurlsQueueRequestExternalAuthPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    applicationsUnfurlsQueueResetExternalAuthRequestsPost: (applicationsUnfurlsQueueResetExternalAuthRequestsPostRequest: ApplicationsUnfurlsQueueResetExternalAuthRequestsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesConfigDelete: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesConfigGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesConfigPut: (authModulesConfigDeleteRequest?: AuthModulesConfigDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesDiscoverOidcGet: (discoveryEndpoint: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesGet: (withDisabled?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesIdLoginsIdentifierChangePost: (id: string, identifier: string, authModulesIdLoginsIdentifierChangePostRequest: AuthModulesIdLoginsIdentifierChangePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesIdLoginsIdentifierDelete: (identifier: string, id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesIdLoginsIdentifierResetPost: (id: string, identifier: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesIdPatch: (id: string, authModulesIdDeleteRequest?: AuthModulesIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesIdSamlMetadataPost: (id: string, authModulesIdSamlMetadataPostRequest: AuthModulesIdSamlMetadataPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesKeykeyGet: (key: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesPost: (authModulesGetRequest: AuthModulesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesReorderPost: (authModulesReorderPostRequest: AuthModulesReorderPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesTestBuiltInPost: (authModulesTestBuiltInPostRequest: AuthModulesTestBuiltInPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesTestLdapPost: (authModulesTestLdapPostRequest: AuthModulesTestLdapPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesThrottledLoginsDelete: (logins: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesThrottledLoginsGet: ($skip?: string, $top?: number, login?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesThrottledLoginsOrgStatusDelete: (options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesThrottledLoginsOrgStatusGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    authModulesUsagesGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    billingAdminOverdraftsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    billingAdminOverdraftsPut: (billingAdminOverdraftsGetRequest: BillingAdminOverdraftsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    billingAdminReportsBillingPeriodGet: (billingPeriod: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    billingAdminReportsTodayGet: (date: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    billingAdminTrialPut: (billingAdminTrialPutRequest: BillingAdminTrialPutRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogAliasaliasGet: (alias: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogExternalIdidGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogGet: ($skip?: string, $top?: number, term?: string, dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, forProfile?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogIdPatch: (id: string, $fields?: string, blogIdDeleteRequest?: BlogIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogImportPost: (blogImportPostRequest: BlogImportPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogPost: (blogGetRequest: BlogGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    blogStatsGet: (dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsAbsenceEventsGet: (dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsBirthdayEventsGet: (dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsBirthdayEventsStarredGet: (dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsEventParticipationsIdPatch: (id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsEventsGet: (dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsEventsIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsHolidaysGet: (startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsGet: ($skip?: string, $top?: number, summaryQuery?: string, locationsQuery?: Array<string>, startingAfter?: string, endingAfter?: string, endingBefore?: string, startingBefore?: string, profiles?: Array<string>, teams?: Array<string>, organizer?: string, includePrivate?: boolean, includeArchived?: boolean, includeMeetingInstances?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsIdConferenceRoomsDelete: (id: string, roomId: string, dateTime: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsIdConferenceRoomsPost: (id: string, calendarsMeetingsIdConferenceRoomsDeleteRequest: CalendarsMeetingsIdConferenceRoomsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsIdDelete: (id: string, targetDate?: string, modificationKind?: RecurrentModification, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsIdParticipationStatusPatch: (id: string, calendarsMeetingsIdParticipationStatusPatchRequest: CalendarsMeetingsIdParticipationStatusPatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsIdPatch: (id: string, $fields?: string, calendarsMeetingsIdDeleteRequest?: CalendarsMeetingsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsNextOccurrenceGet: (meetingId: string, since?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsOccurrencesByMeetingGet: (meetingIds: Array<string>, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsOccurrencesGet: (meetingId: string, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsParticipationStatusesExternalGet: (id: string, emails: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsParticipationStatusesGet: (id: string, profileIds: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsPost: (calendarsMeetingsGetRequest: CalendarsMeetingsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsProfileParticipationGet: (profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMeetingsProfileParticipationRecordsGet: (profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsMembershipEventsGet: (dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    calendarsNonWorkingDaysEventsGet: (dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    callsPost: (callsPostRequest: CallsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsAllChannelsGet: (query: string, $skip?: string, $top?: number, quickFilter?: AllChannelsFilter, sortColumn?: AllChannelsSortColumn, sortOrder?: ColumnSortOrder, publicOnly?: boolean, withArchived?: boolean, subscriber?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelAdministratorGet: (channel: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelAdministratorPatch: (channel: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelArchiveDelete: (channel: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelAttachmentsFilesGet: (channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelAttachmentsGet: (channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelAttachmentsImagesGet: (channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelAttachmentsLinksGet: (channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelAttachmentsVideosGet: (channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelDelete: (channel: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelDescriptionPatch: (channel: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelGet: (channel: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelIconPatch: (channel: string, chatsChannelsChannelIconPatchRequest?: ChatsChannelsChannelIconPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelNamePatch: (channel: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelRestoreArchivedPost: (channel: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelSubscribersTeamsDelete: (channel: string, teams: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelSubscribersTeamsGet: (channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelSubscribersTeamsPost: (channel: string, chatsChannelsChannelSubscribersTeamsDeleteRequest: ChatsChannelsChannelSubscribersTeamsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelSubscribersUsersDelete: (channel: string, profiles: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelSubscribersUsersGet: (channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsChannelSubscribersUsersPost: (channel: string, chatsChannelsChannelSubscribersUsersDeleteRequest: ChatsChannelsChannelSubscribersUsersDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsConversationsChannelConvertPost: (channel: string, chatsChannelsConversationsChannelConvertPostRequest: ChatsChannelsConversationsChannelConvertPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsConversationsChannelSubjectPatch: (channel: string, chatsChannelsConversationsChannelSubjectPatchRequest: ChatsChannelsConversationsChannelSubjectPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsConversationsPost: (chatsChannelsConversationsPostRequest: ChatsChannelsConversationsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsDmPost: (chatsChannelsDmPostRequest: ChatsChannelsDmPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsIsNameFreePost: (chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsChannelsPost: (chatsChannelsPostRequest: ChatsChannelsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesDeleteMessagePost: (chatsMessagesDeleteMessagePostRequest: ChatsMessagesDeleteMessagePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesEditMessagePost: (chatsMessagesEditMessagePostRequest: ChatsMessagesEditMessagePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesGet: (channel: string, sorting: MessagesSorting, batchSize: number, startFromDate?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesImportPost: (chatsMessagesImportPostRequest: ChatsMessagesImportPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesMessageGet: (message: string, channel: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesPinPatch: (chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesPinnedMessagesGet: (channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesSendMessagePost: (chatsMessagesSendMessagePostRequest: ChatsMessagesSendMessagePostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesSendPost: (chatsMessagesSendPostRequest: ChatsMessagesSendPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    chatsMessagesUnpinPatch: (chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    checklistsChecklistItemsPlanItemDelete: (checklist: string, planItem: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    checklistsChecklistItemsPlanItemGet: (checklist: string, planItem: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    checklistsChecklistItemsPlanItemMovePost: (checklist: string, planItem: string, checklistsChecklistItemsPlanItemMovePostRequest: ChecklistsChecklistItemsPlanItemMovePostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    checklistsChecklistItemsPlanItemPatch: (checklist: string, planItem: string, $fields?: string, checklistsChecklistItemsPlanItemDeleteRequest?: ChecklistsChecklistItemsPlanItemDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    checklistsChecklistItemsPost: (checklist: string, checklistsChecklistItemsPostRequest: ChecklistsChecklistItemsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsExtendedTypesGet: (scope?: ExtendedTypeScopeType, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyAllValuesGet: (typeKey: string, $skip?: string, $top?: number, extendedEntityIds?: Array<string>, scope?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyEntityIdValuesGet: (typeKey: string, entityId: string, scope?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyEntityIdValuesPatch: (entityId: string, typeKey: string, customFieldsTypeKeyEntityIdValuesGetRequest: CustomFieldsTypeKeyEntityIdValuesGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyEnumValuesCustomFieldIdGet: (typeKey: string, customFieldId: string, $skip?: string, $top?: number, query?: string, ordering?: EnumValueOrdering, countRecords?: boolean, addedByProfileId?: string, scope?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyEnumValuesCustomFieldIdPost: (typeKey: string, customFieldId: string, customFieldsTypeKeyEnumValuesCustomFieldIdGetRequest: CustomFieldsTypeKeyEnumValuesCustomFieldIdGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsGet: (typeKey: string, withArchived?: boolean, scope?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsIdArchivePost: (typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsIdDelete: (typeKey: string, id: string, scope: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsIdFilterValuesGet: (typeKey: string, id: string, $skip?: string, $top?: number, scope?: string, calculateTotal?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsIdPatch: (typeKey: string, id: string, customFieldsTypeKeyFieldsIdDeleteRequest: CustomFieldsTypeKeyFieldsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsIdRestorePost: (typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsPost: (typeKey: string, customFieldsTypeKeyFieldsGetRequest: CustomFieldsTypeKeyFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsTypeKeyFieldsReorderPost: (typeKey: string, customFieldsTypeKeyFieldsReorderPostRequest: CustomFieldsTypeKeyFieldsReorderPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldArchivePost: (entityType: string, customField: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldDelete: (entityType: string, customField: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePost: (entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesEnumValueToRemoveDelete: (entityType: string, customField: string, enumValueToRemove: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet: (entityType: string, customField: string, query?: string, ordering?: EnumValueOrdering, addedByProfileId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPatch: (entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPost: (entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldGet: (entityType: string, customField: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldPatch: (entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldDeleteRequest?: CustomFieldsV2EntityTypeFieldsCustomFieldDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsCustomFieldRestorePost: (entityType: string, customField: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsGet: (entityType: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsPost: (entityType: string, customFieldsV2EntityTypeFieldsGetRequest: CustomFieldsV2EntityTypeFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2EntityTypeFieldsReorderPost: (entityType: string, customFieldsV2EntityTypeFieldsReorderPostRequest: CustomFieldsV2EntityTypeFieldsReorderPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2ValuesEntityCustomFieldGet: (entity: string, customField: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2ValuesEntityCustomFieldPost: (entity: string, customField: string, customFieldsV2ValuesEntityCustomFieldGetRequest: CustomFieldsV2ValuesEntityCustomFieldGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2ValuesEntityGet: (entity: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    customFieldsV2ValuesEntityPost: (entity: string, customFieldsV2ValuesEntityGetRequest: CustomFieldsV2ValuesEntityGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    emojisAddPost: (emojisAddPostRequest: EmojisAddPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    emojisDeletePost: (emojisDeletePostRequest: EmojisDeletePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    emojisExistsGet: (emoji: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    emojisFrequentlyUsedGet: (options?: AxiosRequestConfig) => Promise<RequestArgs>;
    emojisRecordUsagePost: (emojisRecordUsagePostRequest: EmojisRecordUsagePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    emojisSearchGet: (query: string, $skip?: string, $top?: number, version?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    externalLinkPatternsDelete: (pattern: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    externalLinkPatternsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    externalLinkPatternsPost: (externalLinkPatternsDeleteRequest: ExternalLinkPatternsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    httpApiModelGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsChannelSubscriptionsGet: (channel: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsChannelSubscriptionsIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsChannelSubscriptionsIdPatch: (id: string, $fields?: string, notificationsChannelSubscriptionsIdDeleteRequest?: NotificationsChannelSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsChannelSubscriptionsIdRequestMissingRightsPost: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsChannelSubscriptionsPost: (notificationsChannelSubscriptionsGetRequest: NotificationsChannelSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalCustomSubscriptionsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalCustomSubscriptionsIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalCustomSubscriptionsIdPatch: (id: string, $fields?: string, notificationsPersonalCustomSubscriptionsIdDeleteRequest?: NotificationsPersonalCustomSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalCustomSubscriptionsPost: (notificationsPersonalCustomSubscriptionsGetRequest: NotificationsPersonalCustomSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalSubscriptionsAllPersonalSubscriptionTargetsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalSubscriptionsPersonalSubscriptionSettingsGet: (profile: string, feed: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPost: (notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPost: (notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPrivateFeedsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPrivateFeedsIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPrivateFeedsIdPatch: (id: string, $fields?: string, notificationsPrivateFeedsIdDeleteRequest?: NotificationsPrivateFeedsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    notificationsPrivateFeedsPost: (notificationsPrivateFeedsGetRequest: NotificationsPrivateFeedsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    organizationDomainsCheckGet: (domain: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    organizationDomainsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    organizationDomainsPatch: (organizationDomainsGetRequest: OrganizationDomainsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    organizationGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    organizationJetSalesLicenseActivationUrlGet: (options?: AxiosRequestConfig) => Promise<RequestArgs>;
    organizationJetSalesUrlGet: (options?: AxiosRequestConfig) => Promise<RequestArgs>;
    organizationPatch: (organizationGetRequest: OrganizationGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesCreatePost: (permissionRolesCreatePostRequest: PermissionRolesCreatePostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesGetPost: (applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleId2FaRequirementGet: (roleId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleId2FaRequirementPatch: (roleId: string, permissionRolesRoleId2FaRequirementGetRequest?: PermissionRolesRoleId2FaRequirementGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdDelete: (roleId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdPatch: (roleId: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdPermissionsDelete: (roleId: string, rightCodes: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdPermissionsGet: (roleId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdPermissionsPost: (roleId: string, permissionRolesRoleIdPermissionsDeleteRequest: PermissionRolesRoleIdPermissionsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdProfilesGet: (roleId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdProfilesProfileDelete: (roleId: string, profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdProfilesProfilePost: (roleId: string, profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdResetRolePermissionsToDefaultPost: (roleId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdTeamsGet: (roleId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdTeamsTeamDelete: (roleId: string, team: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionRolesRoleIdTeamsTeamPost: (roleId: string, team: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionsCheckPermissionPost: (permissionsCheckPermissionPostRequest: PermissionsCheckPermissionPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    permissionsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationDslEvaluationsConfigGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationGraphExecutionsIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationGraphExecutionsIdStopPost: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationJobExecutionsCurrentGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationJobsJobIdGet: (jobId: string, project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyDelete: (stepExecId: string, key: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyGet: (stepExecId: string, key: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyPatch: (stepExecId: string, key: string, projectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest: ProjectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationStepExecutionsUsedParametersParamParameterIdGet: (parameterId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationStepExecutionsUsedParametersSecretSecretIdGet: (secretId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsAutomationSubscriptionsLegacyChannelsDelete: (project?: string, jobId?: string, unsubscribedOnly?: boolean, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsCollaboratorprofileGet: (profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsGet: ($skip?: string, $top?: number, term?: string, tag?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsMembermemberGet: (member: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsParamsDefaultBundleGet: (project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsParamsDefaultBundlePost: (projectsParamsDefaultBundleGetRequest: ProjectsParamsDefaultBundleGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsParamsIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsParamsIdPatch: (id: string, projectsParamsIdDeleteRequest: ProjectsParamsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsParamsInDefaultBundleGet: (projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsParamsInDefaultBundlePost: (projectsParamsInDefaultBundleGetRequest: ProjectsParamsInDefaultBundleGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsBoardDelete: (board: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsBoardGet: (board: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsBoardIssuesGet: (board: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsBoardIssuesIssueDelete: (issue: string, board: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsBoardIssuesIssuePost: (issue: string, board: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsBoardPatch: (board: string, projectsPlanningBoardsBoardDeleteRequest?: ProjectsPlanningBoardsBoardDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsSprintsPost: (projectsPlanningBoardsSprintsPostRequest: ProjectsPlanningBoardsSprintsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsSprintsSprintArchiveDelete: (sprint: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsSprintsSprintIssuesGet: (sprint: string, $skip?: string, $top?: number, unresolvedOnly?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsSprintsSprintIssuesIssueDelete: (issue: string, sprint: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsSprintsSprintIssuesIssuePost: (issue: string, sprint: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsSprintsSprintLaunchPost: (sprint: string, projectsPlanningBoardsSprintsSprintLaunchPostRequest: ProjectsPlanningBoardsSprintsSprintLaunchPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPlanningBoardsSprintsSprintPatch: (sprint: string, projectsPlanningBoardsSprintsSprintPatchRequest?: ProjectsPlanningBoardsSprintsSprintPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPost: (projectsGetRequest: ProjectsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPrivateProjectsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsPrivateProjectsProjectRequestAccessPost: (project: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessAdminsGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessAdminsProfilesPost: (project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessAdminsProfilesProfileDelete: (project: string, profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessAdminsTeamsPost: (project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessAdminsTeamsTeamIdDelete: (project: string, teamId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessCollaboratorsGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessCollaboratorsProfilesDelete: (project: string, profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessCollaboratorsProfilesGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessCollaboratorsProfilesPost: (project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessCollaboratorsTeamsDelete: (project: string, teamId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessCollaboratorsTeamsGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessCollaboratorsTeamsPost: (project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessMemberProfilesGet: (project: string, $skip?: string, $top?: number, query?: string, includingAdmins?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessMembersProfilesPost: (project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessMembersProfilesProfileDelete: (project: string, profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessMembersTeamsPost: (project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessMembersTeamsTeamIdDelete: (project: string, teamId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAccessViewersGet: (project: string, term: string, $skip?: string, $top?: number, meOnTop?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentTargetsFavoritesGet: (project: string, sortBy?: string, sortOrder?: ColumnSortOrder, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentTargetsGet: (project: string, search?: string, sortBy?: string, sortOrder?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentTargetsPost: (project: string, projectsProjectAutomationDeploymentTargetsGetRequest: ProjectsProjectAutomationDeploymentTargetsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentTargetsTargetDelete: (project: string, target: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentTargetsTargetGet: (project: string, target: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentTargetsTargetPut: (project: string, target: string, projectsProjectAutomationDeploymentTargetsTargetDeleteRequest?: ProjectsProjectAutomationDeploymentTargetsTargetDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsFailPost: (project: string, projectsProjectAutomationDeploymentsFailPostRequest: ProjectsProjectAutomationDeploymentsFailPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsFinishPost: (project: string, projectsProjectAutomationDeploymentsFinishPostRequest: ProjectsProjectAutomationDeploymentsFinishPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsGet: (project: string, targetIdentifier?: string, jobExecutionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsPut: (project: string, projectsProjectAutomationDeploymentsGetRequest: ProjectsProjectAutomationDeploymentsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsSchedulePost: (project: string, projectsProjectAutomationDeploymentsSchedulePostRequest: ProjectsProjectAutomationDeploymentsSchedulePostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsStartPost: (project: string, projectsProjectAutomationDeploymentsStartPostRequest: ProjectsProjectAutomationDeploymentsStartPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierDelete: (project: string, targetIdentifier: string, deploymentIdentifier: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierGet: (project: string, targetIdentifier: string, deploymentIdentifier: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationGraphExecutionsGet: (project: string, jobId: string, branchFilter?: string, statusFilter?: ExecutionStatus, jobTriggerFilter?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationJobsGet: (project: string, repoFilter: string, branchFilter: string, trigger?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectAutomationJobsJobIdStartPost: (project: string, jobId: string, projectsProjectAutomationJobsJobIdStartPostRequest: ProjectsProjectAutomationJobsJobIdStartPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPost: (project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdRejectSuggestedEditPost: (project: string, discussionId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdReopenSuggestedEditPost: (project: string, discussionId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatch: (project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsCodeDiscussionsPost: (project: string, projectsProjectCodeReviewsCodeDiscussionsPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsCommitSetReviewPost: (project: string, projectsProjectCodeReviewsCommitSetReviewPostRequest: ProjectsProjectCodeReviewsCommitSetReviewPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsGet: (project: string, $skip?: string, $top?: number, state?: CodeReviewStateFilter, text?: string, author?: string, from?: string, to?: string, sort?: ReviewSorting, reviewer?: string, type?: ReviewType, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsMergeRequestsPost: (project: string, projectsProjectCodeReviewsMergeRequestsPostRequest: ProjectsProjectCodeReviewsMergeRequestsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdDescriptionPatch: (project: string, reviewId: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdDetailsGet: (project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdFilesGet: (project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdGet: (project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdMergeFilesGet: (project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdMergePut: (project: string, reviewId: string, projectsProjectCodeReviewsReviewIdMergePutRequest: ProjectsProjectCodeReviewsReviewIdMergePutRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdParticipantsUserDelete: (project: string, reviewId: string, user: string, role: CodeReviewParticipantRole, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdParticipantsUserPost: (project: string, reviewId: string, user: string, projectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest: ProjectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdRebasePut: (project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRebasePutRequest: ProjectsProjectCodeReviewsReviewIdRebasePutRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdRevisionsDelete: (project: string, reviewId: string, revisions: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdRevisionsPost: (project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRevisionsDeleteRequest: ProjectsProjectCodeReviewsReviewIdRevisionsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdStatePatch: (project: string, reviewId: string, projectsProjectCodeReviewsReviewIdStatePatchRequest: ProjectsProjectCodeReviewsReviewIdStatePatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdSuggestedReviewersGet: (project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectCodeReviewsReviewIdTitlePatch: (project: string, reviewId: string, projectsProjectCodeReviewsReviewIdTitlePatchRequest: ProjectsProjectCodeReviewsReviewIdTitlePatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDelete: (project: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsDocumentIdCopyPost: (project: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsDocumentIdDelete: (project: string, documentId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsDocumentIdDeleteForeverDelete: (project: string, documentId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsDocumentIdGet: (project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsDocumentIdMovePatch: (project: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsDocumentIdPatch: (project: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsDocumentIdUnarchivePatch: (project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderDelete: (project: string, folder: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderDocumentsGet: (project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderGet: (project: string, folder: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderIntroductionDelete: (project: string, folder: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderIntroductionDocumentIdPatch: (project: string, folder: string, documentId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderMovePatch: (project: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderPatch: (project: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersFolderSubfoldersGet: (project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsFoldersPost: (project: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectDocumentsPost: (project: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesGet: (project: string, type?: string, query?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesPost: (project: string, projectsProjectPackagesRepositoriesGetRequest: ProjectsProjectPackagesRepositoriesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryCleanupDryPost: (project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest: ProjectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryCleanupPost: (project: string, repository: string, $fields?: string, projectsProjectPackagesRepositoriesRepositoryCleanupPostRequest?: ProjectsProjectPackagesRepositoriesRepositoryCleanupPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet: (project: string, repository: string, connectionId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishPost: (project: string, repository: string, connectionId: string, projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest: ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsGet: (project: string, repository: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryDelete: (project: string, repository: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryFilesFolderfolderPathDelete: (project: string, repository: string, folderPath: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryFilesGet: (project: string, repository: string, parentPath?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathDelete: (project: string, repository: string, filePath: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathGet: (project: string, repository: string, filePath: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryGet: (project: string, repository: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesGet: (project: string, repository: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameDelete: (project: string, repository: string, packageName: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataGet: (project: string, repository: string, packageName: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataPut: (project: string, repository: string, packageName: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest?: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataVersionpackageVersionPut: (project: string, repository: string, packageName: string, packageVersion: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsGet: (project: string, repository: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, connectionId?: string, pinned?: boolean, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionDelete: (project: string, repository: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionGet: (project: string, repository: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryPatch: (project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryDeleteRequest?: ProjectsProjectPackagesRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesRepositoryUrlGet: (project: string, repository: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesGet: (project: string, type: string, repositoryName: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsGet: (project: string, type: string, repositoryName: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionDelete: (project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionGet: (project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesSearchGet: (project: string, type: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPackagesTypesGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPatch: (project: string, $fields?: string, projectsProjectDeleteRequest?: ProjectsProjectDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningBoardsBoardArchiveDelete: (project: string, board: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningBoardsGet: (project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningBoardsPost: (project: string, projectsProjectPlanningBoardsGetRequest: ProjectsProjectPlanningBoardsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningBoardsSprintsGet: (project: string, $skip?: string, $top?: number, board?: string, query?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningBoardsStarredGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsChecklistIdDelete: (project: string, checklistId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsChecklistIdFullChecklistTreeGet: (project: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsChecklistIdImportPost: (project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsChecklistIdPatch: (project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdDeleteRequest?: ProjectsProjectPlanningChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsGet: (project: string, $skip?: string, $top?: number, query?: string, sorting?: ChecklistSorting, descending?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsImportPost: (project: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsPost: (project: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningChecklistsStarredGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesGet: (project: string, sorting: IssuesSorting, descending: boolean, $skip?: string, $top?: number, assigneeId?: Array<string>, createdByProfileId?: string, createdBy?: Array<string>, statuses?: Array<string>, tagId?: string, query?: string, tags?: Array<string>, sprints?: Array<string>, boards?: Array<string>, customFields?: Array<string>, importTransaction?: string, creationTimeFrom?: string, creationTimeTo?: string, dueDateFrom?: string, dueDateTo?: string, topics?: Array<string>, grouping?: IssueListGrouping, deployment?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesImportPost: (project: string, projectsProjectPlanningIssuesImportPostRequest: ProjectsProjectPlanningIssuesImportPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdAttachmentsDelete: (project: string, issueId: string, identities: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdAttachmentsPost: (project: string, issueId: string, projectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdDelete: (project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdPost: (project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsDelete: (project: string, issueId: string, codeReviewIds: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsPost: (project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdCommentsImportPost: (project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommentsImportPostRequest: ProjectsProjectPlanningIssuesIssueIdCommentsImportPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdCommitsDelete: (project: string, issueId: string, repository: string, commitIds: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdCommitsPost: (project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommitsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCommitsDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdDelete: (project: string, issueId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdGet: (project: string, issueId: string, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdPatch: (project: string, issueId: string, projectsProjectPlanningIssuesIssueIdDeleteRequest?: ProjectsProjectPlanningIssuesIssueIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdRestorePost: (project: string, issueId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdDelete: (project: string, issueId: string, tagId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdPost: (project: string, issueId: string, tagId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesIssueIdToggleResolvedPost: (project: string, issueId: string, projectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest: ProjectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesNumbernumberGet: (project: string, number: number, resolveAlias?: boolean, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesPost: (project: string, projectsProjectPlanningIssuesGetRequest: ProjectsProjectPlanningIssuesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergePatch: (project: string, projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest?: ProjectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesStatusesDistributionGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesStatusesGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningIssuesStatusesPatch: (project: string, projectsProjectPlanningIssuesStatusesGetRequest: ProjectsProjectPlanningIssuesStatusesGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningTagsGet: (project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectPlanningTagsPost: (project: string, projectsProjectPlanningTagsGetRequest: ProjectsProjectPlanningTagsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryCherryPickCommitPost: (project: string, repository: string, projectsProjectRepositoriesRepositoryCherryPickCommitPostRequest: ProjectsProjectRepositoriesRepositoryCherryPickCommitPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryCommitBranchesGet: (project: string, repository: string, commit: string, prefix?: string, limit?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryCommitPost: (project: string, repository: string, projectsProjectRepositoriesRepositoryCommitPostRequest: ProjectsProjectRepositoriesRepositoryCommitPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryCommitsGet: (project: string, repository: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryDelete: (project: string, repository: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryDeleteBranchPost: (project: string, repository: string, projectsProjectRepositoriesRepositoryDeleteBranchPostRequest: ProjectsProjectRepositoriesRepositoryDeleteBranchPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryFilesGet: (project: string, repository: string, commit: string, path: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryGcPost: (project: string, repository: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryMergeBranchPost: (project: string, repository: string, projectsProjectRepositoriesRepositoryMergeBranchPostRequest: ProjectsProjectRepositoriesRepositoryMergeBranchPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryPost: (project: string, repository: string, $fields?: string, projectsProjectRepositoriesRepositoryDeleteRequest?: ProjectsProjectRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryReadonlyGet: (project: string, repository: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryReadonlyPost: (project: string, repository: string, projectsProjectRepositoriesRepositoryReadonlyGetRequest: ProjectsProjectRepositoriesRepositoryReadonlyGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryRebaseBranchPost: (project: string, repository: string, projectsProjectRepositoriesRepositoryRebaseBranchPostRequest: ProjectsProjectRepositoriesRepositoryRebaseBranchPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGet: (project: string, repository: string, revision: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksPost: (project: string, repository: string, revision: string, projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest: ProjectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositorySettingsGet: (project: string, repository: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositorySettingsPost: (project: string, repository: string, projectsProjectRepositoriesRepositorySettingsGetRequest: ProjectsProjectRepositoriesRepositorySettingsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectRepositoriesRepositoryUrlGet: (project: string, repository: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDelete: (project: string, responsibilityId: string, profileId: string, role?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdPost: (project: string, responsibilityId: string, profileId: string, projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest?: ProjectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectResponsibilitiesSchemeGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectResponsibilitiesSubjectsPost: (project: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectResponsibilitiesSubjectsSubjectIdPatch: (project: string, subjectId: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectTopicsGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectTopicsIdPatch: (project: string, id: string, $fields?: string, projectsProjectTopicsIdPatchRequest?: ProjectsProjectTopicsIdPatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectTopicsPost: (project: string, projectsProjectTopicsGetRequest: ProjectsProjectTopicsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectTopicsRemoveTopicsPost: (project: string, projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsProjectTopicsSetResponsiblePost: (project: string, projectsProjectTopicsSetResponsiblePostRequest: ProjectsProjectTopicsSetResponsiblePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsRepositoriesFindGet: (term: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsResponsibilitiesPost: (projectsResponsibilitiesPostRequest: ProjectsResponsibilitiesPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsResponsibilitiesResponsibilityIdDelete: (responsibilityId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsResponsibilitiesResponsibilityIdPatch: (responsibilityId: string, projectsResponsibilitiesResponsibilityIdDeleteRequest: ProjectsResponsibilitiesResponsibilityIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsResponsibilitiesSubjectsSubjectIdDelete: (subjectId: string, project?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsRightCoderightCodeGet: (rightCode: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsRightUniqueCoderightGet: (right: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, _private?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsSecretsDefaultBundleGet: (project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsSecretsDefaultBundlePost: (projectsSecretsDefaultBundleGetRequest: ProjectsSecretsDefaultBundleGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsSecretsIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsSecretsIdPatch: (id: string, projectsSecretsIdDeleteRequest: ProjectsSecretsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsSecretsInDefaultBundleGet: (projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsSecretsInDefaultBundlePost: (projectsSecretsInDefaultBundleGetRequest: ProjectsSecretsInDefaultBundleGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsTagsGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsTagsTrackAccessPost: (projectsTagsTrackAccessPostRequest: ProjectsTagsTrackAccessPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsTeamteamGet: (team: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsVaultGet: (project: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsVaultIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsVaultIdPatch: (id: string, projectsVaultIdDeleteRequest: ProjectsVaultIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    projectsVaultPost: (projectsVaultGetRequest: ProjectsVaultGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysCalendarsGet: ($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysCalendarsIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysCalendarsIdPatch: (id: string, publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysCalendarsImportPost: (publicHolidaysCalendarsImportPostRequest: PublicHolidaysCalendarsImportPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysCalendarsPost: (publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysHolidaysGet: ($skip?: string, $top?: number, calendar?: string, location?: string, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysHolidaysIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysHolidaysIdPatch: (id: string, $fields?: string, publicHolidaysHolidaysIdDeleteRequest?: PublicHolidaysHolidaysIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysHolidaysPost: (publicHolidaysHolidaysGetRequest: PublicHolidaysHolidaysGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysHolidaysProfileHolidaysGet: (startDate: string, endDate: string, profile: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    publicHolidaysHolidaysRelatedHolidaysGet: ($skip?: string, $top?: number, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    reactionsItemEmojiDelete: (item: string, emoji: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    reactionsItemEmojiGet: (item: string, emoji: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    reactionsItemEmojiPost: (item: string, emoji: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    reactionsItemGet: (item: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    richTextParseMarkdownPost: (richTextParseMarkdownPostRequest: RichTextParseMarkdownPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsAbsenceEventsGet: (dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsBirthdayEventsGet: (dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsBirthdayEventsStarredGet: (dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsGet: (dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsHolidaysGet: (startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsMeetingParticipationsIdPatch: (id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsMembershipEventsGet: (dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryCalendarEventsNonWorkingDaysEventsGet: (dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationLinksGet: ($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationLinksInvitationLinkIdDelete: (invitationLinkId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationLinksInvitationLinkIdPatch: (invitationLinkId: string, teamDirectoryInvitationLinksInvitationLinkIdDeleteRequest?: TeamDirectoryInvitationLinksInvitationLinkIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationLinksPost: (teamDirectoryInvitationLinksGetRequest: TeamDirectoryInvitationLinksGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationsGet: ($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationsIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationsIdPatch: (id: string, teamDirectoryInvitationsIdDeleteRequest?: TeamDirectoryInvitationsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryInvitationsPost: (teamDirectoryInvitationsGetRequest: TeamDirectoryInvitationsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLanguagesGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationEquipmentTypesGet: (withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationEquipmentTypesNamenameDelete: (name: string, _delete: boolean, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationMapMemberPointsGet: (locationId: string, $skip?: string, $top?: number, includeUnmarked?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationMapMemberPointsLocationPointIdDelete: (locationPointId: string, _delete: boolean, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationMapMemberPointsLocationPointIdPatch: (locationPointId: string, $fields?: string, teamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest?: TeamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationMapMemberPointsPost: (teamDirectoryLocationMapMemberPointsGetRequest: TeamDirectoryLocationMapMemberPointsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsGet: (query?: string, type?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsIdDelete: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsIdMapGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsIdMapPatch: (id: string, teamDirectoryLocationsIdMapGetRequest: TeamDirectoryLocationsIdMapGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsIdPatch: (id: string, $fields?: string, teamDirectoryLocationsIdDeleteRequest?: TeamDirectoryLocationsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsIdRestorePost: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsPost: (teamDirectoryLocationsGetRequest: TeamDirectoryLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsRestorePost: (projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryLocationsWithTimezoneGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMemberLocationsGet: ($skip?: string, $top?: number, profiles?: Array<string>, locations?: Array<string>, since?: string, till?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMemberLocationsMemberLocationIdDelete: (memberLocationId: string, _delete?: boolean, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMemberLocationsMemberLocationIdGet: (memberLocationId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMemberLocationsMemberLocationIdPatch: (memberLocationId: string, $fields?: string, teamDirectoryMemberLocationsMemberLocationIdDeleteRequest?: TeamDirectoryMemberLocationsMemberLocationIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMemberLocationsPost: (teamDirectoryMemberLocationsGetRequest: TeamDirectoryMemberLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipEventsGet: ($skip?: string, $top?: number, teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsGet: ($skip?: string, $top?: number, identifiers?: Array<string>, profiles?: Array<string>, teams?: Array<string>, directTeams?: boolean, roles?: Array<string>, directRoles?: boolean, since?: string, till?: string, requiresApproval?: boolean, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsManagerCandidatesGet: (term: string, $skip?: string, $top?: number, teamId?: string, excludedMemberId?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsMembershipIdDelete: (membershipId: string, _delete?: boolean, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsMembershipIdGet: (membershipId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsMembershipIdPatch: (membershipId: string, $fields?: string, teamDirectoryMembershipsMembershipIdDeleteRequest?: TeamDirectoryMembershipsMembershipIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsMembershipIdRequestRevokePatch: (membershipId: string, teamDirectoryMembershipsMembershipIdRequestRevokePatchRequest: TeamDirectoryMembershipsMembershipIdRequestRevokePatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsMembershipIdRevokeDelete: (membershipId: string, till?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsPost: (teamDirectoryMembershipsGetRequest: TeamDirectoryMembershipsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsRequestsGet: ($skip?: string, $top?: number, teamId?: string, direct?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsRequestsMembershipRequestIdDelete: (membershipRequestId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryMembershipsRequestsMembershipRequestIdPatch: (membershipRequestId: string, teamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest: TeamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesAuthenticationSessionsOwnerGet: (owner: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesAuthenticationSessionsOwnerSessionIdDelete: (owner: string, sessionId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesDashboardsDashboardGet: (dashboard: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesDashboardsDashboardPatch: (dashboard: string, teamDirectoryProfilesDashboardsDashboardGetRequest: TeamDirectoryProfilesDashboardsDashboardGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesEmailemailGet: (email: string, verified?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesGet: ($skip?: string, $top?: number, query?: string, reportPastMembers?: boolean, reportFutureMembers?: boolean, reportCurrentMembers?: boolean, teamId?: string, locationId?: string, roleId?: string, meOnTop?: boolean, order?: ProfileOrder, orgRelation?: ProfileOrgRelation, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesOauthConsentsOwnerApplicationsApplicationDelete: (owner: string, application: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesOauthConsentsOwnerApprovedScopesIdDelete: (owner: string, id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesOauthConsentsOwnerGet: (owner: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesOauthConsentsOwnerInternalApplicationsClientIdDelete: (owner: string, clientId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesOauthConsentsOwnerRefreshTokensIdDelete: (owner: string, id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesPost: (teamDirectoryProfilesGetRequest: TeamDirectoryProfilesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfile2FaRequirementsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfile2FaStatusGet: (profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfile2FaTotpConfirmPost: (profile: string, teamDirectoryProfilesProfile2FaTotpConfirmPostRequest: TeamDirectoryProfilesProfile2FaTotpConfirmPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfile2FaTotpDelete: (profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfile2FaTotpPatch: (profile: string, administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfile2FaTotpPost: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileApplicationPasswordsGet: (profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDelete: (profile: string, passwordId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdPatch: (profile: string, passwordId: string, teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest?: TeamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileApplicationPasswordsPost: (profile: string, teamDirectoryProfilesProfileApplicationPasswordsGetRequest: TeamDirectoryProfilesProfileApplicationPasswordsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsChecklistIdDelete: (profile: string, checklistId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsChecklistIdFullChecklistTreeGet: (profile: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsChecklistIdImportPost: (profile: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsChecklistIdPatch: (profile: string, checklistId: string, teamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest?: TeamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsImportPost: (profile: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsPost: (profile: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileChecklistsStarredGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileConvertToGuestPatch: (profile: string, teamDirectoryProfilesProfileConvertToGuestPatchRequest: TeamDirectoryProfilesProfileConvertToGuestPatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileConvertToMemberPatch: (profile: string, teamDirectoryProfilesProfileConvertToMemberPatchRequest: TeamDirectoryProfilesProfileConvertToMemberPatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDeactivateDelete: (profile: string, at: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDelete: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsDocumentIdCopyPost: (profile: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDelete: (profile: string, documentId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDeleteForeverDelete: (profile: string, documentId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsDocumentIdGet: (profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsDocumentIdMovePatch: (profile: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsDocumentIdPatch: (profile: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsDocumentIdUnarchivePatch: (profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDelete: (profile: string, folder: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDocumentsGet: (profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderGet: (profile: string, folder: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDelete: (profile: string, folder: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDocumentIdPatch: (profile: string, folder: string, documentId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderMovePatch: (profile: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderPatch: (profile: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderSubfoldersGet: (profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsFoldersPost: (profile: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileDocumentsPost: (profile: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileGpgKeysFingerprintDelete: (profile: string, fingerprint: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileGpgKeysFingerprintPatch: (profile: string, fingerprint: string, teamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest?: TeamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileGpgKeysGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileGpgKeysPost: (profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileIsTeamMemberGet: (profile: string, teamIds: Array<string>, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileLeadsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileNavBarMenuItemsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileNavBarMenuItemsPatch: (profile: string, teamDirectoryProfilesProfileNavBarMenuItemsGetRequest: TeamDirectoryProfilesProfileNavBarMenuItemsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileNavBarProjectsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileNavBarProjectsPost: (profile: string, teamDirectoryProfilesProfileNavBarProjectsGetRequest: TeamDirectoryProfilesProfileNavBarProjectsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileNavBarProjectsProjectDelete: (profile: string, project: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileNotificationSettingsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileNotificationSettingsPatch: (profile: string, teamDirectoryProfilesProfileNotificationSettingsGetRequest?: TeamDirectoryProfilesProfileNotificationSettingsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfilePatch: (profile: string, $fields?: string, teamDirectoryProfilesProfileDeleteRequest?: TeamDirectoryProfilesProfileDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfilePermanentTokensCurrentDelete: (profile: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfilePermanentTokensGet: (profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfilePermanentTokensPost: (profile: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdDelete: (profile: string, tokenId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdPatch: (profile: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileReactivatePatch: (profile: string, $fields?: string, teamDirectoryProfilesProfileReactivatePatchRequest?: TeamDirectoryProfilesProfileReactivatePatchRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSettingsGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSettingsPatch: (profile: string, teamDirectoryProfilesProfileSettingsGetRequest?: TeamDirectoryProfilesProfileSettingsGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSpokenLanguagesGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSpokenLanguagesLanguageDelete: (profile: string, language: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSpokenLanguagesPost: (profile: string, teamDirectoryProfilesProfileSpokenLanguagesGetRequest: TeamDirectoryProfilesProfileSpokenLanguagesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSshKeysFingerprintDelete: (profile: string, fingerprint: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSshKeysGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileSshKeysPost: (profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileTimezoneGet: (profile: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileWorkingDaysGet: (profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileWorkingDaysPost: (profile: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdDelete: (profile: string, workingDaysId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdPatch: (profile: string, workingDaysId: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryProfilesWorkingDaysGet: ($skip?: string, $top?: number, profiles?: Array<string>, since?: string, till?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryRolesGet: (query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryRolesIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryRolesIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryRolesIdPatch: (id: string, $fields?: string, teamDirectoryRolesIdDeleteRequest?: TeamDirectoryRolesIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryRolesIdRestorePost: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryRolesPost: (teamDirectoryRolesGetRequest: TeamDirectoryRolesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryStatsGet: (teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsGet: ($skip?: string, $top?: number, query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsIdCancelDisbandingPost: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsIdDelete: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsIdDirectMembersGet: (id: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsIdDisbandDelete: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsIdPatch: (id: string, $fields?: string, teamDirectoryTeamsIdDeleteRequest?: TeamDirectoryTeamsIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsIdRestorePost: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    teamDirectoryTeamsPost: (teamDirectoryTeamsGetRequest: TeamDirectoryTeamsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    timeTrackingItemsGet: (subject: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    timeTrackingItemsItemIdDelete: (itemId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    timeTrackingItemsItemIdPatch: (itemId: string, timeTrackingItemsItemIdDeleteRequest?: TimeTrackingItemsItemIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    timeTrackingItemsPost: (timeTrackingItemsGetRequest: TimeTrackingItemsGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    todoGet: ($skip?: string, $top?: number, open?: boolean, from?: string, till?: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    todoIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    todoIdPatch: (id: string, todoIdDeleteRequest?: TodoIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    todoPost: (todoGetRequest: TodoGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    trustedCertificatesGet: ($fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    trustedCertificatesIdDelete: (id: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    trustedCertificatesIdPatch: (id: string, trustedCertificatesIdDeleteRequest?: TrustedCertificatesIdDeleteRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    trustedCertificatesInfoGet: (data: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    trustedCertificatesPost: (trustedCertificatesGetRequest: TrustedCertificatesGetRequest, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    unfurlsBlockUnfurlGlobalPost: (unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    unfurlsBlockUnfurlPost: (unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    unfurlsCheckBlockedPost: (unfurlsCheckBlockedPostRequest: UnfurlsCheckBlockedPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    unfurlsListBlockedGet: ($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    unfurlsUnblockUnfurlGlobalPost: (unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    unfurlsUnblockUnfurlPost: (unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    uploadsChatPublicUrlChannelMessageAttachmentIdGet: (channel: string, message: string, attachmentId: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    uploadsImageIdGet: (id: string, $fields?: string, options?: AxiosRequestConfig) => Promise<RequestArgs>;
    uploadsPost: (uploadsPostRequest: UploadsPostRequest, options?: AxiosRequestConfig) => Promise<RequestArgs>;
};
export declare const SpaceApiFp: (configuration?: Configuration) => {
    absencesAbsenceReasonsGet(withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<AbsenceReasonRecord>>>;
    absencesAbsenceReasonsIdDelete(id: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    absencesAbsenceReasonsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AbsenceReasonRecord>>;
    absencesAbsenceReasonsIdPatch(id: string, absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AbsenceReasonRecord>>;
    absencesAbsenceReasonsPost(absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AbsenceReasonRecord>>;
    absencesGet($skip?: string, $top?: number, member?: string, members?: Array<string>, location?: string, team?: string, since?: string, till?: string, viewMode?: AbsenceListMode, reason?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AbsencesGet200Response>>;
    absencesIdApprovePost(id: string, absencesIdApprovePostRequest: AbsencesIdApprovePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    absencesIdDelete(id: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    absencesIdDeleteApprovalDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    absencesIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AbsenceRecord>>;
    absencesIdPatch(id: string, absencesIdDeleteRequest: AbsencesIdDeleteRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AbsenceRecord>>;
    absencesMembermemberGet(member: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<AbsenceRecord>>>;
    absencesPost(absencesGetRequest: AbsencesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AbsenceRecord>>;
    administrationSupportPost($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SupportProfileDTO>>;
    administrationUserAgreementEnabledGet(options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<boolean>>;
    administrationUserAgreementEnabledPost(administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    administrationUserAgreementGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<UAUserAgreement>>;
    administrationUserAgreementPatch(administrationUserAgreementGetRequest: AdministrationUserAgreementGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<UAUserAgreement>>;
    administrationUserAgreementStatusGet($skip?: string, $top?: number, query?: string, accepted?: boolean, activeProfilesOnly?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AdministrationUserAgreementStatusGet200Response>>;
    administrationUserAgreementStatusProfileGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<UAUserAgreementStatus>>;
    applicationsApplicationAuthorizationsAuthorizedContextsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PermissionContextApi>>>;
    applicationsApplicationAuthorizationsAuthorizedRightsDelete(application: string, contextIdentifier: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationAuthorizationsAuthorizedRightsGet(application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<RightDTO>>>;
    applicationsApplicationAuthorizationsAuthorizedRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationAuthorizationsRequiredRightsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<RightDTO>>>;
    applicationsApplicationAuthorizationsRequiredRightsPatch(application: string, applicationsApplicationAuthorizationsRequiredRightsGetRequest: ApplicationsApplicationAuthorizationsRequiredRightsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationBearerTokenGet(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    applicationsApplicationClientSecretGet(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    applicationsApplicationClientSecretRegeneratePost(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationDelete(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationForceRemovePost(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ESApp>>;
    applicationsApplicationLastClientCredentialsAccessGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AccessRecord>>;
    applicationsApplicationPatch(application: string, $fields?: string, applicationsApplicationDeleteRequest?: ApplicationsApplicationDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ESApp>>;
    applicationsApplicationPermanentTokensCurrentDelete(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationPermanentTokensGet(application: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ApplicationsApplicationPermanentTokensGet200Response>>;
    applicationsApplicationPermanentTokensPost(application: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ApplicationsApplicationPermanentTokensGet200Response1>>;
    applicationsApplicationPermanentTokensTokenIdDelete(application: string, tokenId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationPermanentTokensTokenIdPatch(application: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationPublicKeysGet(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    applicationsApplicationRestorePost(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationSigningKeyGet(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    applicationsApplicationSigningKeyRegeneratePost(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationSshKeysFingerprintDelete(application: string, fingerprint: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationSshKeysGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<SshKeyData>>>;
    applicationsApplicationSshKeysPost(application: string, applicationsApplicationSshKeysGetRequest: ApplicationsApplicationSshKeysGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationUiExtensionsDisableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationUiExtensionsDisableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationUiExtensionsEnableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationUiExtensionsEnableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationUiExtensionsGet(application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<AppUiExtensionApi>>>;
    applicationsApplicationUnfurlDomainsAuthorizePost(application: string, applicationsApplicationUnfurlDomainsAuthorizePostRequest: ApplicationsApplicationUnfurlDomainsAuthorizePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationUnfurlDomainsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ApplicationUnfurlDomain>>>;
    applicationsApplicationUnfurlPatternsAuthorizePost(application: string, applicationsApplicationUnfurlPatternsAuthorizePostRequest: ApplicationsApplicationUnfurlPatternsAuthorizePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationUnfurlPatternsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ApplicationUnfurlPattern>>>;
    applicationsApplicationVerificationTokenGet(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    applicationsApplicationVerificationTokenRegeneratePost(application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationWebhooksGet(application: string, withArchived?: boolean, query?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ApplicationsApplicationWebhooksGet200Response>>;
    applicationsApplicationWebhooksPost(application: string, applicationsApplicationWebhooksGetRequest: ApplicationsApplicationWebhooksGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<WebhookRecord>>;
    applicationsApplicationWebhooksWebhookIdBearerTokenGet(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersGet(application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<CustomHttpHeaderDTO>>>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest: ApplicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationWebhooksWebhookIdDelete(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationWebhooksWebhookIdPatch(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationWebhooksWebhookIdPost(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationWebhooksWebhookIdSigningKeyGet(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    applicationsApplicationWebhooksWebhookIdSigningKeyRegeneratePost(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsGet(application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<SubscriptionDTO>>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest: ApplicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SubscriptionDTO>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDelete(application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdPatch(application: string, webhookId: string, subscriptionId: string, $fields?: string, applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SubscriptionDTO>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdRequestMissingRightsPost(application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsAuthorizationsAuthorizedApplicationsGet(contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ESApp>>>;
    applicationsPagedGet($skip?: string, $top?: number, name?: string, owner?: Array<string>, withArchived?: boolean, withManaged?: boolean, ordering?: AppsOrdering, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ApplicationsPagedGet200Response>>;
    applicationsPost(applicationsPostRequest: ApplicationsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ESApp>>;
    applicationsUiExtensionsPatch(applicationsUiExtensionsPatchRequest: ApplicationsUiExtensionsPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsUnfurlsDomainsPatch(applicationsUnfurlsDomainsPatchRequest: ApplicationsUnfurlsDomainsPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsUnfurlsPatternsPatch(applicationsUnfurlsPatternsPatchRequest: ApplicationsUnfurlsPatternsPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsUnfurlsQueueContentPost(applicationsUnfurlsQueueContentPostRequest: ApplicationsUnfurlsQueueContentPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PostUnfurlContentResult>>>;
    applicationsUnfurlsQueueGet(batchSize: number, fromEtag?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ApplicationUnfurlQueueItem>>>;
    applicationsUnfurlsQueueRequestExternalAuthPost(applicationsUnfurlsQueueRequestExternalAuthPostRequest: ApplicationsUnfurlsQueueRequestExternalAuthPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    applicationsUnfurlsQueueResetExternalAuthRequestsPost(applicationsUnfurlsQueueResetExternalAuthRequestsPostRequest: ApplicationsUnfurlsQueueResetExternalAuthRequestsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesConfigDelete($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AuthConfig>>;
    authModulesConfigGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AuthConfig>>;
    authModulesConfigPut(authModulesConfigDeleteRequest?: AuthModulesConfigDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesDiscoverOidcGet(discoveryEndpoint: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<OIDCDiscovery>>;
    authModulesGet(withDisabled?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ESAuthModule>>>;
    authModulesIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesIdLoginsIdentifierChangePost(id: string, identifier: string, authModulesIdLoginsIdentifierChangePostRequest: AuthModulesIdLoginsIdentifierChangePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesIdLoginsIdentifierDelete(identifier: string, id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesIdLoginsIdentifierResetPost(id: string, identifier: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesIdPatch(id: string, authModulesIdDeleteRequest?: AuthModulesIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesIdSamlMetadataPost(id: string, authModulesIdSamlMetadataPostRequest: AuthModulesIdSamlMetadataPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SamlMetadataResponse>>;
    authModulesKeykeyGet(key: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ESAuthModule>>;
    authModulesPost(authModulesGetRequest: AuthModulesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ESAuthModule>>;
    authModulesReorderPost(authModulesReorderPostRequest: AuthModulesReorderPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesTestBuiltInPost(authModulesTestBuiltInPostRequest: AuthModulesTestBuiltInPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    authModulesTestLdapPost(authModulesTestLdapPostRequest: AuthModulesTestLdapPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ESDefaultProfileLoginDetails>>;
    authModulesThrottledLoginsDelete(logins: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesThrottledLoginsGet($skip?: string, $top?: number, login?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<AuthModulesThrottledLoginsDelete200Response>>;
    authModulesThrottledLoginsOrgStatusDelete(options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    authModulesThrottledLoginsOrgStatusGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<OrgThrottlingStatus>>;
    authModulesUsagesGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<AuthModuleUsage>>>;
    billingAdminOverdraftsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Overdrafts>>;
    billingAdminOverdraftsPut(billingAdminOverdraftsGetRequest: BillingAdminOverdraftsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    billingAdminReportsBillingPeriodGet(billingPeriod: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<BillingReport>>;
    billingAdminReportsTodayGet(date: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TodayBillingReport>>;
    billingAdminTrialPut(billingAdminTrialPutRequest: BillingAdminTrialPutRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    blogAliasaliasGet(alias: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ArticleRecord>>;
    blogExternalIdidGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ArticleRecord>>;
    blogGet($skip?: string, $top?: number, term?: string, dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, forProfile?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<BlogGet200Response>>;
    blogIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    blogIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ArticleRecord>>;
    blogIdPatch(id: string, $fields?: string, blogIdDeleteRequest?: BlogIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ArticleRecord>>;
    blogImportPost(blogImportPostRequest: BlogImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ArticleImportResult>>>;
    blogPost(blogGetRequest: BlogGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ArticleRecord>>;
    blogStatsGet(dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<BGStats>>;
    calendarsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<AbsenceEvent>>>;
    calendarsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<BirthdayEvent>>>;
    calendarsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<BirthdayEvent>>>;
    calendarsEventParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<MeetingRecord>>;
    calendarsEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<MeetingRecord>>>;
    calendarsEventsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<MeetingRecord>>;
    calendarsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<HolidaysEvent>>>;
    calendarsMeetingsGet($skip?: string, $top?: number, summaryQuery?: string, locationsQuery?: Array<string>, startingAfter?: string, endingAfter?: string, endingBefore?: string, startingBefore?: string, profiles?: Array<string>, teams?: Array<string>, organizer?: string, includePrivate?: boolean, includeArchived?: boolean, includeMeetingInstances?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CalendarsMeetingsGet200Response>>;
    calendarsMeetingsIdConferenceRoomsDelete(id: string, roomId: string, dateTime: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    calendarsMeetingsIdConferenceRoomsPost(id: string, calendarsMeetingsIdConferenceRoomsDeleteRequest: CalendarsMeetingsIdConferenceRoomsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    calendarsMeetingsIdDelete(id: string, targetDate?: string, modificationKind?: RecurrentModification, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DTOMeeting>>;
    calendarsMeetingsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DTOMeeting>>;
    calendarsMeetingsIdParticipationStatusPatch(id: string, calendarsMeetingsIdParticipationStatusPatchRequest: CalendarsMeetingsIdParticipationStatusPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DTOMeeting>>;
    calendarsMeetingsIdPatch(id: string, $fields?: string, calendarsMeetingsIdDeleteRequest?: CalendarsMeetingsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DTOMeeting>>;
    calendarsMeetingsNextOccurrenceGet(meetingId: string, since?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<MeetingOccurrenceTime>>;
    calendarsMeetingsOccurrencesByMeetingGet(meetingIds: Array<string>, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<MeetingWithOccurrenceTime>>>;
    calendarsMeetingsOccurrencesGet(meetingId: string, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<MeetingOccurrenceTime>>>;
    calendarsMeetingsParticipationStatusesExternalGet(id: string, emails: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<EventParticipationStatus>>>;
    calendarsMeetingsParticipationStatusesGet(id: string, profileIds: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<EventParticipationStatus>>>;
    calendarsMeetingsPost(calendarsMeetingsGetRequest: CalendarsMeetingsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DTOMeeting>>;
    calendarsMeetingsProfileParticipationGet(profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<CalendarsMeetingsProfileParticipationGet200ResponseInner>>>;
    calendarsMeetingsProfileParticipationRecordsGet(profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<DTOMeetingRSVP>>>;
    calendarsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<MembershipEvent>>>;
    calendarsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<NonWorkingDaysEvent>>>;
    callsPost(callsPostRequest: CallsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Room>>;
    chatsChannelsAllChannelsGet(query: string, $skip?: string, $top?: number, quickFilter?: AllChannelsFilter, sortColumn?: AllChannelsSortColumn, sortOrder?: ColumnSortOrder, publicOnly?: boolean, withArchived?: boolean, subscriber?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsAllChannelsGet200Response>>;
    chatsChannelsChannelAdministratorGet(channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CPrincipal>>;
    chatsChannelsChannelAdministratorPatch(channel: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelArchiveDelete(channel: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelAttachmentsFilesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>>;
    chatsChannelsChannelAttachmentsGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>>;
    chatsChannelsChannelAttachmentsImagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>>;
    chatsChannelsChannelAttachmentsLinksGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>>;
    chatsChannelsChannelAttachmentsVideosGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>>;
    chatsChannelsChannelDelete(channel: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelDescriptionPatch(channel: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelGet(channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<M2ChannelRecord>>;
    chatsChannelsChannelIconPatch(channel: string, chatsChannelsChannelIconPatchRequest?: ChatsChannelsChannelIconPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelNamePatch(channel: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelRestoreArchivedPost(channel: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelSubscribersTeamsDelete(channel: string, teams: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelSubscribersTeamsGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersTeamsDelete200Response>>;
    chatsChannelsChannelSubscribersTeamsPost(channel: string, chatsChannelsChannelSubscribersTeamsDeleteRequest: ChatsChannelsChannelSubscribersTeamsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelSubscribersUsersDelete(channel: string, profiles: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsChannelSubscribersUsersGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>>;
    chatsChannelsChannelSubscribersUsersPost(channel: string, chatsChannelsChannelSubscribersUsersDeleteRequest: ChatsChannelsChannelSubscribersUsersDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsConversationsChannelConvertPost(channel: string, chatsChannelsConversationsChannelConvertPostRequest: ChatsChannelsConversationsChannelConvertPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<M2ChannelRecord>>;
    chatsChannelsConversationsChannelSubjectPatch(channel: string, chatsChannelsConversationsChannelSubjectPatchRequest: ChatsChannelsConversationsChannelSubjectPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsChannelsConversationsPost(chatsChannelsConversationsPostRequest: ChatsChannelsConversationsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<M2ChannelRecord>>;
    chatsChannelsDmPost(chatsChannelsDmPostRequest: ChatsChannelsDmPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<M2ChannelRecord>>;
    chatsChannelsIsNameFreePost(chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<boolean>>;
    chatsChannelsPost(chatsChannelsPostRequest: ChatsChannelsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<M2ChannelRecord>>;
    chatsMessagesDeleteMessagePost(chatsMessagesDeleteMessagePostRequest: ChatsMessagesDeleteMessagePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsMessagesEditMessagePost(chatsMessagesEditMessagePostRequest: ChatsMessagesEditMessagePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsMessagesGet(channel: string, sorting: MessagesSorting, batchSize: number, startFromDate?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GetMessagesResponse>>;
    chatsMessagesImportPost(chatsMessagesImportPostRequest: ChatsMessagesImportPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsMessagesMessageGet(message: string, channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChannelItemRecord>>;
    chatsMessagesPinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    chatsMessagesPinnedMessagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsMessagesPinnedMessagesGet200Response>>;
    chatsMessagesSendMessagePost(chatsMessagesSendMessagePostRequest: ChatsMessagesSendMessagePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChannelItemRecord>>;
    chatsMessagesSendPost(chatsMessagesSendPostRequest: ChatsMessagesSendPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChannelItemRecord>>;
    chatsMessagesUnpinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    checklistsChecklistItemsPlanItemDelete(checklist: string, planItem: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    checklistsChecklistItemsPlanItemGet(checklist: string, planItem: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PlanItem>>;
    checklistsChecklistItemsPlanItemMovePost(checklist: string, planItem: string, checklistsChecklistItemsPlanItemMovePostRequest: ChecklistsChecklistItemsPlanItemMovePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PlanItem>>;
    checklistsChecklistItemsPlanItemPatch(checklist: string, planItem: string, $fields?: string, checklistsChecklistItemsPlanItemDeleteRequest?: ChecklistsChecklistItemsPlanItemDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PlanItem>>;
    checklistsChecklistItemsPost(checklist: string, checklistsChecklistItemsPostRequest: ChecklistsChecklistItemsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PlanItem>>;
    customFieldsExtendedTypesGet(scope?: ExtendedTypeScopeType, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ExtendedType>>>;
    customFieldsTypeKeyAllValuesGet(typeKey: string, $skip?: string, $top?: number, extendedEntityIds?: Array<string>, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldsTypeKeyAllValuesGet200Response>>;
    customFieldsTypeKeyEntityIdValuesGet(typeKey: string, entityId: string, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldsRecord>>;
    customFieldsTypeKeyEntityIdValuesPatch(entityId: string, typeKey: string, customFieldsTypeKeyEntityIdValuesGetRequest: CustomFieldsTypeKeyEntityIdValuesGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsTypeKeyEnumValuesCustomFieldIdGet(typeKey: string, customFieldId: string, $skip?: string, $top?: number, query?: string, ordering?: EnumValueOrdering, countRecords?: boolean, addedByProfileId?: string, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldsTypeKeyEnumValuesCustomFieldIdGet200Response>>;
    customFieldsTypeKeyEnumValuesCustomFieldIdPost(typeKey: string, customFieldId: string, customFieldsTypeKeyEnumValuesCustomFieldIdGetRequest: CustomFieldsTypeKeyEnumValuesCustomFieldIdGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<EnumValueData>>>;
    customFieldsTypeKeyFieldsGet(typeKey: string, withArchived?: boolean, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<CustomField>>>;
    customFieldsTypeKeyFieldsIdArchivePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsTypeKeyFieldsIdDelete(typeKey: string, id: string, scope: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsTypeKeyFieldsIdFilterValuesGet(typeKey: string, id: string, $skip?: string, $top?: number, scope?: string, calculateTotal?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldsTypeKeyFieldsIdFilterValuesGet200Response>>;
    customFieldsTypeKeyFieldsIdPatch(typeKey: string, id: string, customFieldsTypeKeyFieldsIdDeleteRequest: CustomFieldsTypeKeyFieldsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsTypeKeyFieldsIdRestorePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsTypeKeyFieldsPost(typeKey: string, customFieldsTypeKeyFieldsGetRequest: CustomFieldsTypeKeyFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomField>>;
    customFieldsTypeKeyFieldsReorderPost(typeKey: string, customFieldsTypeKeyFieldsReorderPostRequest: CustomFieldsTypeKeyFieldsReorderPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsCustomFieldArchivePost(entityType: string, customField: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsCustomFieldDelete(entityType: string, customField: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesEnumValueToRemoveDelete(entityType: string, customField: string, enumValueToRemove: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet(entityType: string, customField: string, query?: string, ordering?: EnumValueOrdering, addedByProfileId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet200Response>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CFEnumValue>>;
    customFieldsV2EntityTypeFieldsCustomFieldGet(entityType: string, customField: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldData>>;
    customFieldsV2EntityTypeFieldsCustomFieldPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldDeleteRequest?: CustomFieldsV2EntityTypeFieldsCustomFieldDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsCustomFieldRestorePost(entityType: string, customField: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2EntityTypeFieldsGet(entityType: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<CustomFieldData>>>;
    customFieldsV2EntityTypeFieldsPost(entityType: string, customFieldsV2EntityTypeFieldsGetRequest: CustomFieldsV2EntityTypeFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldData>>;
    customFieldsV2EntityTypeFieldsReorderPost(entityType: string, customFieldsV2EntityTypeFieldsReorderPostRequest: CustomFieldsV2EntityTypeFieldsReorderPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2ValuesEntityCustomFieldGet(entity: string, customField: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CustomFieldValueData>>;
    customFieldsV2ValuesEntityCustomFieldPost(entity: string, customField: string, customFieldsV2ValuesEntityCustomFieldGetRequest: CustomFieldsV2ValuesEntityCustomFieldGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    customFieldsV2ValuesEntityGet(entity: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<CustomFieldValueData>>>;
    customFieldsV2ValuesEntityPost(entity: string, customFieldsV2ValuesEntityGetRequest: CustomFieldsV2ValuesEntityGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    emojisAddPost(emojisAddPostRequest: EmojisAddPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    emojisDeletePost(emojisDeletePostRequest: EmojisDeletePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    emojisExistsGet(emoji: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<boolean>>;
    emojisFrequentlyUsedGet(options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<string>>>;
    emojisRecordUsagePost(emojisRecordUsagePostRequest: EmojisRecordUsagePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    emojisSearchGet(query: string, $skip?: string, $top?: number, version?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<EmojisSearchGet200Response>>;
    externalLinkPatternsDelete(pattern: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    externalLinkPatternsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ExternalLinkPattern>>>;
    externalLinkPatternsPost(externalLinkPatternsDeleteRequest: ExternalLinkPatternsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    httpApiModelGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<HAModel>>;
    notificationsChannelSubscriptionsGet(channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<SubscriptionDTO>>>;
    notificationsChannelSubscriptionsIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    notificationsChannelSubscriptionsIdPatch(id: string, $fields?: string, notificationsChannelSubscriptionsIdDeleteRequest?: NotificationsChannelSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SubscriptionDTO>>;
    notificationsChannelSubscriptionsIdRequestMissingRightsPost(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    notificationsChannelSubscriptionsPost(notificationsChannelSubscriptionsGetRequest: NotificationsChannelSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SubscriptionDTO>>;
    notificationsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<EventSubjectInfoDTO>>>;
    notificationsPersonalCustomSubscriptionsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<SubscriptionDTO>>>;
    notificationsPersonalCustomSubscriptionsIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    notificationsPersonalCustomSubscriptionsIdPatch(id: string, $fields?: string, notificationsPersonalCustomSubscriptionsIdDeleteRequest?: NotificationsPersonalCustomSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SubscriptionDTO>>;
    notificationsPersonalCustomSubscriptionsPost(notificationsPersonalCustomSubscriptionsGetRequest: NotificationsPersonalCustomSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SubscriptionDTO>>;
    notificationsPersonalSubscriptionsAllPersonalSubscriptionTargetsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PersonalSubscriptionTarget>>>;
    notificationsPersonalSubscriptionsPersonalSubscriptionSettingsGet(profile: string, feed: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PersonalSubscriptionSettings>>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    notificationsPrivateFeedsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PrivateFeed>>>;
    notificationsPrivateFeedsIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    notificationsPrivateFeedsIdPatch(id: string, $fields?: string, notificationsPrivateFeedsIdDeleteRequest?: NotificationsPrivateFeedsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PrivateFeed>>;
    notificationsPrivateFeedsPost(notificationsPrivateFeedsGetRequest: NotificationsPrivateFeedsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PrivateFeed>>;
    organizationDomainsCheckGet(domain: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CDomainStatus>>;
    organizationDomainsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<OrgDomainDTO>>>;
    organizationDomainsPatch(organizationDomainsGetRequest: OrganizationDomainsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    organizationGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<OrganizationRecord>>;
    organizationJetSalesLicenseActivationUrlGet(options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    organizationJetSalesUrlGet(options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    organizationPatch(organizationGetRequest: OrganizationGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesCreatePost(permissionRolesCreatePostRequest: PermissionRolesCreatePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<RoleDTO>>;
    permissionRolesGetPost(applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<RoleDTO>>>;
    permissionRolesRoleId2FaRequirementGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TwoFactorAuthenticationRequirement>>;
    permissionRolesRoleId2FaRequirementPatch(roleId: string, permissionRolesRoleId2FaRequirementGetRequest?: PermissionRolesRoleId2FaRequirementGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdDelete(roleId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdPatch(roleId: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdPermissionsDelete(roleId: string, rightCodes: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdPermissionsGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<RightDTO>>>;
    permissionRolesRoleIdPermissionsPost(roleId: string, permissionRolesRoleIdPermissionsDeleteRequest: PermissionRolesRoleIdPermissionsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdProfilesGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDMemberProfile>>>;
    permissionRolesRoleIdProfilesProfileDelete(roleId: string, profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdProfilesProfilePost(roleId: string, profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdResetRolePermissionsToDefaultPost(roleId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdTeamsGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDTeam>>>;
    permissionRolesRoleIdTeamsTeamDelete(roleId: string, team: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionRolesRoleIdTeamsTeamPost(roleId: string, team: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    permissionsCheckPermissionPost(permissionsCheckPermissionPostRequest: PermissionsCheckPermissionPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<boolean>>;
    permissionsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DTORightsWithHierarchy>>;
    projectsAutomationDslEvaluationsConfigGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DslEvaluationConfig>>;
    projectsAutomationGraphExecutionsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<JobExecutionDTO>>;
    projectsAutomationGraphExecutionsIdStopPost(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsAutomationJobExecutionsCurrentGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<JobExecutionDTO>>;
    projectsAutomationJobsJobIdGet(jobId: string, project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<JobDTO>>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyDelete(stepExecId: string, key: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyGet(stepExecId: string, key: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyPatch(stepExecId: string, key: string, projectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest: ProjectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsAutomationStepExecutionsUsedParametersParamParameterIdGet(parameterId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ParameterLastUsageDTO>>>;
    projectsAutomationStepExecutionsUsedParametersSecretSecretIdGet(secretId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ParameterLastUsageDTO>>>;
    projectsAutomationSubscriptionsLegacyChannelsDelete(project?: string, jobId?: string, unsubscribedOnly?: boolean, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsCollaboratorprofileGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsGet200Response>>;
    projectsGet($skip?: string, $top?: number, term?: string, tag?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsGet200Response>>;
    projectsMembermemberGet(member: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsGet200Response>>;
    projectsParamsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsParamsDefaultBundleGet200Response>>;
    projectsParamsDefaultBundlePost(projectsParamsDefaultBundleGetRequest: ProjectsParamsDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsParamsIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsParamsIdPatch(id: string, projectsParamsIdDeleteRequest: ProjectsParamsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsParamsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsParamsDefaultBundleGet200Response>>;
    projectsParamsInDefaultBundlePost(projectsParamsInDefaultBundleGetRequest: ProjectsParamsInDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsPlanningBoardsBoardDelete(board: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsBoardGet(board: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<BoardRecord>>;
    projectsPlanningBoardsBoardIssuesGet(board: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>>;
    projectsPlanningBoardsBoardIssuesIssueDelete(issue: string, board: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsBoardIssuesIssuePost(issue: string, board: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsBoardPatch(board: string, projectsPlanningBoardsBoardDeleteRequest?: ProjectsPlanningBoardsBoardDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsSprintsPost(projectsPlanningBoardsSprintsPostRequest: ProjectsPlanningBoardsSprintsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SprintRecord>>;
    projectsPlanningBoardsSprintsSprintArchiveDelete(sprint: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsSprintsSprintIssuesGet(sprint: string, $skip?: string, $top?: number, unresolvedOnly?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>>;
    projectsPlanningBoardsSprintsSprintIssuesIssueDelete(issue: string, sprint: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsSprintsSprintIssuesIssuePost(issue: string, sprint: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsSprintsSprintLaunchPost(sprint: string, projectsPlanningBoardsSprintsSprintLaunchPostRequest: ProjectsPlanningBoardsSprintsSprintLaunchPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPlanningBoardsSprintsSprintPatch(sprint: string, projectsPlanningBoardsSprintsSprintPatchRequest?: ProjectsPlanningBoardsSprintsSprintPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsPost(projectsGetRequest: ProjectsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PRProject>>;
    projectsPrivateProjectsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PRPrivateProject>>>;
    projectsPrivateProjectsProjectRequestAccessPost(project: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessAdminsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDMemberProfile>>>;
    projectsProjectAccessAdminsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessAdminsProfilesProfileDelete(project: string, profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessAdminsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessAdminsTeamsTeamIdDelete(project: string, teamId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessCollaboratorsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDMemberProfile>>>;
    projectsProjectAccessCollaboratorsProfilesDelete(project: string, profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessCollaboratorsProfilesGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDMemberProfile>>>;
    projectsProjectAccessCollaboratorsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessCollaboratorsTeamsDelete(project: string, teamId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessCollaboratorsTeamsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDTeam>>>;
    projectsProjectAccessCollaboratorsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessMemberProfilesGet(project: string, $skip?: string, $top?: number, query?: string, includingAdmins?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>>;
    projectsProjectAccessMembersProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessMembersProfilesProfileDelete(project: string, profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessMembersTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessMembersTeamsTeamIdDelete(project: string, teamId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAccessViewersGet(project: string, term: string, $skip?: string, $top?: number, meOnTop?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>>;
    projectsProjectAutomationDeploymentTargetsFavoritesGet(project: string, sortBy?: string, sortOrder?: ColumnSortOrder, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<DeployTargetRecord>>>;
    projectsProjectAutomationDeploymentTargetsGet(project: string, search?: string, sortBy?: string, sortOrder?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectAutomationDeploymentTargetsGet200Response>>;
    projectsProjectAutomationDeploymentTargetsPost(project: string, projectsProjectAutomationDeploymentTargetsGetRequest: ProjectsProjectAutomationDeploymentTargetsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeployTargetRecord>>;
    projectsProjectAutomationDeploymentTargetsTargetDelete(project: string, target: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAutomationDeploymentTargetsTargetGet(project: string, target: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeployTargetRecord>>;
    projectsProjectAutomationDeploymentTargetsTargetPut(project: string, target: string, projectsProjectAutomationDeploymentTargetsTargetDeleteRequest?: ProjectsProjectAutomationDeploymentTargetsTargetDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAutomationDeploymentsFailPost(project: string, projectsProjectAutomationDeploymentsFailPostRequest: ProjectsProjectAutomationDeploymentsFailPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeploymentRecord>>;
    projectsProjectAutomationDeploymentsFinishPost(project: string, projectsProjectAutomationDeploymentsFinishPostRequest: ProjectsProjectAutomationDeploymentsFinishPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeploymentRecord>>;
    projectsProjectAutomationDeploymentsGet(project: string, targetIdentifier?: string, jobExecutionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectAutomationDeploymentsGet200Response>>;
    projectsProjectAutomationDeploymentsPut(project: string, projectsProjectAutomationDeploymentsGetRequest: ProjectsProjectAutomationDeploymentsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeploymentRecord>>;
    projectsProjectAutomationDeploymentsSchedulePost(project: string, projectsProjectAutomationDeploymentsSchedulePostRequest: ProjectsProjectAutomationDeploymentsSchedulePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeploymentRecord>>;
    projectsProjectAutomationDeploymentsStartPost(project: string, projectsProjectAutomationDeploymentsStartPostRequest: ProjectsProjectAutomationDeploymentsStartPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeploymentRecord>>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierDelete(project: string, targetIdentifier: string, deploymentIdentifier: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierGet(project: string, targetIdentifier: string, deploymentIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DeploymentRecord>>;
    projectsProjectAutomationGraphExecutionsGet(project: string, jobId: string, branchFilter?: string, statusFilter?: ExecutionStatus, jobTriggerFilter?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectAutomationGraphExecutionsGet200Response>>;
    projectsProjectAutomationJobsGet(project: string, repoFilter: string, branchFilter: string, trigger?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectAutomationJobsGet200Response>>;
    projectsProjectAutomationJobsJobIdStartPost(project: string, jobId: string, projectsProjectAutomationJobsJobIdStartPostRequest: ProjectsProjectAutomationJobsJobIdStartPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<LaunchResult>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPost(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdRejectSuggestedEditPost(project: string, discussionId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdReopenSuggestedEditPost(project: string, discussionId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatch(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsCodeDiscussionsPost(project: string, projectsProjectCodeReviewsCodeDiscussionsPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CodeDiscussionRecord>>;
    projectsProjectCodeReviewsCommitSetReviewPost(project: string, projectsProjectCodeReviewsCommitSetReviewPostRequest: ProjectsProjectCodeReviewsCommitSetReviewPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CommitSetReviewRecord>>;
    projectsProjectCodeReviewsGet(project: string, $skip?: string, $top?: number, state?: CodeReviewStateFilter, text?: string, author?: string, from?: string, to?: string, sort?: ReviewSorting, reviewer?: string, type?: ReviewType, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectCodeReviewsGet200Response>>;
    projectsProjectCodeReviewsMergeRequestsPost(project: string, projectsProjectCodeReviewsMergeRequestsPostRequest: ProjectsProjectCodeReviewsMergeRequestsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<MergeRequestRecord>>;
    projectsProjectCodeReviewsReviewIdDescriptionPatch(project: string, reviewId: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsReviewIdDetailsGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CodeReviewDetailedInfo>>;
    projectsProjectCodeReviewsReviewIdFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectCodeReviewsReviewIdFilesGet200Response>>;
    projectsProjectCodeReviewsReviewIdGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CodeReviewRecord>>;
    projectsProjectCodeReviewsReviewIdMergeFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectCodeReviewsReviewIdMergeFilesGet200Response>>;
    projectsProjectCodeReviewsReviewIdMergePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdMergePutRequest: ProjectsProjectCodeReviewsReviewIdMergePutRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GitMergeResultHttp>>;
    projectsProjectCodeReviewsReviewIdParticipantsUserDelete(project: string, reviewId: string, user: string, role: CodeReviewParticipantRole, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsReviewIdParticipantsUserPost(project: string, reviewId: string, user: string, projectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest: ProjectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsReviewIdRebasePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRebasePutRequest: ProjectsProjectCodeReviewsReviewIdRebasePutRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GitRebaseResultHttp>>;
    projectsProjectCodeReviewsReviewIdRevisionsDelete(project: string, reviewId: string, revisions: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsReviewIdRevisionsPost(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRevisionsDeleteRequest: ProjectsProjectCodeReviewsReviewIdRevisionsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsReviewIdStatePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdStatePatchRequest: ProjectsProjectCodeReviewsReviewIdStatePatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectCodeReviewsReviewIdSuggestedReviewersGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDMemberProfile>>>;
    projectsProjectCodeReviewsReviewIdTitlePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdTitlePatchRequest: ProjectsProjectCodeReviewsReviewIdTitlePatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDelete(project: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDocumentsDocumentIdCopyPost(project: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    projectsProjectDocumentsDocumentIdDelete(project: string, documentId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDocumentsDocumentIdDeleteForeverDelete(project: string, documentId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDocumentsDocumentIdGet(project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    projectsProjectDocumentsDocumentIdMovePatch(project: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    projectsProjectDocumentsDocumentIdPatch(project: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    projectsProjectDocumentsDocumentIdUnarchivePatch(project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    projectsProjectDocumentsFoldersFolderDelete(project: string, folder: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDocumentsFoldersFolderDocumentsGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response>>;
    projectsProjectDocumentsFoldersFolderGet(project: string, folder: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DocumentFolder>>;
    projectsProjectDocumentsFoldersFolderIntroductionDelete(project: string, folder: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDocumentsFoldersFolderIntroductionDocumentIdPatch(project: string, folder: string, documentId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDocumentsFoldersFolderMovePatch(project: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DocumentFolder>>;
    projectsProjectDocumentsFoldersFolderPatch(project: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectDocumentsFoldersFolderSubfoldersGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response>>;
    projectsProjectDocumentsFoldersPost(project: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DocumentFolder>>;
    projectsProjectDocumentsPost(project: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    projectsProjectGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PRProject>>;
    projectsProjectPackagesRepositoriesGet(project: string, type?: string, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ProjectPackageRepository>>>;
    projectsProjectPackagesRepositoriesPost(project: string, projectsProjectPackagesRepositoriesGetRequest: ProjectsProjectPackagesRepositoriesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectPackageRepository>>;
    projectsProjectPackagesRepositoriesRepositoryCleanupDryPost(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest: ProjectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DryCleanupResults>>;
    projectsProjectPackagesRepositoriesRepositoryCleanupPost(project: string, repository: string, $fields?: string, projectsProjectPackagesRepositoriesRepositoryCleanupPostRequest?: ProjectsProjectPackagesRepositoriesRepositoryCleanupPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PackagesExecutionResult>>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet(project: string, repository: string, connectionId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet200Response>>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishPost(project: string, repository: string, connectionId: string, projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest: ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PackageRepositoryConnection>>>;
    projectsProjectPackagesRepositoriesRepositoryDelete(project: string, repository: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryFilesFolderfolderPathDelete(project: string, repository: string, folderPath: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryFilesGet(project: string, repository: string, parentPath?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<FileData>>>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathDelete(project: string, repository: string, filePath: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathGet(project: string, repository: string, filePath: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<FileDetails>>;
    projectsProjectPackagesRepositoriesRepositoryGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectPackageRepository>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesGet(project: string, repository: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameDelete(project: string, repository: string, packageName: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataGet(project: string, repository: string, packageName: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PackageMetadata>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataPut(project: string, repository: string, packageName: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest?: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataVersionpackageVersionPut(project: string, repository: string, packageName: string, packageVersion: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsGet(project: string, repository: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, connectionId?: string, pinned?: boolean, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, repository: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, repository: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PackageVersionDetails>>;
    projectsProjectPackagesRepositoriesRepositoryPatch(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryDeleteRequest?: ProjectsProjectPackagesRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesRepositoryUrlGet(project: string, repository: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesGet(project: string, type: string, repositoryName: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsGet(project: string, type: string, repositoryName: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PackageVersionDetails>>;
    projectsProjectPackagesSearchGet(project: string, type: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>>;
    projectsProjectPackagesTypesGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PackageType>>>;
    projectsProjectPatch(project: string, $fields?: string, projectsProjectDeleteRequest?: ProjectsProjectDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PRProject>>;
    projectsProjectPlanningBoardsBoardArchiveDelete(project: string, board: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningBoardsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPlanningBoardsGet200Response>>;
    projectsProjectPlanningBoardsPost(project: string, projectsProjectPlanningBoardsGetRequest: ProjectsProjectPlanningBoardsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<BoardRecord>>;
    projectsProjectPlanningBoardsSprintsGet(project: string, $skip?: string, $top?: number, board?: string, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPlanningBoardsSprintsGet200Response>>;
    projectsProjectPlanningBoardsStarredGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<BoardRecord>>>;
    projectsProjectPlanningChecklistsChecklistIdDelete(project: string, checklistId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningChecklistsChecklistIdFullChecklistTreeGet(project: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PlanItemChildren>>>;
    projectsProjectPlanningChecklistsChecklistIdImportPost(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningChecklistsChecklistIdPatch(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdDeleteRequest?: ProjectsProjectPlanningChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningChecklistsGet(project: string, $skip?: string, $top?: number, query?: string, sorting?: ChecklistSorting, descending?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPlanningChecklistsGet200Response>>;
    projectsProjectPlanningChecklistsImportPost(project: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Checklist>>;
    projectsProjectPlanningChecklistsPost(project: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Checklist>>;
    projectsProjectPlanningChecklistsStarredGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<Checklist>>>;
    projectsProjectPlanningIssuesGet(project: string, sorting: IssuesSorting, descending: boolean, $skip?: string, $top?: number, assigneeId?: Array<string>, createdByProfileId?: string, createdBy?: Array<string>, statuses?: Array<string>, tagId?: string, query?: string, tags?: Array<string>, sprints?: Array<string>, boards?: Array<string>, customFields?: Array<string>, importTransaction?: string, creationTimeFrom?: string, creationTimeTo?: string, dueDateFrom?: string, dueDateTo?: string, topics?: Array<string>, grouping?: IssueListGrouping, deployment?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>>;
    projectsProjectPlanningIssuesImportPost(project: string, projectsProjectPlanningIssuesImportPostRequest: ProjectsProjectPlanningIssuesImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<IssueImportResult>>;
    projectsProjectPlanningIssuesIssueIdAttachmentsDelete(project: string, issueId: string, identities: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdAttachmentsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdDelete(project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdPost(project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsDelete(project: string, issueId: string, codeReviewIds: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdCommentsImportPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommentsImportPostRequest: ProjectsProjectPlanningIssuesIssueIdCommentsImportPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<string>>>;
    projectsProjectPlanningIssuesIssueIdCommitsDelete(project: string, issueId: string, repository: string, commitIds: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdCommitsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommitsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCommitsDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdDelete(project: string, issueId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdGet(project: string, issueId: string, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Issue>>;
    projectsProjectPlanningIssuesIssueIdPatch(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdDeleteRequest?: ProjectsProjectPlanningIssuesIssueIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdRestorePost(project: string, issueId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdDelete(project: string, issueId: string, tagId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdPost(project: string, issueId: string, tagId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesIssueIdToggleResolvedPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest: ProjectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesNumbernumberGet(project: string, number: number, resolveAlias?: boolean, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Issue>>;
    projectsProjectPlanningIssuesPost(project: string, projectsProjectPlanningIssuesGetRequest: ProjectsProjectPlanningIssuesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Issue>>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<IssueStatus>>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergePatch(project: string, projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest?: ProjectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningIssuesStatusesDistributionGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<IssueStatusWithUsages>>>;
    projectsProjectPlanningIssuesStatusesGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<IssueStatus>>>;
    projectsProjectPlanningIssuesStatusesPatch(project: string, projectsProjectPlanningIssuesStatusesGetRequest: ProjectsProjectPlanningIssuesStatusesGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectPlanningTagsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectPlanningTagsGet200Response>>;
    projectsProjectPlanningTagsPost(project: string, projectsProjectPlanningTagsGetRequest: ProjectsProjectPlanningTagsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PlanningTag>>;
    projectsProjectRepositoriesRepositoryCherryPickCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCherryPickCommitPostRequest: ProjectsProjectRepositoriesRepositoryCherryPickCommitPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GitCherryPickResult>>;
    projectsProjectRepositoriesRepositoryCommitBranchesGet(project: string, repository: string, commit: string, prefix?: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<BranchInfo>>>;
    projectsProjectRepositoriesRepositoryCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCommitPostRequest: ProjectsProjectRepositoriesRepositoryCommitPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GitCommitResult>>;
    projectsProjectRepositoriesRepositoryCommitsGet(project: string, repository: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectRepositoriesRepositoryCommitsGet200Response>>;
    projectsProjectRepositoriesRepositoryDelete(project: string, repository: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectRepositoriesRepositoryDeleteBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryDeleteBranchPostRequest: ProjectsProjectRepositoriesRepositoryDeleteBranchPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectRepositoriesRepositoryFilesGet(project: string, repository: string, commit: string, path: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<GitFile>>>;
    projectsProjectRepositoriesRepositoryGcPost(project: string, repository: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectRepositoriesRepositoryMergeBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryMergeBranchPostRequest: ProjectsProjectRepositoriesRepositoryMergeBranchPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GitMergeBranchResult>>;
    projectsProjectRepositoriesRepositoryPost(project: string, repository: string, $fields?: string, projectsProjectRepositoriesRepositoryDeleteRequest?: ProjectsProjectRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PRRepositoryInfo>>;
    projectsProjectRepositoriesRepositoryReadonlyGet(project: string, repository: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<boolean>>;
    projectsProjectRepositoriesRepositoryReadonlyPost(project: string, repository: string, projectsProjectRepositoriesRepositoryReadonlyGetRequest: ProjectsProjectRepositoriesRepositoryReadonlyGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectRepositoriesRepositoryRebaseBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryRebaseBranchPostRequest: ProjectsProjectRepositoriesRepositoryRebaseBranchPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GitRebaseResult>>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGet(project: string, repository: string, revision: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ExternalCheckDTO>>>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksPost(project: string, repository: string, revision: string, projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest: ProjectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectRepositoriesRepositorySettingsGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GitRepositorySettings>>;
    projectsProjectRepositoriesRepositorySettingsPost(project: string, repository: string, projectsProjectRepositoriesRepositorySettingsGetRequest: ProjectsProjectRepositoriesRepositorySettingsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectRepositoriesRepositoryUrlGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<RepositoryUrls>>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDelete(project: string, responsibilityId: string, profileId: string, role?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdPost(project: string, responsibilityId: string, profileId: string, projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest?: ProjectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectResponsibilitiesSchemeGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<SubjectResponsibilitiesTable>>>;
    projectsProjectResponsibilitiesSubjectsPost(project: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsProjectResponsibilitiesSubjectsSubjectIdPatch(project: string, subjectId: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectTopicsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<Topic>>>;
    projectsProjectTopicsIdPatch(project: string, id: string, $fields?: string, projectsProjectTopicsIdPatchRequest?: ProjectsProjectTopicsIdPatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Topic>>;
    projectsProjectTopicsPost(project: string, projectsProjectTopicsGetRequest: ProjectsProjectTopicsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Topic>>;
    projectsProjectTopicsRemoveTopicsPost(project: string, projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsProjectTopicsSetResponsiblePost(project: string, projectsProjectTopicsSetResponsiblePostRequest: ProjectsProjectTopicsSetResponsiblePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsRepositoriesFindGet(term: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsRepositoriesFindGet200Response>>;
    projectsResponsibilitiesPost(projectsResponsibilitiesPostRequest: ProjectsResponsibilitiesPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsResponsibilitiesResponsibilityIdDelete(responsibilityId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsResponsibilitiesResponsibilityIdPatch(responsibilityId: string, projectsResponsibilitiesResponsibilityIdDeleteRequest: ProjectsResponsibilitiesResponsibilityIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsResponsibilitiesSubjectsSubjectIdDelete(subjectId: string, project?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsRightCoderightCodeGet(rightCode: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsGet200Response>>;
    projectsRightUniqueCoderightGet(right: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, _private?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsGet200Response>>;
    projectsSecretsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsSecretsDefaultBundleGet200Response>>;
    projectsSecretsDefaultBundlePost(projectsSecretsDefaultBundleGetRequest: ProjectsSecretsDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsSecretsIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsSecretsIdPatch(id: string, projectsSecretsIdDeleteRequest: ProjectsSecretsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsSecretsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsSecretsDefaultBundleGet200Response>>;
    projectsSecretsInDefaultBundlePost(projectsSecretsInDefaultBundleGetRequest: ProjectsSecretsInDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    projectsTagsGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PRTag>>>;
    projectsTagsTrackAccessPost(projectsTagsTrackAccessPostRequest: ProjectsTagsTrackAccessPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsTeamteamGet(team: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsGet200Response>>;
    projectsVaultGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<VaultConnectionRecord>>>;
    projectsVaultIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsVaultIdPatch(id: string, projectsVaultIdDeleteRequest: ProjectsVaultIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    projectsVaultPost(projectsVaultGetRequest: ProjectsVaultGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    publicHolidaysCalendarsGet($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PublicHolidaysCalendarsGet200Response>>;
    publicHolidaysCalendarsIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    publicHolidaysCalendarsIdPatch(id: string, publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PublicHolidayCalendarRecord>>;
    publicHolidaysCalendarsImportPost(publicHolidaysCalendarsImportPostRequest: PublicHolidaysCalendarsImportPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    publicHolidaysCalendarsPost(publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PublicHolidayCalendarRecord>>;
    publicHolidaysHolidaysGet($skip?: string, $top?: number, calendar?: string, location?: string, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PublicHolidaysHolidaysGet200Response>>;
    publicHolidaysHolidaysIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    publicHolidaysHolidaysIdPatch(id: string, $fields?: string, publicHolidaysHolidaysIdDeleteRequest?: PublicHolidaysHolidaysIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PublicHoliday>>;
    publicHolidaysHolidaysPost(publicHolidaysHolidaysGetRequest: PublicHolidaysHolidaysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PublicHoliday>>;
    publicHolidaysHolidaysProfileHolidaysGet(startDate: string, endDate: string, profile: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PublicHoliday>>>;
    publicHolidaysHolidaysRelatedHolidaysGet($skip?: string, $top?: number, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<PublicHolidaysHolidaysGet200Response>>;
    reactionsItemEmojiDelete(item: string, emoji: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    reactionsItemEmojiGet(item: string, emoji: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<CPrincipal>>>;
    reactionsItemEmojiPost(item: string, emoji: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    reactionsItemGet(item: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<EmojiReaction>>>;
    richTextParseMarkdownPost(richTextParseMarkdownPostRequest: RichTextParseMarkdownPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<RtDocument>>;
    teamDirectoryCalendarEventsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<AbsenceEvent>>>;
    teamDirectoryCalendarEventsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<BirthdayEvent>>>;
    teamDirectoryCalendarEventsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<BirthdayEvent>>>;
    teamDirectoryCalendarEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<MeetingRecord>>>;
    teamDirectoryCalendarEventsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<HolidaysEvent>>>;
    teamDirectoryCalendarEventsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<MeetingRecord>>;
    teamDirectoryCalendarEventsMeetingParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<MeetingRecord>>;
    teamDirectoryCalendarEventsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<MembershipEvent>>>;
    teamDirectoryCalendarEventsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<NonWorkingDaysEvent>>>;
    teamDirectoryInvitationLinksGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryInvitationLinksGet200Response>>;
    teamDirectoryInvitationLinksInvitationLinkIdDelete(invitationLinkId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryInvitationLinksInvitationLinkIdPatch(invitationLinkId: string, teamDirectoryInvitationLinksInvitationLinkIdDeleteRequest?: TeamDirectoryInvitationLinksInvitationLinkIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryInvitationLinksPost(teamDirectoryInvitationLinksGetRequest: TeamDirectoryInvitationLinksGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryInvitationLinksGet200Response1>>;
    teamDirectoryInvitationsGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryInvitationsGet200Response>>;
    teamDirectoryInvitationsIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryInvitationsIdPatch(id: string, teamDirectoryInvitationsIdDeleteRequest?: TeamDirectoryInvitationsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryInvitationsPost(teamDirectoryInvitationsGetRequest: TeamDirectoryInvitationsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Invitation>>;
    teamDirectoryLanguagesGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDLanguage>>>;
    teamDirectoryLocationEquipmentTypesGet(withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDLocationEquipmentType>>>;
    teamDirectoryLocationEquipmentTypesNamenameDelete(name: string, _delete: boolean, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryLocationMapMemberPointsGet(locationId: string, $skip?: string, $top?: number, includeUnmarked?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryLocationMapMemberPointsGet200Response>>;
    teamDirectoryLocationMapMemberPointsLocationPointIdDelete(locationPointId: string, _delete: boolean, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryLocationMapMemberPointsLocationPointIdPatch(locationPointId: string, $fields?: string, teamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest?: TeamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocationMapPoint>>;
    teamDirectoryLocationMapMemberPointsPost(teamDirectoryLocationMapMemberPointsGetRequest: TeamDirectoryLocationMapMemberPointsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocationMapPoint>>;
    teamDirectoryLocationsGet(query?: string, type?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDLocation>>>;
    teamDirectoryLocationsIdDelete(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDLocation>>>;
    teamDirectoryLocationsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocation>>;
    teamDirectoryLocationsIdMapGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocationMap>>;
    teamDirectoryLocationsIdMapPatch(id: string, teamDirectoryLocationsIdMapGetRequest: TeamDirectoryLocationsIdMapGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocationMap>>;
    teamDirectoryLocationsIdPatch(id: string, $fields?: string, teamDirectoryLocationsIdDeleteRequest?: TeamDirectoryLocationsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocation>>;
    teamDirectoryLocationsIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocation>>;
    teamDirectoryLocationsPost(teamDirectoryLocationsGetRequest: TeamDirectoryLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDLocation>>;
    teamDirectoryLocationsRestorePost(projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDLocation>>>;
    teamDirectoryLocationsWithTimezoneGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDLocationWithTimeZone>>>;
    teamDirectoryMemberLocationsGet($skip?: string, $top?: number, profiles?: Array<string>, locations?: Array<string>, since?: string, till?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryMemberLocationsGet200Response>>;
    teamDirectoryMemberLocationsMemberLocationIdDelete(memberLocationId: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryMemberLocationsMemberLocationIdGet(memberLocationId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberLocation>>;
    teamDirectoryMemberLocationsMemberLocationIdPatch(memberLocationId: string, $fields?: string, teamDirectoryMemberLocationsMemberLocationIdDeleteRequest?: TeamDirectoryMemberLocationsMemberLocationIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberLocation>>;
    teamDirectoryMemberLocationsPost(teamDirectoryMemberLocationsGetRequest: TeamDirectoryMemberLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberLocation>>;
    teamDirectoryMembershipEventsGet($skip?: string, $top?: number, teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryMembershipEventsGet200Response>>;
    teamDirectoryMembershipsGet($skip?: string, $top?: number, identifiers?: Array<string>, profiles?: Array<string>, teams?: Array<string>, directTeams?: boolean, roles?: Array<string>, directRoles?: boolean, since?: string, till?: string, requiresApproval?: boolean, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryMembershipsGet200Response>>;
    teamDirectoryMembershipsManagerCandidatesGet(term: string, $skip?: string, $top?: number, teamId?: string, excludedMemberId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>>;
    teamDirectoryMembershipsMembershipIdDelete(membershipId: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryMembershipsMembershipIdGet(membershipId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMembership>>;
    teamDirectoryMembershipsMembershipIdPatch(membershipId: string, $fields?: string, teamDirectoryMembershipsMembershipIdDeleteRequest?: TeamDirectoryMembershipsMembershipIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMembership>>;
    teamDirectoryMembershipsMembershipIdRequestRevokePatch(membershipId: string, teamDirectoryMembershipsMembershipIdRequestRevokePatchRequest: TeamDirectoryMembershipsMembershipIdRequestRevokePatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryMembershipsMembershipIdRevokeDelete(membershipId: string, till?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryMembershipsPost(teamDirectoryMembershipsGetRequest: TeamDirectoryMembershipsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMembership>>;
    teamDirectoryMembershipsRequestsGet($skip?: string, $top?: number, teamId?: string, direct?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryMembershipsRequestsGet200Response>>;
    teamDirectoryMembershipsRequestsMembershipRequestIdDelete(membershipRequestId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMembership>>;
    teamDirectoryMembershipsRequestsMembershipRequestIdPatch(membershipRequestId: string, teamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest: TeamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesAuthenticationSessionsOwnerGet(owner: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ESAuthenticationSession>>>;
    teamDirectoryProfilesAuthenticationSessionsOwnerSessionIdDelete(owner: string, sessionId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesDashboardsDashboardGet(dashboard: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DashboardPreferencesRecord>>;
    teamDirectoryProfilesDashboardsDashboardPatch(dashboard: string, teamDirectoryProfilesDashboardsDashboardGetRequest: TeamDirectoryProfilesDashboardsDashboardGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesEmailemailGet(email: string, verified?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    teamDirectoryProfilesGet($skip?: string, $top?: number, query?: string, reportPastMembers?: boolean, reportFutureMembers?: boolean, reportCurrentMembers?: boolean, teamId?: string, locationId?: string, roleId?: string, meOnTop?: boolean, order?: ProfileOrder, orgRelation?: ProfileOrgRelation, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>>;
    teamDirectoryProfilesOauthConsentsOwnerApplicationsApplicationDelete(owner: string, application: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesOauthConsentsOwnerApprovedScopesIdDelete(owner: string, id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesOauthConsentsOwnerGet(owner: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<ESOAuthConsent>>>;
    teamDirectoryProfilesOauthConsentsOwnerInternalApplicationsClientIdDelete(owner: string, clientId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesOauthConsentsOwnerRefreshTokensIdDelete(owner: string, id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesPost(teamDirectoryProfilesGetRequest: TeamDirectoryProfilesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    teamDirectoryProfilesProfile2FaRequirementsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<Profile2FARequirement>>>;
    teamDirectoryProfilesProfile2FaStatusGet(profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TwoFactorAuthenticationStatus>>;
    teamDirectoryProfilesProfile2FaTotpConfirmPost(profile: string, teamDirectoryProfilesProfile2FaTotpConfirmPostRequest: TeamDirectoryProfilesProfile2FaTotpConfirmPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfile2FaTotpDelete(profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfile2FaTotpPatch(profile: string, administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfile2FaTotpPost(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TwoFactorAuthenticationSecret>>;
    teamDirectoryProfilesProfileApplicationPasswordsGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response>>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDelete(profile: string, passwordId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdPatch(profile: string, passwordId: string, teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest?: TeamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileApplicationPasswordsPost(profile: string, teamDirectoryProfilesProfileApplicationPasswordsGetRequest: TeamDirectoryProfilesProfileApplicationPasswordsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response1>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdDelete(profile: string, checklistId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdFullChecklistTreeGet(profile: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PlanItemChildren>>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdImportPost(profile: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdPatch(profile: string, checklistId: string, teamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest?: TeamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileChecklistsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<Checklist>>>;
    teamDirectoryProfilesProfileChecklistsImportPost(profile: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Checklist>>;
    teamDirectoryProfilesProfileChecklistsPost(profile: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Checklist>>;
    teamDirectoryProfilesProfileChecklistsStarredGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<Checklist>>>;
    teamDirectoryProfilesProfileConvertToGuestPatch(profile: string, teamDirectoryProfilesProfileConvertToGuestPatchRequest: TeamDirectoryProfilesProfileConvertToGuestPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DryRunResult>>;
    teamDirectoryProfilesProfileConvertToMemberPatch(profile: string, teamDirectoryProfilesProfileConvertToMemberPatchRequest: TeamDirectoryProfilesProfileConvertToMemberPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DryRunResult>>;
    teamDirectoryProfilesProfileDeactivateDelete(profile: string, at: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    teamDirectoryProfilesProfileDelete(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdCopyPost(profile: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDelete(profile: string, documentId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDeleteForeverDelete(profile: string, documentId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdGet(profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdMovePatch(profile: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdPatch(profile: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdUnarchivePatch(profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDelete(profile: string, folder: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDocumentsGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderGet(profile: string, folder: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DocumentFolder>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDelete(profile: string, folder: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDocumentIdPatch(profile: string, folder: string, documentId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderMovePatch(profile: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DocumentFolder>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderPatch(profile: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderSubfoldersGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response>>;
    teamDirectoryProfilesProfileDocumentsFoldersPost(profile: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<DocumentFolder>>;
    teamDirectoryProfilesProfileDocumentsPost(profile: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Document>>;
    teamDirectoryProfilesProfileGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    teamDirectoryProfilesProfileGpgKeysFingerprintDelete(profile: string, fingerprint: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileGpgKeysFingerprintPatch(profile: string, fingerprint: string, teamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest?: TeamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileGpgKeysGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<GpgKeyData>>>;
    teamDirectoryProfilesProfileGpgKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GpgKeyData>>;
    teamDirectoryProfilesProfileIsTeamMemberGet(profile: string, teamIds: Array<string>, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<boolean>>;
    teamDirectoryProfilesProfileLeadsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDMemberProfile>>>;
    teamDirectoryProfilesProfileNavBarMenuItemsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<NavBarMenuItem>>>;
    teamDirectoryProfilesProfileNavBarMenuItemsPatch(profile: string, teamDirectoryProfilesProfileNavBarMenuItemsGetRequest: TeamDirectoryProfilesProfileNavBarMenuItemsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileNavBarProjectsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<PRProject>>>;
    teamDirectoryProfilesProfileNavBarProjectsPost(profile: string, teamDirectoryProfilesProfileNavBarProjectsGetRequest: TeamDirectoryProfilesProfileNavBarProjectsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileNavBarProjectsProjectDelete(profile: string, project: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileNotificationSettingsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<GlobalNotificationSettings>>;
    teamDirectoryProfilesProfileNotificationSettingsPatch(profile: string, teamDirectoryProfilesProfileNotificationSettingsGetRequest?: TeamDirectoryProfilesProfileNotificationSettingsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfilePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileDeleteRequest?: TeamDirectoryProfilesProfileDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    teamDirectoryProfilesProfilePermanentTokensCurrentDelete(profile: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfilePermanentTokensGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryProfilesProfilePermanentTokensGet200Response>>;
    teamDirectoryProfilesProfilePermanentTokensPost(profile: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryProfilesProfilePermanentTokensGet200Response1>>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdDelete(profile: string, tokenId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdPatch(profile: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileReactivatePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileReactivatePatchRequest?: TeamDirectoryProfilesProfileReactivatePatchRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDMemberProfile>>;
    teamDirectoryProfilesProfileSettingsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<SettingsValue>>;
    teamDirectoryProfilesProfileSettingsPatch(profile: string, teamDirectoryProfilesProfileSettingsGetRequest?: TeamDirectoryProfilesProfileSettingsGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileSpokenLanguagesGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDProfileLanguage>>>;
    teamDirectoryProfilesProfileSpokenLanguagesLanguageDelete(profile: string, language: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileSpokenLanguagesPost(profile: string, teamDirectoryProfilesProfileSpokenLanguagesGetRequest: TeamDirectoryProfilesProfileSpokenLanguagesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDProfileLanguage>>;
    teamDirectoryProfilesProfileSshKeysFingerprintDelete(profile: string, fingerprint: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileSshKeysGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<SshKeyData>>>;
    teamDirectoryProfilesProfileSshKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileTimezoneGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ATimeZone>>;
    teamDirectoryProfilesProfileWorkingDaysGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryProfilesProfileWorkingDaysGet200Response>>;
    teamDirectoryProfilesProfileWorkingDaysPost(profile: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDWorkingDays>>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdDelete(profile: string, workingDaysId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdPatch(profile: string, workingDaysId: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDWorkingDays>>;
    teamDirectoryProfilesWorkingDaysGet($skip?: string, $top?: number, profiles?: Array<string>, since?: string, till?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TeamDirectoryProfilesWorkingDaysGet200Response>>;
    teamDirectoryRolesGet(query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDRole>>>;
    teamDirectoryRolesIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryRolesIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDRole>>;
    teamDirectoryRolesIdPatch(id: string, $fields?: string, teamDirectoryRolesIdDeleteRequest?: TeamDirectoryRolesIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDRole>>;
    teamDirectoryRolesIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDRole>>;
    teamDirectoryRolesPost(teamDirectoryRolesGetRequest: TeamDirectoryRolesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDRole>>;
    teamDirectoryStatsGet(teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDStats>>;
    teamDirectoryTeamsGet($skip?: string, $top?: number, query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersTeamsDelete200Response>>;
    teamDirectoryTeamsIdCancelDisbandingPost(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    teamDirectoryTeamsIdDelete(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDTeam>>>;
    teamDirectoryTeamsIdDirectMembersGet(id: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>>;
    teamDirectoryTeamsIdDisbandDelete(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TDTeam>>>;
    teamDirectoryTeamsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDTeam>>;
    teamDirectoryTeamsIdPatch(id: string, $fields?: string, teamDirectoryTeamsIdDeleteRequest?: TeamDirectoryTeamsIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDTeam>>;
    teamDirectoryTeamsIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDTeam>>;
    teamDirectoryTeamsPost(teamDirectoryTeamsGetRequest: TeamDirectoryTeamsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TDTeam>>;
    timeTrackingItemsGet(subject: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TimeTrackingItemsGet200Response>>;
    timeTrackingItemsItemIdDelete(itemId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    timeTrackingItemsItemIdPatch(itemId: string, timeTrackingItemsItemIdDeleteRequest?: TimeTrackingItemsItemIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    timeTrackingItemsPost(timeTrackingItemsGetRequest: TimeTrackingItemsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TimeTrackingItem>>;
    todoGet($skip?: string, $top?: number, open?: boolean, from?: string, till?: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TodoGet200Response>>;
    todoIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    todoIdPatch(id: string, todoIdDeleteRequest?: TodoIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    todoPost(todoGetRequest: TodoGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TodoItemRecord>>;
    trustedCertificatesGet($fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<Array<TrustedCertificate>>>;
    trustedCertificatesIdDelete(id: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    trustedCertificatesIdPatch(id: string, trustedCertificatesIdDeleteRequest?: TrustedCertificatesIdDeleteRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    trustedCertificatesInfoGet(data: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<CertificateInfo>>;
    trustedCertificatesPost(trustedCertificatesGetRequest: TrustedCertificatesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<TrustedCertificate>>;
    unfurlsBlockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    unfurlsBlockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    unfurlsCheckBlockedPost(unfurlsCheckBlockedPostRequest: UnfurlsCheckBlockedPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<boolean>>;
    unfurlsListBlockedGet($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<UnfurlsListBlockedGet200Response>>;
    unfurlsUnblockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    unfurlsUnblockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<void>>;
    uploadsChatPublicUrlChannelMessageAttachmentIdGet(channel: string, message: string, attachmentId: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
    uploadsImageIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<ImageAttachmentMeta>>;
    uploadsPost(uploadsPostRequest: UploadsPostRequest, options?: AxiosRequestConfig): Promise<(axios?: AxiosInstance, basePath?: string) => AxiosPromise<string>>;
};
export declare const SpaceApiFactory: (configuration?: Configuration, basePath?: string, axios?: AxiosInstance) => {
    absencesAbsenceReasonsGet(withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<Array<AbsenceReasonRecord>>;
    absencesAbsenceReasonsIdDelete(id: string, _delete?: boolean, options?: any): AxiosPromise<void>;
    absencesAbsenceReasonsIdGet(id: string, $fields?: string, options?: any): AxiosPromise<AbsenceReasonRecord>;
    absencesAbsenceReasonsIdPatch(id: string, absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: any): AxiosPromise<AbsenceReasonRecord>;
    absencesAbsenceReasonsPost(absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: any): AxiosPromise<AbsenceReasonRecord>;
    absencesGet($skip?: string, $top?: number, member?: string, members?: Array<string>, location?: string, team?: string, since?: string, till?: string, viewMode?: AbsenceListMode, reason?: string, $fields?: string, options?: any): AxiosPromise<AbsencesGet200Response>;
    absencesIdApprovePost(id: string, absencesIdApprovePostRequest: AbsencesIdApprovePostRequest, options?: any): AxiosPromise<void>;
    absencesIdDelete(id: string, _delete?: boolean, options?: any): AxiosPromise<void>;
    absencesIdDeleteApprovalDelete(id: string, options?: any): AxiosPromise<void>;
    absencesIdGet(id: string, $fields?: string, options?: any): AxiosPromise<AbsenceRecord>;
    absencesIdPatch(id: string, absencesIdDeleteRequest: AbsencesIdDeleteRequest, $fields?: string, options?: any): AxiosPromise<AbsenceRecord>;
    absencesMembermemberGet(member: string, $fields?: string, options?: any): AxiosPromise<Array<AbsenceRecord>>;
    absencesPost(absencesGetRequest: AbsencesGetRequest, $fields?: string, options?: any): AxiosPromise<AbsenceRecord>;
    administrationSupportPost($fields?: string, options?: any): AxiosPromise<SupportProfileDTO>;
    administrationUserAgreementEnabledGet(options?: any): AxiosPromise<boolean>;
    administrationUserAgreementEnabledPost(administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: any): AxiosPromise<void>;
    administrationUserAgreementGet($fields?: string, options?: any): AxiosPromise<UAUserAgreement>;
    administrationUserAgreementPatch(administrationUserAgreementGetRequest: AdministrationUserAgreementGetRequest, $fields?: string, options?: any): AxiosPromise<UAUserAgreement>;
    administrationUserAgreementStatusGet($skip?: string, $top?: number, query?: string, accepted?: boolean, activeProfilesOnly?: boolean, $fields?: string, options?: any): AxiosPromise<AdministrationUserAgreementStatusGet200Response>;
    administrationUserAgreementStatusProfileGet(profile: string, $fields?: string, options?: any): AxiosPromise<UAUserAgreementStatus>;
    applicationsApplicationAuthorizationsAuthorizedContextsGet(application: string, $fields?: string, options?: any): AxiosPromise<Array<PermissionContextApi>>;
    applicationsApplicationAuthorizationsAuthorizedRightsDelete(application: string, contextIdentifier: string, options?: any): AxiosPromise<void>;
    applicationsApplicationAuthorizationsAuthorizedRightsGet(application: string, contextIdentifier: string, $fields?: string, options?: any): AxiosPromise<Array<RightDTO>>;
    applicationsApplicationAuthorizationsAuthorizedRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationAuthorizationsRequiredRightsGet(application: string, $fields?: string, options?: any): AxiosPromise<Array<RightDTO>>;
    applicationsApplicationAuthorizationsRequiredRightsPatch(application: string, applicationsApplicationAuthorizationsRequiredRightsGetRequest: ApplicationsApplicationAuthorizationsRequiredRightsGetRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationBearerTokenGet(application: string, options?: any): AxiosPromise<string>;
    applicationsApplicationClientSecretGet(application: string, options?: any): AxiosPromise<string>;
    applicationsApplicationClientSecretRegeneratePost(application: string, options?: any): AxiosPromise<void>;
    applicationsApplicationDelete(application: string, options?: any): AxiosPromise<void>;
    applicationsApplicationForceRemovePost(application: string, options?: any): AxiosPromise<void>;
    applicationsApplicationGet(application: string, $fields?: string, options?: any): AxiosPromise<ESApp>;
    applicationsApplicationLastClientCredentialsAccessGet(application: string, $fields?: string, options?: any): AxiosPromise<AccessRecord>;
    applicationsApplicationPatch(application: string, $fields?: string, applicationsApplicationDeleteRequest?: ApplicationsApplicationDeleteRequest, options?: any): AxiosPromise<ESApp>;
    applicationsApplicationPermanentTokensCurrentDelete(application: string, options?: any): AxiosPromise<void>;
    applicationsApplicationPermanentTokensGet(application: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ApplicationsApplicationPermanentTokensGet200Response>;
    applicationsApplicationPermanentTokensPost(application: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: any): AxiosPromise<ApplicationsApplicationPermanentTokensGet200Response1>;
    applicationsApplicationPermanentTokensTokenIdDelete(application: string, tokenId: string, options?: any): AxiosPromise<void>;
    applicationsApplicationPermanentTokensTokenIdPatch(application: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationPublicKeysGet(application: string, options?: any): AxiosPromise<string>;
    applicationsApplicationRestorePost(application: string, options?: any): AxiosPromise<void>;
    applicationsApplicationSigningKeyGet(application: string, options?: any): AxiosPromise<string>;
    applicationsApplicationSigningKeyRegeneratePost(application: string, options?: any): AxiosPromise<void>;
    applicationsApplicationSshKeysFingerprintDelete(application: string, fingerprint: string, options?: any): AxiosPromise<void>;
    applicationsApplicationSshKeysGet(application: string, $fields?: string, options?: any): AxiosPromise<Array<SshKeyData>>;
    applicationsApplicationSshKeysPost(application: string, applicationsApplicationSshKeysGetRequest: ApplicationsApplicationSshKeysGetRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationUiExtensionsDisableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationUiExtensionsDisableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationUiExtensionsEnableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationUiExtensionsEnableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationUiExtensionsGet(application: string, contextIdentifier: string, $fields?: string, options?: any): AxiosPromise<Array<AppUiExtensionApi>>;
    applicationsApplicationUnfurlDomainsAuthorizePost(application: string, applicationsApplicationUnfurlDomainsAuthorizePostRequest: ApplicationsApplicationUnfurlDomainsAuthorizePostRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationUnfurlDomainsGet(application: string, $fields?: string, options?: any): AxiosPromise<Array<ApplicationUnfurlDomain>>;
    applicationsApplicationUnfurlPatternsAuthorizePost(application: string, applicationsApplicationUnfurlPatternsAuthorizePostRequest: ApplicationsApplicationUnfurlPatternsAuthorizePostRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationUnfurlPatternsGet(application: string, $fields?: string, options?: any): AxiosPromise<Array<ApplicationUnfurlPattern>>;
    applicationsApplicationVerificationTokenGet(application: string, options?: any): AxiosPromise<string>;
    applicationsApplicationVerificationTokenRegeneratePost(application: string, options?: any): AxiosPromise<void>;
    applicationsApplicationWebhooksGet(application: string, withArchived?: boolean, query?: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ApplicationsApplicationWebhooksGet200Response>;
    applicationsApplicationWebhooksPost(application: string, applicationsApplicationWebhooksGetRequest: ApplicationsApplicationWebhooksGetRequest, $fields?: string, options?: any): AxiosPromise<WebhookRecord>;
    applicationsApplicationWebhooksWebhookIdBearerTokenGet(application: string, webhookId: string, options?: any): AxiosPromise<string>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersGet(application: string, webhookId: string, $fields?: string, options?: any): AxiosPromise<Array<CustomHttpHeaderDTO>>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest: ApplicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdDelete(application: string, webhookId: string, options?: any): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdPatch(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdDeleteRequest, options?: any): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdPost(application: string, webhookId: string, options?: any): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdSigningKeyGet(application: string, webhookId: string, options?: any): AxiosPromise<string>;
    applicationsApplicationWebhooksWebhookIdSigningKeyRegeneratePost(application: string, webhookId: string, options?: any): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsGet(application: string, webhookId: string, $fields?: string, options?: any): AxiosPromise<Array<SubscriptionDTO>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest: ApplicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest, $fields?: string, options?: any): AxiosPromise<SubscriptionDTO>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDelete(application: string, webhookId: string, subscriptionId: string, options?: any): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdPatch(application: string, webhookId: string, subscriptionId: string, $fields?: string, applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest, options?: any): AxiosPromise<SubscriptionDTO>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdRequestMissingRightsPost(application: string, webhookId: string, subscriptionId: string, options?: any): AxiosPromise<void>;
    applicationsAuthorizationsAuthorizedApplicationsGet(contextIdentifier: string, $fields?: string, options?: any): AxiosPromise<Array<ESApp>>;
    applicationsPagedGet($skip?: string, $top?: number, name?: string, owner?: Array<string>, withArchived?: boolean, withManaged?: boolean, ordering?: AppsOrdering, $fields?: string, options?: any): AxiosPromise<ApplicationsPagedGet200Response>;
    applicationsPost(applicationsPostRequest: ApplicationsPostRequest, $fields?: string, options?: any): AxiosPromise<ESApp>;
    applicationsUiExtensionsPatch(applicationsUiExtensionsPatchRequest: ApplicationsUiExtensionsPatchRequest, options?: any): AxiosPromise<void>;
    applicationsUnfurlsDomainsPatch(applicationsUnfurlsDomainsPatchRequest: ApplicationsUnfurlsDomainsPatchRequest, options?: any): AxiosPromise<void>;
    applicationsUnfurlsPatternsPatch(applicationsUnfurlsPatternsPatchRequest: ApplicationsUnfurlsPatternsPatchRequest, options?: any): AxiosPromise<void>;
    applicationsUnfurlsQueueContentPost(applicationsUnfurlsQueueContentPostRequest: ApplicationsUnfurlsQueueContentPostRequest, $fields?: string, options?: any): AxiosPromise<Array<PostUnfurlContentResult>>;
    applicationsUnfurlsQueueGet(batchSize: number, fromEtag?: number, $fields?: string, options?: any): AxiosPromise<Array<ApplicationUnfurlQueueItem>>;
    applicationsUnfurlsQueueRequestExternalAuthPost(applicationsUnfurlsQueueRequestExternalAuthPostRequest: ApplicationsUnfurlsQueueRequestExternalAuthPostRequest, options?: any): AxiosPromise<void>;
    applicationsUnfurlsQueueResetExternalAuthRequestsPost(applicationsUnfurlsQueueResetExternalAuthRequestsPostRequest: ApplicationsUnfurlsQueueResetExternalAuthRequestsPostRequest, options?: any): AxiosPromise<void>;
    authModulesConfigDelete($fields?: string, options?: any): AxiosPromise<AuthConfig>;
    authModulesConfigGet($fields?: string, options?: any): AxiosPromise<AuthConfig>;
    authModulesConfigPut(authModulesConfigDeleteRequest?: AuthModulesConfigDeleteRequest, options?: any): AxiosPromise<void>;
    authModulesDiscoverOidcGet(discoveryEndpoint: string, $fields?: string, options?: any): AxiosPromise<OIDCDiscovery>;
    authModulesGet(withDisabled?: boolean, $fields?: string, options?: any): AxiosPromise<Array<ESAuthModule>>;
    authModulesIdDelete(id: string, options?: any): AxiosPromise<void>;
    authModulesIdLoginsIdentifierChangePost(id: string, identifier: string, authModulesIdLoginsIdentifierChangePostRequest: AuthModulesIdLoginsIdentifierChangePostRequest, options?: any): AxiosPromise<void>;
    authModulesIdLoginsIdentifierDelete(identifier: string, id: string, options?: any): AxiosPromise<void>;
    authModulesIdLoginsIdentifierResetPost(id: string, identifier: string, options?: any): AxiosPromise<void>;
    authModulesIdPatch(id: string, authModulesIdDeleteRequest?: AuthModulesIdDeleteRequest, options?: any): AxiosPromise<void>;
    authModulesIdSamlMetadataPost(id: string, authModulesIdSamlMetadataPostRequest: AuthModulesIdSamlMetadataPostRequest, $fields?: string, options?: any): AxiosPromise<SamlMetadataResponse>;
    authModulesKeykeyGet(key: string, $fields?: string, options?: any): AxiosPromise<ESAuthModule>;
    authModulesPost(authModulesGetRequest: AuthModulesGetRequest, $fields?: string, options?: any): AxiosPromise<ESAuthModule>;
    authModulesReorderPost(authModulesReorderPostRequest: AuthModulesReorderPostRequest, options?: any): AxiosPromise<void>;
    authModulesTestBuiltInPost(authModulesTestBuiltInPostRequest: AuthModulesTestBuiltInPostRequest, $fields?: string, options?: any): AxiosPromise<TDMemberProfile>;
    authModulesTestLdapPost(authModulesTestLdapPostRequest: AuthModulesTestLdapPostRequest, $fields?: string, options?: any): AxiosPromise<ESDefaultProfileLoginDetails>;
    authModulesThrottledLoginsDelete(logins: Array<string>, options?: any): AxiosPromise<void>;
    authModulesThrottledLoginsGet($skip?: string, $top?: number, login?: string, $fields?: string, options?: any): AxiosPromise<AuthModulesThrottledLoginsDelete200Response>;
    authModulesThrottledLoginsOrgStatusDelete(options?: any): AxiosPromise<void>;
    authModulesThrottledLoginsOrgStatusGet($fields?: string, options?: any): AxiosPromise<OrgThrottlingStatus>;
    authModulesUsagesGet($fields?: string, options?: any): AxiosPromise<Array<AuthModuleUsage>>;
    billingAdminOverdraftsGet($fields?: string, options?: any): AxiosPromise<Overdrafts>;
    billingAdminOverdraftsPut(billingAdminOverdraftsGetRequest: BillingAdminOverdraftsGetRequest, options?: any): AxiosPromise<void>;
    billingAdminReportsBillingPeriodGet(billingPeriod: string, $fields?: string, options?: any): AxiosPromise<BillingReport>;
    billingAdminReportsTodayGet(date: string, $fields?: string, options?: any): AxiosPromise<TodayBillingReport>;
    billingAdminTrialPut(billingAdminTrialPutRequest: BillingAdminTrialPutRequest, options?: any): AxiosPromise<void>;
    blogAliasaliasGet(alias: string, $fields?: string, options?: any): AxiosPromise<ArticleRecord>;
    blogExternalIdidGet(id: string, $fields?: string, options?: any): AxiosPromise<ArticleRecord>;
    blogGet($skip?: string, $top?: number, term?: string, dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, forProfile?: string, $fields?: string, options?: any): AxiosPromise<BlogGet200Response>;
    blogIdDelete(id: string, options?: any): AxiosPromise<void>;
    blogIdGet(id: string, $fields?: string, options?: any): AxiosPromise<ArticleRecord>;
    blogIdPatch(id: string, $fields?: string, blogIdDeleteRequest?: BlogIdDeleteRequest, options?: any): AxiosPromise<ArticleRecord>;
    blogImportPost(blogImportPostRequest: BlogImportPostRequest, $fields?: string, options?: any): AxiosPromise<Array<ArticleImportResult>>;
    blogPost(blogGetRequest: BlogGetRequest, $fields?: string, options?: any): AxiosPromise<ArticleRecord>;
    blogStatsGet(dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, $fields?: string, options?: any): AxiosPromise<BGStats>;
    calendarsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<AbsenceEvent>>;
    calendarsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<BirthdayEvent>>;
    calendarsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: any): AxiosPromise<Array<BirthdayEvent>>;
    calendarsEventParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: any): AxiosPromise<MeetingRecord>;
    calendarsEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: any): AxiosPromise<Array<MeetingRecord>>;
    calendarsEventsIdGet(id: string, $fields?: string, options?: any): AxiosPromise<MeetingRecord>;
    calendarsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: any): AxiosPromise<Array<HolidaysEvent>>;
    calendarsMeetingsGet($skip?: string, $top?: number, summaryQuery?: string, locationsQuery?: Array<string>, startingAfter?: string, endingAfter?: string, endingBefore?: string, startingBefore?: string, profiles?: Array<string>, teams?: Array<string>, organizer?: string, includePrivate?: boolean, includeArchived?: boolean, includeMeetingInstances?: boolean, $fields?: string, options?: any): AxiosPromise<CalendarsMeetingsGet200Response>;
    calendarsMeetingsIdConferenceRoomsDelete(id: string, roomId: string, dateTime: string, options?: any): AxiosPromise<void>;
    calendarsMeetingsIdConferenceRoomsPost(id: string, calendarsMeetingsIdConferenceRoomsDeleteRequest: CalendarsMeetingsIdConferenceRoomsDeleteRequest, options?: any): AxiosPromise<void>;
    calendarsMeetingsIdDelete(id: string, targetDate?: string, modificationKind?: RecurrentModification, $fields?: string, options?: any): AxiosPromise<DTOMeeting>;
    calendarsMeetingsIdGet(id: string, $fields?: string, options?: any): AxiosPromise<DTOMeeting>;
    calendarsMeetingsIdParticipationStatusPatch(id: string, calendarsMeetingsIdParticipationStatusPatchRequest: CalendarsMeetingsIdParticipationStatusPatchRequest, $fields?: string, options?: any): AxiosPromise<DTOMeeting>;
    calendarsMeetingsIdPatch(id: string, $fields?: string, calendarsMeetingsIdDeleteRequest?: CalendarsMeetingsIdDeleteRequest, options?: any): AxiosPromise<DTOMeeting>;
    calendarsMeetingsNextOccurrenceGet(meetingId: string, since?: string, $fields?: string, options?: any): AxiosPromise<MeetingOccurrenceTime>;
    calendarsMeetingsOccurrencesByMeetingGet(meetingIds: Array<string>, since: string, until: string, limit?: number, $fields?: string, options?: any): AxiosPromise<Array<MeetingWithOccurrenceTime>>;
    calendarsMeetingsOccurrencesGet(meetingId: string, since: string, until: string, limit?: number, $fields?: string, options?: any): AxiosPromise<Array<MeetingOccurrenceTime>>;
    calendarsMeetingsParticipationStatusesExternalGet(id: string, emails: Array<string>, options?: any): AxiosPromise<Array<EventParticipationStatus>>;
    calendarsMeetingsParticipationStatusesGet(id: string, profileIds: Array<string>, options?: any): AxiosPromise<Array<EventParticipationStatus>>;
    calendarsMeetingsPost(calendarsMeetingsGetRequest: CalendarsMeetingsGetRequest, $fields?: string, options?: any): AxiosPromise<DTOMeeting>;
    calendarsMeetingsProfileParticipationGet(profileId: string, events: Array<string>, $fields?: string, options?: any): AxiosPromise<Array<CalendarsMeetingsProfileParticipationGet200ResponseInner>>;
    calendarsMeetingsProfileParticipationRecordsGet(profileId: string, events: Array<string>, $fields?: string, options?: any): AxiosPromise<Array<DTOMeetingRSVP>>;
    calendarsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<MembershipEvent>>;
    calendarsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<NonWorkingDaysEvent>>;
    callsPost(callsPostRequest: CallsPostRequest, $fields?: string, options?: any): AxiosPromise<Room>;
    chatsChannelsAllChannelsGet(query: string, $skip?: string, $top?: number, quickFilter?: AllChannelsFilter, sortColumn?: AllChannelsSortColumn, sortOrder?: ColumnSortOrder, publicOnly?: boolean, withArchived?: boolean, subscriber?: string, $fields?: string, options?: any): AxiosPromise<ChatsChannelsAllChannelsGet200Response>;
    chatsChannelsChannelAdministratorGet(channel: string, $fields?: string, options?: any): AxiosPromise<CPrincipal>;
    chatsChannelsChannelAdministratorPatch(channel: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: any): AxiosPromise<void>;
    chatsChannelsChannelArchiveDelete(channel: string, options?: any): AxiosPromise<void>;
    chatsChannelsChannelAttachmentsFilesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsImagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsLinksGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsVideosGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelDelete(channel: string, options?: any): AxiosPromise<void>;
    chatsChannelsChannelDescriptionPatch(channel: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: any): AxiosPromise<void>;
    chatsChannelsChannelGet(channel: string, $fields?: string, options?: any): AxiosPromise<M2ChannelRecord>;
    chatsChannelsChannelIconPatch(channel: string, chatsChannelsChannelIconPatchRequest?: ChatsChannelsChannelIconPatchRequest, options?: any): AxiosPromise<void>;
    chatsChannelsChannelNamePatch(channel: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: any): AxiosPromise<void>;
    chatsChannelsChannelRestoreArchivedPost(channel: string, options?: any): AxiosPromise<void>;
    chatsChannelsChannelSubscribersTeamsDelete(channel: string, teams: Array<string>, options?: any): AxiosPromise<void>;
    chatsChannelsChannelSubscribersTeamsGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersTeamsDelete200Response>;
    chatsChannelsChannelSubscribersTeamsPost(channel: string, chatsChannelsChannelSubscribersTeamsDeleteRequest: ChatsChannelsChannelSubscribersTeamsDeleteRequest, options?: any): AxiosPromise<void>;
    chatsChannelsChannelSubscribersUsersDelete(channel: string, profiles: Array<string>, options?: any): AxiosPromise<void>;
    chatsChannelsChannelSubscribersUsersGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    chatsChannelsChannelSubscribersUsersPost(channel: string, chatsChannelsChannelSubscribersUsersDeleteRequest: ChatsChannelsChannelSubscribersUsersDeleteRequest, options?: any): AxiosPromise<void>;
    chatsChannelsConversationsChannelConvertPost(channel: string, chatsChannelsConversationsChannelConvertPostRequest: ChatsChannelsConversationsChannelConvertPostRequest, $fields?: string, options?: any): AxiosPromise<M2ChannelRecord>;
    chatsChannelsConversationsChannelSubjectPatch(channel: string, chatsChannelsConversationsChannelSubjectPatchRequest: ChatsChannelsConversationsChannelSubjectPatchRequest, options?: any): AxiosPromise<void>;
    chatsChannelsConversationsPost(chatsChannelsConversationsPostRequest: ChatsChannelsConversationsPostRequest, $fields?: string, options?: any): AxiosPromise<M2ChannelRecord>;
    chatsChannelsDmPost(chatsChannelsDmPostRequest: ChatsChannelsDmPostRequest, $fields?: string, options?: any): AxiosPromise<M2ChannelRecord>;
    chatsChannelsIsNameFreePost(chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: any): AxiosPromise<boolean>;
    chatsChannelsPost(chatsChannelsPostRequest: ChatsChannelsPostRequest, $fields?: string, options?: any): AxiosPromise<M2ChannelRecord>;
    chatsMessagesDeleteMessagePost(chatsMessagesDeleteMessagePostRequest: ChatsMessagesDeleteMessagePostRequest, options?: any): AxiosPromise<void>;
    chatsMessagesEditMessagePost(chatsMessagesEditMessagePostRequest: ChatsMessagesEditMessagePostRequest, options?: any): AxiosPromise<void>;
    chatsMessagesGet(channel: string, sorting: MessagesSorting, batchSize: number, startFromDate?: string, $fields?: string, options?: any): AxiosPromise<GetMessagesResponse>;
    chatsMessagesImportPost(chatsMessagesImportPostRequest: ChatsMessagesImportPostRequest, options?: any): AxiosPromise<void>;
    chatsMessagesMessageGet(message: string, channel: string, $fields?: string, options?: any): AxiosPromise<ChannelItemRecord>;
    chatsMessagesPinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: any): AxiosPromise<void>;
    chatsMessagesPinnedMessagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ChatsMessagesPinnedMessagesGet200Response>;
    chatsMessagesSendMessagePost(chatsMessagesSendMessagePostRequest: ChatsMessagesSendMessagePostRequest, $fields?: string, options?: any): AxiosPromise<ChannelItemRecord>;
    chatsMessagesSendPost(chatsMessagesSendPostRequest: ChatsMessagesSendPostRequest, $fields?: string, options?: any): AxiosPromise<ChannelItemRecord>;
    chatsMessagesUnpinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: any): AxiosPromise<void>;
    checklistsChecklistItemsPlanItemDelete(checklist: string, planItem: string, options?: any): AxiosPromise<void>;
    checklistsChecklistItemsPlanItemGet(checklist: string, planItem: string, $fields?: string, options?: any): AxiosPromise<PlanItem>;
    checklistsChecklistItemsPlanItemMovePost(checklist: string, planItem: string, checklistsChecklistItemsPlanItemMovePostRequest: ChecklistsChecklistItemsPlanItemMovePostRequest, $fields?: string, options?: any): AxiosPromise<PlanItem>;
    checklistsChecklistItemsPlanItemPatch(checklist: string, planItem: string, $fields?: string, checklistsChecklistItemsPlanItemDeleteRequest?: ChecklistsChecklistItemsPlanItemDeleteRequest, options?: any): AxiosPromise<PlanItem>;
    checklistsChecklistItemsPost(checklist: string, checklistsChecklistItemsPostRequest: ChecklistsChecklistItemsPostRequest, $fields?: string, options?: any): AxiosPromise<PlanItem>;
    customFieldsExtendedTypesGet(scope?: ExtendedTypeScopeType, $fields?: string, options?: any): AxiosPromise<Array<ExtendedType>>;
    customFieldsTypeKeyAllValuesGet(typeKey: string, $skip?: string, $top?: number, extendedEntityIds?: Array<string>, scope?: string, $fields?: string, options?: any): AxiosPromise<CustomFieldsTypeKeyAllValuesGet200Response>;
    customFieldsTypeKeyEntityIdValuesGet(typeKey: string, entityId: string, scope?: string, $fields?: string, options?: any): AxiosPromise<CustomFieldsRecord>;
    customFieldsTypeKeyEntityIdValuesPatch(entityId: string, typeKey: string, customFieldsTypeKeyEntityIdValuesGetRequest: CustomFieldsTypeKeyEntityIdValuesGetRequest, options?: any): AxiosPromise<void>;
    customFieldsTypeKeyEnumValuesCustomFieldIdGet(typeKey: string, customFieldId: string, $skip?: string, $top?: number, query?: string, ordering?: EnumValueOrdering, countRecords?: boolean, addedByProfileId?: string, scope?: string, $fields?: string, options?: any): AxiosPromise<CustomFieldsTypeKeyEnumValuesCustomFieldIdGet200Response>;
    customFieldsTypeKeyEnumValuesCustomFieldIdPost(typeKey: string, customFieldId: string, customFieldsTypeKeyEnumValuesCustomFieldIdGetRequest: CustomFieldsTypeKeyEnumValuesCustomFieldIdGetRequest, $fields?: string, options?: any): AxiosPromise<Array<EnumValueData>>;
    customFieldsTypeKeyFieldsGet(typeKey: string, withArchived?: boolean, scope?: string, $fields?: string, options?: any): AxiosPromise<Array<CustomField>>;
    customFieldsTypeKeyFieldsIdArchivePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: any): AxiosPromise<void>;
    customFieldsTypeKeyFieldsIdDelete(typeKey: string, id: string, scope: string, options?: any): AxiosPromise<void>;
    customFieldsTypeKeyFieldsIdFilterValuesGet(typeKey: string, id: string, $skip?: string, $top?: number, scope?: string, calculateTotal?: boolean, $fields?: string, options?: any): AxiosPromise<CustomFieldsTypeKeyFieldsIdFilterValuesGet200Response>;
    customFieldsTypeKeyFieldsIdPatch(typeKey: string, id: string, customFieldsTypeKeyFieldsIdDeleteRequest: CustomFieldsTypeKeyFieldsIdDeleteRequest, options?: any): AxiosPromise<void>;
    customFieldsTypeKeyFieldsIdRestorePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: any): AxiosPromise<void>;
    customFieldsTypeKeyFieldsPost(typeKey: string, customFieldsTypeKeyFieldsGetRequest: CustomFieldsTypeKeyFieldsGetRequest, $fields?: string, options?: any): AxiosPromise<CustomField>;
    customFieldsTypeKeyFieldsReorderPost(typeKey: string, customFieldsTypeKeyFieldsReorderPostRequest: CustomFieldsTypeKeyFieldsReorderPostRequest, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldArchivePost(entityType: string, customField: string, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldDelete(entityType: string, customField: string, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesEnumValueToRemoveDelete(entityType: string, customField: string, enumValueToRemove: string, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet(entityType: string, customField: string, query?: string, ordering?: EnumValueOrdering, addedByProfileId?: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet200Response>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest, $fields?: string, options?: any): AxiosPromise<CFEnumValue>;
    customFieldsV2EntityTypeFieldsCustomFieldGet(entityType: string, customField: string, $fields?: string, options?: any): AxiosPromise<CustomFieldData>;
    customFieldsV2EntityTypeFieldsCustomFieldPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldDeleteRequest?: CustomFieldsV2EntityTypeFieldsCustomFieldDeleteRequest, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldRestorePost(entityType: string, customField: string, options?: any): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsGet(entityType: string, withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<Array<CustomFieldData>>;
    customFieldsV2EntityTypeFieldsPost(entityType: string, customFieldsV2EntityTypeFieldsGetRequest: CustomFieldsV2EntityTypeFieldsGetRequest, $fields?: string, options?: any): AxiosPromise<CustomFieldData>;
    customFieldsV2EntityTypeFieldsReorderPost(entityType: string, customFieldsV2EntityTypeFieldsReorderPostRequest: CustomFieldsV2EntityTypeFieldsReorderPostRequest, options?: any): AxiosPromise<void>;
    customFieldsV2ValuesEntityCustomFieldGet(entity: string, customField: string, $fields?: string, options?: any): AxiosPromise<CustomFieldValueData>;
    customFieldsV2ValuesEntityCustomFieldPost(entity: string, customField: string, customFieldsV2ValuesEntityCustomFieldGetRequest: CustomFieldsV2ValuesEntityCustomFieldGetRequest, options?: any): AxiosPromise<void>;
    customFieldsV2ValuesEntityGet(entity: string, $fields?: string, options?: any): AxiosPromise<Array<CustomFieldValueData>>;
    customFieldsV2ValuesEntityPost(entity: string, customFieldsV2ValuesEntityGetRequest: CustomFieldsV2ValuesEntityGetRequest, options?: any): AxiosPromise<void>;
    emojisAddPost(emojisAddPostRequest: EmojisAddPostRequest, options?: any): AxiosPromise<void>;
    emojisDeletePost(emojisDeletePostRequest: EmojisDeletePostRequest, options?: any): AxiosPromise<void>;
    emojisExistsGet(emoji: string, options?: any): AxiosPromise<boolean>;
    emojisFrequentlyUsedGet(options?: any): AxiosPromise<Array<string>>;
    emojisRecordUsagePost(emojisRecordUsagePostRequest: EmojisRecordUsagePostRequest, options?: any): AxiosPromise<void>;
    emojisSearchGet(query: string, $skip?: string, $top?: number, version?: string, $fields?: string, options?: any): AxiosPromise<EmojisSearchGet200Response>;
    externalLinkPatternsDelete(pattern: string, options?: any): AxiosPromise<void>;
    externalLinkPatternsGet($fields?: string, options?: any): AxiosPromise<Array<ExternalLinkPattern>>;
    externalLinkPatternsPost(externalLinkPatternsDeleteRequest: ExternalLinkPatternsDeleteRequest, options?: any): AxiosPromise<void>;
    httpApiModelGet($fields?: string, options?: any): AxiosPromise<HAModel>;
    notificationsChannelSubscriptionsGet(channel: string, $fields?: string, options?: any): AxiosPromise<Array<SubscriptionDTO>>;
    notificationsChannelSubscriptionsIdDelete(id: string, options?: any): AxiosPromise<void>;
    notificationsChannelSubscriptionsIdPatch(id: string, $fields?: string, notificationsChannelSubscriptionsIdDeleteRequest?: NotificationsChannelSubscriptionsIdDeleteRequest, options?: any): AxiosPromise<SubscriptionDTO>;
    notificationsChannelSubscriptionsIdRequestMissingRightsPost(id: string, options?: any): AxiosPromise<void>;
    notificationsChannelSubscriptionsPost(notificationsChannelSubscriptionsGetRequest: NotificationsChannelSubscriptionsGetRequest, $fields?: string, options?: any): AxiosPromise<SubscriptionDTO>;
    notificationsGet($fields?: string, options?: any): AxiosPromise<Array<EventSubjectInfoDTO>>;
    notificationsPersonalCustomSubscriptionsGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<SubscriptionDTO>>;
    notificationsPersonalCustomSubscriptionsIdDelete(id: string, options?: any): AxiosPromise<void>;
    notificationsPersonalCustomSubscriptionsIdPatch(id: string, $fields?: string, notificationsPersonalCustomSubscriptionsIdDeleteRequest?: NotificationsPersonalCustomSubscriptionsIdDeleteRequest, options?: any): AxiosPromise<SubscriptionDTO>;
    notificationsPersonalCustomSubscriptionsPost(notificationsPersonalCustomSubscriptionsGetRequest: NotificationsPersonalCustomSubscriptionsGetRequest, $fields?: string, options?: any): AxiosPromise<SubscriptionDTO>;
    notificationsPersonalSubscriptionsAllPersonalSubscriptionTargetsGet($fields?: string, options?: any): AxiosPromise<Array<PersonalSubscriptionTarget>>;
    notificationsPersonalSubscriptionsPersonalSubscriptionSettingsGet(profile: string, feed: string, $fields?: string, options?: any): AxiosPromise<PersonalSubscriptionSettings>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest, options?: any): AxiosPromise<void>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest, options?: any): AxiosPromise<void>;
    notificationsPrivateFeedsGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<PrivateFeed>>;
    notificationsPrivateFeedsIdDelete(id: string, options?: any): AxiosPromise<void>;
    notificationsPrivateFeedsIdPatch(id: string, $fields?: string, notificationsPrivateFeedsIdDeleteRequest?: NotificationsPrivateFeedsIdDeleteRequest, options?: any): AxiosPromise<PrivateFeed>;
    notificationsPrivateFeedsPost(notificationsPrivateFeedsGetRequest: NotificationsPrivateFeedsGetRequest, $fields?: string, options?: any): AxiosPromise<PrivateFeed>;
    organizationDomainsCheckGet(domain: string, options?: any): AxiosPromise<CDomainStatus>;
    organizationDomainsGet($fields?: string, options?: any): AxiosPromise<Array<OrgDomainDTO>>;
    organizationDomainsPatch(organizationDomainsGetRequest: OrganizationDomainsGetRequest, options?: any): AxiosPromise<void>;
    organizationGet($fields?: string, options?: any): AxiosPromise<OrganizationRecord>;
    organizationJetSalesLicenseActivationUrlGet(options?: any): AxiosPromise<string>;
    organizationJetSalesUrlGet(options?: any): AxiosPromise<string>;
    organizationPatch(organizationGetRequest: OrganizationGetRequest, options?: any): AxiosPromise<void>;
    permissionRolesCreatePost(permissionRolesCreatePostRequest: PermissionRolesCreatePostRequest, $fields?: string, options?: any): AxiosPromise<RoleDTO>;
    permissionRolesGetPost(applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, $fields?: string, options?: any): AxiosPromise<Array<RoleDTO>>;
    permissionRolesRoleId2FaRequirementGet(roleId: string, $fields?: string, options?: any): AxiosPromise<TwoFactorAuthenticationRequirement>;
    permissionRolesRoleId2FaRequirementPatch(roleId: string, permissionRolesRoleId2FaRequirementGetRequest?: PermissionRolesRoleId2FaRequirementGetRequest, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdDelete(roleId: string, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdPatch(roleId: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdPermissionsDelete(roleId: string, rightCodes: Array<string>, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdPermissionsGet(roleId: string, $fields?: string, options?: any): AxiosPromise<Array<RightDTO>>;
    permissionRolesRoleIdPermissionsPost(roleId: string, permissionRolesRoleIdPermissionsDeleteRequest: PermissionRolesRoleIdPermissionsDeleteRequest, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdProfilesGet(roleId: string, $fields?: string, options?: any): AxiosPromise<Array<TDMemberProfile>>;
    permissionRolesRoleIdProfilesProfileDelete(roleId: string, profile: string, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdProfilesProfilePost(roleId: string, profile: string, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdResetRolePermissionsToDefaultPost(roleId: string, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdTeamsGet(roleId: string, $fields?: string, options?: any): AxiosPromise<Array<TDTeam>>;
    permissionRolesRoleIdTeamsTeamDelete(roleId: string, team: string, options?: any): AxiosPromise<void>;
    permissionRolesRoleIdTeamsTeamPost(roleId: string, team: string, options?: any): AxiosPromise<void>;
    permissionsCheckPermissionPost(permissionsCheckPermissionPostRequest: PermissionsCheckPermissionPostRequest, options?: any): AxiosPromise<boolean>;
    permissionsGet($fields?: string, options?: any): AxiosPromise<DTORightsWithHierarchy>;
    projectsAutomationDslEvaluationsConfigGet($fields?: string, options?: any): AxiosPromise<DslEvaluationConfig>;
    projectsAutomationGraphExecutionsIdGet(id: string, $fields?: string, options?: any): AxiosPromise<JobExecutionDTO>;
    projectsAutomationGraphExecutionsIdStopPost(id: string, options?: any): AxiosPromise<void>;
    projectsAutomationJobExecutionsCurrentGet($fields?: string, options?: any): AxiosPromise<JobExecutionDTO>;
    projectsAutomationJobsJobIdGet(jobId: string, project: string, $fields?: string, options?: any): AxiosPromise<JobDTO>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyDelete(stepExecId: string, key: string, options?: any): AxiosPromise<void>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyGet(stepExecId: string, key: string, options?: any): AxiosPromise<string>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyPatch(stepExecId: string, key: string, projectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest: ProjectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest, options?: any): AxiosPromise<void>;
    projectsAutomationStepExecutionsUsedParametersParamParameterIdGet(parameterId: string, $fields?: string, options?: any): AxiosPromise<Array<ParameterLastUsageDTO>>;
    projectsAutomationStepExecutionsUsedParametersSecretSecretIdGet(secretId: string, $fields?: string, options?: any): AxiosPromise<Array<ParameterLastUsageDTO>>;
    projectsAutomationSubscriptionsLegacyChannelsDelete(project?: string, jobId?: string, unsubscribedOnly?: boolean, options?: any): AxiosPromise<void>;
    projectsCollaboratorprofileGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsGet200Response>;
    projectsGet($skip?: string, $top?: number, term?: string, tag?: string, starred?: boolean, $fields?: string, options?: any): AxiosPromise<ProjectsGet200Response>;
    projectsMembermemberGet(member: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsGet200Response>;
    projectsParamsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsParamsDefaultBundleGet200Response>;
    projectsParamsDefaultBundlePost(projectsParamsDefaultBundleGetRequest: ProjectsParamsDefaultBundleGetRequest, options?: any): AxiosPromise<string>;
    projectsParamsIdDelete(id: string, options?: any): AxiosPromise<void>;
    projectsParamsIdPatch(id: string, projectsParamsIdDeleteRequest: ProjectsParamsIdDeleteRequest, options?: any): AxiosPromise<void>;
    projectsParamsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsParamsDefaultBundleGet200Response>;
    projectsParamsInDefaultBundlePost(projectsParamsInDefaultBundleGetRequest: ProjectsParamsInDefaultBundleGetRequest, options?: any): AxiosPromise<string>;
    projectsPlanningBoardsBoardDelete(board: string, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsBoardGet(board: string, $fields?: string, options?: any): AxiosPromise<BoardRecord>;
    projectsPlanningBoardsBoardIssuesGet(board: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>;
    projectsPlanningBoardsBoardIssuesIssueDelete(issue: string, board: string, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsBoardIssuesIssuePost(issue: string, board: string, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsBoardPatch(board: string, projectsPlanningBoardsBoardDeleteRequest?: ProjectsPlanningBoardsBoardDeleteRequest, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsSprintsPost(projectsPlanningBoardsSprintsPostRequest: ProjectsPlanningBoardsSprintsPostRequest, $fields?: string, options?: any): AxiosPromise<SprintRecord>;
    projectsPlanningBoardsSprintsSprintArchiveDelete(sprint: string, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintIssuesGet(sprint: string, $skip?: string, $top?: number, unresolvedOnly?: boolean, $fields?: string, options?: any): AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>;
    projectsPlanningBoardsSprintsSprintIssuesIssueDelete(issue: string, sprint: string, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintIssuesIssuePost(issue: string, sprint: string, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintLaunchPost(sprint: string, projectsPlanningBoardsSprintsSprintLaunchPostRequest: ProjectsPlanningBoardsSprintsSprintLaunchPostRequest, options?: any): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintPatch(sprint: string, projectsPlanningBoardsSprintsSprintPatchRequest?: ProjectsPlanningBoardsSprintsSprintPatchRequest, options?: any): AxiosPromise<void>;
    projectsPost(projectsGetRequest: ProjectsGetRequest, $fields?: string, options?: any): AxiosPromise<PRProject>;
    projectsPrivateProjectsGet($fields?: string, options?: any): AxiosPromise<Array<PRPrivateProject>>;
    projectsPrivateProjectsProjectRequestAccessPost(project: string, options?: any): AxiosPromise<void>;
    projectsProjectAccessAdminsGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectAccessAdminsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectAccessAdminsProfilesProfileDelete(project: string, profile: string, options?: any): AxiosPromise<void>;
    projectsProjectAccessAdminsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectAccessAdminsTeamsTeamIdDelete(project: string, teamId: string, options?: any): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectAccessCollaboratorsProfilesDelete(project: string, profile: string, options?: any): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsProfilesGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectAccessCollaboratorsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsTeamsDelete(project: string, teamId: string, options?: any): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsTeamsGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<TDTeam>>;
    projectsProjectAccessCollaboratorsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectAccessMemberProfilesGet(project: string, $skip?: string, $top?: number, query?: string, includingAdmins?: boolean, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    projectsProjectAccessMembersProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectAccessMembersProfilesProfileDelete(project: string, profile: string, options?: any): AxiosPromise<void>;
    projectsProjectAccessMembersTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectAccessMembersTeamsTeamIdDelete(project: string, teamId: string, options?: any): AxiosPromise<void>;
    projectsProjectAccessViewersGet(project: string, term: string, $skip?: string, $top?: number, meOnTop?: boolean, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    projectsProjectAutomationDeploymentTargetsFavoritesGet(project: string, sortBy?: string, sortOrder?: ColumnSortOrder, $fields?: string, options?: any): AxiosPromise<Array<DeployTargetRecord>>;
    projectsProjectAutomationDeploymentTargetsGet(project: string, search?: string, sortBy?: string, sortOrder?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectAutomationDeploymentTargetsGet200Response>;
    projectsProjectAutomationDeploymentTargetsPost(project: string, projectsProjectAutomationDeploymentTargetsGetRequest: ProjectsProjectAutomationDeploymentTargetsGetRequest, $fields?: string, options?: any): AxiosPromise<DeployTargetRecord>;
    projectsProjectAutomationDeploymentTargetsTargetDelete(project: string, target: string, options?: any): AxiosPromise<void>;
    projectsProjectAutomationDeploymentTargetsTargetGet(project: string, target: string, $fields?: string, options?: any): AxiosPromise<DeployTargetRecord>;
    projectsProjectAutomationDeploymentTargetsTargetPut(project: string, target: string, projectsProjectAutomationDeploymentTargetsTargetDeleteRequest?: ProjectsProjectAutomationDeploymentTargetsTargetDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectAutomationDeploymentsFailPost(project: string, projectsProjectAutomationDeploymentsFailPostRequest: ProjectsProjectAutomationDeploymentsFailPostRequest, $fields?: string, options?: any): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsFinishPost(project: string, projectsProjectAutomationDeploymentsFinishPostRequest: ProjectsProjectAutomationDeploymentsFinishPostRequest, $fields?: string, options?: any): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsGet(project: string, targetIdentifier?: string, jobExecutionId?: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectAutomationDeploymentsGet200Response>;
    projectsProjectAutomationDeploymentsPut(project: string, projectsProjectAutomationDeploymentsGetRequest: ProjectsProjectAutomationDeploymentsGetRequest, $fields?: string, options?: any): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsSchedulePost(project: string, projectsProjectAutomationDeploymentsSchedulePostRequest: ProjectsProjectAutomationDeploymentsSchedulePostRequest, $fields?: string, options?: any): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsStartPost(project: string, projectsProjectAutomationDeploymentsStartPostRequest: ProjectsProjectAutomationDeploymentsStartPostRequest, $fields?: string, options?: any): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierDelete(project: string, targetIdentifier: string, deploymentIdentifier: string, options?: any): AxiosPromise<void>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierGet(project: string, targetIdentifier: string, deploymentIdentifier: string, $fields?: string, options?: any): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationGraphExecutionsGet(project: string, jobId: string, branchFilter?: string, statusFilter?: ExecutionStatus, jobTriggerFilter?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectAutomationGraphExecutionsGet200Response>;
    projectsProjectAutomationJobsGet(project: string, repoFilter: string, branchFilter: string, trigger?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectAutomationJobsGet200Response>;
    projectsProjectAutomationJobsJobIdStartPost(project: string, jobId: string, projectsProjectAutomationJobsJobIdStartPostRequest: ProjectsProjectAutomationJobsJobIdStartPostRequest, $fields?: string, options?: any): AxiosPromise<LaunchResult>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPost(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdRejectSuggestedEditPost(project: string, discussionId: string, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdReopenSuggestedEditPost(project: string, discussionId: string, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatch(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsPost(project: string, projectsProjectCodeReviewsCodeDiscussionsPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsPostRequest, $fields?: string, options?: any): AxiosPromise<CodeDiscussionRecord>;
    projectsProjectCodeReviewsCommitSetReviewPost(project: string, projectsProjectCodeReviewsCommitSetReviewPostRequest: ProjectsProjectCodeReviewsCommitSetReviewPostRequest, $fields?: string, options?: any): AxiosPromise<CommitSetReviewRecord>;
    projectsProjectCodeReviewsGet(project: string, $skip?: string, $top?: number, state?: CodeReviewStateFilter, text?: string, author?: string, from?: string, to?: string, sort?: ReviewSorting, reviewer?: string, type?: ReviewType, $fields?: string, options?: any): AxiosPromise<ProjectsProjectCodeReviewsGet200Response>;
    projectsProjectCodeReviewsMergeRequestsPost(project: string, projectsProjectCodeReviewsMergeRequestsPostRequest: ProjectsProjectCodeReviewsMergeRequestsPostRequest, $fields?: string, options?: any): AxiosPromise<MergeRequestRecord>;
    projectsProjectCodeReviewsReviewIdDescriptionPatch(project: string, reviewId: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdDetailsGet(project: string, reviewId: string, $fields?: string, options?: any): AxiosPromise<CodeReviewDetailedInfo>;
    projectsProjectCodeReviewsReviewIdFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectCodeReviewsReviewIdFilesGet200Response>;
    projectsProjectCodeReviewsReviewIdGet(project: string, reviewId: string, $fields?: string, options?: any): AxiosPromise<CodeReviewRecord>;
    projectsProjectCodeReviewsReviewIdMergeFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectCodeReviewsReviewIdMergeFilesGet200Response>;
    projectsProjectCodeReviewsReviewIdMergePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdMergePutRequest: ProjectsProjectCodeReviewsReviewIdMergePutRequest, $fields?: string, options?: any): AxiosPromise<GitMergeResultHttp>;
    projectsProjectCodeReviewsReviewIdParticipantsUserDelete(project: string, reviewId: string, user: string, role: CodeReviewParticipantRole, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdParticipantsUserPost(project: string, reviewId: string, user: string, projectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest: ProjectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdRebasePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRebasePutRequest: ProjectsProjectCodeReviewsReviewIdRebasePutRequest, $fields?: string, options?: any): AxiosPromise<GitRebaseResultHttp>;
    projectsProjectCodeReviewsReviewIdRevisionsDelete(project: string, reviewId: string, revisions: Array<string>, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdRevisionsPost(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRevisionsDeleteRequest: ProjectsProjectCodeReviewsReviewIdRevisionsDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdStatePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdStatePatchRequest: ProjectsProjectCodeReviewsReviewIdStatePatchRequest, options?: any): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdSuggestedReviewersGet(project: string, reviewId: string, $fields?: string, options?: any): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectCodeReviewsReviewIdTitlePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdTitlePatchRequest: ProjectsProjectCodeReviewsReviewIdTitlePatchRequest, options?: any): AxiosPromise<void>;
    projectsProjectDelete(project: string, options?: any): AxiosPromise<void>;
    projectsProjectDocumentsDocumentIdCopyPost(project: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: any): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdDelete(project: string, documentId: string, options?: any): AxiosPromise<void>;
    projectsProjectDocumentsDocumentIdDeleteForeverDelete(project: string, documentId: string, options?: any): AxiosPromise<void>;
    projectsProjectDocumentsDocumentIdGet(project: string, documentId: string, $fields?: string, options?: any): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdMovePatch(project: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: any): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdPatch(project: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: any): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdUnarchivePatch(project: string, documentId: string, $fields?: string, options?: any): AxiosPromise<Document>;
    projectsProjectDocumentsFoldersFolderDelete(project: string, folder: string, options?: any): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderDocumentsGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response>;
    projectsProjectDocumentsFoldersFolderGet(project: string, folder: string, $fields?: string, options?: any): AxiosPromise<DocumentFolder>;
    projectsProjectDocumentsFoldersFolderIntroductionDelete(project: string, folder: string, options?: any): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderIntroductionDocumentIdPatch(project: string, folder: string, documentId: string, options?: any): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderMovePatch(project: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: any): AxiosPromise<DocumentFolder>;
    projectsProjectDocumentsFoldersFolderPatch(project: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: any): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderSubfoldersGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response>;
    projectsProjectDocumentsFoldersPost(project: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: any): AxiosPromise<DocumentFolder>;
    projectsProjectDocumentsPost(project: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: any): AxiosPromise<Document>;
    projectsProjectGet(project: string, $fields?: string, options?: any): AxiosPromise<PRProject>;
    projectsProjectPackagesRepositoriesGet(project: string, type?: string, query?: string, $fields?: string, options?: any): AxiosPromise<Array<ProjectPackageRepository>>;
    projectsProjectPackagesRepositoriesPost(project: string, projectsProjectPackagesRepositoriesGetRequest: ProjectsProjectPackagesRepositoriesGetRequest, $fields?: string, options?: any): AxiosPromise<ProjectPackageRepository>;
    projectsProjectPackagesRepositoriesRepositoryCleanupDryPost(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest: ProjectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest, $fields?: string, options?: any): AxiosPromise<DryCleanupResults>;
    projectsProjectPackagesRepositoriesRepositoryCleanupPost(project: string, repository: string, $fields?: string, projectsProjectPackagesRepositoriesRepositoryCleanupPostRequest?: ProjectsProjectPackagesRepositoriesRepositoryCleanupPostRequest, options?: any): AxiosPromise<PackagesExecutionResult>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet(project: string, repository: string, connectionId: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet200Response>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishPost(project: string, repository: string, connectionId: string, projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest: ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest, options?: any): AxiosPromise<string>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsGet(project: string, repository: string, $fields?: string, options?: any): AxiosPromise<Array<PackageRepositoryConnection>>;
    projectsProjectPackagesRepositoriesRepositoryDelete(project: string, repository: string, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryFilesFolderfolderPathDelete(project: string, repository: string, folderPath: string, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryFilesGet(project: string, repository: string, parentPath?: string, $fields?: string, options?: any): AxiosPromise<Array<FileData>>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathDelete(project: string, repository: string, filePath: string, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathGet(project: string, repository: string, filePath: string, $fields?: string, options?: any): AxiosPromise<FileDetails>;
    projectsProjectPackagesRepositoriesRepositoryGet(project: string, repository: string, $fields?: string, options?: any): AxiosPromise<ProjectPackageRepository>;
    projectsProjectPackagesRepositoriesRepositoryPackagesGet(project: string, repository: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameDelete(project: string, repository: string, packageName: string, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataGet(project: string, repository: string, packageName: string, $fields?: string, options?: any): AxiosPromise<PackageMetadata>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataPut(project: string, repository: string, packageName: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest?: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataVersionpackageVersionPut(project: string, repository: string, packageName: string, packageVersion: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsGet(project: string, repository: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, connectionId?: string, pinned?: boolean, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, repository: string, packageName: string, packageVersion: string, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, repository: string, packageName: string, packageVersion: string, $fields?: string, options?: any): AxiosPromise<PackageVersionDetails>;
    projectsProjectPackagesRepositoriesRepositoryPatch(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryDeleteRequest?: ProjectsProjectPackagesRepositoriesRepositoryDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryUrlGet(project: string, repository: string, options?: any): AxiosPromise<string>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesGet(project: string, type: string, repositoryName: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsGet(project: string, type: string, repositoryName: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, options?: any): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, $fields?: string, options?: any): AxiosPromise<PackageVersionDetails>;
    projectsProjectPackagesSearchGet(project: string, type: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>;
    projectsProjectPackagesTypesGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<PackageType>>;
    projectsProjectPatch(project: string, $fields?: string, projectsProjectDeleteRequest?: ProjectsProjectDeleteRequest, options?: any): AxiosPromise<PRProject>;
    projectsProjectPlanningBoardsBoardArchiveDelete(project: string, board: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningBoardsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPlanningBoardsGet200Response>;
    projectsProjectPlanningBoardsPost(project: string, projectsProjectPlanningBoardsGetRequest: ProjectsProjectPlanningBoardsGetRequest, $fields?: string, options?: any): AxiosPromise<BoardRecord>;
    projectsProjectPlanningBoardsSprintsGet(project: string, $skip?: string, $top?: number, board?: string, query?: string, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPlanningBoardsSprintsGet200Response>;
    projectsProjectPlanningBoardsStarredGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<BoardRecord>>;
    projectsProjectPlanningChecklistsChecklistIdDelete(project: string, checklistId: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningChecklistsChecklistIdFullChecklistTreeGet(project: string, checklistId: string, $fields?: string, options?: any): AxiosPromise<Array<PlanItemChildren>>;
    projectsProjectPlanningChecklistsChecklistIdImportPost(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningChecklistsChecklistIdPatch(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdDeleteRequest?: ProjectsProjectPlanningChecklistsChecklistIdDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningChecklistsGet(project: string, $skip?: string, $top?: number, query?: string, sorting?: ChecklistSorting, descending?: boolean, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPlanningChecklistsGet200Response>;
    projectsProjectPlanningChecklistsImportPost(project: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: any): AxiosPromise<Checklist>;
    projectsProjectPlanningChecklistsPost(project: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: any): AxiosPromise<Checklist>;
    projectsProjectPlanningChecklistsStarredGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<Checklist>>;
    projectsProjectPlanningIssuesGet(project: string, sorting: IssuesSorting, descending: boolean, $skip?: string, $top?: number, assigneeId?: Array<string>, createdByProfileId?: string, createdBy?: Array<string>, statuses?: Array<string>, tagId?: string, query?: string, tags?: Array<string>, sprints?: Array<string>, boards?: Array<string>, customFields?: Array<string>, importTransaction?: string, creationTimeFrom?: string, creationTimeTo?: string, dueDateFrom?: string, dueDateTo?: string, topics?: Array<string>, grouping?: IssueListGrouping, deployment?: string, $fields?: string, options?: any): AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>;
    projectsProjectPlanningIssuesImportPost(project: string, projectsProjectPlanningIssuesImportPostRequest: ProjectsProjectPlanningIssuesImportPostRequest, $fields?: string, options?: any): AxiosPromise<IssueImportResult>;
    projectsProjectPlanningIssuesIssueIdAttachmentsDelete(project: string, issueId: string, identities: Array<string>, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdAttachmentsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdDelete(project: string, issueId: string, checklistId: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdPost(project: string, issueId: string, checklistId: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsDelete(project: string, issueId: string, codeReviewIds: Array<string>, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCommentsImportPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommentsImportPostRequest: ProjectsProjectPlanningIssuesIssueIdCommentsImportPostRequest, options?: any): AxiosPromise<Array<string>>;
    projectsProjectPlanningIssuesIssueIdCommitsDelete(project: string, issueId: string, repository: string, commitIds: Array<string>, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCommitsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommitsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCommitsDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdDelete(project: string, issueId: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdGet(project: string, issueId: string, withDeleted?: boolean, $fields?: string, options?: any): AxiosPromise<Issue>;
    projectsProjectPlanningIssuesIssueIdPatch(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdDeleteRequest?: ProjectsProjectPlanningIssuesIssueIdDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdRestorePost(project: string, issueId: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdDelete(project: string, issueId: string, tagId: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdPost(project: string, issueId: string, tagId: string, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdToggleResolvedPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest: ProjectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesNumbernumberGet(project: string, number: number, resolveAlias?: boolean, withDeleted?: boolean, $fields?: string, options?: any): AxiosPromise<Issue>;
    projectsProjectPlanningIssuesPost(project: string, projectsProjectPlanningIssuesGetRequest: ProjectsProjectPlanningIssuesGetRequest, $fields?: string, options?: any): AxiosPromise<Issue>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGet(project: string, $fields?: string, options?: any): AxiosPromise<IssueStatus>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergePatch(project: string, projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest?: ProjectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningIssuesStatusesDistributionGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<IssueStatusWithUsages>>;
    projectsProjectPlanningIssuesStatusesGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<IssueStatus>>;
    projectsProjectPlanningIssuesStatusesPatch(project: string, projectsProjectPlanningIssuesStatusesGetRequest: ProjectsProjectPlanningIssuesStatusesGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectPlanningTagsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: any): AxiosPromise<ProjectsProjectPlanningTagsGet200Response>;
    projectsProjectPlanningTagsPost(project: string, projectsProjectPlanningTagsGetRequest: ProjectsProjectPlanningTagsGetRequest, $fields?: string, options?: any): AxiosPromise<PlanningTag>;
    projectsProjectRepositoriesRepositoryCherryPickCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCherryPickCommitPostRequest: ProjectsProjectRepositoriesRepositoryCherryPickCommitPostRequest, $fields?: string, options?: any): AxiosPromise<GitCherryPickResult>;
    projectsProjectRepositoriesRepositoryCommitBranchesGet(project: string, repository: string, commit: string, prefix?: string, limit?: number, $fields?: string, options?: any): AxiosPromise<Array<BranchInfo>>;
    projectsProjectRepositoriesRepositoryCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCommitPostRequest: ProjectsProjectRepositoriesRepositoryCommitPostRequest, $fields?: string, options?: any): AxiosPromise<GitCommitResult>;
    projectsProjectRepositoriesRepositoryCommitsGet(project: string, repository: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: any): AxiosPromise<ProjectsProjectRepositoriesRepositoryCommitsGet200Response>;
    projectsProjectRepositoriesRepositoryDelete(project: string, repository: string, options?: any): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryDeleteBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryDeleteBranchPostRequest: ProjectsProjectRepositoriesRepositoryDeleteBranchPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryFilesGet(project: string, repository: string, commit: string, path: string, $fields?: string, options?: any): AxiosPromise<Array<GitFile>>;
    projectsProjectRepositoriesRepositoryGcPost(project: string, repository: string, options?: any): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryMergeBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryMergeBranchPostRequest: ProjectsProjectRepositoriesRepositoryMergeBranchPostRequest, $fields?: string, options?: any): AxiosPromise<GitMergeBranchResult>;
    projectsProjectRepositoriesRepositoryPost(project: string, repository: string, $fields?: string, projectsProjectRepositoriesRepositoryDeleteRequest?: ProjectsProjectRepositoriesRepositoryDeleteRequest, options?: any): AxiosPromise<PRRepositoryInfo>;
    projectsProjectRepositoriesRepositoryReadonlyGet(project: string, repository: string, options?: any): AxiosPromise<boolean>;
    projectsProjectRepositoriesRepositoryReadonlyPost(project: string, repository: string, projectsProjectRepositoriesRepositoryReadonlyGetRequest: ProjectsProjectRepositoriesRepositoryReadonlyGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryRebaseBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryRebaseBranchPostRequest: ProjectsProjectRepositoriesRepositoryRebaseBranchPostRequest, $fields?: string, options?: any): AxiosPromise<GitRebaseResult>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGet(project: string, repository: string, revision: string, $fields?: string, options?: any): AxiosPromise<Array<ExternalCheckDTO>>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksPost(project: string, repository: string, revision: string, projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest: ProjectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectRepositoriesRepositorySettingsGet(project: string, repository: string, $fields?: string, options?: any): AxiosPromise<GitRepositorySettings>;
    projectsProjectRepositoriesRepositorySettingsPost(project: string, repository: string, projectsProjectRepositoriesRepositorySettingsGetRequest: ProjectsProjectRepositoriesRepositorySettingsGetRequest, options?: any): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryUrlGet(project: string, repository: string, $fields?: string, options?: any): AxiosPromise<RepositoryUrls>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDelete(project: string, responsibilityId: string, profileId: string, role?: string, options?: any): AxiosPromise<void>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdPost(project: string, responsibilityId: string, profileId: string, projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest?: ProjectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest, options?: any): AxiosPromise<void>;
    projectsProjectResponsibilitiesSchemeGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<SubjectResponsibilitiesTable>>;
    projectsProjectResponsibilitiesSubjectsPost(project: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: any): AxiosPromise<string>;
    projectsProjectResponsibilitiesSubjectsSubjectIdPatch(project: string, subjectId: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectTopicsGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<Topic>>;
    projectsProjectTopicsIdPatch(project: string, id: string, $fields?: string, projectsProjectTopicsIdPatchRequest?: ProjectsProjectTopicsIdPatchRequest, options?: any): AxiosPromise<Topic>;
    projectsProjectTopicsPost(project: string, projectsProjectTopicsGetRequest: ProjectsProjectTopicsGetRequest, $fields?: string, options?: any): AxiosPromise<Topic>;
    projectsProjectTopicsRemoveTopicsPost(project: string, projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, options?: any): AxiosPromise<void>;
    projectsProjectTopicsSetResponsiblePost(project: string, projectsProjectTopicsSetResponsiblePostRequest: ProjectsProjectTopicsSetResponsiblePostRequest, options?: any): AxiosPromise<void>;
    projectsRepositoriesFindGet(term: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsRepositoriesFindGet200Response>;
    projectsResponsibilitiesPost(projectsResponsibilitiesPostRequest: ProjectsResponsibilitiesPostRequest, options?: any): AxiosPromise<string>;
    projectsResponsibilitiesResponsibilityIdDelete(responsibilityId: string, options?: any): AxiosPromise<void>;
    projectsResponsibilitiesResponsibilityIdPatch(responsibilityId: string, projectsResponsibilitiesResponsibilityIdDeleteRequest: ProjectsResponsibilitiesResponsibilityIdDeleteRequest, options?: any): AxiosPromise<void>;
    projectsResponsibilitiesSubjectsSubjectIdDelete(subjectId: string, project?: string, options?: any): AxiosPromise<void>;
    projectsRightCoderightCodeGet(rightCode: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, $fields?: string, options?: any): AxiosPromise<ProjectsGet200Response>;
    projectsRightUniqueCoderightGet(right: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, _private?: boolean, $fields?: string, options?: any): AxiosPromise<ProjectsGet200Response>;
    projectsSecretsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsSecretsDefaultBundleGet200Response>;
    projectsSecretsDefaultBundlePost(projectsSecretsDefaultBundleGetRequest: ProjectsSecretsDefaultBundleGetRequest, options?: any): AxiosPromise<string>;
    projectsSecretsIdDelete(id: string, options?: any): AxiosPromise<void>;
    projectsSecretsIdPatch(id: string, projectsSecretsIdDeleteRequest: ProjectsSecretsIdDeleteRequest, options?: any): AxiosPromise<void>;
    projectsSecretsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsSecretsDefaultBundleGet200Response>;
    projectsSecretsInDefaultBundlePost(projectsSecretsInDefaultBundleGetRequest: ProjectsSecretsInDefaultBundleGetRequest, options?: any): AxiosPromise<string>;
    projectsTagsGet($fields?: string, options?: any): AxiosPromise<Array<PRTag>>;
    projectsTagsTrackAccessPost(projectsTagsTrackAccessPostRequest: ProjectsTagsTrackAccessPostRequest, options?: any): AxiosPromise<void>;
    projectsTeamteamGet(team: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsGet200Response>;
    projectsVaultGet(project: string, $fields?: string, options?: any): AxiosPromise<Array<VaultConnectionRecord>>;
    projectsVaultIdDelete(id: string, options?: any): AxiosPromise<void>;
    projectsVaultIdPatch(id: string, projectsVaultIdDeleteRequest: ProjectsVaultIdDeleteRequest, options?: any): AxiosPromise<void>;
    projectsVaultPost(projectsVaultGetRequest: ProjectsVaultGetRequest, options?: any): AxiosPromise<string>;
    publicHolidaysCalendarsGet($skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<PublicHolidaysCalendarsGet200Response>;
    publicHolidaysCalendarsIdDelete(id: string, options?: any): AxiosPromise<void>;
    publicHolidaysCalendarsIdPatch(id: string, publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: any): AxiosPromise<PublicHolidayCalendarRecord>;
    publicHolidaysCalendarsImportPost(publicHolidaysCalendarsImportPostRequest: PublicHolidaysCalendarsImportPostRequest, options?: any): AxiosPromise<string>;
    publicHolidaysCalendarsPost(publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: any): AxiosPromise<PublicHolidayCalendarRecord>;
    publicHolidaysHolidaysGet($skip?: string, $top?: number, calendar?: string, location?: string, startDate?: string, endDate?: string, $fields?: string, options?: any): AxiosPromise<PublicHolidaysHolidaysGet200Response>;
    publicHolidaysHolidaysIdDelete(id: string, options?: any): AxiosPromise<void>;
    publicHolidaysHolidaysIdPatch(id: string, $fields?: string, publicHolidaysHolidaysIdDeleteRequest?: PublicHolidaysHolidaysIdDeleteRequest, options?: any): AxiosPromise<PublicHoliday>;
    publicHolidaysHolidaysPost(publicHolidaysHolidaysGetRequest: PublicHolidaysHolidaysGetRequest, $fields?: string, options?: any): AxiosPromise<PublicHoliday>;
    publicHolidaysHolidaysProfileHolidaysGet(startDate: string, endDate: string, profile: string, workingDays?: boolean, $fields?: string, options?: any): AxiosPromise<Array<PublicHoliday>>;
    publicHolidaysHolidaysRelatedHolidaysGet($skip?: string, $top?: number, startDate?: string, endDate?: string, $fields?: string, options?: any): AxiosPromise<PublicHolidaysHolidaysGet200Response>;
    reactionsItemEmojiDelete(item: string, emoji: string, options?: any): AxiosPromise<void>;
    reactionsItemEmojiGet(item: string, emoji: string, $fields?: string, options?: any): AxiosPromise<Array<CPrincipal>>;
    reactionsItemEmojiPost(item: string, emoji: string, options?: any): AxiosPromise<void>;
    reactionsItemGet(item: string, $fields?: string, options?: any): AxiosPromise<Array<EmojiReaction>>;
    richTextParseMarkdownPost(richTextParseMarkdownPostRequest: RichTextParseMarkdownPostRequest, $fields?: string, options?: any): AxiosPromise<RtDocument>;
    teamDirectoryCalendarEventsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<AbsenceEvent>>;
    teamDirectoryCalendarEventsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<BirthdayEvent>>;
    teamDirectoryCalendarEventsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: any): AxiosPromise<Array<BirthdayEvent>>;
    teamDirectoryCalendarEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: any): AxiosPromise<Array<MeetingRecord>>;
    teamDirectoryCalendarEventsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: any): AxiosPromise<Array<HolidaysEvent>>;
    teamDirectoryCalendarEventsIdGet(id: string, $fields?: string, options?: any): AxiosPromise<MeetingRecord>;
    teamDirectoryCalendarEventsMeetingParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: any): AxiosPromise<MeetingRecord>;
    teamDirectoryCalendarEventsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<MembershipEvent>>;
    teamDirectoryCalendarEventsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: any): AxiosPromise<Array<NonWorkingDaysEvent>>;
    teamDirectoryInvitationLinksGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: any): AxiosPromise<TeamDirectoryInvitationLinksGet200Response>;
    teamDirectoryInvitationLinksInvitationLinkIdDelete(invitationLinkId: string, options?: any): AxiosPromise<void>;
    teamDirectoryInvitationLinksInvitationLinkIdPatch(invitationLinkId: string, teamDirectoryInvitationLinksInvitationLinkIdDeleteRequest?: TeamDirectoryInvitationLinksInvitationLinkIdDeleteRequest, options?: any): AxiosPromise<void>;
    teamDirectoryInvitationLinksPost(teamDirectoryInvitationLinksGetRequest: TeamDirectoryInvitationLinksGetRequest, $fields?: string, options?: any): AxiosPromise<TeamDirectoryInvitationLinksGet200Response1>;
    teamDirectoryInvitationsGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: any): AxiosPromise<TeamDirectoryInvitationsGet200Response>;
    teamDirectoryInvitationsIdDelete(id: string, options?: any): AxiosPromise<void>;
    teamDirectoryInvitationsIdPatch(id: string, teamDirectoryInvitationsIdDeleteRequest?: TeamDirectoryInvitationsIdDeleteRequest, options?: any): AxiosPromise<void>;
    teamDirectoryInvitationsPost(teamDirectoryInvitationsGetRequest: TeamDirectoryInvitationsGetRequest, $fields?: string, options?: any): AxiosPromise<Invitation>;
    teamDirectoryLanguagesGet($fields?: string, options?: any): AxiosPromise<Array<TDLanguage>>;
    teamDirectoryLocationEquipmentTypesGet(withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<Array<TDLocationEquipmentType>>;
    teamDirectoryLocationEquipmentTypesNamenameDelete(name: string, _delete: boolean, options?: any): AxiosPromise<void>;
    teamDirectoryLocationMapMemberPointsGet(locationId: string, $skip?: string, $top?: number, includeUnmarked?: boolean, $fields?: string, options?: any): AxiosPromise<TeamDirectoryLocationMapMemberPointsGet200Response>;
    teamDirectoryLocationMapMemberPointsLocationPointIdDelete(locationPointId: string, _delete: boolean, options?: any): AxiosPromise<void>;
    teamDirectoryLocationMapMemberPointsLocationPointIdPatch(locationPointId: string, $fields?: string, teamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest?: TeamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest, options?: any): AxiosPromise<TDLocationMapPoint>;
    teamDirectoryLocationMapMemberPointsPost(teamDirectoryLocationMapMemberPointsGetRequest: TeamDirectoryLocationMapMemberPointsGetRequest, $fields?: string, options?: any): AxiosPromise<TDLocationMapPoint>;
    teamDirectoryLocationsGet(query?: string, type?: string, withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<Array<TDLocation>>;
    teamDirectoryLocationsIdDelete(id: string, $fields?: string, options?: any): AxiosPromise<Array<TDLocation>>;
    teamDirectoryLocationsIdGet(id: string, $fields?: string, options?: any): AxiosPromise<TDLocation>;
    teamDirectoryLocationsIdMapGet(id: string, $fields?: string, options?: any): AxiosPromise<TDLocationMap>;
    teamDirectoryLocationsIdMapPatch(id: string, teamDirectoryLocationsIdMapGetRequest: TeamDirectoryLocationsIdMapGetRequest, $fields?: string, options?: any): AxiosPromise<TDLocationMap>;
    teamDirectoryLocationsIdPatch(id: string, $fields?: string, teamDirectoryLocationsIdDeleteRequest?: TeamDirectoryLocationsIdDeleteRequest, options?: any): AxiosPromise<TDLocation>;
    teamDirectoryLocationsIdRestorePost(id: string, $fields?: string, options?: any): AxiosPromise<TDLocation>;
    teamDirectoryLocationsPost(teamDirectoryLocationsGetRequest: TeamDirectoryLocationsGetRequest, $fields?: string, options?: any): AxiosPromise<TDLocation>;
    teamDirectoryLocationsRestorePost(projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, $fields?: string, options?: any): AxiosPromise<Array<TDLocation>>;
    teamDirectoryLocationsWithTimezoneGet($fields?: string, options?: any): AxiosPromise<Array<TDLocationWithTimeZone>>;
    teamDirectoryMemberLocationsGet($skip?: string, $top?: number, profiles?: Array<string>, locations?: Array<string>, since?: string, till?: string, withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<TeamDirectoryMemberLocationsGet200Response>;
    teamDirectoryMemberLocationsMemberLocationIdDelete(memberLocationId: string, _delete?: boolean, options?: any): AxiosPromise<void>;
    teamDirectoryMemberLocationsMemberLocationIdGet(memberLocationId: string, $fields?: string, options?: any): AxiosPromise<TDMemberLocation>;
    teamDirectoryMemberLocationsMemberLocationIdPatch(memberLocationId: string, $fields?: string, teamDirectoryMemberLocationsMemberLocationIdDeleteRequest?: TeamDirectoryMemberLocationsMemberLocationIdDeleteRequest, options?: any): AxiosPromise<TDMemberLocation>;
    teamDirectoryMemberLocationsPost(teamDirectoryMemberLocationsGetRequest: TeamDirectoryMemberLocationsGetRequest, $fields?: string, options?: any): AxiosPromise<TDMemberLocation>;
    teamDirectoryMembershipEventsGet($skip?: string, $top?: number, teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: any): AxiosPromise<TeamDirectoryMembershipEventsGet200Response>;
    teamDirectoryMembershipsGet($skip?: string, $top?: number, identifiers?: Array<string>, profiles?: Array<string>, teams?: Array<string>, directTeams?: boolean, roles?: Array<string>, directRoles?: boolean, since?: string, till?: string, requiresApproval?: boolean, withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<TeamDirectoryMembershipsGet200Response>;
    teamDirectoryMembershipsManagerCandidatesGet(term: string, $skip?: string, $top?: number, teamId?: string, excludedMemberId?: string, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    teamDirectoryMembershipsMembershipIdDelete(membershipId: string, _delete?: boolean, options?: any): AxiosPromise<void>;
    teamDirectoryMembershipsMembershipIdGet(membershipId: string, $fields?: string, options?: any): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsMembershipIdPatch(membershipId: string, $fields?: string, teamDirectoryMembershipsMembershipIdDeleteRequest?: TeamDirectoryMembershipsMembershipIdDeleteRequest, options?: any): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsMembershipIdRequestRevokePatch(membershipId: string, teamDirectoryMembershipsMembershipIdRequestRevokePatchRequest: TeamDirectoryMembershipsMembershipIdRequestRevokePatchRequest, options?: any): AxiosPromise<void>;
    teamDirectoryMembershipsMembershipIdRevokeDelete(membershipId: string, till?: string, options?: any): AxiosPromise<void>;
    teamDirectoryMembershipsPost(teamDirectoryMembershipsGetRequest: TeamDirectoryMembershipsGetRequest, $fields?: string, options?: any): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsRequestsGet($skip?: string, $top?: number, teamId?: string, direct?: boolean, $fields?: string, options?: any): AxiosPromise<TeamDirectoryMembershipsRequestsGet200Response>;
    teamDirectoryMembershipsRequestsMembershipRequestIdDelete(membershipRequestId: string, $fields?: string, options?: any): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsRequestsMembershipRequestIdPatch(membershipRequestId: string, teamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest: TeamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesAuthenticationSessionsOwnerGet(owner: string, $fields?: string, options?: any): AxiosPromise<Array<ESAuthenticationSession>>;
    teamDirectoryProfilesAuthenticationSessionsOwnerSessionIdDelete(owner: string, sessionId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesDashboardsDashboardGet(dashboard: string, $fields?: string, options?: any): AxiosPromise<DashboardPreferencesRecord>;
    teamDirectoryProfilesDashboardsDashboardPatch(dashboard: string, teamDirectoryProfilesDashboardsDashboardGetRequest: TeamDirectoryProfilesDashboardsDashboardGetRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesEmailemailGet(email: string, verified?: boolean, $fields?: string, options?: any): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesGet($skip?: string, $top?: number, query?: string, reportPastMembers?: boolean, reportFutureMembers?: boolean, reportCurrentMembers?: boolean, teamId?: string, locationId?: string, roleId?: string, meOnTop?: boolean, order?: ProfileOrder, orgRelation?: ProfileOrgRelation, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    teamDirectoryProfilesOauthConsentsOwnerApplicationsApplicationDelete(owner: string, application: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesOauthConsentsOwnerApprovedScopesIdDelete(owner: string, id: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesOauthConsentsOwnerGet(owner: string, $fields?: string, options?: any): AxiosPromise<Array<ESOAuthConsent>>;
    teamDirectoryProfilesOauthConsentsOwnerInternalApplicationsClientIdDelete(owner: string, clientId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesOauthConsentsOwnerRefreshTokensIdDelete(owner: string, id: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesPost(teamDirectoryProfilesGetRequest: TeamDirectoryProfilesGetRequest, $fields?: string, options?: any): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfile2FaRequirementsGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<Profile2FARequirement>>;
    teamDirectoryProfilesProfile2FaStatusGet(profile: string, options?: any): AxiosPromise<TwoFactorAuthenticationStatus>;
    teamDirectoryProfilesProfile2FaTotpConfirmPost(profile: string, teamDirectoryProfilesProfile2FaTotpConfirmPostRequest: TeamDirectoryProfilesProfile2FaTotpConfirmPostRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfile2FaTotpDelete(profile: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfile2FaTotpPatch(profile: string, administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfile2FaTotpPost(profile: string, $fields?: string, options?: any): AxiosPromise<TwoFactorAuthenticationSecret>;
    teamDirectoryProfilesProfileApplicationPasswordsGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDelete(profile: string, passwordId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdPatch(profile: string, passwordId: string, teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest?: TeamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileApplicationPasswordsPost(profile: string, teamDirectoryProfilesProfileApplicationPasswordsGetRequest: TeamDirectoryProfilesProfileApplicationPasswordsGetRequest, $fields?: string, options?: any): AxiosPromise<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response1>;
    teamDirectoryProfilesProfileChecklistsChecklistIdDelete(profile: string, checklistId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileChecklistsChecklistIdFullChecklistTreeGet(profile: string, checklistId: string, $fields?: string, options?: any): AxiosPromise<Array<PlanItemChildren>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdImportPost(profile: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileChecklistsChecklistIdPatch(profile: string, checklistId: string, teamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest?: TeamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileChecklistsGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<Checklist>>;
    teamDirectoryProfilesProfileChecklistsImportPost(profile: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: any): AxiosPromise<Checklist>;
    teamDirectoryProfilesProfileChecklistsPost(profile: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: any): AxiosPromise<Checklist>;
    teamDirectoryProfilesProfileChecklistsStarredGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<Checklist>>;
    teamDirectoryProfilesProfileConvertToGuestPatch(profile: string, teamDirectoryProfilesProfileConvertToGuestPatchRequest: TeamDirectoryProfilesProfileConvertToGuestPatchRequest, $fields?: string, options?: any): AxiosPromise<DryRunResult>;
    teamDirectoryProfilesProfileConvertToMemberPatch(profile: string, teamDirectoryProfilesProfileConvertToMemberPatchRequest: TeamDirectoryProfilesProfileConvertToMemberPatchRequest, $fields?: string, options?: any): AxiosPromise<DryRunResult>;
    teamDirectoryProfilesProfileDeactivateDelete(profile: string, at: string, $fields?: string, options?: any): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileDelete(profile: string, $fields?: string, options?: any): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileDocumentsDocumentIdCopyPost(profile: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: any): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDelete(profile: string, documentId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDeleteForeverDelete(profile: string, documentId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsDocumentIdGet(profile: string, documentId: string, $fields?: string, options?: any): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdMovePatch(profile: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: any): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdPatch(profile: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: any): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdUnarchivePatch(profile: string, documentId: string, $fields?: string, options?: any): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDelete(profile: string, folder: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDocumentsGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderGet(profile: string, folder: string, $fields?: string, options?: any): AxiosPromise<DocumentFolder>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDelete(profile: string, folder: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDocumentIdPatch(profile: string, folder: string, documentId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderMovePatch(profile: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: any): AxiosPromise<DocumentFolder>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderPatch(profile: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderSubfoldersGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response>;
    teamDirectoryProfilesProfileDocumentsFoldersPost(profile: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: any): AxiosPromise<DocumentFolder>;
    teamDirectoryProfilesProfileDocumentsPost(profile: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: any): AxiosPromise<Document>;
    teamDirectoryProfilesProfileGet(profile: string, $fields?: string, options?: any): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileGpgKeysFingerprintDelete(profile: string, fingerprint: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileGpgKeysFingerprintPatch(profile: string, fingerprint: string, teamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest?: TeamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileGpgKeysGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<GpgKeyData>>;
    teamDirectoryProfilesProfileGpgKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, $fields?: string, options?: any): AxiosPromise<GpgKeyData>;
    teamDirectoryProfilesProfileIsTeamMemberGet(profile: string, teamIds: Array<string>, options?: any): AxiosPromise<boolean>;
    teamDirectoryProfilesProfileLeadsGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<TDMemberProfile>>;
    teamDirectoryProfilesProfileNavBarMenuItemsGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<NavBarMenuItem>>;
    teamDirectoryProfilesProfileNavBarMenuItemsPatch(profile: string, teamDirectoryProfilesProfileNavBarMenuItemsGetRequest: TeamDirectoryProfilesProfileNavBarMenuItemsGetRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileNavBarProjectsGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<PRProject>>;
    teamDirectoryProfilesProfileNavBarProjectsPost(profile: string, teamDirectoryProfilesProfileNavBarProjectsGetRequest: TeamDirectoryProfilesProfileNavBarProjectsGetRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileNavBarProjectsProjectDelete(profile: string, project: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileNotificationSettingsGet(profile: string, $fields?: string, options?: any): AxiosPromise<GlobalNotificationSettings>;
    teamDirectoryProfilesProfileNotificationSettingsPatch(profile: string, teamDirectoryProfilesProfileNotificationSettingsGetRequest?: TeamDirectoryProfilesProfileNotificationSettingsGetRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfilePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileDeleteRequest?: TeamDirectoryProfilesProfileDeleteRequest, options?: any): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfilePermanentTokensCurrentDelete(profile: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfilePermanentTokensGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<TeamDirectoryProfilesProfilePermanentTokensGet200Response>;
    teamDirectoryProfilesProfilePermanentTokensPost(profile: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: any): AxiosPromise<TeamDirectoryProfilesProfilePermanentTokensGet200Response1>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdDelete(profile: string, tokenId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdPatch(profile: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileReactivatePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileReactivatePatchRequest?: TeamDirectoryProfilesProfileReactivatePatchRequest, options?: any): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileSettingsGet(profile: string, $fields?: string, options?: any): AxiosPromise<SettingsValue>;
    teamDirectoryProfilesProfileSettingsPatch(profile: string, teamDirectoryProfilesProfileSettingsGetRequest?: TeamDirectoryProfilesProfileSettingsGetRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileSpokenLanguagesGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<TDProfileLanguage>>;
    teamDirectoryProfilesProfileSpokenLanguagesLanguageDelete(profile: string, language: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileSpokenLanguagesPost(profile: string, teamDirectoryProfilesProfileSpokenLanguagesGetRequest: TeamDirectoryProfilesProfileSpokenLanguagesGetRequest, $fields?: string, options?: any): AxiosPromise<TDProfileLanguage>;
    teamDirectoryProfilesProfileSshKeysFingerprintDelete(profile: string, fingerprint: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileSshKeysGet(profile: string, $fields?: string, options?: any): AxiosPromise<Array<SshKeyData>>;
    teamDirectoryProfilesProfileSshKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileTimezoneGet(profile: string, $fields?: string, options?: any): AxiosPromise<ATimeZone>;
    teamDirectoryProfilesProfileWorkingDaysGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<TeamDirectoryProfilesProfileWorkingDaysGet200Response>;
    teamDirectoryProfilesProfileWorkingDaysPost(profile: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: any): AxiosPromise<TDWorkingDays>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdDelete(profile: string, workingDaysId: string, options?: any): AxiosPromise<void>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdPatch(profile: string, workingDaysId: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: any): AxiosPromise<TDWorkingDays>;
    teamDirectoryProfilesWorkingDaysGet($skip?: string, $top?: number, profiles?: Array<string>, since?: string, till?: string, $fields?: string, options?: any): AxiosPromise<TeamDirectoryProfilesWorkingDaysGet200Response>;
    teamDirectoryRolesGet(query?: string, withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<Array<TDRole>>;
    teamDirectoryRolesIdDelete(id: string, options?: any): AxiosPromise<void>;
    teamDirectoryRolesIdGet(id: string, $fields?: string, options?: any): AxiosPromise<TDRole>;
    teamDirectoryRolesIdPatch(id: string, $fields?: string, teamDirectoryRolesIdDeleteRequest?: TeamDirectoryRolesIdDeleteRequest, options?: any): AxiosPromise<TDRole>;
    teamDirectoryRolesIdRestorePost(id: string, $fields?: string, options?: any): AxiosPromise<TDRole>;
    teamDirectoryRolesPost(teamDirectoryRolesGetRequest: TeamDirectoryRolesGetRequest, $fields?: string, options?: any): AxiosPromise<TDRole>;
    teamDirectoryStatsGet(teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: any): AxiosPromise<TDStats>;
    teamDirectoryTeamsGet($skip?: string, $top?: number, query?: string, withArchived?: boolean, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersTeamsDelete200Response>;
    teamDirectoryTeamsIdCancelDisbandingPost(id: string, options?: any): AxiosPromise<void>;
    teamDirectoryTeamsIdDelete(id: string, $fields?: string, options?: any): AxiosPromise<Array<TDTeam>>;
    teamDirectoryTeamsIdDirectMembersGet(id: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: any): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    teamDirectoryTeamsIdDisbandDelete(id: string, $fields?: string, options?: any): AxiosPromise<Array<TDTeam>>;
    teamDirectoryTeamsIdGet(id: string, $fields?: string, options?: any): AxiosPromise<TDTeam>;
    teamDirectoryTeamsIdPatch(id: string, $fields?: string, teamDirectoryTeamsIdDeleteRequest?: TeamDirectoryTeamsIdDeleteRequest, options?: any): AxiosPromise<TDTeam>;
    teamDirectoryTeamsIdRestorePost(id: string, $fields?: string, options?: any): AxiosPromise<TDTeam>;
    teamDirectoryTeamsPost(teamDirectoryTeamsGetRequest: TeamDirectoryTeamsGetRequest, $fields?: string, options?: any): AxiosPromise<TDTeam>;
    timeTrackingItemsGet(subject: string, $skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<TimeTrackingItemsGet200Response>;
    timeTrackingItemsItemIdDelete(itemId: string, options?: any): AxiosPromise<void>;
    timeTrackingItemsItemIdPatch(itemId: string, timeTrackingItemsItemIdDeleteRequest?: TimeTrackingItemsItemIdDeleteRequest, options?: any): AxiosPromise<void>;
    timeTrackingItemsPost(timeTrackingItemsGetRequest: TimeTrackingItemsGetRequest, $fields?: string, options?: any): AxiosPromise<TimeTrackingItem>;
    todoGet($skip?: string, $top?: number, open?: boolean, from?: string, till?: string, $fields?: string, options?: any): AxiosPromise<TodoGet200Response>;
    todoIdDelete(id: string, options?: any): AxiosPromise<void>;
    todoIdPatch(id: string, todoIdDeleteRequest?: TodoIdDeleteRequest, options?: any): AxiosPromise<void>;
    todoPost(todoGetRequest: TodoGetRequest, $fields?: string, options?: any): AxiosPromise<TodoItemRecord>;
    trustedCertificatesGet($fields?: string, options?: any): AxiosPromise<Array<TrustedCertificate>>;
    trustedCertificatesIdDelete(id: string, options?: any): AxiosPromise<void>;
    trustedCertificatesIdPatch(id: string, trustedCertificatesIdDeleteRequest?: TrustedCertificatesIdDeleteRequest, options?: any): AxiosPromise<void>;
    trustedCertificatesInfoGet(data: string, $fields?: string, options?: any): AxiosPromise<CertificateInfo>;
    trustedCertificatesPost(trustedCertificatesGetRequest: TrustedCertificatesGetRequest, $fields?: string, options?: any): AxiosPromise<TrustedCertificate>;
    unfurlsBlockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: any): AxiosPromise<void>;
    unfurlsBlockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: any): AxiosPromise<void>;
    unfurlsCheckBlockedPost(unfurlsCheckBlockedPostRequest: UnfurlsCheckBlockedPostRequest, options?: any): AxiosPromise<boolean>;
    unfurlsListBlockedGet($skip?: string, $top?: number, $fields?: string, options?: any): AxiosPromise<UnfurlsListBlockedGet200Response>;
    unfurlsUnblockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: any): AxiosPromise<void>;
    unfurlsUnblockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: any): AxiosPromise<void>;
    uploadsChatPublicUrlChannelMessageAttachmentIdGet(channel: string, message: string, attachmentId: string, options?: any): AxiosPromise<string>;
    uploadsImageIdGet(id: string, $fields?: string, options?: any): AxiosPromise<ImageAttachmentMeta>;
    uploadsPost(uploadsPostRequest: UploadsPostRequest, options?: any): AxiosPromise<string>;
};
export interface SpaceApiInterface {
    absencesAbsenceReasonsGet(withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<AbsenceReasonRecord>>;
    absencesAbsenceReasonsIdDelete(id: string, _delete?: boolean, options?: AxiosRequestConfig): AxiosPromise<void>;
    absencesAbsenceReasonsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AbsenceReasonRecord>;
    absencesAbsenceReasonsIdPatch(id: string, absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AbsenceReasonRecord>;
    absencesAbsenceReasonsPost(absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AbsenceReasonRecord>;
    absencesGet($skip?: string, $top?: number, member?: string, members?: Array<string>, location?: string, team?: string, since?: string, till?: string, viewMode?: AbsenceListMode, reason?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AbsencesGet200Response>;
    absencesIdApprovePost(id: string, absencesIdApprovePostRequest: AbsencesIdApprovePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    absencesIdDelete(id: string, _delete?: boolean, options?: AxiosRequestConfig): AxiosPromise<void>;
    absencesIdDeleteApprovalDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    absencesIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AbsenceRecord>;
    absencesIdPatch(id: string, absencesIdDeleteRequest: AbsencesIdDeleteRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AbsenceRecord>;
    absencesMembermemberGet(member: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<AbsenceRecord>>;
    absencesPost(absencesGetRequest: AbsencesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AbsenceRecord>;
    administrationSupportPost($fields?: string, options?: AxiosRequestConfig): AxiosPromise<SupportProfileDTO>;
    administrationUserAgreementEnabledGet(options?: AxiosRequestConfig): AxiosPromise<boolean>;
    administrationUserAgreementEnabledPost(administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    administrationUserAgreementGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<UAUserAgreement>;
    administrationUserAgreementPatch(administrationUserAgreementGetRequest: AdministrationUserAgreementGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<UAUserAgreement>;
    administrationUserAgreementStatusGet($skip?: string, $top?: number, query?: string, accepted?: boolean, activeProfilesOnly?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AdministrationUserAgreementStatusGet200Response>;
    administrationUserAgreementStatusProfileGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<UAUserAgreementStatus>;
    applicationsApplicationAuthorizationsAuthorizedContextsGet(application: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PermissionContextApi>>;
    applicationsApplicationAuthorizationsAuthorizedRightsDelete(application: string, contextIdentifier: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationAuthorizationsAuthorizedRightsGet(application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<RightDTO>>;
    applicationsApplicationAuthorizationsAuthorizedRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationAuthorizationsRequiredRightsGet(application: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<RightDTO>>;
    applicationsApplicationAuthorizationsRequiredRightsPatch(application: string, applicationsApplicationAuthorizationsRequiredRightsGetRequest: ApplicationsApplicationAuthorizationsRequiredRightsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationBearerTokenGet(application: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    applicationsApplicationClientSecretGet(application: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    applicationsApplicationClientSecretRegeneratePost(application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationDelete(application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationForceRemovePost(application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationGet(application: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ESApp>;
    applicationsApplicationLastClientCredentialsAccessGet(application: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AccessRecord>;
    applicationsApplicationPatch(application: string, $fields?: string, applicationsApplicationDeleteRequest?: ApplicationsApplicationDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<ESApp>;
    applicationsApplicationPermanentTokensCurrentDelete(application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationPermanentTokensGet(application: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ApplicationsApplicationPermanentTokensGet200Response>;
    applicationsApplicationPermanentTokensPost(application: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ApplicationsApplicationPermanentTokensGet200Response1>;
    applicationsApplicationPermanentTokensTokenIdDelete(application: string, tokenId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationPermanentTokensTokenIdPatch(application: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationPublicKeysGet(application: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    applicationsApplicationRestorePost(application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationSigningKeyGet(application: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    applicationsApplicationSigningKeyRegeneratePost(application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationSshKeysFingerprintDelete(application: string, fingerprint: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationSshKeysGet(application: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<SshKeyData>>;
    applicationsApplicationSshKeysPost(application: string, applicationsApplicationSshKeysGetRequest: ApplicationsApplicationSshKeysGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationUiExtensionsDisableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationUiExtensionsDisableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationUiExtensionsEnableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationUiExtensionsEnableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationUiExtensionsGet(application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<AppUiExtensionApi>>;
    applicationsApplicationUnfurlDomainsAuthorizePost(application: string, applicationsApplicationUnfurlDomainsAuthorizePostRequest: ApplicationsApplicationUnfurlDomainsAuthorizePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationUnfurlDomainsGet(application: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ApplicationUnfurlDomain>>;
    applicationsApplicationUnfurlPatternsAuthorizePost(application: string, applicationsApplicationUnfurlPatternsAuthorizePostRequest: ApplicationsApplicationUnfurlPatternsAuthorizePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationUnfurlPatternsGet(application: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ApplicationUnfurlPattern>>;
    applicationsApplicationVerificationTokenGet(application: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    applicationsApplicationVerificationTokenRegeneratePost(application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationWebhooksGet(application: string, withArchived?: boolean, query?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ApplicationsApplicationWebhooksGet200Response>;
    applicationsApplicationWebhooksPost(application: string, applicationsApplicationWebhooksGetRequest: ApplicationsApplicationWebhooksGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<WebhookRecord>;
    applicationsApplicationWebhooksWebhookIdBearerTokenGet(application: string, webhookId: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersGet(application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<CustomHttpHeaderDTO>>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest: ApplicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdDelete(application: string, webhookId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdPatch(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdPost(application: string, webhookId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdSigningKeyGet(application: string, webhookId: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    applicationsApplicationWebhooksWebhookIdSigningKeyRegeneratePost(application: string, webhookId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsGet(application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<SubscriptionDTO>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest: ApplicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<SubscriptionDTO>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDelete(application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdPatch(application: string, webhookId: string, subscriptionId: string, $fields?: string, applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<SubscriptionDTO>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdRequestMissingRightsPost(application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsAuthorizationsAuthorizedApplicationsGet(contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ESApp>>;
    applicationsPagedGet($skip?: string, $top?: number, name?: string, owner?: Array<string>, withArchived?: boolean, withManaged?: boolean, ordering?: AppsOrdering, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ApplicationsPagedGet200Response>;
    applicationsPost(applicationsPostRequest: ApplicationsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ESApp>;
    applicationsUiExtensionsPatch(applicationsUiExtensionsPatchRequest: ApplicationsUiExtensionsPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsUnfurlsDomainsPatch(applicationsUnfurlsDomainsPatchRequest: ApplicationsUnfurlsDomainsPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsUnfurlsPatternsPatch(applicationsUnfurlsPatternsPatchRequest: ApplicationsUnfurlsPatternsPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsUnfurlsQueueContentPost(applicationsUnfurlsQueueContentPostRequest: ApplicationsUnfurlsQueueContentPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PostUnfurlContentResult>>;
    applicationsUnfurlsQueueGet(batchSize: number, fromEtag?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ApplicationUnfurlQueueItem>>;
    applicationsUnfurlsQueueRequestExternalAuthPost(applicationsUnfurlsQueueRequestExternalAuthPostRequest: ApplicationsUnfurlsQueueRequestExternalAuthPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    applicationsUnfurlsQueueResetExternalAuthRequestsPost(applicationsUnfurlsQueueResetExternalAuthRequestsPostRequest: ApplicationsUnfurlsQueueResetExternalAuthRequestsPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesConfigDelete($fields?: string, options?: AxiosRequestConfig): AxiosPromise<AuthConfig>;
    authModulesConfigGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<AuthConfig>;
    authModulesConfigPut(authModulesConfigDeleteRequest?: AuthModulesConfigDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesDiscoverOidcGet(discoveryEndpoint: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<OIDCDiscovery>;
    authModulesGet(withDisabled?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ESAuthModule>>;
    authModulesIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesIdLoginsIdentifierChangePost(id: string, identifier: string, authModulesIdLoginsIdentifierChangePostRequest: AuthModulesIdLoginsIdentifierChangePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesIdLoginsIdentifierDelete(identifier: string, id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesIdLoginsIdentifierResetPost(id: string, identifier: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesIdPatch(id: string, authModulesIdDeleteRequest?: AuthModulesIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesIdSamlMetadataPost(id: string, authModulesIdSamlMetadataPostRequest: AuthModulesIdSamlMetadataPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<SamlMetadataResponse>;
    authModulesKeykeyGet(key: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ESAuthModule>;
    authModulesPost(authModulesGetRequest: AuthModulesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ESAuthModule>;
    authModulesReorderPost(authModulesReorderPostRequest: AuthModulesReorderPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesTestBuiltInPost(authModulesTestBuiltInPostRequest: AuthModulesTestBuiltInPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    authModulesTestLdapPost(authModulesTestLdapPostRequest: AuthModulesTestLdapPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ESDefaultProfileLoginDetails>;
    authModulesThrottledLoginsDelete(logins: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesThrottledLoginsGet($skip?: string, $top?: number, login?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<AuthModulesThrottledLoginsDelete200Response>;
    authModulesThrottledLoginsOrgStatusDelete(options?: AxiosRequestConfig): AxiosPromise<void>;
    authModulesThrottledLoginsOrgStatusGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<OrgThrottlingStatus>;
    authModulesUsagesGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<AuthModuleUsage>>;
    billingAdminOverdraftsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Overdrafts>;
    billingAdminOverdraftsPut(billingAdminOverdraftsGetRequest: BillingAdminOverdraftsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    billingAdminReportsBillingPeriodGet(billingPeriod: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<BillingReport>;
    billingAdminReportsTodayGet(date: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TodayBillingReport>;
    billingAdminTrialPut(billingAdminTrialPutRequest: BillingAdminTrialPutRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    blogAliasaliasGet(alias: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ArticleRecord>;
    blogExternalIdidGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ArticleRecord>;
    blogGet($skip?: string, $top?: number, term?: string, dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, forProfile?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<BlogGet200Response>;
    blogIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    blogIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ArticleRecord>;
    blogIdPatch(id: string, $fields?: string, blogIdDeleteRequest?: BlogIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<ArticleRecord>;
    blogImportPost(blogImportPostRequest: BlogImportPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ArticleImportResult>>;
    blogPost(blogGetRequest: BlogGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ArticleRecord>;
    blogStatsGet(dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<BGStats>;
    calendarsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<AbsenceEvent>>;
    calendarsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<BirthdayEvent>>;
    calendarsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<BirthdayEvent>>;
    calendarsEventParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<MeetingRecord>;
    calendarsEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<MeetingRecord>>;
    calendarsEventsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<MeetingRecord>;
    calendarsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<HolidaysEvent>>;
    calendarsMeetingsGet($skip?: string, $top?: number, summaryQuery?: string, locationsQuery?: Array<string>, startingAfter?: string, endingAfter?: string, endingBefore?: string, startingBefore?: string, profiles?: Array<string>, teams?: Array<string>, organizer?: string, includePrivate?: boolean, includeArchived?: boolean, includeMeetingInstances?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CalendarsMeetingsGet200Response>;
    calendarsMeetingsIdConferenceRoomsDelete(id: string, roomId: string, dateTime: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    calendarsMeetingsIdConferenceRoomsPost(id: string, calendarsMeetingsIdConferenceRoomsDeleteRequest: CalendarsMeetingsIdConferenceRoomsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    calendarsMeetingsIdDelete(id: string, targetDate?: string, modificationKind?: RecurrentModification, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DTOMeeting>;
    calendarsMeetingsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DTOMeeting>;
    calendarsMeetingsIdParticipationStatusPatch(id: string, calendarsMeetingsIdParticipationStatusPatchRequest: CalendarsMeetingsIdParticipationStatusPatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DTOMeeting>;
    calendarsMeetingsIdPatch(id: string, $fields?: string, calendarsMeetingsIdDeleteRequest?: CalendarsMeetingsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<DTOMeeting>;
    calendarsMeetingsNextOccurrenceGet(meetingId: string, since?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<MeetingOccurrenceTime>;
    calendarsMeetingsOccurrencesByMeetingGet(meetingIds: Array<string>, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<MeetingWithOccurrenceTime>>;
    calendarsMeetingsOccurrencesGet(meetingId: string, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<MeetingOccurrenceTime>>;
    calendarsMeetingsParticipationStatusesExternalGet(id: string, emails: Array<string>, options?: AxiosRequestConfig): AxiosPromise<Array<EventParticipationStatus>>;
    calendarsMeetingsParticipationStatusesGet(id: string, profileIds: Array<string>, options?: AxiosRequestConfig): AxiosPromise<Array<EventParticipationStatus>>;
    calendarsMeetingsPost(calendarsMeetingsGetRequest: CalendarsMeetingsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DTOMeeting>;
    calendarsMeetingsProfileParticipationGet(profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<CalendarsMeetingsProfileParticipationGet200ResponseInner>>;
    calendarsMeetingsProfileParticipationRecordsGet(profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<DTOMeetingRSVP>>;
    calendarsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<MembershipEvent>>;
    calendarsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<NonWorkingDaysEvent>>;
    callsPost(callsPostRequest: CallsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Room>;
    chatsChannelsAllChannelsGet(query: string, $skip?: string, $top?: number, quickFilter?: AllChannelsFilter, sortColumn?: AllChannelsSortColumn, sortOrder?: ColumnSortOrder, publicOnly?: boolean, withArchived?: boolean, subscriber?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsAllChannelsGet200Response>;
    chatsChannelsChannelAdministratorGet(channel: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CPrincipal>;
    chatsChannelsChannelAdministratorPatch(channel: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelArchiveDelete(channel: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelAttachmentsFilesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsImagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsLinksGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelAttachmentsVideosGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelAttachmentsGet200Response>;
    chatsChannelsChannelDelete(channel: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelDescriptionPatch(channel: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelGet(channel: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<M2ChannelRecord>;
    chatsChannelsChannelIconPatch(channel: string, chatsChannelsChannelIconPatchRequest?: ChatsChannelsChannelIconPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelNamePatch(channel: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelRestoreArchivedPost(channel: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelSubscribersTeamsDelete(channel: string, teams: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelSubscribersTeamsGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersTeamsDelete200Response>;
    chatsChannelsChannelSubscribersTeamsPost(channel: string, chatsChannelsChannelSubscribersTeamsDeleteRequest: ChatsChannelsChannelSubscribersTeamsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelSubscribersUsersDelete(channel: string, profiles: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsChannelSubscribersUsersGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    chatsChannelsChannelSubscribersUsersPost(channel: string, chatsChannelsChannelSubscribersUsersDeleteRequest: ChatsChannelsChannelSubscribersUsersDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsConversationsChannelConvertPost(channel: string, chatsChannelsConversationsChannelConvertPostRequest: ChatsChannelsConversationsChannelConvertPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<M2ChannelRecord>;
    chatsChannelsConversationsChannelSubjectPatch(channel: string, chatsChannelsConversationsChannelSubjectPatchRequest: ChatsChannelsConversationsChannelSubjectPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsChannelsConversationsPost(chatsChannelsConversationsPostRequest: ChatsChannelsConversationsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<M2ChannelRecord>;
    chatsChannelsDmPost(chatsChannelsDmPostRequest: ChatsChannelsDmPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<M2ChannelRecord>;
    chatsChannelsIsNameFreePost(chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): AxiosPromise<boolean>;
    chatsChannelsPost(chatsChannelsPostRequest: ChatsChannelsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<M2ChannelRecord>;
    chatsMessagesDeleteMessagePost(chatsMessagesDeleteMessagePostRequest: ChatsMessagesDeleteMessagePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsMessagesEditMessagePost(chatsMessagesEditMessagePostRequest: ChatsMessagesEditMessagePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsMessagesGet(channel: string, sorting: MessagesSorting, batchSize: number, startFromDate?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GetMessagesResponse>;
    chatsMessagesImportPost(chatsMessagesImportPostRequest: ChatsMessagesImportPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsMessagesMessageGet(message: string, channel: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChannelItemRecord>;
    chatsMessagesPinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    chatsMessagesPinnedMessagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsMessagesPinnedMessagesGet200Response>;
    chatsMessagesSendMessagePost(chatsMessagesSendMessagePostRequest: ChatsMessagesSendMessagePostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChannelItemRecord>;
    chatsMessagesSendPost(chatsMessagesSendPostRequest: ChatsMessagesSendPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChannelItemRecord>;
    chatsMessagesUnpinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    checklistsChecklistItemsPlanItemDelete(checklist: string, planItem: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    checklistsChecklistItemsPlanItemGet(checklist: string, planItem: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PlanItem>;
    checklistsChecklistItemsPlanItemMovePost(checklist: string, planItem: string, checklistsChecklistItemsPlanItemMovePostRequest: ChecklistsChecklistItemsPlanItemMovePostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PlanItem>;
    checklistsChecklistItemsPlanItemPatch(checklist: string, planItem: string, $fields?: string, checklistsChecklistItemsPlanItemDeleteRequest?: ChecklistsChecklistItemsPlanItemDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<PlanItem>;
    checklistsChecklistItemsPost(checklist: string, checklistsChecklistItemsPostRequest: ChecklistsChecklistItemsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PlanItem>;
    customFieldsExtendedTypesGet(scope?: ExtendedTypeScopeType, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ExtendedType>>;
    customFieldsTypeKeyAllValuesGet(typeKey: string, $skip?: string, $top?: number, extendedEntityIds?: Array<string>, scope?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldsTypeKeyAllValuesGet200Response>;
    customFieldsTypeKeyEntityIdValuesGet(typeKey: string, entityId: string, scope?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldsRecord>;
    customFieldsTypeKeyEntityIdValuesPatch(entityId: string, typeKey: string, customFieldsTypeKeyEntityIdValuesGetRequest: CustomFieldsTypeKeyEntityIdValuesGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsTypeKeyEnumValuesCustomFieldIdGet(typeKey: string, customFieldId: string, $skip?: string, $top?: number, query?: string, ordering?: EnumValueOrdering, countRecords?: boolean, addedByProfileId?: string, scope?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldsTypeKeyEnumValuesCustomFieldIdGet200Response>;
    customFieldsTypeKeyEnumValuesCustomFieldIdPost(typeKey: string, customFieldId: string, customFieldsTypeKeyEnumValuesCustomFieldIdGetRequest: CustomFieldsTypeKeyEnumValuesCustomFieldIdGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<EnumValueData>>;
    customFieldsTypeKeyFieldsGet(typeKey: string, withArchived?: boolean, scope?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<CustomField>>;
    customFieldsTypeKeyFieldsIdArchivePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsTypeKeyFieldsIdDelete(typeKey: string, id: string, scope: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsTypeKeyFieldsIdFilterValuesGet(typeKey: string, id: string, $skip?: string, $top?: number, scope?: string, calculateTotal?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldsTypeKeyFieldsIdFilterValuesGet200Response>;
    customFieldsTypeKeyFieldsIdPatch(typeKey: string, id: string, customFieldsTypeKeyFieldsIdDeleteRequest: CustomFieldsTypeKeyFieldsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsTypeKeyFieldsIdRestorePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsTypeKeyFieldsPost(typeKey: string, customFieldsTypeKeyFieldsGetRequest: CustomFieldsTypeKeyFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomField>;
    customFieldsTypeKeyFieldsReorderPost(typeKey: string, customFieldsTypeKeyFieldsReorderPostRequest: CustomFieldsTypeKeyFieldsReorderPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldArchivePost(entityType: string, customField: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldDelete(entityType: string, customField: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesEnumValueToRemoveDelete(entityType: string, customField: string, enumValueToRemove: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet(entityType: string, customField: string, query?: string, ordering?: EnumValueOrdering, addedByProfileId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet200Response>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CFEnumValue>;
    customFieldsV2EntityTypeFieldsCustomFieldGet(entityType: string, customField: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldData>;
    customFieldsV2EntityTypeFieldsCustomFieldPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldDeleteRequest?: CustomFieldsV2EntityTypeFieldsCustomFieldDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsCustomFieldRestorePost(entityType: string, customField: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2EntityTypeFieldsGet(entityType: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<CustomFieldData>>;
    customFieldsV2EntityTypeFieldsPost(entityType: string, customFieldsV2EntityTypeFieldsGetRequest: CustomFieldsV2EntityTypeFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldData>;
    customFieldsV2EntityTypeFieldsReorderPost(entityType: string, customFieldsV2EntityTypeFieldsReorderPostRequest: CustomFieldsV2EntityTypeFieldsReorderPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2ValuesEntityCustomFieldGet(entity: string, customField: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CustomFieldValueData>;
    customFieldsV2ValuesEntityCustomFieldPost(entity: string, customField: string, customFieldsV2ValuesEntityCustomFieldGetRequest: CustomFieldsV2ValuesEntityCustomFieldGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    customFieldsV2ValuesEntityGet(entity: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<CustomFieldValueData>>;
    customFieldsV2ValuesEntityPost(entity: string, customFieldsV2ValuesEntityGetRequest: CustomFieldsV2ValuesEntityGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    emojisAddPost(emojisAddPostRequest: EmojisAddPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    emojisDeletePost(emojisDeletePostRequest: EmojisDeletePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    emojisExistsGet(emoji: string, options?: AxiosRequestConfig): AxiosPromise<boolean>;
    emojisFrequentlyUsedGet(options?: AxiosRequestConfig): AxiosPromise<Array<string>>;
    emojisRecordUsagePost(emojisRecordUsagePostRequest: EmojisRecordUsagePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    emojisSearchGet(query: string, $skip?: string, $top?: number, version?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<EmojisSearchGet200Response>;
    externalLinkPatternsDelete(pattern: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    externalLinkPatternsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ExternalLinkPattern>>;
    externalLinkPatternsPost(externalLinkPatternsDeleteRequest: ExternalLinkPatternsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    httpApiModelGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<HAModel>;
    notificationsChannelSubscriptionsGet(channel: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<SubscriptionDTO>>;
    notificationsChannelSubscriptionsIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    notificationsChannelSubscriptionsIdPatch(id: string, $fields?: string, notificationsChannelSubscriptionsIdDeleteRequest?: NotificationsChannelSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<SubscriptionDTO>;
    notificationsChannelSubscriptionsIdRequestMissingRightsPost(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    notificationsChannelSubscriptionsPost(notificationsChannelSubscriptionsGetRequest: NotificationsChannelSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<SubscriptionDTO>;
    notificationsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<EventSubjectInfoDTO>>;
    notificationsPersonalCustomSubscriptionsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<SubscriptionDTO>>;
    notificationsPersonalCustomSubscriptionsIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    notificationsPersonalCustomSubscriptionsIdPatch(id: string, $fields?: string, notificationsPersonalCustomSubscriptionsIdDeleteRequest?: NotificationsPersonalCustomSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<SubscriptionDTO>;
    notificationsPersonalCustomSubscriptionsPost(notificationsPersonalCustomSubscriptionsGetRequest: NotificationsPersonalCustomSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<SubscriptionDTO>;
    notificationsPersonalSubscriptionsAllPersonalSubscriptionTargetsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PersonalSubscriptionTarget>>;
    notificationsPersonalSubscriptionsPersonalSubscriptionSettingsGet(profile: string, feed: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PersonalSubscriptionSettings>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    notificationsPrivateFeedsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PrivateFeed>>;
    notificationsPrivateFeedsIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    notificationsPrivateFeedsIdPatch(id: string, $fields?: string, notificationsPrivateFeedsIdDeleteRequest?: NotificationsPrivateFeedsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<PrivateFeed>;
    notificationsPrivateFeedsPost(notificationsPrivateFeedsGetRequest: NotificationsPrivateFeedsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PrivateFeed>;
    organizationDomainsCheckGet(domain: string, options?: AxiosRequestConfig): AxiosPromise<CDomainStatus>;
    organizationDomainsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<OrgDomainDTO>>;
    organizationDomainsPatch(organizationDomainsGetRequest: OrganizationDomainsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    organizationGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<OrganizationRecord>;
    organizationJetSalesLicenseActivationUrlGet(options?: AxiosRequestConfig): AxiosPromise<string>;
    organizationJetSalesUrlGet(options?: AxiosRequestConfig): AxiosPromise<string>;
    organizationPatch(organizationGetRequest: OrganizationGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesCreatePost(permissionRolesCreatePostRequest: PermissionRolesCreatePostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<RoleDTO>;
    permissionRolesGetPost(applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<RoleDTO>>;
    permissionRolesRoleId2FaRequirementGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TwoFactorAuthenticationRequirement>;
    permissionRolesRoleId2FaRequirementPatch(roleId: string, permissionRolesRoleId2FaRequirementGetRequest?: PermissionRolesRoleId2FaRequirementGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdDelete(roleId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdPatch(roleId: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdPermissionsDelete(roleId: string, rightCodes: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdPermissionsGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<RightDTO>>;
    permissionRolesRoleIdPermissionsPost(roleId: string, permissionRolesRoleIdPermissionsDeleteRequest: PermissionRolesRoleIdPermissionsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdProfilesGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDMemberProfile>>;
    permissionRolesRoleIdProfilesProfileDelete(roleId: string, profile: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdProfilesProfilePost(roleId: string, profile: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdResetRolePermissionsToDefaultPost(roleId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdTeamsGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDTeam>>;
    permissionRolesRoleIdTeamsTeamDelete(roleId: string, team: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionRolesRoleIdTeamsTeamPost(roleId: string, team: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    permissionsCheckPermissionPost(permissionsCheckPermissionPostRequest: PermissionsCheckPermissionPostRequest, options?: AxiosRequestConfig): AxiosPromise<boolean>;
    permissionsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<DTORightsWithHierarchy>;
    projectsAutomationDslEvaluationsConfigGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<DslEvaluationConfig>;
    projectsAutomationGraphExecutionsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<JobExecutionDTO>;
    projectsAutomationGraphExecutionsIdStopPost(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsAutomationJobExecutionsCurrentGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<JobExecutionDTO>;
    projectsAutomationJobsJobIdGet(jobId: string, project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<JobDTO>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyDelete(stepExecId: string, key: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyGet(stepExecId: string, key: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyPatch(stepExecId: string, key: string, projectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest: ProjectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsAutomationStepExecutionsUsedParametersParamParameterIdGet(parameterId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ParameterLastUsageDTO>>;
    projectsAutomationStepExecutionsUsedParametersSecretSecretIdGet(secretId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ParameterLastUsageDTO>>;
    projectsAutomationSubscriptionsLegacyChannelsDelete(project?: string, jobId?: string, unsubscribedOnly?: boolean, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsCollaboratorprofileGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsGet200Response>;
    projectsGet($skip?: string, $top?: number, term?: string, tag?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsGet200Response>;
    projectsMembermemberGet(member: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsGet200Response>;
    projectsParamsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsParamsDefaultBundleGet200Response>;
    projectsParamsDefaultBundlePost(projectsParamsDefaultBundleGetRequest: ProjectsParamsDefaultBundleGetRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsParamsIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsParamsIdPatch(id: string, projectsParamsIdDeleteRequest: ProjectsParamsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsParamsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsParamsDefaultBundleGet200Response>;
    projectsParamsInDefaultBundlePost(projectsParamsInDefaultBundleGetRequest: ProjectsParamsInDefaultBundleGetRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsPlanningBoardsBoardDelete(board: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsBoardGet(board: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<BoardRecord>;
    projectsPlanningBoardsBoardIssuesGet(board: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>;
    projectsPlanningBoardsBoardIssuesIssueDelete(issue: string, board: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsBoardIssuesIssuePost(issue: string, board: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsBoardPatch(board: string, projectsPlanningBoardsBoardDeleteRequest?: ProjectsPlanningBoardsBoardDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsSprintsPost(projectsPlanningBoardsSprintsPostRequest: ProjectsPlanningBoardsSprintsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<SprintRecord>;
    projectsPlanningBoardsSprintsSprintArchiveDelete(sprint: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintIssuesGet(sprint: string, $skip?: string, $top?: number, unresolvedOnly?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>;
    projectsPlanningBoardsSprintsSprintIssuesIssueDelete(issue: string, sprint: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintIssuesIssuePost(issue: string, sprint: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintLaunchPost(sprint: string, projectsPlanningBoardsSprintsSprintLaunchPostRequest: ProjectsPlanningBoardsSprintsSprintLaunchPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPlanningBoardsSprintsSprintPatch(sprint: string, projectsPlanningBoardsSprintsSprintPatchRequest?: ProjectsPlanningBoardsSprintsSprintPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsPost(projectsGetRequest: ProjectsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PRProject>;
    projectsPrivateProjectsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PRPrivateProject>>;
    projectsPrivateProjectsProjectRequestAccessPost(project: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessAdminsGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectAccessAdminsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessAdminsProfilesProfileDelete(project: string, profile: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessAdminsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessAdminsTeamsTeamIdDelete(project: string, teamId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectAccessCollaboratorsProfilesDelete(project: string, profile: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsProfilesGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectAccessCollaboratorsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsTeamsDelete(project: string, teamId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessCollaboratorsTeamsGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDTeam>>;
    projectsProjectAccessCollaboratorsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessMemberProfilesGet(project: string, $skip?: string, $top?: number, query?: string, includingAdmins?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    projectsProjectAccessMembersProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessMembersProfilesProfileDelete(project: string, profile: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessMembersTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessMembersTeamsTeamIdDelete(project: string, teamId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAccessViewersGet(project: string, term: string, $skip?: string, $top?: number, meOnTop?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    projectsProjectAutomationDeploymentTargetsFavoritesGet(project: string, sortBy?: string, sortOrder?: ColumnSortOrder, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<DeployTargetRecord>>;
    projectsProjectAutomationDeploymentTargetsGet(project: string, search?: string, sortBy?: string, sortOrder?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectAutomationDeploymentTargetsGet200Response>;
    projectsProjectAutomationDeploymentTargetsPost(project: string, projectsProjectAutomationDeploymentTargetsGetRequest: ProjectsProjectAutomationDeploymentTargetsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeployTargetRecord>;
    projectsProjectAutomationDeploymentTargetsTargetDelete(project: string, target: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAutomationDeploymentTargetsTargetGet(project: string, target: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeployTargetRecord>;
    projectsProjectAutomationDeploymentTargetsTargetPut(project: string, target: string, projectsProjectAutomationDeploymentTargetsTargetDeleteRequest?: ProjectsProjectAutomationDeploymentTargetsTargetDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAutomationDeploymentsFailPost(project: string, projectsProjectAutomationDeploymentsFailPostRequest: ProjectsProjectAutomationDeploymentsFailPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsFinishPost(project: string, projectsProjectAutomationDeploymentsFinishPostRequest: ProjectsProjectAutomationDeploymentsFinishPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsGet(project: string, targetIdentifier?: string, jobExecutionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectAutomationDeploymentsGet200Response>;
    projectsProjectAutomationDeploymentsPut(project: string, projectsProjectAutomationDeploymentsGetRequest: ProjectsProjectAutomationDeploymentsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsSchedulePost(project: string, projectsProjectAutomationDeploymentsSchedulePostRequest: ProjectsProjectAutomationDeploymentsSchedulePostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsStartPost(project: string, projectsProjectAutomationDeploymentsStartPostRequest: ProjectsProjectAutomationDeploymentsStartPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierDelete(project: string, targetIdentifier: string, deploymentIdentifier: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierGet(project: string, targetIdentifier: string, deploymentIdentifier: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DeploymentRecord>;
    projectsProjectAutomationGraphExecutionsGet(project: string, jobId: string, branchFilter?: string, statusFilter?: ExecutionStatus, jobTriggerFilter?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectAutomationGraphExecutionsGet200Response>;
    projectsProjectAutomationJobsGet(project: string, repoFilter: string, branchFilter: string, trigger?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectAutomationJobsGet200Response>;
    projectsProjectAutomationJobsJobIdStartPost(project: string, jobId: string, projectsProjectAutomationJobsJobIdStartPostRequest: ProjectsProjectAutomationJobsJobIdStartPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<LaunchResult>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPost(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdRejectSuggestedEditPost(project: string, discussionId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdReopenSuggestedEditPost(project: string, discussionId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatch(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsCodeDiscussionsPost(project: string, projectsProjectCodeReviewsCodeDiscussionsPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CodeDiscussionRecord>;
    projectsProjectCodeReviewsCommitSetReviewPost(project: string, projectsProjectCodeReviewsCommitSetReviewPostRequest: ProjectsProjectCodeReviewsCommitSetReviewPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CommitSetReviewRecord>;
    projectsProjectCodeReviewsGet(project: string, $skip?: string, $top?: number, state?: CodeReviewStateFilter, text?: string, author?: string, from?: string, to?: string, sort?: ReviewSorting, reviewer?: string, type?: ReviewType, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectCodeReviewsGet200Response>;
    projectsProjectCodeReviewsMergeRequestsPost(project: string, projectsProjectCodeReviewsMergeRequestsPostRequest: ProjectsProjectCodeReviewsMergeRequestsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<MergeRequestRecord>;
    projectsProjectCodeReviewsReviewIdDescriptionPatch(project: string, reviewId: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdDetailsGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CodeReviewDetailedInfo>;
    projectsProjectCodeReviewsReviewIdFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectCodeReviewsReviewIdFilesGet200Response>;
    projectsProjectCodeReviewsReviewIdGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CodeReviewRecord>;
    projectsProjectCodeReviewsReviewIdMergeFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectCodeReviewsReviewIdMergeFilesGet200Response>;
    projectsProjectCodeReviewsReviewIdMergePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdMergePutRequest: ProjectsProjectCodeReviewsReviewIdMergePutRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GitMergeResultHttp>;
    projectsProjectCodeReviewsReviewIdParticipantsUserDelete(project: string, reviewId: string, user: string, role: CodeReviewParticipantRole, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdParticipantsUserPost(project: string, reviewId: string, user: string, projectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest: ProjectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdRebasePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRebasePutRequest: ProjectsProjectCodeReviewsReviewIdRebasePutRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GitRebaseResultHttp>;
    projectsProjectCodeReviewsReviewIdRevisionsDelete(project: string, reviewId: string, revisions: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdRevisionsPost(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRevisionsDeleteRequest: ProjectsProjectCodeReviewsReviewIdRevisionsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdStatePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdStatePatchRequest: ProjectsProjectCodeReviewsReviewIdStatePatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectCodeReviewsReviewIdSuggestedReviewersGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDMemberProfile>>;
    projectsProjectCodeReviewsReviewIdTitlePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdTitlePatchRequest: ProjectsProjectCodeReviewsReviewIdTitlePatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDelete(project: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDocumentsDocumentIdCopyPost(project: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdDelete(project: string, documentId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDocumentsDocumentIdDeleteForeverDelete(project: string, documentId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDocumentsDocumentIdGet(project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdMovePatch(project: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdPatch(project: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<Document>;
    projectsProjectDocumentsDocumentIdUnarchivePatch(project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    projectsProjectDocumentsFoldersFolderDelete(project: string, folder: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderDocumentsGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response>;
    projectsProjectDocumentsFoldersFolderGet(project: string, folder: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DocumentFolder>;
    projectsProjectDocumentsFoldersFolderIntroductionDelete(project: string, folder: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderIntroductionDocumentIdPatch(project: string, folder: string, documentId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderMovePatch(project: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DocumentFolder>;
    projectsProjectDocumentsFoldersFolderPatch(project: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectDocumentsFoldersFolderSubfoldersGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response>;
    projectsProjectDocumentsFoldersPost(project: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DocumentFolder>;
    projectsProjectDocumentsPost(project: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    projectsProjectGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PRProject>;
    projectsProjectPackagesRepositoriesGet(project: string, type?: string, query?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ProjectPackageRepository>>;
    projectsProjectPackagesRepositoriesPost(project: string, projectsProjectPackagesRepositoriesGetRequest: ProjectsProjectPackagesRepositoriesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectPackageRepository>;
    projectsProjectPackagesRepositoriesRepositoryCleanupDryPost(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest: ProjectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DryCleanupResults>;
    projectsProjectPackagesRepositoriesRepositoryCleanupPost(project: string, repository: string, $fields?: string, projectsProjectPackagesRepositoriesRepositoryCleanupPostRequest?: ProjectsProjectPackagesRepositoriesRepositoryCleanupPostRequest, options?: AxiosRequestConfig): AxiosPromise<PackagesExecutionResult>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet(project: string, repository: string, connectionId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet200Response>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishPost(project: string, repository: string, connectionId: string, projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest: ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PackageRepositoryConnection>>;
    projectsProjectPackagesRepositoriesRepositoryDelete(project: string, repository: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryFilesFolderfolderPathDelete(project: string, repository: string, folderPath: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryFilesGet(project: string, repository: string, parentPath?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<FileData>>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathDelete(project: string, repository: string, filePath: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathGet(project: string, repository: string, filePath: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<FileDetails>;
    projectsProjectPackagesRepositoriesRepositoryGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectPackageRepository>;
    projectsProjectPackagesRepositoriesRepositoryPackagesGet(project: string, repository: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameDelete(project: string, repository: string, packageName: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataGet(project: string, repository: string, packageName: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PackageMetadata>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataPut(project: string, repository: string, packageName: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest?: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataVersionpackageVersionPut(project: string, repository: string, packageName: string, packageVersion: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsGet(project: string, repository: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, connectionId?: string, pinned?: boolean, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, repository: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, repository: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PackageVersionDetails>;
    projectsProjectPackagesRepositoriesRepositoryPatch(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryDeleteRequest?: ProjectsProjectPackagesRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesRepositoryUrlGet(project: string, repository: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesGet(project: string, type: string, repositoryName: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsGet(project: string, type: string, repositoryName: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PackageVersionDetails>;
    projectsProjectPackagesSearchGet(project: string, type: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response>;
    projectsProjectPackagesTypesGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PackageType>>;
    projectsProjectPatch(project: string, $fields?: string, projectsProjectDeleteRequest?: ProjectsProjectDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<PRProject>;
    projectsProjectPlanningBoardsBoardArchiveDelete(project: string, board: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningBoardsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPlanningBoardsGet200Response>;
    projectsProjectPlanningBoardsPost(project: string, projectsProjectPlanningBoardsGetRequest: ProjectsProjectPlanningBoardsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<BoardRecord>;
    projectsProjectPlanningBoardsSprintsGet(project: string, $skip?: string, $top?: number, board?: string, query?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPlanningBoardsSprintsGet200Response>;
    projectsProjectPlanningBoardsStarredGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<BoardRecord>>;
    projectsProjectPlanningChecklistsChecklistIdDelete(project: string, checklistId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningChecklistsChecklistIdFullChecklistTreeGet(project: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PlanItemChildren>>;
    projectsProjectPlanningChecklistsChecklistIdImportPost(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningChecklistsChecklistIdPatch(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdDeleteRequest?: ProjectsProjectPlanningChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningChecklistsGet(project: string, $skip?: string, $top?: number, query?: string, sorting?: ChecklistSorting, descending?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPlanningChecklistsGet200Response>;
    projectsProjectPlanningChecklistsImportPost(project: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Checklist>;
    projectsProjectPlanningChecklistsPost(project: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Checklist>;
    projectsProjectPlanningChecklistsStarredGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<Checklist>>;
    projectsProjectPlanningIssuesGet(project: string, sorting: IssuesSorting, descending: boolean, $skip?: string, $top?: number, assigneeId?: Array<string>, createdByProfileId?: string, createdBy?: Array<string>, statuses?: Array<string>, tagId?: string, query?: string, tags?: Array<string>, sprints?: Array<string>, boards?: Array<string>, customFields?: Array<string>, importTransaction?: string, creationTimeFrom?: string, creationTimeTo?: string, dueDateFrom?: string, dueDateTo?: string, topics?: Array<string>, grouping?: IssueListGrouping, deployment?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response>;
    projectsProjectPlanningIssuesImportPost(project: string, projectsProjectPlanningIssuesImportPostRequest: ProjectsProjectPlanningIssuesImportPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<IssueImportResult>;
    projectsProjectPlanningIssuesIssueIdAttachmentsDelete(project: string, issueId: string, identities: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdAttachmentsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdDelete(project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdPost(project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsDelete(project: string, issueId: string, codeReviewIds: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCommentsImportPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommentsImportPostRequest: ProjectsProjectPlanningIssuesIssueIdCommentsImportPostRequest, options?: AxiosRequestConfig): AxiosPromise<Array<string>>;
    projectsProjectPlanningIssuesIssueIdCommitsDelete(project: string, issueId: string, repository: string, commitIds: Array<string>, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdCommitsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommitsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCommitsDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdDelete(project: string, issueId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdGet(project: string, issueId: string, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Issue>;
    projectsProjectPlanningIssuesIssueIdPatch(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdDeleteRequest?: ProjectsProjectPlanningIssuesIssueIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdRestorePost(project: string, issueId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdDelete(project: string, issueId: string, tagId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdPost(project: string, issueId: string, tagId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesIssueIdToggleResolvedPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest: ProjectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesNumbernumberGet(project: string, number: number, resolveAlias?: boolean, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Issue>;
    projectsProjectPlanningIssuesPost(project: string, projectsProjectPlanningIssuesGetRequest: ProjectsProjectPlanningIssuesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Issue>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<IssueStatus>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergePatch(project: string, projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest?: ProjectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningIssuesStatusesDistributionGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<IssueStatusWithUsages>>;
    projectsProjectPlanningIssuesStatusesGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<IssueStatus>>;
    projectsProjectPlanningIssuesStatusesPatch(project: string, projectsProjectPlanningIssuesStatusesGetRequest: ProjectsProjectPlanningIssuesStatusesGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectPlanningTagsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectPlanningTagsGet200Response>;
    projectsProjectPlanningTagsPost(project: string, projectsProjectPlanningTagsGetRequest: ProjectsProjectPlanningTagsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PlanningTag>;
    projectsProjectRepositoriesRepositoryCherryPickCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCherryPickCommitPostRequest: ProjectsProjectRepositoriesRepositoryCherryPickCommitPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GitCherryPickResult>;
    projectsProjectRepositoriesRepositoryCommitBranchesGet(project: string, repository: string, commit: string, prefix?: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<BranchInfo>>;
    projectsProjectRepositoriesRepositoryCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCommitPostRequest: ProjectsProjectRepositoriesRepositoryCommitPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GitCommitResult>;
    projectsProjectRepositoriesRepositoryCommitsGet(project: string, repository: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectRepositoriesRepositoryCommitsGet200Response>;
    projectsProjectRepositoriesRepositoryDelete(project: string, repository: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryDeleteBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryDeleteBranchPostRequest: ProjectsProjectRepositoriesRepositoryDeleteBranchPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryFilesGet(project: string, repository: string, commit: string, path: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<GitFile>>;
    projectsProjectRepositoriesRepositoryGcPost(project: string, repository: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryMergeBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryMergeBranchPostRequest: ProjectsProjectRepositoriesRepositoryMergeBranchPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GitMergeBranchResult>;
    projectsProjectRepositoriesRepositoryPost(project: string, repository: string, $fields?: string, projectsProjectRepositoriesRepositoryDeleteRequest?: ProjectsProjectRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<PRRepositoryInfo>;
    projectsProjectRepositoriesRepositoryReadonlyGet(project: string, repository: string, options?: AxiosRequestConfig): AxiosPromise<boolean>;
    projectsProjectRepositoriesRepositoryReadonlyPost(project: string, repository: string, projectsProjectRepositoriesRepositoryReadonlyGetRequest: ProjectsProjectRepositoriesRepositoryReadonlyGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryRebaseBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryRebaseBranchPostRequest: ProjectsProjectRepositoriesRepositoryRebaseBranchPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GitRebaseResult>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGet(project: string, repository: string, revision: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ExternalCheckDTO>>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksPost(project: string, repository: string, revision: string, projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest: ProjectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectRepositoriesRepositorySettingsGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GitRepositorySettings>;
    projectsProjectRepositoriesRepositorySettingsPost(project: string, repository: string, projectsProjectRepositoriesRepositorySettingsGetRequest: ProjectsProjectRepositoriesRepositorySettingsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectRepositoriesRepositoryUrlGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<RepositoryUrls>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDelete(project: string, responsibilityId: string, profileId: string, role?: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdPost(project: string, responsibilityId: string, profileId: string, projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest?: ProjectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectResponsibilitiesSchemeGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<SubjectResponsibilitiesTable>>;
    projectsProjectResponsibilitiesSubjectsPost(project: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsProjectResponsibilitiesSubjectsSubjectIdPatch(project: string, subjectId: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectTopicsGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<Topic>>;
    projectsProjectTopicsIdPatch(project: string, id: string, $fields?: string, projectsProjectTopicsIdPatchRequest?: ProjectsProjectTopicsIdPatchRequest, options?: AxiosRequestConfig): AxiosPromise<Topic>;
    projectsProjectTopicsPost(project: string, projectsProjectTopicsGetRequest: ProjectsProjectTopicsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Topic>;
    projectsProjectTopicsRemoveTopicsPost(project: string, projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsProjectTopicsSetResponsiblePost(project: string, projectsProjectTopicsSetResponsiblePostRequest: ProjectsProjectTopicsSetResponsiblePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsRepositoriesFindGet(term: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsRepositoriesFindGet200Response>;
    projectsResponsibilitiesPost(projectsResponsibilitiesPostRequest: ProjectsResponsibilitiesPostRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsResponsibilitiesResponsibilityIdDelete(responsibilityId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsResponsibilitiesResponsibilityIdPatch(responsibilityId: string, projectsResponsibilitiesResponsibilityIdDeleteRequest: ProjectsResponsibilitiesResponsibilityIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsResponsibilitiesSubjectsSubjectIdDelete(subjectId: string, project?: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsRightCoderightCodeGet(rightCode: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsGet200Response>;
    projectsRightUniqueCoderightGet(right: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, _private?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsGet200Response>;
    projectsSecretsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsSecretsDefaultBundleGet200Response>;
    projectsSecretsDefaultBundlePost(projectsSecretsDefaultBundleGetRequest: ProjectsSecretsDefaultBundleGetRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsSecretsIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsSecretsIdPatch(id: string, projectsSecretsIdDeleteRequest: ProjectsSecretsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsSecretsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsSecretsDefaultBundleGet200Response>;
    projectsSecretsInDefaultBundlePost(projectsSecretsInDefaultBundleGetRequest: ProjectsSecretsInDefaultBundleGetRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    projectsTagsGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PRTag>>;
    projectsTagsTrackAccessPost(projectsTagsTrackAccessPostRequest: ProjectsTagsTrackAccessPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsTeamteamGet(team: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsGet200Response>;
    projectsVaultGet(project: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<VaultConnectionRecord>>;
    projectsVaultIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsVaultIdPatch(id: string, projectsVaultIdDeleteRequest: ProjectsVaultIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    projectsVaultPost(projectsVaultGetRequest: ProjectsVaultGetRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    publicHolidaysCalendarsGet($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PublicHolidaysCalendarsGet200Response>;
    publicHolidaysCalendarsIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    publicHolidaysCalendarsIdPatch(id: string, publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PublicHolidayCalendarRecord>;
    publicHolidaysCalendarsImportPost(publicHolidaysCalendarsImportPostRequest: PublicHolidaysCalendarsImportPostRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
    publicHolidaysCalendarsPost(publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PublicHolidayCalendarRecord>;
    publicHolidaysHolidaysGet($skip?: string, $top?: number, calendar?: string, location?: string, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PublicHolidaysHolidaysGet200Response>;
    publicHolidaysHolidaysIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    publicHolidaysHolidaysIdPatch(id: string, $fields?: string, publicHolidaysHolidaysIdDeleteRequest?: PublicHolidaysHolidaysIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<PublicHoliday>;
    publicHolidaysHolidaysPost(publicHolidaysHolidaysGetRequest: PublicHolidaysHolidaysGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PublicHoliday>;
    publicHolidaysHolidaysProfileHolidaysGet(startDate: string, endDate: string, profile: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PublicHoliday>>;
    publicHolidaysHolidaysRelatedHolidaysGet($skip?: string, $top?: number, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<PublicHolidaysHolidaysGet200Response>;
    reactionsItemEmojiDelete(item: string, emoji: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    reactionsItemEmojiGet(item: string, emoji: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<CPrincipal>>;
    reactionsItemEmojiPost(item: string, emoji: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    reactionsItemGet(item: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<EmojiReaction>>;
    richTextParseMarkdownPost(richTextParseMarkdownPostRequest: RichTextParseMarkdownPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<RtDocument>;
    teamDirectoryCalendarEventsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<AbsenceEvent>>;
    teamDirectoryCalendarEventsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<BirthdayEvent>>;
    teamDirectoryCalendarEventsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<BirthdayEvent>>;
    teamDirectoryCalendarEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<MeetingRecord>>;
    teamDirectoryCalendarEventsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<HolidaysEvent>>;
    teamDirectoryCalendarEventsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<MeetingRecord>;
    teamDirectoryCalendarEventsMeetingParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<MeetingRecord>;
    teamDirectoryCalendarEventsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<MembershipEvent>>;
    teamDirectoryCalendarEventsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<NonWorkingDaysEvent>>;
    teamDirectoryInvitationLinksGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryInvitationLinksGet200Response>;
    teamDirectoryInvitationLinksInvitationLinkIdDelete(invitationLinkId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryInvitationLinksInvitationLinkIdPatch(invitationLinkId: string, teamDirectoryInvitationLinksInvitationLinkIdDeleteRequest?: TeamDirectoryInvitationLinksInvitationLinkIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryInvitationLinksPost(teamDirectoryInvitationLinksGetRequest: TeamDirectoryInvitationLinksGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryInvitationLinksGet200Response1>;
    teamDirectoryInvitationsGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryInvitationsGet200Response>;
    teamDirectoryInvitationsIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryInvitationsIdPatch(id: string, teamDirectoryInvitationsIdDeleteRequest?: TeamDirectoryInvitationsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryInvitationsPost(teamDirectoryInvitationsGetRequest: TeamDirectoryInvitationsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Invitation>;
    teamDirectoryLanguagesGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDLanguage>>;
    teamDirectoryLocationEquipmentTypesGet(withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDLocationEquipmentType>>;
    teamDirectoryLocationEquipmentTypesNamenameDelete(name: string, _delete: boolean, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryLocationMapMemberPointsGet(locationId: string, $skip?: string, $top?: number, includeUnmarked?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryLocationMapMemberPointsGet200Response>;
    teamDirectoryLocationMapMemberPointsLocationPointIdDelete(locationPointId: string, _delete: boolean, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryLocationMapMemberPointsLocationPointIdPatch(locationPointId: string, $fields?: string, teamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest?: TeamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<TDLocationMapPoint>;
    teamDirectoryLocationMapMemberPointsPost(teamDirectoryLocationMapMemberPointsGetRequest: TeamDirectoryLocationMapMemberPointsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDLocationMapPoint>;
    teamDirectoryLocationsGet(query?: string, type?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDLocation>>;
    teamDirectoryLocationsIdDelete(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDLocation>>;
    teamDirectoryLocationsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDLocation>;
    teamDirectoryLocationsIdMapGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDLocationMap>;
    teamDirectoryLocationsIdMapPatch(id: string, teamDirectoryLocationsIdMapGetRequest: TeamDirectoryLocationsIdMapGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDLocationMap>;
    teamDirectoryLocationsIdPatch(id: string, $fields?: string, teamDirectoryLocationsIdDeleteRequest?: TeamDirectoryLocationsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<TDLocation>;
    teamDirectoryLocationsIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDLocation>;
    teamDirectoryLocationsPost(teamDirectoryLocationsGetRequest: TeamDirectoryLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDLocation>;
    teamDirectoryLocationsRestorePost(projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDLocation>>;
    teamDirectoryLocationsWithTimezoneGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDLocationWithTimeZone>>;
    teamDirectoryMemberLocationsGet($skip?: string, $top?: number, profiles?: Array<string>, locations?: Array<string>, since?: string, till?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryMemberLocationsGet200Response>;
    teamDirectoryMemberLocationsMemberLocationIdDelete(memberLocationId: string, _delete?: boolean, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryMemberLocationsMemberLocationIdGet(memberLocationId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberLocation>;
    teamDirectoryMemberLocationsMemberLocationIdPatch(memberLocationId: string, $fields?: string, teamDirectoryMemberLocationsMemberLocationIdDeleteRequest?: TeamDirectoryMemberLocationsMemberLocationIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<TDMemberLocation>;
    teamDirectoryMemberLocationsPost(teamDirectoryMemberLocationsGetRequest: TeamDirectoryMemberLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberLocation>;
    teamDirectoryMembershipEventsGet($skip?: string, $top?: number, teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryMembershipEventsGet200Response>;
    teamDirectoryMembershipsGet($skip?: string, $top?: number, identifiers?: Array<string>, profiles?: Array<string>, teams?: Array<string>, directTeams?: boolean, roles?: Array<string>, directRoles?: boolean, since?: string, till?: string, requiresApproval?: boolean, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryMembershipsGet200Response>;
    teamDirectoryMembershipsManagerCandidatesGet(term: string, $skip?: string, $top?: number, teamId?: string, excludedMemberId?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    teamDirectoryMembershipsMembershipIdDelete(membershipId: string, _delete?: boolean, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryMembershipsMembershipIdGet(membershipId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsMembershipIdPatch(membershipId: string, $fields?: string, teamDirectoryMembershipsMembershipIdDeleteRequest?: TeamDirectoryMembershipsMembershipIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsMembershipIdRequestRevokePatch(membershipId: string, teamDirectoryMembershipsMembershipIdRequestRevokePatchRequest: TeamDirectoryMembershipsMembershipIdRequestRevokePatchRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryMembershipsMembershipIdRevokeDelete(membershipId: string, till?: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryMembershipsPost(teamDirectoryMembershipsGetRequest: TeamDirectoryMembershipsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsRequestsGet($skip?: string, $top?: number, teamId?: string, direct?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryMembershipsRequestsGet200Response>;
    teamDirectoryMembershipsRequestsMembershipRequestIdDelete(membershipRequestId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMembership>;
    teamDirectoryMembershipsRequestsMembershipRequestIdPatch(membershipRequestId: string, teamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest: TeamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesAuthenticationSessionsOwnerGet(owner: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ESAuthenticationSession>>;
    teamDirectoryProfilesAuthenticationSessionsOwnerSessionIdDelete(owner: string, sessionId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesDashboardsDashboardGet(dashboard: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DashboardPreferencesRecord>;
    teamDirectoryProfilesDashboardsDashboardPatch(dashboard: string, teamDirectoryProfilesDashboardsDashboardGetRequest: TeamDirectoryProfilesDashboardsDashboardGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesEmailemailGet(email: string, verified?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesGet($skip?: string, $top?: number, query?: string, reportPastMembers?: boolean, reportFutureMembers?: boolean, reportCurrentMembers?: boolean, teamId?: string, locationId?: string, roleId?: string, meOnTop?: boolean, order?: ProfileOrder, orgRelation?: ProfileOrgRelation, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    teamDirectoryProfilesOauthConsentsOwnerApplicationsApplicationDelete(owner: string, application: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesOauthConsentsOwnerApprovedScopesIdDelete(owner: string, id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesOauthConsentsOwnerGet(owner: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<ESOAuthConsent>>;
    teamDirectoryProfilesOauthConsentsOwnerInternalApplicationsClientIdDelete(owner: string, clientId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesOauthConsentsOwnerRefreshTokensIdDelete(owner: string, id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesPost(teamDirectoryProfilesGetRequest: TeamDirectoryProfilesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfile2FaRequirementsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<Profile2FARequirement>>;
    teamDirectoryProfilesProfile2FaStatusGet(profile: string, options?: AxiosRequestConfig): AxiosPromise<TwoFactorAuthenticationStatus>;
    teamDirectoryProfilesProfile2FaTotpConfirmPost(profile: string, teamDirectoryProfilesProfile2FaTotpConfirmPostRequest: TeamDirectoryProfilesProfile2FaTotpConfirmPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfile2FaTotpDelete(profile: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfile2FaTotpPatch(profile: string, administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfile2FaTotpPost(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TwoFactorAuthenticationSecret>;
    teamDirectoryProfilesProfileApplicationPasswordsGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDelete(profile: string, passwordId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdPatch(profile: string, passwordId: string, teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest?: TeamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileApplicationPasswordsPost(profile: string, teamDirectoryProfilesProfileApplicationPasswordsGetRequest: TeamDirectoryProfilesProfileApplicationPasswordsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response1>;
    teamDirectoryProfilesProfileChecklistsChecklistIdDelete(profile: string, checklistId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileChecklistsChecklistIdFullChecklistTreeGet(profile: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PlanItemChildren>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdImportPost(profile: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileChecklistsChecklistIdPatch(profile: string, checklistId: string, teamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest?: TeamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileChecklistsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<Checklist>>;
    teamDirectoryProfilesProfileChecklistsImportPost(profile: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Checklist>;
    teamDirectoryProfilesProfileChecklistsPost(profile: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Checklist>;
    teamDirectoryProfilesProfileChecklistsStarredGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<Checklist>>;
    teamDirectoryProfilesProfileConvertToGuestPatch(profile: string, teamDirectoryProfilesProfileConvertToGuestPatchRequest: TeamDirectoryProfilesProfileConvertToGuestPatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DryRunResult>;
    teamDirectoryProfilesProfileConvertToMemberPatch(profile: string, teamDirectoryProfilesProfileConvertToMemberPatchRequest: TeamDirectoryProfilesProfileConvertToMemberPatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DryRunResult>;
    teamDirectoryProfilesProfileDeactivateDelete(profile: string, at: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileDelete(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileDocumentsDocumentIdCopyPost(profile: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDelete(profile: string, documentId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDeleteForeverDelete(profile: string, documentId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsDocumentIdGet(profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdMovePatch(profile: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdPatch(profile: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsDocumentIdUnarchivePatch(profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDelete(profile: string, folder: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDocumentsGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderGet(profile: string, folder: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DocumentFolder>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDelete(profile: string, folder: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDocumentIdPatch(profile: string, folder: string, documentId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderMovePatch(profile: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DocumentFolder>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderPatch(profile: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderSubfoldersGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response>;
    teamDirectoryProfilesProfileDocumentsFoldersPost(profile: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<DocumentFolder>;
    teamDirectoryProfilesProfileDocumentsPost(profile: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Document>;
    teamDirectoryProfilesProfileGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileGpgKeysFingerprintDelete(profile: string, fingerprint: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileGpgKeysFingerprintPatch(profile: string, fingerprint: string, teamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest?: TeamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileGpgKeysGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<GpgKeyData>>;
    teamDirectoryProfilesProfileGpgKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GpgKeyData>;
    teamDirectoryProfilesProfileIsTeamMemberGet(profile: string, teamIds: Array<string>, options?: AxiosRequestConfig): AxiosPromise<boolean>;
    teamDirectoryProfilesProfileLeadsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDMemberProfile>>;
    teamDirectoryProfilesProfileNavBarMenuItemsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<NavBarMenuItem>>;
    teamDirectoryProfilesProfileNavBarMenuItemsPatch(profile: string, teamDirectoryProfilesProfileNavBarMenuItemsGetRequest: TeamDirectoryProfilesProfileNavBarMenuItemsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileNavBarProjectsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<PRProject>>;
    teamDirectoryProfilesProfileNavBarProjectsPost(profile: string, teamDirectoryProfilesProfileNavBarProjectsGetRequest: TeamDirectoryProfilesProfileNavBarProjectsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileNavBarProjectsProjectDelete(profile: string, project: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileNotificationSettingsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<GlobalNotificationSettings>;
    teamDirectoryProfilesProfileNotificationSettingsPatch(profile: string, teamDirectoryProfilesProfileNotificationSettingsGetRequest?: TeamDirectoryProfilesProfileNotificationSettingsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfilePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileDeleteRequest?: TeamDirectoryProfilesProfileDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfilePermanentTokensCurrentDelete(profile: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfilePermanentTokensGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryProfilesProfilePermanentTokensGet200Response>;
    teamDirectoryProfilesProfilePermanentTokensPost(profile: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryProfilesProfilePermanentTokensGet200Response1>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdDelete(profile: string, tokenId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdPatch(profile: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileReactivatePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileReactivatePatchRequest?: TeamDirectoryProfilesProfileReactivatePatchRequest, options?: AxiosRequestConfig): AxiosPromise<TDMemberProfile>;
    teamDirectoryProfilesProfileSettingsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<SettingsValue>;
    teamDirectoryProfilesProfileSettingsPatch(profile: string, teamDirectoryProfilesProfileSettingsGetRequest?: TeamDirectoryProfilesProfileSettingsGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileSpokenLanguagesGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDProfileLanguage>>;
    teamDirectoryProfilesProfileSpokenLanguagesLanguageDelete(profile: string, language: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileSpokenLanguagesPost(profile: string, teamDirectoryProfilesProfileSpokenLanguagesGetRequest: TeamDirectoryProfilesProfileSpokenLanguagesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDProfileLanguage>;
    teamDirectoryProfilesProfileSshKeysFingerprintDelete(profile: string, fingerprint: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileSshKeysGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<SshKeyData>>;
    teamDirectoryProfilesProfileSshKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileTimezoneGet(profile: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ATimeZone>;
    teamDirectoryProfilesProfileWorkingDaysGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryProfilesProfileWorkingDaysGet200Response>;
    teamDirectoryProfilesProfileWorkingDaysPost(profile: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDWorkingDays>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdDelete(profile: string, workingDaysId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdPatch(profile: string, workingDaysId: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDWorkingDays>;
    teamDirectoryProfilesWorkingDaysGet($skip?: string, $top?: number, profiles?: Array<string>, since?: string, till?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TeamDirectoryProfilesWorkingDaysGet200Response>;
    teamDirectoryRolesGet(query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDRole>>;
    teamDirectoryRolesIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryRolesIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDRole>;
    teamDirectoryRolesIdPatch(id: string, $fields?: string, teamDirectoryRolesIdDeleteRequest?: TeamDirectoryRolesIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<TDRole>;
    teamDirectoryRolesIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDRole>;
    teamDirectoryRolesPost(teamDirectoryRolesGetRequest: TeamDirectoryRolesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDRole>;
    teamDirectoryStatsGet(teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDStats>;
    teamDirectoryTeamsGet($skip?: string, $top?: number, query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersTeamsDelete200Response>;
    teamDirectoryTeamsIdCancelDisbandingPost(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    teamDirectoryTeamsIdDelete(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDTeam>>;
    teamDirectoryTeamsIdDirectMembersGet(id: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ChatsChannelsChannelSubscribersUsersDelete200Response>;
    teamDirectoryTeamsIdDisbandDelete(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TDTeam>>;
    teamDirectoryTeamsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDTeam>;
    teamDirectoryTeamsIdPatch(id: string, $fields?: string, teamDirectoryTeamsIdDeleteRequest?: TeamDirectoryTeamsIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<TDTeam>;
    teamDirectoryTeamsIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDTeam>;
    teamDirectoryTeamsPost(teamDirectoryTeamsGetRequest: TeamDirectoryTeamsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TDTeam>;
    timeTrackingItemsGet(subject: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TimeTrackingItemsGet200Response>;
    timeTrackingItemsItemIdDelete(itemId: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    timeTrackingItemsItemIdPatch(itemId: string, timeTrackingItemsItemIdDeleteRequest?: TimeTrackingItemsItemIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    timeTrackingItemsPost(timeTrackingItemsGetRequest: TimeTrackingItemsGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TimeTrackingItem>;
    todoGet($skip?: string, $top?: number, open?: boolean, from?: string, till?: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TodoGet200Response>;
    todoIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    todoIdPatch(id: string, todoIdDeleteRequest?: TodoIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    todoPost(todoGetRequest: TodoGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TodoItemRecord>;
    trustedCertificatesGet($fields?: string, options?: AxiosRequestConfig): AxiosPromise<Array<TrustedCertificate>>;
    trustedCertificatesIdDelete(id: string, options?: AxiosRequestConfig): AxiosPromise<void>;
    trustedCertificatesIdPatch(id: string, trustedCertificatesIdDeleteRequest?: TrustedCertificatesIdDeleteRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    trustedCertificatesInfoGet(data: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<CertificateInfo>;
    trustedCertificatesPost(trustedCertificatesGetRequest: TrustedCertificatesGetRequest, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<TrustedCertificate>;
    unfurlsBlockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    unfurlsBlockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    unfurlsCheckBlockedPost(unfurlsCheckBlockedPostRequest: UnfurlsCheckBlockedPostRequest, options?: AxiosRequestConfig): AxiosPromise<boolean>;
    unfurlsListBlockedGet($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<UnfurlsListBlockedGet200Response>;
    unfurlsUnblockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    unfurlsUnblockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): AxiosPromise<void>;
    uploadsChatPublicUrlChannelMessageAttachmentIdGet(channel: string, message: string, attachmentId: string, options?: AxiosRequestConfig): AxiosPromise<string>;
    uploadsImageIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): AxiosPromise<ImageAttachmentMeta>;
    uploadsPost(uploadsPostRequest: UploadsPostRequest, options?: AxiosRequestConfig): AxiosPromise<string>;
}
export declare class SpaceApi extends BaseAPI implements SpaceApiInterface {
    absencesAbsenceReasonsGet(withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceReasonRecord[], any>>;
    absencesAbsenceReasonsIdDelete(id: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    absencesAbsenceReasonsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceReasonRecord, any>>;
    absencesAbsenceReasonsIdPatch(id: string, absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceReasonRecord, any>>;
    absencesAbsenceReasonsPost(absencesAbsenceReasonsGetRequest: AbsencesAbsenceReasonsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceReasonRecord, any>>;
    absencesGet($skip?: string, $top?: number, member?: string, members?: Array<string>, location?: string, team?: string, since?: string, till?: string, viewMode?: AbsenceListMode, reason?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsencesGet200Response, any>>;
    absencesIdApprovePost(id: string, absencesIdApprovePostRequest: AbsencesIdApprovePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    absencesIdDelete(id: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    absencesIdDeleteApprovalDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    absencesIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceRecord, any>>;
    absencesIdPatch(id: string, absencesIdDeleteRequest: AbsencesIdDeleteRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceRecord, any>>;
    absencesMembermemberGet(member: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceRecord[], any>>;
    absencesPost(absencesGetRequest: AbsencesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceRecord, any>>;
    administrationSupportPost($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SupportProfileDTO, any>>;
    administrationUserAgreementEnabledGet(options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<boolean, any>>;
    administrationUserAgreementEnabledPost(administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    administrationUserAgreementGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<UAUserAgreement, any>>;
    administrationUserAgreementPatch(administrationUserAgreementGetRequest: AdministrationUserAgreementGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<UAUserAgreement, any>>;
    administrationUserAgreementStatusGet($skip?: string, $top?: number, query?: string, accepted?: boolean, activeProfilesOnly?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AdministrationUserAgreementStatusGet200Response, any>>;
    administrationUserAgreementStatusProfileGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<UAUserAgreementStatus, any>>;
    applicationsApplicationAuthorizationsAuthorizedContextsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PermissionContextApi[], any>>;
    applicationsApplicationAuthorizationsAuthorizedRightsDelete(application: string, contextIdentifier: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationAuthorizationsAuthorizedRightsGet(application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<RightDTO[], any>>;
    applicationsApplicationAuthorizationsAuthorizedRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatch(application: string, applicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest: ApplicationsApplicationAuthorizationsAuthorizedRightsRequestRightsPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationAuthorizationsRequiredRightsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<RightDTO[], any>>;
    applicationsApplicationAuthorizationsRequiredRightsPatch(application: string, applicationsApplicationAuthorizationsRequiredRightsGetRequest: ApplicationsApplicationAuthorizationsRequiredRightsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationBearerTokenGet(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    applicationsApplicationClientSecretGet(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    applicationsApplicationClientSecretRegeneratePost(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationDelete(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationForceRemovePost(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESApp, any>>;
    applicationsApplicationLastClientCredentialsAccessGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AccessRecord, any>>;
    applicationsApplicationPatch(application: string, $fields?: string, applicationsApplicationDeleteRequest?: ApplicationsApplicationDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESApp, any>>;
    applicationsApplicationPermanentTokensCurrentDelete(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationPermanentTokensGet(application: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ApplicationsApplicationPermanentTokensGet200Response, any>>;
    applicationsApplicationPermanentTokensPost(application: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ApplicationsApplicationPermanentTokensGet200Response1, any>>;
    applicationsApplicationPermanentTokensTokenIdDelete(application: string, tokenId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationPermanentTokensTokenIdPatch(application: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationPublicKeysGet(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    applicationsApplicationRestorePost(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationSigningKeyGet(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    applicationsApplicationSigningKeyRegeneratePost(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationSshKeysFingerprintDelete(application: string, fingerprint: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationSshKeysGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SshKeyData[], any>>;
    applicationsApplicationSshKeysPost(application: string, applicationsApplicationSshKeysGetRequest: ApplicationsApplicationSshKeysGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationUiExtensionsDisableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationUiExtensionsDisableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationUiExtensionsEnableForEverybodyPatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationUiExtensionsEnableForMePatch(application: string, applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationUiExtensionsGet(application: string, contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AppUiExtensionApi[], any>>;
    applicationsApplicationUnfurlDomainsAuthorizePost(application: string, applicationsApplicationUnfurlDomainsAuthorizePostRequest: ApplicationsApplicationUnfurlDomainsAuthorizePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationUnfurlDomainsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ApplicationUnfurlDomain[], any>>;
    applicationsApplicationUnfurlPatternsAuthorizePost(application: string, applicationsApplicationUnfurlPatternsAuthorizePostRequest: ApplicationsApplicationUnfurlPatternsAuthorizePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationUnfurlPatternsGet(application: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ApplicationUnfurlPattern[], any>>;
    applicationsApplicationVerificationTokenGet(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    applicationsApplicationVerificationTokenRegeneratePost(application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationWebhooksGet(application: string, withArchived?: boolean, query?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ApplicationsApplicationWebhooksGet200Response, any>>;
    applicationsApplicationWebhooksPost(application: string, applicationsApplicationWebhooksGetRequest: ApplicationsApplicationWebhooksGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<WebhookRecord, any>>;
    applicationsApplicationWebhooksWebhookIdBearerTokenGet(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersGet(application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomHttpHeaderDTO[], any>>;
    applicationsApplicationWebhooksWebhookIdCustomHeadersPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest: ApplicationsApplicationWebhooksWebhookIdCustomHeadersGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationWebhooksWebhookIdDelete(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationWebhooksWebhookIdPatch(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationWebhooksWebhookIdPost(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationWebhooksWebhookIdSigningKeyGet(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    applicationsApplicationWebhooksWebhookIdSigningKeyRegeneratePost(application: string, webhookId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsGet(application: string, webhookId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO[], any>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsPost(application: string, webhookId: string, applicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest: ApplicationsApplicationWebhooksWebhookIdSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO, any>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDelete(application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdPatch(application: string, webhookId: string, subscriptionId: string, $fields?: string, applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest?: ApplicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO, any>>;
    applicationsApplicationWebhooksWebhookIdSubscriptionsSubscriptionIdRequestMissingRightsPost(application: string, webhookId: string, subscriptionId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsAuthorizationsAuthorizedApplicationsGet(contextIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESApp[], any>>;
    applicationsPagedGet($skip?: string, $top?: number, name?: string, owner?: Array<string>, withArchived?: boolean, withManaged?: boolean, ordering?: AppsOrdering, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ApplicationsPagedGet200Response, any>>;
    applicationsPost(applicationsPostRequest: ApplicationsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESApp, any>>;
    applicationsUiExtensionsPatch(applicationsUiExtensionsPatchRequest: ApplicationsUiExtensionsPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsUnfurlsDomainsPatch(applicationsUnfurlsDomainsPatchRequest: ApplicationsUnfurlsDomainsPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsUnfurlsPatternsPatch(applicationsUnfurlsPatternsPatchRequest: ApplicationsUnfurlsPatternsPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsUnfurlsQueueContentPost(applicationsUnfurlsQueueContentPostRequest: ApplicationsUnfurlsQueueContentPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PostUnfurlContentResult[], any>>;
    applicationsUnfurlsQueueGet(batchSize: number, fromEtag?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ApplicationUnfurlQueueItem[], any>>;
    applicationsUnfurlsQueueRequestExternalAuthPost(applicationsUnfurlsQueueRequestExternalAuthPostRequest: ApplicationsUnfurlsQueueRequestExternalAuthPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    applicationsUnfurlsQueueResetExternalAuthRequestsPost(applicationsUnfurlsQueueResetExternalAuthRequestsPostRequest: ApplicationsUnfurlsQueueResetExternalAuthRequestsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesConfigDelete($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AuthConfig, any>>;
    authModulesConfigGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AuthConfig, any>>;
    authModulesConfigPut(authModulesConfigDeleteRequest?: AuthModulesConfigDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesDiscoverOidcGet(discoveryEndpoint: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<OIDCDiscovery, any>>;
    authModulesGet(withDisabled?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESAuthModule[], any>>;
    authModulesIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesIdLoginsIdentifierChangePost(id: string, identifier: string, authModulesIdLoginsIdentifierChangePostRequest: AuthModulesIdLoginsIdentifierChangePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesIdLoginsIdentifierDelete(identifier: string, id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesIdLoginsIdentifierResetPost(id: string, identifier: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesIdPatch(id: string, authModulesIdDeleteRequest?: AuthModulesIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesIdSamlMetadataPost(id: string, authModulesIdSamlMetadataPostRequest: AuthModulesIdSamlMetadataPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SamlMetadataResponse, any>>;
    authModulesKeykeyGet(key: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESAuthModule, any>>;
    authModulesPost(authModulesGetRequest: AuthModulesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESAuthModule, any>>;
    authModulesReorderPost(authModulesReorderPostRequest: AuthModulesReorderPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesTestBuiltInPost(authModulesTestBuiltInPostRequest: AuthModulesTestBuiltInPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    authModulesTestLdapPost(authModulesTestLdapPostRequest: AuthModulesTestLdapPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESDefaultProfileLoginDetails, any>>;
    authModulesThrottledLoginsDelete(logins: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesThrottledLoginsGet($skip?: string, $top?: number, login?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AuthModulesThrottledLoginsDelete200Response, any>>;
    authModulesThrottledLoginsOrgStatusDelete(options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    authModulesThrottledLoginsOrgStatusGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<OrgThrottlingStatus, any>>;
    authModulesUsagesGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AuthModuleUsage[], any>>;
    billingAdminOverdraftsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Overdrafts, any>>;
    billingAdminOverdraftsPut(billingAdminOverdraftsGetRequest: BillingAdminOverdraftsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    billingAdminReportsBillingPeriodGet(billingPeriod: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BillingReport, any>>;
    billingAdminReportsTodayGet(date: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TodayBillingReport, any>>;
    billingAdminTrialPut(billingAdminTrialPutRequest: BillingAdminTrialPutRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    blogAliasaliasGet(alias: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ArticleRecord, any>>;
    blogExternalIdidGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ArticleRecord, any>>;
    blogGet($skip?: string, $top?: number, term?: string, dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, forProfile?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BlogGet200Response, any>>;
    blogIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    blogIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ArticleRecord, any>>;
    blogIdPatch(id: string, $fields?: string, blogIdDeleteRequest?: BlogIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ArticleRecord, any>>;
    blogImportPost(blogImportPostRequest: BlogImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ArticleImportResult[], any>>;
    blogPost(blogGetRequest: BlogGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ArticleRecord, any>>;
    blogStatsGet(dateFrom?: string, dateTo?: string, authorId?: string, teamId?: string, locationId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BGStats, any>>;
    calendarsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceEvent[], any>>;
    calendarsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BirthdayEvent[], any>>;
    calendarsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BirthdayEvent[], any>>;
    calendarsEventParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingRecord, any>>;
    calendarsEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingRecord[], any>>;
    calendarsEventsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingRecord, any>>;
    calendarsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<HolidaysEvent[], any>>;
    calendarsMeetingsGet($skip?: string, $top?: number, summaryQuery?: string, locationsQuery?: Array<string>, startingAfter?: string, endingAfter?: string, endingBefore?: string, startingBefore?: string, profiles?: Array<string>, teams?: Array<string>, organizer?: string, includePrivate?: boolean, includeArchived?: boolean, includeMeetingInstances?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CalendarsMeetingsGet200Response, any>>;
    calendarsMeetingsIdConferenceRoomsDelete(id: string, roomId: string, dateTime: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    calendarsMeetingsIdConferenceRoomsPost(id: string, calendarsMeetingsIdConferenceRoomsDeleteRequest: CalendarsMeetingsIdConferenceRoomsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    calendarsMeetingsIdDelete(id: string, targetDate?: string, modificationKind?: RecurrentModification, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DTOMeeting, any>>;
    calendarsMeetingsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DTOMeeting, any>>;
    calendarsMeetingsIdParticipationStatusPatch(id: string, calendarsMeetingsIdParticipationStatusPatchRequest: CalendarsMeetingsIdParticipationStatusPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DTOMeeting, any>>;
    calendarsMeetingsIdPatch(id: string, $fields?: string, calendarsMeetingsIdDeleteRequest?: CalendarsMeetingsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DTOMeeting, any>>;
    calendarsMeetingsNextOccurrenceGet(meetingId: string, since?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingOccurrenceTime, any>>;
    calendarsMeetingsOccurrencesByMeetingGet(meetingIds: Array<string>, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingWithOccurrenceTime[], any>>;
    calendarsMeetingsOccurrencesGet(meetingId: string, since: string, until: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingOccurrenceTime[], any>>;
    calendarsMeetingsParticipationStatusesExternalGet(id: string, emails: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<EventParticipationStatus[], any>>;
    calendarsMeetingsParticipationStatusesGet(id: string, profileIds: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<EventParticipationStatus[], any>>;
    calendarsMeetingsPost(calendarsMeetingsGetRequest: CalendarsMeetingsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DTOMeeting, any>>;
    calendarsMeetingsProfileParticipationGet(profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CalendarsMeetingsProfileParticipationGet200ResponseInner[], any>>;
    calendarsMeetingsProfileParticipationRecordsGet(profileId: string, events: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DTOMeetingRSVP[], any>>;
    calendarsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MembershipEvent[], any>>;
    calendarsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<NonWorkingDaysEvent[], any>>;
    callsPost(callsPostRequest: CallsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Room, any>>;
    chatsChannelsAllChannelsGet(query: string, $skip?: string, $top?: number, quickFilter?: AllChannelsFilter, sortColumn?: AllChannelsSortColumn, sortOrder?: ColumnSortOrder, publicOnly?: boolean, withArchived?: boolean, subscriber?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsAllChannelsGet200Response, any>>;
    chatsChannelsChannelAdministratorGet(channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CPrincipal, any>>;
    chatsChannelsChannelAdministratorPatch(channel: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelArchiveDelete(channel: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelAttachmentsFilesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelAttachmentsGet200Response, any>>;
    chatsChannelsChannelAttachmentsGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelAttachmentsGet200Response, any>>;
    chatsChannelsChannelAttachmentsImagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelAttachmentsGet200Response, any>>;
    chatsChannelsChannelAttachmentsLinksGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelAttachmentsGet200Response, any>>;
    chatsChannelsChannelAttachmentsVideosGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelAttachmentsGet200Response, any>>;
    chatsChannelsChannelDelete(channel: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelDescriptionPatch(channel: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelGet(channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<M2ChannelRecord, any>>;
    chatsChannelsChannelIconPatch(channel: string, chatsChannelsChannelIconPatchRequest?: ChatsChannelsChannelIconPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelNamePatch(channel: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelRestoreArchivedPost(channel: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelSubscribersTeamsDelete(channel: string, teams: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelSubscribersTeamsGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersTeamsDelete200Response, any>>;
    chatsChannelsChannelSubscribersTeamsPost(channel: string, chatsChannelsChannelSubscribersTeamsDeleteRequest: ChatsChannelsChannelSubscribersTeamsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelSubscribersUsersDelete(channel: string, profiles: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsChannelSubscribersUsersGet(channel: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersUsersDelete200Response, any>>;
    chatsChannelsChannelSubscribersUsersPost(channel: string, chatsChannelsChannelSubscribersUsersDeleteRequest: ChatsChannelsChannelSubscribersUsersDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsConversationsChannelConvertPost(channel: string, chatsChannelsConversationsChannelConvertPostRequest: ChatsChannelsConversationsChannelConvertPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<M2ChannelRecord, any>>;
    chatsChannelsConversationsChannelSubjectPatch(channel: string, chatsChannelsConversationsChannelSubjectPatchRequest: ChatsChannelsConversationsChannelSubjectPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsChannelsConversationsPost(chatsChannelsConversationsPostRequest: ChatsChannelsConversationsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<M2ChannelRecord, any>>;
    chatsChannelsDmPost(chatsChannelsDmPostRequest: ChatsChannelsDmPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<M2ChannelRecord, any>>;
    chatsChannelsIsNameFreePost(chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<boolean, any>>;
    chatsChannelsPost(chatsChannelsPostRequest: ChatsChannelsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<M2ChannelRecord, any>>;
    chatsMessagesDeleteMessagePost(chatsMessagesDeleteMessagePostRequest: ChatsMessagesDeleteMessagePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsMessagesEditMessagePost(chatsMessagesEditMessagePostRequest: ChatsMessagesEditMessagePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsMessagesGet(channel: string, sorting: MessagesSorting, batchSize: number, startFromDate?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GetMessagesResponse, any>>;
    chatsMessagesImportPost(chatsMessagesImportPostRequest: ChatsMessagesImportPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsMessagesMessageGet(message: string, channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChannelItemRecord, any>>;
    chatsMessagesPinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    chatsMessagesPinnedMessagesGet(channel: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsMessagesPinnedMessagesGet200Response, any>>;
    chatsMessagesSendMessagePost(chatsMessagesSendMessagePostRequest: ChatsMessagesSendMessagePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChannelItemRecord, any>>;
    chatsMessagesSendPost(chatsMessagesSendPostRequest: ChatsMessagesSendPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChannelItemRecord, any>>;
    chatsMessagesUnpinPatch(chatsMessagesPinPatchRequest: ChatsMessagesPinPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    checklistsChecklistItemsPlanItemDelete(checklist: string, planItem: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    checklistsChecklistItemsPlanItemGet(checklist: string, planItem: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PlanItem, any>>;
    checklistsChecklistItemsPlanItemMovePost(checklist: string, planItem: string, checklistsChecklistItemsPlanItemMovePostRequest: ChecklistsChecklistItemsPlanItemMovePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PlanItem, any>>;
    checklistsChecklistItemsPlanItemPatch(checklist: string, planItem: string, $fields?: string, checklistsChecklistItemsPlanItemDeleteRequest?: ChecklistsChecklistItemsPlanItemDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PlanItem, any>>;
    checklistsChecklistItemsPost(checklist: string, checklistsChecklistItemsPostRequest: ChecklistsChecklistItemsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PlanItem, any>>;
    customFieldsExtendedTypesGet(scope?: ExtendedTypeScopeType, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ExtendedType[], any>>;
    customFieldsTypeKeyAllValuesGet(typeKey: string, $skip?: string, $top?: number, extendedEntityIds?: Array<string>, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldsTypeKeyAllValuesGet200Response, any>>;
    customFieldsTypeKeyEntityIdValuesGet(typeKey: string, entityId: string, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldsRecord, any>>;
    customFieldsTypeKeyEntityIdValuesPatch(entityId: string, typeKey: string, customFieldsTypeKeyEntityIdValuesGetRequest: CustomFieldsTypeKeyEntityIdValuesGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsTypeKeyEnumValuesCustomFieldIdGet(typeKey: string, customFieldId: string, $skip?: string, $top?: number, query?: string, ordering?: EnumValueOrdering, countRecords?: boolean, addedByProfileId?: string, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldsTypeKeyEnumValuesCustomFieldIdGet200Response, any>>;
    customFieldsTypeKeyEnumValuesCustomFieldIdPost(typeKey: string, customFieldId: string, customFieldsTypeKeyEnumValuesCustomFieldIdGetRequest: CustomFieldsTypeKeyEnumValuesCustomFieldIdGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<EnumValueData[], any>>;
    customFieldsTypeKeyFieldsGet(typeKey: string, withArchived?: boolean, scope?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomField[], any>>;
    customFieldsTypeKeyFieldsIdArchivePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsTypeKeyFieldsIdDelete(typeKey: string, id: string, scope: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsTypeKeyFieldsIdFilterValuesGet(typeKey: string, id: string, $skip?: string, $top?: number, scope?: string, calculateTotal?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldsTypeKeyFieldsIdFilterValuesGet200Response, any>>;
    customFieldsTypeKeyFieldsIdPatch(typeKey: string, id: string, customFieldsTypeKeyFieldsIdDeleteRequest: CustomFieldsTypeKeyFieldsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsTypeKeyFieldsIdRestorePost(typeKey: string, id: string, customFieldsTypeKeyFieldsIdArchivePostRequest: CustomFieldsTypeKeyFieldsIdArchivePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsTypeKeyFieldsPost(typeKey: string, customFieldsTypeKeyFieldsGetRequest: CustomFieldsTypeKeyFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomField, any>>;
    customFieldsTypeKeyFieldsReorderPost(typeKey: string, customFieldsTypeKeyFieldsReorderPostRequest: CustomFieldsTypeKeyFieldsReorderPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldArchivePost(entityType: string, customField: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldDelete(entityType: string, customField: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesBulkUpdatePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesEnumValueToRemoveDelete(entityType: string, customField: string, enumValueToRemove: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet(entityType: string, customField: string, query?: string, ordering?: EnumValueOrdering, addedByProfileId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGet200Response, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest1, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldEnumValuesPost(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest: CustomFieldsV2EntityTypeFieldsCustomFieldEnumValuesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CFEnumValue, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldGet(entityType: string, customField: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldData, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldPatch(entityType: string, customField: string, customFieldsV2EntityTypeFieldsCustomFieldDeleteRequest?: CustomFieldsV2EntityTypeFieldsCustomFieldDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsCustomFieldRestorePost(entityType: string, customField: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2EntityTypeFieldsGet(entityType: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldData[], any>>;
    customFieldsV2EntityTypeFieldsPost(entityType: string, customFieldsV2EntityTypeFieldsGetRequest: CustomFieldsV2EntityTypeFieldsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldData, any>>;
    customFieldsV2EntityTypeFieldsReorderPost(entityType: string, customFieldsV2EntityTypeFieldsReorderPostRequest: CustomFieldsV2EntityTypeFieldsReorderPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2ValuesEntityCustomFieldGet(entity: string, customField: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldValueData, any>>;
    customFieldsV2ValuesEntityCustomFieldPost(entity: string, customField: string, customFieldsV2ValuesEntityCustomFieldGetRequest: CustomFieldsV2ValuesEntityCustomFieldGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    customFieldsV2ValuesEntityGet(entity: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CustomFieldValueData[], any>>;
    customFieldsV2ValuesEntityPost(entity: string, customFieldsV2ValuesEntityGetRequest: CustomFieldsV2ValuesEntityGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    emojisAddPost(emojisAddPostRequest: EmojisAddPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    emojisDeletePost(emojisDeletePostRequest: EmojisDeletePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    emojisExistsGet(emoji: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<boolean, any>>;
    emojisFrequentlyUsedGet(options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string[], any>>;
    emojisRecordUsagePost(emojisRecordUsagePostRequest: EmojisRecordUsagePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    emojisSearchGet(query: string, $skip?: string, $top?: number, version?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<EmojisSearchGet200Response, any>>;
    externalLinkPatternsDelete(pattern: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    externalLinkPatternsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ExternalLinkPattern[], any>>;
    externalLinkPatternsPost(externalLinkPatternsDeleteRequest: ExternalLinkPatternsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    httpApiModelGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<HAModel, any>>;
    notificationsChannelSubscriptionsGet(channel: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO[], any>>;
    notificationsChannelSubscriptionsIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    notificationsChannelSubscriptionsIdPatch(id: string, $fields?: string, notificationsChannelSubscriptionsIdDeleteRequest?: NotificationsChannelSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO, any>>;
    notificationsChannelSubscriptionsIdRequestMissingRightsPost(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    notificationsChannelSubscriptionsPost(notificationsChannelSubscriptionsGetRequest: NotificationsChannelSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO, any>>;
    notificationsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<EventSubjectInfoDTO[], any>>;
    notificationsPersonalCustomSubscriptionsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO[], any>>;
    notificationsPersonalCustomSubscriptionsIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    notificationsPersonalCustomSubscriptionsIdPatch(id: string, $fields?: string, notificationsPersonalCustomSubscriptionsIdDeleteRequest?: NotificationsPersonalCustomSubscriptionsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO, any>>;
    notificationsPersonalCustomSubscriptionsPost(notificationsPersonalCustomSubscriptionsGetRequest: NotificationsPersonalCustomSubscriptionsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubscriptionDTO, any>>;
    notificationsPersonalSubscriptionsAllPersonalSubscriptionTargetsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PersonalSubscriptionTarget[], any>>;
    notificationsPersonalSubscriptionsPersonalSubscriptionSettingsGet(profile: string, feed: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PersonalSubscriptionSettings, any>>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionSubjectPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPost(notificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest: NotificationsPersonalSubscriptionsUpdatePersonalSubscriptionTargetPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    notificationsPrivateFeedsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PrivateFeed[], any>>;
    notificationsPrivateFeedsIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    notificationsPrivateFeedsIdPatch(id: string, $fields?: string, notificationsPrivateFeedsIdDeleteRequest?: NotificationsPrivateFeedsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PrivateFeed, any>>;
    notificationsPrivateFeedsPost(notificationsPrivateFeedsGetRequest: NotificationsPrivateFeedsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PrivateFeed, any>>;
    organizationDomainsCheckGet(domain: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CDomainStatus, any>>;
    organizationDomainsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<OrgDomainDTO[], any>>;
    organizationDomainsPatch(organizationDomainsGetRequest: OrganizationDomainsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    organizationGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<OrganizationRecord, any>>;
    organizationJetSalesLicenseActivationUrlGet(options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    organizationJetSalesUrlGet(options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    organizationPatch(organizationGetRequest: OrganizationGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesCreatePost(permissionRolesCreatePostRequest: PermissionRolesCreatePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<RoleDTO, any>>;
    permissionRolesGetPost(applicationsApplicationUiExtensionsDisableForEverybodyPatchRequest: ApplicationsApplicationUiExtensionsDisableForEverybodyPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<RoleDTO[], any>>;
    permissionRolesRoleId2FaRequirementGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TwoFactorAuthenticationRequirement, any>>;
    permissionRolesRoleId2FaRequirementPatch(roleId: string, permissionRolesRoleId2FaRequirementGetRequest?: PermissionRolesRoleId2FaRequirementGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdDelete(roleId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdPatch(roleId: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdPermissionsDelete(roleId: string, rightCodes: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdPermissionsGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<RightDTO[], any>>;
    permissionRolesRoleIdPermissionsPost(roleId: string, permissionRolesRoleIdPermissionsDeleteRequest: PermissionRolesRoleIdPermissionsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdProfilesGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile[], any>>;
    permissionRolesRoleIdProfilesProfileDelete(roleId: string, profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdProfilesProfilePost(roleId: string, profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdResetRolePermissionsToDefaultPost(roleId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdTeamsGet(roleId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam[], any>>;
    permissionRolesRoleIdTeamsTeamDelete(roleId: string, team: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionRolesRoleIdTeamsTeamPost(roleId: string, team: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    permissionsCheckPermissionPost(permissionsCheckPermissionPostRequest: PermissionsCheckPermissionPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<boolean, any>>;
    permissionsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DTORightsWithHierarchy, any>>;
    projectsAutomationDslEvaluationsConfigGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DslEvaluationConfig, any>>;
    projectsAutomationGraphExecutionsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<JobExecutionDTO, any>>;
    projectsAutomationGraphExecutionsIdStopPost(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsAutomationJobExecutionsCurrentGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<JobExecutionDTO, any>>;
    projectsAutomationJobsJobIdGet(jobId: string, project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<JobDTO, any>>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyDelete(stepExecId: string, key: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyGet(stepExecId: string, key: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsAutomationStepExecutionsStepExecIdParametersKeyPatch(stepExecId: string, key: string, projectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest: ProjectsAutomationStepExecutionsStepExecIdParametersKeyDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsAutomationStepExecutionsUsedParametersParamParameterIdGet(parameterId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ParameterLastUsageDTO[], any>>;
    projectsAutomationStepExecutionsUsedParametersSecretSecretIdGet(secretId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ParameterLastUsageDTO[], any>>;
    projectsAutomationSubscriptionsLegacyChannelsDelete(project?: string, jobId?: string, unsubscribedOnly?: boolean, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsCollaboratorprofileGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsGet200Response, any>>;
    projectsGet($skip?: string, $top?: number, term?: string, tag?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsGet200Response, any>>;
    projectsMembermemberGet(member: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsGet200Response, any>>;
    projectsParamsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsParamsDefaultBundleGet200Response, any>>;
    projectsParamsDefaultBundlePost(projectsParamsDefaultBundleGetRequest: ProjectsParamsDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsParamsIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsParamsIdPatch(id: string, projectsParamsIdDeleteRequest: ProjectsParamsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsParamsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsParamsDefaultBundleGet200Response, any>>;
    projectsParamsInDefaultBundlePost(projectsParamsInDefaultBundleGetRequest: ProjectsParamsInDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsPlanningBoardsBoardDelete(board: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsBoardGet(board: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BoardRecord, any>>;
    projectsPlanningBoardsBoardIssuesGet(board: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response, any>>;
    projectsPlanningBoardsBoardIssuesIssueDelete(issue: string, board: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsBoardIssuesIssuePost(issue: string, board: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsBoardPatch(board: string, projectsPlanningBoardsBoardDeleteRequest?: ProjectsPlanningBoardsBoardDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsSprintsPost(projectsPlanningBoardsSprintsPostRequest: ProjectsPlanningBoardsSprintsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SprintRecord, any>>;
    projectsPlanningBoardsSprintsSprintArchiveDelete(sprint: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsSprintsSprintIssuesGet(sprint: string, $skip?: string, $top?: number, unresolvedOnly?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response, any>>;
    projectsPlanningBoardsSprintsSprintIssuesIssueDelete(issue: string, sprint: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsSprintsSprintIssuesIssuePost(issue: string, sprint: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsSprintsSprintLaunchPost(sprint: string, projectsPlanningBoardsSprintsSprintLaunchPostRequest: ProjectsPlanningBoardsSprintsSprintLaunchPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPlanningBoardsSprintsSprintPatch(sprint: string, projectsPlanningBoardsSprintsSprintPatchRequest?: ProjectsPlanningBoardsSprintsSprintPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsPost(projectsGetRequest: ProjectsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PRProject, any>>;
    projectsPrivateProjectsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PRPrivateProject[], any>>;
    projectsPrivateProjectsProjectRequestAccessPost(project: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessAdminsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile[], any>>;
    projectsProjectAccessAdminsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessAdminsProfilesProfileDelete(project: string, profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessAdminsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessAdminsTeamsTeamIdDelete(project: string, teamId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessCollaboratorsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile[], any>>;
    projectsProjectAccessCollaboratorsProfilesDelete(project: string, profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessCollaboratorsProfilesGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile[], any>>;
    projectsProjectAccessCollaboratorsProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessCollaboratorsTeamsDelete(project: string, teamId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessCollaboratorsTeamsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam[], any>>;
    projectsProjectAccessCollaboratorsTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessMemberProfilesGet(project: string, $skip?: string, $top?: number, query?: string, includingAdmins?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersUsersDelete200Response, any>>;
    projectsProjectAccessMembersProfilesPost(project: string, chatsChannelsChannelAdministratorGetRequest: ChatsChannelsChannelAdministratorGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessMembersProfilesProfileDelete(project: string, profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessMembersTeamsPost(project: string, projectsProjectAccessAdminsTeamsPostRequest: ProjectsProjectAccessAdminsTeamsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessMembersTeamsTeamIdDelete(project: string, teamId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAccessViewersGet(project: string, term: string, $skip?: string, $top?: number, meOnTop?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersUsersDelete200Response, any>>;
    projectsProjectAutomationDeploymentTargetsFavoritesGet(project: string, sortBy?: string, sortOrder?: ColumnSortOrder, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeployTargetRecord[], any>>;
    projectsProjectAutomationDeploymentTargetsGet(project: string, search?: string, sortBy?: string, sortOrder?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectAutomationDeploymentTargetsGet200Response, any>>;
    projectsProjectAutomationDeploymentTargetsPost(project: string, projectsProjectAutomationDeploymentTargetsGetRequest: ProjectsProjectAutomationDeploymentTargetsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeployTargetRecord, any>>;
    projectsProjectAutomationDeploymentTargetsTargetDelete(project: string, target: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAutomationDeploymentTargetsTargetGet(project: string, target: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeployTargetRecord, any>>;
    projectsProjectAutomationDeploymentTargetsTargetPut(project: string, target: string, projectsProjectAutomationDeploymentTargetsTargetDeleteRequest?: ProjectsProjectAutomationDeploymentTargetsTargetDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAutomationDeploymentsFailPost(project: string, projectsProjectAutomationDeploymentsFailPostRequest: ProjectsProjectAutomationDeploymentsFailPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeploymentRecord, any>>;
    projectsProjectAutomationDeploymentsFinishPost(project: string, projectsProjectAutomationDeploymentsFinishPostRequest: ProjectsProjectAutomationDeploymentsFinishPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeploymentRecord, any>>;
    projectsProjectAutomationDeploymentsGet(project: string, targetIdentifier?: string, jobExecutionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectAutomationDeploymentsGet200Response, any>>;
    projectsProjectAutomationDeploymentsPut(project: string, projectsProjectAutomationDeploymentsGetRequest: ProjectsProjectAutomationDeploymentsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeploymentRecord, any>>;
    projectsProjectAutomationDeploymentsSchedulePost(project: string, projectsProjectAutomationDeploymentsSchedulePostRequest: ProjectsProjectAutomationDeploymentsSchedulePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeploymentRecord, any>>;
    projectsProjectAutomationDeploymentsStartPost(project: string, projectsProjectAutomationDeploymentsStartPostRequest: ProjectsProjectAutomationDeploymentsStartPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeploymentRecord, any>>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierDelete(project: string, targetIdentifier: string, deploymentIdentifier: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectAutomationDeploymentsTargetIdentifierDeploymentIdentifierGet(project: string, targetIdentifier: string, deploymentIdentifier: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DeploymentRecord, any>>;
    projectsProjectAutomationGraphExecutionsGet(project: string, jobId: string, branchFilter?: string, statusFilter?: ExecutionStatus, jobTriggerFilter?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectAutomationGraphExecutionsGet200Response, any>>;
    projectsProjectAutomationJobsGet(project: string, repoFilter: string, branchFilter: string, trigger?: JobTriggerType, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectAutomationJobsGet200Response, any>>;
    projectsProjectAutomationJobsJobIdStartPost(project: string, jobId: string, projectsProjectAutomationJobsJobIdStartPostRequest: ProjectsProjectAutomationJobsJobIdStartPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<LaunchResult, any>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPost(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdAcceptSuggestedEditPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdRejectSuggestedEditPost(project: string, discussionId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdReopenSuggestedEditPost(project: string, discussionId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatch(project: string, discussionId: string, projectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest: ProjectsProjectCodeReviewsCodeDiscussionsDiscussionIdSuggestedEditPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsCodeDiscussionsPost(project: string, projectsProjectCodeReviewsCodeDiscussionsPostRequest: ProjectsProjectCodeReviewsCodeDiscussionsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CodeDiscussionRecord, any>>;
    projectsProjectCodeReviewsCommitSetReviewPost(project: string, projectsProjectCodeReviewsCommitSetReviewPostRequest: ProjectsProjectCodeReviewsCommitSetReviewPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CommitSetReviewRecord, any>>;
    projectsProjectCodeReviewsGet(project: string, $skip?: string, $top?: number, state?: CodeReviewStateFilter, text?: string, author?: string, from?: string, to?: string, sort?: ReviewSorting, reviewer?: string, type?: ReviewType, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectCodeReviewsGet200Response, any>>;
    projectsProjectCodeReviewsMergeRequestsPost(project: string, projectsProjectCodeReviewsMergeRequestsPostRequest: ProjectsProjectCodeReviewsMergeRequestsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MergeRequestRecord, any>>;
    projectsProjectCodeReviewsReviewIdDescriptionPatch(project: string, reviewId: string, chatsChannelsChannelDescriptionPatchRequest: ChatsChannelsChannelDescriptionPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsReviewIdDetailsGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CodeReviewDetailedInfo, any>>;
    projectsProjectCodeReviewsReviewIdFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectCodeReviewsReviewIdFilesGet200Response, any>>;
    projectsProjectCodeReviewsReviewIdGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CodeReviewRecord, any>>;
    projectsProjectCodeReviewsReviewIdMergeFilesGet(project: string, reviewId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectCodeReviewsReviewIdMergeFilesGet200Response, any>>;
    projectsProjectCodeReviewsReviewIdMergePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdMergePutRequest: ProjectsProjectCodeReviewsReviewIdMergePutRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitMergeResultHttp, any>>;
    projectsProjectCodeReviewsReviewIdParticipantsUserDelete(project: string, reviewId: string, user: string, role: CodeReviewParticipantRole, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsReviewIdParticipantsUserPost(project: string, reviewId: string, user: string, projectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest: ProjectsProjectCodeReviewsReviewIdParticipantsUserDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsReviewIdRebasePut(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRebasePutRequest: ProjectsProjectCodeReviewsReviewIdRebasePutRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitRebaseResultHttp, any>>;
    projectsProjectCodeReviewsReviewIdRevisionsDelete(project: string, reviewId: string, revisions: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsReviewIdRevisionsPost(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdRevisionsDeleteRequest: ProjectsProjectCodeReviewsReviewIdRevisionsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsReviewIdStatePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdStatePatchRequest: ProjectsProjectCodeReviewsReviewIdStatePatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectCodeReviewsReviewIdSuggestedReviewersGet(project: string, reviewId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile[], any>>;
    projectsProjectCodeReviewsReviewIdTitlePatch(project: string, reviewId: string, projectsProjectCodeReviewsReviewIdTitlePatchRequest: ProjectsProjectCodeReviewsReviewIdTitlePatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDelete(project: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDocumentsDocumentIdCopyPost(project: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    projectsProjectDocumentsDocumentIdDelete(project: string, documentId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDocumentsDocumentIdDeleteForeverDelete(project: string, documentId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDocumentsDocumentIdGet(project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    projectsProjectDocumentsDocumentIdMovePatch(project: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    projectsProjectDocumentsDocumentIdPatch(project: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    projectsProjectDocumentsDocumentIdUnarchivePatch(project: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    projectsProjectDocumentsFoldersFolderDelete(project: string, folder: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDocumentsFoldersFolderDocumentsGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response, any>>;
    projectsProjectDocumentsFoldersFolderGet(project: string, folder: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DocumentFolder, any>>;
    projectsProjectDocumentsFoldersFolderIntroductionDelete(project: string, folder: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDocumentsFoldersFolderIntroductionDocumentIdPatch(project: string, folder: string, documentId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDocumentsFoldersFolderMovePatch(project: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DocumentFolder, any>>;
    projectsProjectDocumentsFoldersFolderPatch(project: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectDocumentsFoldersFolderSubfoldersGet(project: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response, any>>;
    projectsProjectDocumentsFoldersPost(project: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DocumentFolder, any>>;
    projectsProjectDocumentsPost(project: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    projectsProjectGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PRProject, any>>;
    projectsProjectPackagesRepositoriesGet(project: string, type?: string, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectPackageRepository[], any>>;
    projectsProjectPackagesRepositoriesPost(project: string, projectsProjectPackagesRepositoriesGetRequest: ProjectsProjectPackagesRepositoriesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectPackageRepository, any>>;
    projectsProjectPackagesRepositoriesRepositoryCleanupDryPost(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest: ProjectsProjectPackagesRepositoriesRepositoryCleanupDryPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DryCleanupResults, any>>;
    projectsProjectPackagesRepositoriesRepositoryCleanupPost(project: string, repository: string, $fields?: string, projectsProjectPackagesRepositoriesRepositoryCleanupPostRequest?: ProjectsProjectPackagesRepositoriesRepositoryCleanupPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PackagesExecutionResult, any>>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet(project: string, repository: string, connectionId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGet200Response, any>>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishPost(project: string, repository: string, connectionId: string, projectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest: ProjectsProjectPackagesRepositoriesRepositoryConnectionsConnectionIdPublishGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsProjectPackagesRepositoriesRepositoryConnectionsGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PackageRepositoryConnection[], any>>;
    projectsProjectPackagesRepositoriesRepositoryDelete(project: string, repository: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryFilesFolderfolderPathDelete(project: string, repository: string, folderPath: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryFilesGet(project: string, repository: string, parentPath?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<FileData[], any>>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathDelete(project: string, repository: string, filePath: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryFilesNamefilePathGet(project: string, repository: string, filePath: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<FileDetails, any>>;
    projectsProjectPackagesRepositoriesRepositoryGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectPackageRepository, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesGet(project: string, repository: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameDelete(project: string, repository: string, packageName: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataGet(project: string, repository: string, packageName: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PackageMetadata, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataPut(project: string, repository: string, packageName: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest?: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameMetadataVersionpackageVersionPut(project: string, repository: string, packageName: string, packageVersion: string, projectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest: ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameMetadataVersionPackageVersionPutRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsGet(project: string, repository: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, connectionId?: string, pinned?: boolean, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, repository: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryPackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, repository: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PackageVersionDetails, any>>;
    projectsProjectPackagesRepositoriesRepositoryPatch(project: string, repository: string, projectsProjectPackagesRepositoriesRepositoryDeleteRequest?: ProjectsProjectPackagesRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesRepositoryUrlGet(project: string, repository: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesGet(project: string, type: string, repositoryName: string, query: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPackagesRepositoriesRepositoryPackagesGet200Response, any>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsGet(project: string, type: string, repositoryName: string, packageName: string, query: string, sortColumn: PackagesSortColumn, sortOrder: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response, any>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionDelete(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPackagesRepositoriesTypetypeRepositoryrepositoryNamePackagesNamepackageNameVersionsVersionpackageVersionGet(project: string, type: string, repositoryName: string, packageName: string, packageVersion: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PackageVersionDetails, any>>;
    projectsProjectPackagesSearchGet(project: string, type: string, query: string, connectionId?: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPackagesRepositoriesRepositoryPackagesNamePackageNameVersionsGet200Response, any>>;
    projectsProjectPackagesTypesGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PackageType[], any>>;
    projectsProjectPatch(project: string, $fields?: string, projectsProjectDeleteRequest?: ProjectsProjectDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PRProject, any>>;
    projectsProjectPlanningBoardsBoardArchiveDelete(project: string, board: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningBoardsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPlanningBoardsGet200Response, any>>;
    projectsProjectPlanningBoardsPost(project: string, projectsProjectPlanningBoardsGetRequest: ProjectsProjectPlanningBoardsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BoardRecord, any>>;
    projectsProjectPlanningBoardsSprintsGet(project: string, $skip?: string, $top?: number, board?: string, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPlanningBoardsSprintsGet200Response, any>>;
    projectsProjectPlanningBoardsStarredGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BoardRecord[], any>>;
    projectsProjectPlanningChecklistsChecklistIdDelete(project: string, checklistId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningChecklistsChecklistIdFullChecklistTreeGet(project: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PlanItemChildren[], any>>;
    projectsProjectPlanningChecklistsChecklistIdImportPost(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningChecklistsChecklistIdPatch(project: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdDeleteRequest?: ProjectsProjectPlanningChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningChecklistsGet(project: string, $skip?: string, $top?: number, query?: string, sorting?: ChecklistSorting, descending?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPlanningChecklistsGet200Response, any>>;
    projectsProjectPlanningChecklistsImportPost(project: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Checklist, any>>;
    projectsProjectPlanningChecklistsPost(project: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Checklist, any>>;
    projectsProjectPlanningChecklistsStarredGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Checklist[], any>>;
    projectsProjectPlanningIssuesGet(project: string, sorting: IssuesSorting, descending: boolean, $skip?: string, $top?: number, assigneeId?: Array<string>, createdByProfileId?: string, createdBy?: Array<string>, statuses?: Array<string>, tagId?: string, query?: string, tags?: Array<string>, sprints?: Array<string>, boards?: Array<string>, customFields?: Array<string>, importTransaction?: string, creationTimeFrom?: string, creationTimeTo?: string, dueDateFrom?: string, dueDateTo?: string, topics?: Array<string>, grouping?: IssueListGrouping, deployment?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsPlanningBoardsSprintsSprintIssuesGet200Response, any>>;
    projectsProjectPlanningIssuesImportPost(project: string, projectsProjectPlanningIssuesImportPostRequest: ProjectsProjectPlanningIssuesImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<IssueImportResult, any>>;
    projectsProjectPlanningIssuesIssueIdAttachmentsDelete(project: string, issueId: string, identities: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdAttachmentsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdAttachmentsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdDelete(project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdChecklistsChecklistIdPost(project: string, issueId: string, checklistId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsDelete(project: string, issueId: string, codeReviewIds: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdCodeReviewsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCodeReviewsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdCommentsImportPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommentsImportPostRequest: ProjectsProjectPlanningIssuesIssueIdCommentsImportPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string[], any>>;
    projectsProjectPlanningIssuesIssueIdCommitsDelete(project: string, issueId: string, repository: string, commitIds: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdCommitsPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdCommitsDeleteRequest: ProjectsProjectPlanningIssuesIssueIdCommitsDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdDelete(project: string, issueId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdGet(project: string, issueId: string, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Issue, any>>;
    projectsProjectPlanningIssuesIssueIdPatch(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdDeleteRequest?: ProjectsProjectPlanningIssuesIssueIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdRestorePost(project: string, issueId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdDelete(project: string, issueId: string, tagId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdTagsTagIdPost(project: string, issueId: string, tagId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesIssueIdToggleResolvedPost(project: string, issueId: string, projectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest: ProjectsProjectPlanningIssuesIssueIdToggleResolvedPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesNumbernumberGet(project: string, number: number, resolveAlias?: boolean, withDeleted?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Issue, any>>;
    projectsProjectPlanningIssuesPost(project: string, projectsProjectPlanningIssuesGetRequest: ProjectsProjectPlanningIssuesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Issue, any>>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<IssueStatus, any>>;
    projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergePatch(project: string, projectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest?: ProjectsProjectPlanningIssuesStatusesAutoUpdateOnMergeRequestMergeGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningIssuesStatusesDistributionGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<IssueStatusWithUsages[], any>>;
    projectsProjectPlanningIssuesStatusesGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<IssueStatus[], any>>;
    projectsProjectPlanningIssuesStatusesPatch(project: string, projectsProjectPlanningIssuesStatusesGetRequest: ProjectsProjectPlanningIssuesStatusesGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectPlanningTagsGet(project: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectPlanningTagsGet200Response, any>>;
    projectsProjectPlanningTagsPost(project: string, projectsProjectPlanningTagsGetRequest: ProjectsProjectPlanningTagsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PlanningTag, any>>;
    projectsProjectRepositoriesRepositoryCherryPickCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCherryPickCommitPostRequest: ProjectsProjectRepositoriesRepositoryCherryPickCommitPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitCherryPickResult, any>>;
    projectsProjectRepositoriesRepositoryCommitBranchesGet(project: string, repository: string, commit: string, prefix?: string, limit?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BranchInfo[], any>>;
    projectsProjectRepositoriesRepositoryCommitPost(project: string, repository: string, projectsProjectRepositoriesRepositoryCommitPostRequest: ProjectsProjectRepositoriesRepositoryCommitPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitCommitResult, any>>;
    projectsProjectRepositoriesRepositoryCommitsGet(project: string, repository: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectRepositoriesRepositoryCommitsGet200Response, any>>;
    projectsProjectRepositoriesRepositoryDelete(project: string, repository: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectRepositoriesRepositoryDeleteBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryDeleteBranchPostRequest: ProjectsProjectRepositoriesRepositoryDeleteBranchPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectRepositoriesRepositoryFilesGet(project: string, repository: string, commit: string, path: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitFile[], any>>;
    projectsProjectRepositoriesRepositoryGcPost(project: string, repository: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectRepositoriesRepositoryMergeBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryMergeBranchPostRequest: ProjectsProjectRepositoriesRepositoryMergeBranchPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitMergeBranchResult, any>>;
    projectsProjectRepositoriesRepositoryPost(project: string, repository: string, $fields?: string, projectsProjectRepositoriesRepositoryDeleteRequest?: ProjectsProjectRepositoriesRepositoryDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PRRepositoryInfo, any>>;
    projectsProjectRepositoriesRepositoryReadonlyGet(project: string, repository: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<boolean, any>>;
    projectsProjectRepositoriesRepositoryReadonlyPost(project: string, repository: string, projectsProjectRepositoriesRepositoryReadonlyGetRequest: ProjectsProjectRepositoriesRepositoryReadonlyGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectRepositoriesRepositoryRebaseBranchPost(project: string, repository: string, projectsProjectRepositoriesRepositoryRebaseBranchPostRequest: ProjectsProjectRepositoriesRepositoryRebaseBranchPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitRebaseResult, any>>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGet(project: string, repository: string, revision: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ExternalCheckDTO[], any>>;
    projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksPost(project: string, repository: string, revision: string, projectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest: ProjectsProjectRepositoriesRepositoryRevisionsRevisionExternalChecksGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectRepositoriesRepositorySettingsGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GitRepositorySettings, any>>;
    projectsProjectRepositoriesRepositorySettingsPost(project: string, repository: string, projectsProjectRepositoriesRepositorySettingsGetRequest: ProjectsProjectRepositoriesRepositorySettingsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectRepositoriesRepositoryUrlGet(project: string, repository: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<RepositoryUrls, any>>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDelete(project: string, responsibilityId: string, profileId: string, role?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdPost(project: string, responsibilityId: string, profileId: string, projectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest?: ProjectsProjectResponsibilitiesResponsibilityIdAssigneesProfileIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectResponsibilitiesSchemeGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SubjectResponsibilitiesTable[], any>>;
    projectsProjectResponsibilitiesSubjectsPost(project: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsProjectResponsibilitiesSubjectsSubjectIdPatch(project: string, subjectId: string, projectsProjectResponsibilitiesSubjectsPostRequest?: ProjectsProjectResponsibilitiesSubjectsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectTopicsGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Topic[], any>>;
    projectsProjectTopicsIdPatch(project: string, id: string, $fields?: string, projectsProjectTopicsIdPatchRequest?: ProjectsProjectTopicsIdPatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Topic, any>>;
    projectsProjectTopicsPost(project: string, projectsProjectTopicsGetRequest: ProjectsProjectTopicsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Topic, any>>;
    projectsProjectTopicsRemoveTopicsPost(project: string, projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsProjectTopicsSetResponsiblePost(project: string, projectsProjectTopicsSetResponsiblePostRequest: ProjectsProjectTopicsSetResponsiblePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsRepositoriesFindGet(term: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsRepositoriesFindGet200Response, any>>;
    projectsResponsibilitiesPost(projectsResponsibilitiesPostRequest: ProjectsResponsibilitiesPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsResponsibilitiesResponsibilityIdDelete(responsibilityId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsResponsibilitiesResponsibilityIdPatch(responsibilityId: string, projectsResponsibilitiesResponsibilityIdDeleteRequest: ProjectsResponsibilitiesResponsibilityIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsResponsibilitiesSubjectsSubjectIdDelete(subjectId: string, project?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsRightCoderightCodeGet(rightCode: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsGet200Response, any>>;
    projectsRightUniqueCoderightGet(right: string, $skip?: string, $top?: number, term?: string, path?: string, starred?: boolean, _private?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsGet200Response, any>>;
    projectsSecretsDefaultBundleGet(project: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsSecretsDefaultBundleGet200Response, any>>;
    projectsSecretsDefaultBundlePost(projectsSecretsDefaultBundleGetRequest: ProjectsSecretsDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsSecretsIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsSecretsIdPatch(id: string, projectsSecretsIdDeleteRequest: ProjectsSecretsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsSecretsInDefaultBundleGet(projectId: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsSecretsDefaultBundleGet200Response, any>>;
    projectsSecretsInDefaultBundlePost(projectsSecretsInDefaultBundleGetRequest: ProjectsSecretsInDefaultBundleGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    projectsTagsGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PRTag[], any>>;
    projectsTagsTrackAccessPost(projectsTagsTrackAccessPostRequest: ProjectsTagsTrackAccessPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsTeamteamGet(team: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsGet200Response, any>>;
    projectsVaultGet(project: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<VaultConnectionRecord[], any>>;
    projectsVaultIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsVaultIdPatch(id: string, projectsVaultIdDeleteRequest: ProjectsVaultIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    projectsVaultPost(projectsVaultGetRequest: ProjectsVaultGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    publicHolidaysCalendarsGet($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHolidaysCalendarsGet200Response, any>>;
    publicHolidaysCalendarsIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    publicHolidaysCalendarsIdPatch(id: string, publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHolidayCalendarRecord, any>>;
    publicHolidaysCalendarsImportPost(publicHolidaysCalendarsImportPostRequest: PublicHolidaysCalendarsImportPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    publicHolidaysCalendarsPost(publicHolidaysCalendarsGetRequest: PublicHolidaysCalendarsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHolidayCalendarRecord, any>>;
    publicHolidaysHolidaysGet($skip?: string, $top?: number, calendar?: string, location?: string, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHolidaysHolidaysGet200Response, any>>;
    publicHolidaysHolidaysIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    publicHolidaysHolidaysIdPatch(id: string, $fields?: string, publicHolidaysHolidaysIdDeleteRequest?: PublicHolidaysHolidaysIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHoliday, any>>;
    publicHolidaysHolidaysPost(publicHolidaysHolidaysGetRequest: PublicHolidaysHolidaysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHoliday, any>>;
    publicHolidaysHolidaysProfileHolidaysGet(startDate: string, endDate: string, profile: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHoliday[], any>>;
    publicHolidaysHolidaysRelatedHolidaysGet($skip?: string, $top?: number, startDate?: string, endDate?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PublicHolidaysHolidaysGet200Response, any>>;
    reactionsItemEmojiDelete(item: string, emoji: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    reactionsItemEmojiGet(item: string, emoji: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CPrincipal[], any>>;
    reactionsItemEmojiPost(item: string, emoji: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    reactionsItemGet(item: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<EmojiReaction[], any>>;
    richTextParseMarkdownPost(richTextParseMarkdownPostRequest: RichTextParseMarkdownPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<RtDocument, any>>;
    teamDirectoryCalendarEventsAbsenceEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<AbsenceEvent[], any>>;
    teamDirectoryCalendarEventsBirthdayEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BirthdayEvent[], any>>;
    teamDirectoryCalendarEventsBirthdayEventsStarredGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<BirthdayEvent[], any>>;
    teamDirectoryCalendarEventsGet(dateFrom: string, dateTo: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingRecord[], any>>;
    teamDirectoryCalendarEventsHolidaysGet(startDate: string, endDate: string, team?: string, location?: string, role?: string, workingDays?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<HolidaysEvent[], any>>;
    teamDirectoryCalendarEventsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingRecord, any>>;
    teamDirectoryCalendarEventsMeetingParticipationsIdPatch(id: string, calendarsEventParticipationsIdPatchRequest: CalendarsEventParticipationsIdPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MeetingRecord, any>>;
    teamDirectoryCalendarEventsMembershipEventsGet(dateFrom: string, dateTo: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<MembershipEvent[], any>>;
    teamDirectoryCalendarEventsNonWorkingDaysEventsGet(dateFrom: string, dateTo: string, member?: string, team?: string, location?: string, role?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<NonWorkingDaysEvent[], any>>;
    teamDirectoryInvitationLinksGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryInvitationLinksGet200Response, any>>;
    teamDirectoryInvitationLinksInvitationLinkIdDelete(invitationLinkId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryInvitationLinksInvitationLinkIdPatch(invitationLinkId: string, teamDirectoryInvitationLinksInvitationLinkIdDeleteRequest?: TeamDirectoryInvitationLinksInvitationLinkIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryInvitationLinksPost(teamDirectoryInvitationLinksGetRequest: TeamDirectoryInvitationLinksGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryInvitationLinksGet200Response1, any>>;
    teamDirectoryInvitationsGet($skip?: string, $top?: number, withDeleted?: boolean, projects?: Array<string>, teams?: Array<string>, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryInvitationsGet200Response, any>>;
    teamDirectoryInvitationsIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryInvitationsIdPatch(id: string, teamDirectoryInvitationsIdDeleteRequest?: TeamDirectoryInvitationsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryInvitationsPost(teamDirectoryInvitationsGetRequest: TeamDirectoryInvitationsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Invitation, any>>;
    teamDirectoryLanguagesGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLanguage[], any>>;
    teamDirectoryLocationEquipmentTypesGet(withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocationEquipmentType[], any>>;
    teamDirectoryLocationEquipmentTypesNamenameDelete(name: string, _delete: boolean, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryLocationMapMemberPointsGet(locationId: string, $skip?: string, $top?: number, includeUnmarked?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryLocationMapMemberPointsGet200Response, any>>;
    teamDirectoryLocationMapMemberPointsLocationPointIdDelete(locationPointId: string, _delete: boolean, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryLocationMapMemberPointsLocationPointIdPatch(locationPointId: string, $fields?: string, teamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest?: TeamDirectoryLocationMapMemberPointsLocationPointIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocationMapPoint, any>>;
    teamDirectoryLocationMapMemberPointsPost(teamDirectoryLocationMapMemberPointsGetRequest: TeamDirectoryLocationMapMemberPointsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocationMapPoint, any>>;
    teamDirectoryLocationsGet(query?: string, type?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocation[], any>>;
    teamDirectoryLocationsIdDelete(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocation[], any>>;
    teamDirectoryLocationsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocation, any>>;
    teamDirectoryLocationsIdMapGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocationMap, any>>;
    teamDirectoryLocationsIdMapPatch(id: string, teamDirectoryLocationsIdMapGetRequest: TeamDirectoryLocationsIdMapGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocationMap, any>>;
    teamDirectoryLocationsIdPatch(id: string, $fields?: string, teamDirectoryLocationsIdDeleteRequest?: TeamDirectoryLocationsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocation, any>>;
    teamDirectoryLocationsIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocation, any>>;
    teamDirectoryLocationsPost(teamDirectoryLocationsGetRequest: TeamDirectoryLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocation, any>>;
    teamDirectoryLocationsRestorePost(projectsProjectTopicsRemoveTopicsPostRequest: ProjectsProjectTopicsRemoveTopicsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocation[], any>>;
    teamDirectoryLocationsWithTimezoneGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDLocationWithTimeZone[], any>>;
    teamDirectoryMemberLocationsGet($skip?: string, $top?: number, profiles?: Array<string>, locations?: Array<string>, since?: string, till?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryMemberLocationsGet200Response, any>>;
    teamDirectoryMemberLocationsMemberLocationIdDelete(memberLocationId: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryMemberLocationsMemberLocationIdGet(memberLocationId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberLocation, any>>;
    teamDirectoryMemberLocationsMemberLocationIdPatch(memberLocationId: string, $fields?: string, teamDirectoryMemberLocationsMemberLocationIdDeleteRequest?: TeamDirectoryMemberLocationsMemberLocationIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberLocation, any>>;
    teamDirectoryMemberLocationsPost(teamDirectoryMemberLocationsGetRequest: TeamDirectoryMemberLocationsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberLocation, any>>;
    teamDirectoryMembershipEventsGet($skip?: string, $top?: number, teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryMembershipEventsGet200Response, any>>;
    teamDirectoryMembershipsGet($skip?: string, $top?: number, identifiers?: Array<string>, profiles?: Array<string>, teams?: Array<string>, directTeams?: boolean, roles?: Array<string>, directRoles?: boolean, since?: string, till?: string, requiresApproval?: boolean, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryMembershipsGet200Response, any>>;
    teamDirectoryMembershipsManagerCandidatesGet(term: string, $skip?: string, $top?: number, teamId?: string, excludedMemberId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersUsersDelete200Response, any>>;
    teamDirectoryMembershipsMembershipIdDelete(membershipId: string, _delete?: boolean, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryMembershipsMembershipIdGet(membershipId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMembership, any>>;
    teamDirectoryMembershipsMembershipIdPatch(membershipId: string, $fields?: string, teamDirectoryMembershipsMembershipIdDeleteRequest?: TeamDirectoryMembershipsMembershipIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMembership, any>>;
    teamDirectoryMembershipsMembershipIdRequestRevokePatch(membershipId: string, teamDirectoryMembershipsMembershipIdRequestRevokePatchRequest: TeamDirectoryMembershipsMembershipIdRequestRevokePatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryMembershipsMembershipIdRevokeDelete(membershipId: string, till?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryMembershipsPost(teamDirectoryMembershipsGetRequest: TeamDirectoryMembershipsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMembership, any>>;
    teamDirectoryMembershipsRequestsGet($skip?: string, $top?: number, teamId?: string, direct?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryMembershipsRequestsGet200Response, any>>;
    teamDirectoryMembershipsRequestsMembershipRequestIdDelete(membershipRequestId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMembership, any>>;
    teamDirectoryMembershipsRequestsMembershipRequestIdPatch(membershipRequestId: string, teamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest: TeamDirectoryMembershipsRequestsMembershipRequestIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesAuthenticationSessionsOwnerGet(owner: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESAuthenticationSession[], any>>;
    teamDirectoryProfilesAuthenticationSessionsOwnerSessionIdDelete(owner: string, sessionId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesDashboardsDashboardGet(dashboard: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DashboardPreferencesRecord, any>>;
    teamDirectoryProfilesDashboardsDashboardPatch(dashboard: string, teamDirectoryProfilesDashboardsDashboardGetRequest: TeamDirectoryProfilesDashboardsDashboardGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesEmailemailGet(email: string, verified?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    teamDirectoryProfilesGet($skip?: string, $top?: number, query?: string, reportPastMembers?: boolean, reportFutureMembers?: boolean, reportCurrentMembers?: boolean, teamId?: string, locationId?: string, roleId?: string, meOnTop?: boolean, order?: ProfileOrder, orgRelation?: ProfileOrgRelation, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersUsersDelete200Response, any>>;
    teamDirectoryProfilesOauthConsentsOwnerApplicationsApplicationDelete(owner: string, application: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesOauthConsentsOwnerApprovedScopesIdDelete(owner: string, id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesOauthConsentsOwnerGet(owner: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ESOAuthConsent[], any>>;
    teamDirectoryProfilesOauthConsentsOwnerInternalApplicationsClientIdDelete(owner: string, clientId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesOauthConsentsOwnerRefreshTokensIdDelete(owner: string, id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesPost(teamDirectoryProfilesGetRequest: TeamDirectoryProfilesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    teamDirectoryProfilesProfile2FaRequirementsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Profile2FARequirement[], any>>;
    teamDirectoryProfilesProfile2FaStatusGet(profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TwoFactorAuthenticationStatus, any>>;
    teamDirectoryProfilesProfile2FaTotpConfirmPost(profile: string, teamDirectoryProfilesProfile2FaTotpConfirmPostRequest: TeamDirectoryProfilesProfile2FaTotpConfirmPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfile2FaTotpDelete(profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfile2FaTotpPatch(profile: string, administrationUserAgreementEnabledGetRequest: AdministrationUserAgreementEnabledGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfile2FaTotpPost(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TwoFactorAuthenticationSecret, any>>;
    teamDirectoryProfilesProfileApplicationPasswordsGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response, any>>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDelete(profile: string, passwordId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileApplicationPasswordsPasswordIdPatch(profile: string, passwordId: string, teamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest?: TeamDirectoryProfilesProfileApplicationPasswordsPasswordIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileApplicationPasswordsPost(profile: string, teamDirectoryProfilesProfileApplicationPasswordsGetRequest: TeamDirectoryProfilesProfileApplicationPasswordsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryProfilesProfileApplicationPasswordsGet200Response1, any>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdDelete(profile: string, checklistId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdFullChecklistTreeGet(profile: string, checklistId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PlanItemChildren[], any>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdImportPost(profile: string, checklistId: string, projectsProjectPlanningChecklistsChecklistIdImportPostRequest: ProjectsProjectPlanningChecklistsChecklistIdImportPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileChecklistsChecklistIdPatch(profile: string, checklistId: string, teamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest?: TeamDirectoryProfilesProfileChecklistsChecklistIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileChecklistsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Checklist[], any>>;
    teamDirectoryProfilesProfileChecklistsImportPost(profile: string, projectsProjectPlanningChecklistsImportPostRequest: ProjectsProjectPlanningChecklistsImportPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Checklist, any>>;
    teamDirectoryProfilesProfileChecklistsPost(profile: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Checklist, any>>;
    teamDirectoryProfilesProfileChecklistsStarredGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Checklist[], any>>;
    teamDirectoryProfilesProfileConvertToGuestPatch(profile: string, teamDirectoryProfilesProfileConvertToGuestPatchRequest: TeamDirectoryProfilesProfileConvertToGuestPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DryRunResult, any>>;
    teamDirectoryProfilesProfileConvertToMemberPatch(profile: string, teamDirectoryProfilesProfileConvertToMemberPatchRequest: TeamDirectoryProfilesProfileConvertToMemberPatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DryRunResult, any>>;
    teamDirectoryProfilesProfileDeactivateDelete(profile: string, at: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    teamDirectoryProfilesProfileDelete(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdCopyPost(profile: string, documentId: string, projectsProjectDocumentsDocumentIdCopyPostRequest: ProjectsProjectDocumentsDocumentIdCopyPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDelete(profile: string, documentId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdDeleteForeverDelete(profile: string, documentId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdGet(profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdMovePatch(profile: string, documentId: string, projectsProjectDocumentsDocumentIdMovePatchRequest: ProjectsProjectDocumentsDocumentIdMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdPatch(profile: string, documentId: string, $fields?: string, projectsProjectDocumentsDocumentIdDeleteRequest?: ProjectsProjectDocumentsDocumentIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    teamDirectoryProfilesProfileDocumentsDocumentIdUnarchivePatch(profile: string, documentId: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDelete(profile: string, folder: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderDocumentsGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectDocumentsFoldersFolderDocumentsGet200Response, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderGet(profile: string, folder: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DocumentFolder, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDelete(profile: string, folder: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderIntroductionDocumentIdPatch(profile: string, folder: string, documentId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderMovePatch(profile: string, folder: string, projectsProjectDocumentsFoldersFolderMovePatchRequest: ProjectsProjectDocumentsFoldersFolderMovePatchRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DocumentFolder, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderPatch(profile: string, folder: string, chatsChannelsIsNameFreePostRequest: ChatsChannelsIsNameFreePostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersFolderSubfoldersGet(profile: string, folder: string, withArchived?: boolean, sortBy?: string, order?: ColumnSortOrder, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ProjectsProjectDocumentsFoldersFolderSubfoldersGet200Response, any>>;
    teamDirectoryProfilesProfileDocumentsFoldersPost(profile: string, projectsProjectDocumentsFoldersPostRequest: ProjectsProjectDocumentsFoldersPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<DocumentFolder, any>>;
    teamDirectoryProfilesProfileDocumentsPost(profile: string, projectsProjectDocumentsPostRequest: ProjectsProjectDocumentsPostRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<Document, any>>;
    teamDirectoryProfilesProfileGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    teamDirectoryProfilesProfileGpgKeysFingerprintDelete(profile: string, fingerprint: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileGpgKeysFingerprintPatch(profile: string, fingerprint: string, teamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest?: TeamDirectoryProfilesProfileGpgKeysFingerprintDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileGpgKeysGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GpgKeyData[], any>>;
    teamDirectoryProfilesProfileGpgKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GpgKeyData, any>>;
    teamDirectoryProfilesProfileIsTeamMemberGet(profile: string, teamIds: Array<string>, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<boolean, any>>;
    teamDirectoryProfilesProfileLeadsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile[], any>>;
    teamDirectoryProfilesProfileNavBarMenuItemsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<NavBarMenuItem[], any>>;
    teamDirectoryProfilesProfileNavBarMenuItemsPatch(profile: string, teamDirectoryProfilesProfileNavBarMenuItemsGetRequest: TeamDirectoryProfilesProfileNavBarMenuItemsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileNavBarProjectsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<PRProject[], any>>;
    teamDirectoryProfilesProfileNavBarProjectsPost(profile: string, teamDirectoryProfilesProfileNavBarProjectsGetRequest: TeamDirectoryProfilesProfileNavBarProjectsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileNavBarProjectsProjectDelete(profile: string, project: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileNotificationSettingsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<GlobalNotificationSettings, any>>;
    teamDirectoryProfilesProfileNotificationSettingsPatch(profile: string, teamDirectoryProfilesProfileNotificationSettingsGetRequest?: TeamDirectoryProfilesProfileNotificationSettingsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfilePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileDeleteRequest?: TeamDirectoryProfilesProfileDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    teamDirectoryProfilesProfilePermanentTokensCurrentDelete(profile: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfilePermanentTokensGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryProfilesProfilePermanentTokensGet200Response, any>>;
    teamDirectoryProfilesProfilePermanentTokensPost(profile: string, applicationsApplicationPermanentTokensGetRequest: ApplicationsApplicationPermanentTokensGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryProfilesProfilePermanentTokensGet200Response1, any>>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdDelete(profile: string, tokenId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfilePermanentTokensTokenIdPatch(profile: string, tokenId: string, applicationsApplicationPermanentTokensTokenIdDeleteRequest?: ApplicationsApplicationPermanentTokensTokenIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileReactivatePatch(profile: string, $fields?: string, teamDirectoryProfilesProfileReactivatePatchRequest?: TeamDirectoryProfilesProfileReactivatePatchRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDMemberProfile, any>>;
    teamDirectoryProfilesProfileSettingsGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SettingsValue, any>>;
    teamDirectoryProfilesProfileSettingsPatch(profile: string, teamDirectoryProfilesProfileSettingsGetRequest?: TeamDirectoryProfilesProfileSettingsGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileSpokenLanguagesGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDProfileLanguage[], any>>;
    teamDirectoryProfilesProfileSpokenLanguagesLanguageDelete(profile: string, language: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileSpokenLanguagesPost(profile: string, teamDirectoryProfilesProfileSpokenLanguagesGetRequest: TeamDirectoryProfilesProfileSpokenLanguagesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDProfileLanguage, any>>;
    teamDirectoryProfilesProfileSshKeysFingerprintDelete(profile: string, fingerprint: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileSshKeysGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<SshKeyData[], any>>;
    teamDirectoryProfilesProfileSshKeysPost(profile: string, teamDirectoryProfilesProfileGpgKeysGetRequest: TeamDirectoryProfilesProfileGpgKeysGetRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileTimezoneGet(profile: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ATimeZone, any>>;
    teamDirectoryProfilesProfileWorkingDaysGet(profile: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryProfilesProfileWorkingDaysGet200Response, any>>;
    teamDirectoryProfilesProfileWorkingDaysPost(profile: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDWorkingDays, any>>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdDelete(profile: string, workingDaysId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryProfilesProfileWorkingDaysWorkingDaysIdPatch(profile: string, workingDaysId: string, teamDirectoryProfilesProfileWorkingDaysGetRequest: TeamDirectoryProfilesProfileWorkingDaysGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDWorkingDays, any>>;
    teamDirectoryProfilesWorkingDaysGet($skip?: string, $top?: number, profiles?: Array<string>, since?: string, till?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TeamDirectoryProfilesWorkingDaysGet200Response, any>>;
    teamDirectoryRolesGet(query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDRole[], any>>;
    teamDirectoryRolesIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryRolesIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDRole, any>>;
    teamDirectoryRolesIdPatch(id: string, $fields?: string, teamDirectoryRolesIdDeleteRequest?: TeamDirectoryRolesIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDRole, any>>;
    teamDirectoryRolesIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDRole, any>>;
    teamDirectoryRolesPost(teamDirectoryRolesGetRequest: TeamDirectoryRolesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDRole, any>>;
    teamDirectoryStatsGet(teamId?: string, locationId?: string, roleId?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDStats, any>>;
    teamDirectoryTeamsGet($skip?: string, $top?: number, query?: string, withArchived?: boolean, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersTeamsDelete200Response, any>>;
    teamDirectoryTeamsIdCancelDisbandingPost(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    teamDirectoryTeamsIdDelete(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam[], any>>;
    teamDirectoryTeamsIdDirectMembersGet(id: string, $skip?: string, $top?: number, query?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ChatsChannelsChannelSubscribersUsersDelete200Response, any>>;
    teamDirectoryTeamsIdDisbandDelete(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam[], any>>;
    teamDirectoryTeamsIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam, any>>;
    teamDirectoryTeamsIdPatch(id: string, $fields?: string, teamDirectoryTeamsIdDeleteRequest?: TeamDirectoryTeamsIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam, any>>;
    teamDirectoryTeamsIdRestorePost(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam, any>>;
    teamDirectoryTeamsPost(teamDirectoryTeamsGetRequest: TeamDirectoryTeamsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TDTeam, any>>;
    timeTrackingItemsGet(subject: string, $skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TimeTrackingItemsGet200Response, any>>;
    timeTrackingItemsItemIdDelete(itemId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    timeTrackingItemsItemIdPatch(itemId: string, timeTrackingItemsItemIdDeleteRequest?: TimeTrackingItemsItemIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    timeTrackingItemsPost(timeTrackingItemsGetRequest: TimeTrackingItemsGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TimeTrackingItem, any>>;
    todoGet($skip?: string, $top?: number, open?: boolean, from?: string, till?: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TodoGet200Response, any>>;
    todoIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    todoIdPatch(id: string, todoIdDeleteRequest?: TodoIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    todoPost(todoGetRequest: TodoGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TodoItemRecord, any>>;
    trustedCertificatesGet($fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TrustedCertificate[], any>>;
    trustedCertificatesIdDelete(id: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    trustedCertificatesIdPatch(id: string, trustedCertificatesIdDeleteRequest?: TrustedCertificatesIdDeleteRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    trustedCertificatesInfoGet(data: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<CertificateInfo, any>>;
    trustedCertificatesPost(trustedCertificatesGetRequest: TrustedCertificatesGetRequest, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<TrustedCertificate, any>>;
    unfurlsBlockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    unfurlsBlockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    unfurlsCheckBlockedPost(unfurlsCheckBlockedPostRequest: UnfurlsCheckBlockedPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<boolean, any>>;
    unfurlsListBlockedGet($skip?: string, $top?: number, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<UnfurlsListBlockedGet200Response, any>>;
    unfurlsUnblockUnfurlGlobalPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    unfurlsUnblockUnfurlPost(unfurlsBlockUnfurlPostRequest: UnfurlsBlockUnfurlPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<void, any>>;
    uploadsChatPublicUrlChannelMessageAttachmentIdGet(channel: string, message: string, attachmentId: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
    uploadsImageIdGet(id: string, $fields?: string, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<ImageAttachmentMeta, any>>;
    uploadsPost(uploadsPostRequest: UploadsPostRequest, options?: AxiosRequestConfig): Promise<import("axios").AxiosResponse<string, any>>;
}
